/* cm_ioCR.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "integer.h"
#include "fun.h"

extern MPCR *CR[];
extern POLYCR PCR[];
extern MPMATCR *CRM[];
extern MPMATPCR *PCRM[];

extern char cCR[];
extern char cPCR[];
extern char cCRM[];
extern char cPCRM[];

void FINPUT_CR(char *filename)
/*
 * Reads all M0 complex rational numbers from the specified file into CR[].
 */
{
	FILE *infile;
	unsigned int i, u;
	char *p, *Fgets(), *line;

	infile = Fopen_r(filename);
	for (i = 0; i < M0; i++)
	{
		line = Fgets(infile);
		p = line;
		FREEMPCR(CR[i]);
		CR[i] = INPUTCR(&p, &u);
		free(line);
	}
	fclose(infile);
	return;
}

void FINPUTMAT_CR(char *filename, USI m)
/*
 * complex matrices CRM[m],... are read from the specified file.  If the file
 * could not be opened, a diagnostic is printed.
 */
{
	int i;
	unsigned int j, k;
	FILE *infile;

	infile = Fopen_r(filename);
	fscanf(infile, "%u", &k);
	fgetc(infile);
	j = (m + k <= M0) ? m + k: M0;
	for (i = m; i < j; i++)
	{
		FREEMATCR(CRM[i]);
		CRM[i] = FINPUTMATCR(infile);
	}
	fclose(infile);
}

void FINPUT_PCR(char *filename)
/*
 * The M0 complex rational polynomials are read from the specified file into 
 * PCR[]. If the file could not be opened, a diagnostic is printed.
 */
{
	FILE *infile;
	unsigned int i;

	infile = Fopen_r(filename);
	for (i = 0; i < M0; i++)
	{
		DELETE_PCR(PCR[i]);
		PCR[i] = INPUT1PCR(infile);
	}
	fclose(infile);
	return;
}

void FINPUTMAT_PCR(char *filename)
/*
 * The M0 complex rational polynomial matrices are read from the specified file
 * into PCRM[]. If the file could not be opened, a diagnostic is printed.
 */
{
	FILE *infile;
	unsigned int i;

	infile = Fopen_r(filename);
	for (i = 0; i < M0; i++)
	{
		FREEMATPCR(PCRM[i]);
		PCRM[i] = FINPUTMATPCR(infile);
	}
	fclose(infile);
	return;
}

void PRINTE_CR()
{
	unsigned int j, l;
	char c;

	c = drawmenu(23);
	switch (c)
	{
	case 'q':
		clearscreen();
		Flush();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("CR[%u] = ", j);
				PRINTCR(CR[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("CR[%u] = ", j);
		PRINTCR(CR[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTD_CR(USI d)
{
	unsigned int j, l;
	char c;

	c = drawmenu(24);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("CR[%u] = ", j);
				PRINTDCR(d, CR[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("CR[%u] = ", j);
		PRINTDCR(d, CR[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINT_CR()
{
	unsigned int d;
	char c;

	c = drawmenu(21);
	switch (c)
	{
	case 'e':
		PRINTE_CR();
		break;
	case 'r':
		if (scanf10(&d) == 0)
			break;
		PRINTD_CR(d);
		break;
	case 'q':
		Flush();
		break;
	}
}

void PRINT_PCR()
{
	unsigned int j, l;
	char c;

	c = drawmenu(18);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("PCR[%u] = ", j);
				PRINTPCR(PCR[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("PCR[%u] = ", j);
		PRINTPCR(PCR[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTMAT_CR()
{
	unsigned int j, l;
	char c;

	c = drawmenu(19);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("CRM[%u] = \n", j);
				PRINTMATCR(0, CRM[j]->R - 1, 0, CRM[j]->C - 1, CRM[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("CRM[%u] = \n", j);
		PRINTMATCR(0, CRM[j]->R - 1, 0, CRM[j]->C - 1, CRM[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTMATD_CR(USI d)
{
	unsigned int j, l;
	char c, filename[50];
	FILE *outfile;

	c = drawmenu(25);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("CRM[%u] = \n", j);
				PRINTMATDCR(d, 0, CRM[j]->R - 1, 0, CRM[j]->C - 1, CRM[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		printf("print CRM[%u] to a file to be named\n", j);
		if(GetYN()){
			printf("enter the output filename:");
			gets(filename);
			outfile = fopen(filename, "w");
			FPRINTMATDCR(outfile, d, 0, CRM[j]->R - 1, 0, CRM[j]->C - 1, CRM[j]);
			fclose(outfile);
			printf("CRM[%u] sent to output file %s:\n", j, filename);
			GetReturn();
			break;
			   }
		clearscreen();
		printf("CRM[%u] = \n", j);
		PRINTMATDCR(d, 0, CRM[j]->R - 1, 0, CRM[j]->C - 1, CRM[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTMAT_PCR()
{
	unsigned int j, l;
	char c;

	c = drawmenu(20);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("PCRM[%u] = \n", j);
				PRINTMATPCR(0, PCRM[j]->R - 1, 0, PCRM[j]->C - 1, PCRM[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("PCRM[%u] = \n", j);
		PRINTMATPCR(0, PCRM[j]->R - 1, 0, PCRM[j]->C - 1, PCRM[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void FPRINT_CR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		FPRINTCR(outfile, CR[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void LPRINT_CR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "CR[%u] = ", i);
		FPRINTCR(outfile, CR[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void FPRINTMAT_CR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	fprintf(outfile, "%u\n", M0);
	for (i = 0; i < M0; i++)
		FPRINTMATCR(outfile, 0, CRM[i]->R - 1, 0, CRM[i]->C - 1, CRM[i]);
	fclose(outfile);
	return;
}

void LPRINTMAT_CR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "CRM[%u] = \n\n", i);
		FPRINTMATCR(outfile, 0, CRM[i]->R - 1, 0, CRM[i]->C - 1, CRM[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void LPRINTMATD_CR(USI d, char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	fprintf(outfile, "printing matrices to %u decimal places:\n", d);
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "CRM[%u] = \n\n", i);
		FPRINTMATDCR(outfile, d, 0, CRM[i]->R - 1, 0, CRM[i]->C - 1, CRM[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void FPRINT_PCR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		FPRINT1PCR(outfile, PCR[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void LPRINT_PCR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "PCR[%u] = ", i);
		FPRINTPCR(outfile, PCR[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void FPRINTMAT_PCR(char *filename)
{
	unsigned i;
	FILE *outfile;

	outfile = fopen(filename, "w");
	for (i = 0; i < M0; i++)
		FPRINTMAT1PCR(outfile, 0, PCRM[i]->R - 1, 0, PCRM[i]->C - 1, PCRM[i]);
	fclose(outfile);
}

void LPRINTMAT_PCR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "PCRM[%u] = \n\n", i);
		FPRINTMATPCR(outfile, 0, PCRM[i]->R - 1, 0, PCRM[i]->C - 1, PCRM[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void PRINTCR_()
{
	char c;
	unsigned int GoOnFlag = 1, d;

	while (GoOnFlag)
	{
		c = drawmenu(17);
		switch (c)
		{
		case 'n':
			c = drawmenu(21);
			switch (c)
			{
			case 'e':
				PRINT_CR();
				break;
			case 'r':
				if (scanf10(&d) == 0)
					break;
				PRINTD_CR(d);
				break;
			case 'q':
				Flush();
				break;
			}
			break;
		case 'p':
			PRINT_PCR();
			break;
		case 'm':
			c = drawmenu(22);
			switch (c)
			{
			case 'e':
				PRINTMAT_CR();
				break;
			case 'r':
				if (scanf10(&d) == 0)
					break;
				PRINTMATD_CR(d);
				break;
			case 'q':
				Flush();
				break;
			}
			break;
		case 'M':
			PRINTMAT_PCR();
			break;
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		}
	}
}

void LPRINTCR_()
{
	unsigned int d;
	char c;
	unsigned int GoOnFlag = 1;
	char tmpfile[20];
	char *mktemp();

	strcpy(tmpfile,"cmXXXXXX");
	mktemp(tmpfile);
	while (GoOnFlag)
	{
		c = drawmenu(26);
		Flush();
		switch (c)
		{
		case 'n':
			clearscreen();
			LPRINT_CR(tmpfile);
			printfile(tmpfile);
			printf("complex numbers sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'p':
			clearscreen();
			LPRINT_PCR(tmpfile);
			printfile(tmpfile);
			printf("polynomials sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'm':
			c = drawmenu(22);
			switch (c)
			{
			case 'r':
				clearscreen();
				if (scanf10(&d) == 0)
					break;
				LPRINTMATD_CR(d, tmpfile);
				printfile(tmpfile);
				printf("decimal matrices sent to line-printer.\n");
				GetReturn();
				remove(tmpfile);
				break;
			case 'e':
				Flush();
				clearscreen();
				LPRINTMAT_CR(tmpfile);
				printfile(tmpfile);
				printf("complex matrices sent to line-printer.\n");
				GetReturn();
				remove(tmpfile);
				break;
			case 'q':
				Flush();
				break;
			}
			break;
		case 'M':
			clearscreen();
			LPRINTMAT_PCR(tmpfile);
			printfile(tmpfile);
			printf("polynomial matrices sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'q':
			GoOnFlag = 0;
			break;
		}
	}
}

void INPUT_CR()
{
	unsigned int j, u;
	char str[BUFLEN], *head, *gets();

	if (scanf01(&j) == 0)
		return;
	clearscreen();
	printf("enter CR[%u]: ", j);
	FREEMPCR(CR[j]);
	head = str;
	gets(head);
	CR[j] = INPUTCR(&head, &u);
	printf("CR[%u] = ", j);
	PRINTCR(CR[j]);
	printf("\n\n");
	GetReturn();
	return;
}

void INPUT_PCR()
{
	unsigned int j, u;

	if (scanf01(&j) == 0)
		return;
	clearscreen();
	printf("enter PCR[%u]: ", j);
	DELETE_PCR(PCR[j]);
	PCR[j] = INPUTPCR(&u);
	printf("PCR[%u] = ", j);
	PRINTPCR(PCR[j]);
	printf("\n\n");
	GetReturn();
	return;
}

void ALTER_MATCR(USI l, USI m)
/*
 * alter matrix *CRM[l] to *CRM[m].
 */
{
	unsigned int GoOnFlag = 1, Flag, i, j, n, r, *CA, u;
	int e;
	char c, str[BUFLEN], *head, *gets(), *line, *p;
	MPMATCR *Temp;

	while (GoOnFlag)
	{
		c = drawmenu(15);
		switch (c)
		{
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		case 'r':
			if (scanf10(&i) == 0)
				break;
			if (i > CRM[l]->R)
				printf("i > number of rows of matrix CRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (CRM[l]->R == 1)
				printf("matrix CRM[%u] has one row\n\n", l);
			else
			{
				clearscreen();
				printf("deleting row %u of CRM[%u] =\n", i, l); 
				PRINTMATCR(0, CRM[l]->R - 1, 0, CRM[l]->C - 1, CRM[l]);
				Temp = CRM[m];
				CRM[m] = DELETE_ROWCR(i, CRM[l]);
				FREEMATCR(Temp);
				printf("-> CRM[%u] = \n\n", m);
				PRINTMATCR(0, CRM[m]->R - 1, 0, CRM[m]->C - 1, CRM[m]);
			}
			GetReturn();
			break;
		case 'c':
			if (scanf10(&j) == 0)
				break;
			if (j > CRM[l]->C)
				printf("j > number of columns of matrix CRM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else if (CRM[l]->C == 1)
				printf("matrix CRM[%u] has one column\n\n", l);
			else
			{
				clearscreen();
				printf("deleting column %u of CRM[%u] =\n", j, l); 
				PRINTMATCR(0, CRM[l]->R - 1, 0, CRM[l]->C - 1, CRM[l]);
				Temp = CRM[m];
				CRM[m] = DELETE_COLCR(j, CRM[l]);
				FREEMATCR(Temp);
				printf("-> CRM[%u] = \n\n", m);
				PRINTMATCR(0, CRM[m]->R - 1, 0, CRM[m]->C - 1, CRM[m]);
			}
			GetReturn();
			break;
		case 'a':
			if (scanf20(&i, &j) == 0)
				break;
			if (i > CRM[l]->R)
				printf("i > number of rows of matrix CRM[%u]\n\n", l);
			else if (j > CRM[l]->C)
				printf("j > number of columns of matrix CRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement %u-%u element of matrix CRM[%u] :", i, j, l);
				head = str;
				gets(head);
				Temp = CRM[m];
				CRM[m] = COPYMATCR(CRM[l]);
				FREEMPCR(elt(CRM[m], i - 1, j - 1));
				elt(CRM[m], i - 1, j - 1) = INPUTCR(&head, &u);
				if (u == 0)
					fprintf(stderr, "input corrupted\n");
				printf("CRM[%u] ->  \n\n", m);
				PRINTMATCR(0, CRM[m]->R - 1, 0, CRM[m]->C - 1, CRM[m]);
				FREEMATCR(Temp);
			}
			GetReturn();
			break;
		case 'f':
			if (scanf10(&i) == 0)
				break;
			if (i > CRM[l]->R)
				printf("i > number of rows of matrix CRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else
			{
				printf("enter the replacement row %u of matrix CRM[%u] :", i, l);
				Temp = CRM[m];
				CRM[m] = COPYMATCR(CRM[l]);
				p = line = Fgets(stdin);
				for (j = 0; j < CRM[m]->C; j++)
				{
					FREEMPCR(elt(CRM[m], i - 1, j));
					elt(CRM[m], i - 1, j) = INPUTCR(&p, &u);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				free(line);
				printf("CRM[%u] ->  \n\n", m);
				PRINTMATCR(0, CRM[m]->R - 1, 0, CRM[m]->C - 1, CRM[m]);
				FREEMATCR(Temp);
			}
			GetReturn();
			break;
		case 'g':
			if (scanf10(&j) == 0)
				break;
			if (j > CRM[l]->C)
				printf("j > number of columns of matrix CRM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement column %u of matrix CRM[%u] :", j, l);
				Temp = CRM[m];
				CRM[m] = COPYMATCR(CRM[l]);
				p = line = Fgets(stdin);
				for (i = 0; i < CRM[m]->R; i++)
				{
					FREEMPCR(elt(CRM[m], i, j - 1));
					elt(CRM[m], i, j - 1) = INPUTCR(&p, &u);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				free(line);
				printf("CRM[%u] ->  \n\n", m);
				PRINTMATCR(0, CRM[m]->R - 1, 0, CRM[m]->C - 1, CRM[m]);
				FREEMATCR(Temp);
			}
			GetReturn();
			break;
		case 's':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of rows in the submatrix:", CRM[l]->R);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= CRM[l]->R)
			{
				printf("n >= %u\n", CRM[l]->R);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, CRM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", CRM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a row index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= CRM[l]->R)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, CRM[l]->R);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = CRM[m];
			CRM[m] = SUBMATROWSCR(CRM[l], CA, n);
			FREEMATCR(Temp);
			printf("CRM[%u] ->  \n\n", m);
			PRINTMATCR(0, CRM[m]->R - 1, 0, CRM[m]->C - 1, CRM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		case 't':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of columns in the submatrix:", CRM[l]->C);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= CRM[l]->C)
			{
				printf("n >= %u\n", CRM[l]->C);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, CRM[l]->C);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", CRM[l]->C);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a column index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= CRM[l]->C)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, CRM[l]->C);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = CRM[m];
			CRM[m] = SUBMATCOLUMNSCR(CRM[l], CA, n);
			FREEMATCR(Temp);
			printf("CRM[%u] ->  \n\n", m);
			PRINTMATCR(0, CRM[m]->R - 1, 0, CRM[m]->C - 1, CRM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		}
	}
}

void ALTER_MAT_CR()
{
	unsigned int l, m;

	if (scanf02(&l, &m) == 0)
		return;
	clearscreen();
	printf("altering matrix CRM[%u] = \n\n", l);
	PRINTMATCR(0, CRM[l]->R - 1, 0, CRM[l]->C - 1, CRM[l]);
	GetReturn();
	ALTER_MATCR(l, m);
	return;
}

void ALTER_MATPCR(USI l, USI m)
/*
 * alter matrix *PCRM[l] to *PCRM[m].
 */
{
	unsigned int GoOnFlag = 1, Flag, i, j, n, r, *CA, u;
	int e;
	char c, *gets();
	MPMATPCR *Temp;

	while (GoOnFlag)
	{
		c = drawmenu(16);
		switch (c)
		{
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		case 'r':
			if (scanf10(&i) == 0)
				break;
			if (i > PCRM[l]->R)
				printf("i > number of rows of matrix PCRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (PCRM[l]->R == 1)
				printf("matrix PCRM[%u] has one row\n\n", l);
			else
			{
				clearscreen();
				printf("deleting row %u of PCRM[%u] =\n", i, l); 
				PRINTMATPCR(0, PCRM[l]->R - 1, 0, PCRM[l]->C - 1, PCRM[l]);
				Temp = PCRM[m];
				PCRM[m] = DELETE_ROWPCR(i, PCRM[l]);
				FREEMATPCR(Temp);
				printf("-> PCRM[%u] = \n\n", m);
				PRINTMATPCR(0, PCRM[m]->R - 1, 0, PCRM[m]->C - 1, PCRM[m]);
			}
			GetReturn();
			break;
		case 'c':
			if (scanf10(&j) == 0)
				break;
			if (j > PCRM[l]->C)
				printf("j > number of columns of matrix PCRM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else if (PCRM[l]->C == 1)
				printf("matrix PCRM[%u] has one column\n\n", l);
			else
			{
				clearscreen();
				printf("deleting column %u of PCRM[%u] =\n", j, l); 
				PRINTMATPCR(0, PCRM[l]->R - 1, 0, PCRM[l]->C - 1, PCRM[l]);
				Temp = PCRM[m];
				PCRM[m] = DELETE_COLPCR(j, PCRM[l]);
				FREEMATPCR(Temp);
				printf("-> PCRM[%u] = \n\n", m);
				PRINTMATPCR(0, PCRM[m]->R - 1, 0, PCRM[m]->C - 1, PCRM[m]);
			}
			GetReturn();
			break;
		case 'a':
			if (scanf20(&i, &j) == 0)
				break;
			if (i > PCRM[l]->R)
				printf("i > number of rows of matrix PCRM[%u]\n\n", l);
			else if (j > PCRM[l]->C)
				printf("j > number of columns of matrix PCRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				clearscreen();
				printf("enter the replacement %u-%u element of matrix PCRM[%u]:", i, j, l);
				Temp = PCRM[m];
				PCRM[m] = COPYMATPCR(PCRM[l]);
				DELETE_PCR(PCRM[m]->V[i - 1][j - 1]);
				PCRM[m]->V[i - 1][j - 1] = INPUTPCR(&u);
				printf("PCRM[%u] ->  \n\n", m);
				PRINTMATPCR(0, PCRM[m]->R - 1, 0, PCRM[m]->C - 1, PCRM[m]);
				FREEMATPCR(Temp);
			}
			GetReturn();
			break;
		case 'f':
			if (scanf10(&i) == 0)
				break;
			if (i > PCRM[l]->R)
				printf("i > number of rows of matrix PCRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else
			{
				printf("enter the replacement row %u of matrix PCRM[%u] =\n ", i, l);
				PRINTMATPCR(0, PCRM[l]->R - 1, 0, PCRM[l]->C - 1, PCRM[l]);
				printf(":");
				Temp = PCRM[m];
				PCRM[m] = COPYMATPCR(PCRM[l]);
				for (j = 0; j < PCRM[m]->C; j++)
				{
					printf("Entering polynomial[%u][%u]:\n", i - 1, j);
					DELETE_PCR(PCRM[m]->V[i - 1][j]);
					PCRM[m]->V[i - 1][j] = INPUTPCR(&u);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				printf("PCRM[%u] ->  \n\n", m);
				PRINTMATPCR(0, PCRM[m]->R - 1, 0, PCRM[m]->C - 1, PCRM[m]);
				FREEMATPCR(Temp);
			}
			GetReturn();
			break;
		case 'g':
			if (scanf10(&j) == 0)
				break;
			if (j > PCRM[l]->C)
				printf("j > number of columns of matrix PCRM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement column %u of matrix PCRM[%u] =\n", j, l);
				PRINTMATPCR(0, PCRM[l]->R - 1, 0, PCRM[l]->C - 1, PCRM[l]);
				printf(":");
				Temp = PCRM[m];
				PCRM[m] = COPYMATPCR(PCRM[l]);
				for (i = 0; i < PCRM[m]->R; i++)
				{
					printf("Entering polynomial[%u][%u]:\n", i, j - 1);
					DELETE_PCR(PCRM[m]->V[i][j - 1]);
					PCRM[m]->V[i][j - 1] = INPUTPCR(&u);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				printf("PCRM[%u] ->  \n\n", m);
				PRINTMATPCR(0, PCRM[m]->R - 1, 0, PCRM[m]->C - 1, PCRM[m]);
				FREEMATPCR(Temp);
			}
			GetReturn();
			break;
		case 's':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of rows in the submatrix:", PCRM[l]->R);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= PCRM[l]->R)
			{
				printf("n >= %u\n", PCRM[l]->R);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, PCRM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", PCRM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a row index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= PCRM[l]->R)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, PCRM[l]->R);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = PCRM[m];
			PCRM[m] = SUBMATROWSPCR(PCRM[l], CA, n);
			FREEMATPCR(Temp);
			printf("PCRM[%u] ->  \n\n", m);
			PRINTMATPCR(0, PCRM[m]->R - 1, 0, PCRM[m]->C - 1, PCRM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		case 't':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of columns in the submatrix:", PCRM[l]->C);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= PCRM[l]->C)
			{
				printf("n >= %u\n", PCRM[l]->C);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, PCRM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", PCRM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a column index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= PCRM[l]->C)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, PCRM[l]->C);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = PCRM[m];
			PCRM[m] = SUBMATCOLUMNSPCR(PCRM[l], CA, n);
			FREEMATPCR(Temp);
			printf("PCRM[%u] ->  \n\n", m);
			PRINTMATPCR(0, PCRM[m]->R - 1, 0, PCRM[m]->C - 1, PCRM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		}
	}
}

void ALTER_MAT_PCR()
{
	unsigned int l, m;

	if (scanf02(&l, &m) == 0)
		return;
	clearscreen();
	printf("altering matrix PCRM[%u] = \n\n", l);
	PRINTMATPCR(0, PCRM[l]->R - 1, 0, PCRM[l]->C - 1, PCRM[l]);
	GetReturn();
	ALTER_MATPCR(l, m);
	return;
}

void INPUTMAT_CR()
{
	unsigned int k;

	if (scanf01(&k) == 0)
		return;
	clearscreen();
	printf("enter CRM[%u]:\n\n", k);
	FREEMATCR(CRM[k]);
	CRM[k] = INPUTMATCR();
	printf("\n");
	printf("CRM[%u] = \n", k);
	PRINTMATCR(0, CRM[k]->R - 1, 0, CRM[k]->C - 1, CRM[k]);
	printf("\n");
	printf("Do you wish to alter CRM[%u]? :", k);
	if (GetYN())
		ALTER_MATCR(k, k);
	return;
}

void INPUTMAT_PCR()
{
	unsigned int k;
	
	if (scanf01(&k) == 0)
		return;
	clearscreen();
	printf("enter PCRM[%u]:\n\n", k);
	FREEMATPCR(PCRM[k]);
	PCRM[k] = INPUTMATPCR();
	printf("\n");
	printf("PCRM[%u] = \n", k);
	PRINTMATPCR(0, PCRM[k]->R - 1, 0, PCRM[k]->C - 1, PCRM[k]);
	printf("\n");
	printf("Do you wish to alter PCRM[%u]? :", k);
	if (GetYN())
		ALTER_MATPCR(k, k);
	return;
}

void FINPUTMATFILE_CR()
{
	char filename[50];
	unsigned int m;
	FILE *fp;

	getchar();
	clearscreen();
	printf("enter the name of your matrix file:  ");
	gets(filename);
	if ((fp = fopen(filename, "r")) == NULL)
	{
		printf("The file `%s' does not exist!\n",filename);
		GetReturn();
	}
	else
	{
		fclose(fp);
		printf("\n");
		printf("To store the matrices at consecutive array positions starting at CRM[m],\n\n");
		printf("please enter a non-negative integer m, 0<=m<=%u:  ", M0 - 1);
		if (scanf01(&m) == 0)
			return;
		FINPUTMAT_CR(filename, m);
		printf("\n");
		printf("matrices from file %s successfully inputted, starting at CRM[%u]:\n", filename, m);
		GetReturn();
	}
	return;
}

void FINPUTMATBC_CR()
{
	unsigned int m, n, i, j, k;
	char nsr[100], dsr[100], nsi[100], dsi[100];
	MPI *Numerator, *Denominator;

	if (scanf01(&k) == 0)
		return;
	clearscreen();
	printf("entering CRM[%u]:\n\n", k);
	printf("enter the number of rows and number of columns: ");
	if (scanf20(&m, &n) == 0)
		return;
	if (m == 0 || n == 0)
	{
		printf("one of the integers was zero!\n");
		GetReturn();
		return;
	}
	printf("Enter integer expressions for the numerators and denominators of Re(a[i][j]) and Im(a[i][j]):\n\n");
	printf("e.g. i^2 - i*j + i + j^5\n\n");
	printf("enter the numerator of Re(a[i][j]): ");
	scanf("%99[^\n]", nsr);
	Flush();
	printf("enter the denominator of Re(a[i][j]): ");
	scanf("%99[^\n]", dsr);
	Flush();
	printf("enter the numerator of Im(a[i][j]): ");
	scanf("%99[^\n]", nsi);
	Flush();
	printf("enter the denominator of Im(a[i][j]): ");
	scanf("%99[^\n]", dsi);
	Flush();
	FREEMATCR(CRM[k]);
	CRM[k] = BUILDMATCR(m, n);
	for (i = 1; i <= m; i++)
		for (j = 1; j <= n; j++)
		{
			elt(CRM[k], i - 1, j - 1) = BUILDMPCR();
			Numerator = Parse(nsr, i, j);
			Denominator = Parse(dsr, i, j);
			elt(CRM[k], i - 1, j - 1)->R = RATIOI(Numerator, Denominator);
			FREEMPI(Numerator);
			FREEMPI(Denominator);
			Numerator = Parse(nsi, i, j);
			Denominator = Parse(dsi, i, j);
			elt(CRM[k], i - 1, j - 1)->I = RATIOI(Numerator, Denominator);
			FREEMPI(Numerator);
			FREEMPI(Denominator);
		}
	printf("\n");
	printf("CRM[%u] = \n", k);
	PRINTMATCR(0, CRM[k]->R - 1, 0, CRM[k]->C - 1, CRM[k]);
	printf("\n");
	printf("Do you wish to alter CRM[%d]? : ", k);
	if (GetYN())
		ALTER_MATCR(k, k);
	return;
}

void BANDMAT_CR()
{
	unsigned int k, m, j, u;
	MPCR *A, *B;
	char *line, *p;

	if (scanf21(&k, &m, &j) == 0)
		return;
	clearscreen();
	if (m < 2)
		printf("m < 2\n");
	else if (k > m - 1)
		printf("k >= m\n");
	else
	{
		printf("Enter the diagonal element:");
		p = line = Gets();
		A = INPUTCR(&p, &u);
		free(line);
		printf("\n");
		printf("Enter the band element:");
		p = line = Gets();
		B = INPUTCR(&p, &u);
		free(line);
		printf("\n");
		FREEMATCR(CRM[j]);
		CRM[j] = BANDMATCR(A, B, k, m);
		FREEMPCR(A);
		FREEMPCR(B);
		printf("CRM[%d] = \n\n", j);
		PRINTMATCR(0, CRM[j]->R - 1, 0, CRM[j]->C - 1, CRM[j]); 
	}
	GetReturn();
	return;
}

void IDENTITY_CR()
{
	unsigned int m, j;

	if (scanf11(&m, &j) == 0)
		return;
	clearscreen();
	if (m == 0)
		printf("m = 0\n");
	else
	{
		FREEMATCR(CRM[j]);
		CRM[j] = IDENTITYCR(m);
		printf("CRM[%d] = \n\n", j);
		PRINTMATCR(0, CRM[j]->R - 1, 0, CRM[j]->C - 1, CRM[j]); 
	}
	GetReturn();
	return;
}

void JORDAN_CR()
{
	unsigned int m, k, l;

	if (scanf12(&m, &k, &l) == 0)
		return;
	clearscreen();
	if (m == 0)
		printf("m = 0\n");
	else
	{
		FREEMATCR(CRM[l]);
		CRM[l] = JORDANCR(m, CR[k]);
		printf("CRM[%u] = \n\n", l);
		PRINTMATCR(0, CRM[l]->R - 1, 0, CRM[l]->C - 1, CRM[l]); 
	}
	GetReturn();
	return;
}

void COMPANION_CR()
{
	unsigned int j, k;
	MPCR *G;

	if (scanf02(&j, &k) == 0)
		return;
	clearscreen();
	if (DEGREEPCR(PCR[j]) <= 0)
		printf("degree of PCR[%u] is <= 0\n", j);
	else
	{
		G = LEADPCR(PCR[j]);
		if (!EQONECR(G))
			printf("PCR[%u] is not monic\n", j);
		else
		{
			printf("The companion matrix of PCR[%u] = ", j);
			PRINTPCR(PCR[j]);
			printf("\n");
			FREEMATCR(CRM[k]);
			CRM[k] = COMPANIONCR(PCR[j]);
			printf("-> CRM[%u] = \n\n", k);
			PRINTMATCR(0, CRM[k]->R - 1, 0, CRM[k]->C - 1, CRM[k]); 
		}
		FREEMPCR(G);
	}
	GetReturn();
	return;
}

void INPUTCR_()
{
	char c;

	unsigned int GoOnFlag = 1;
	while (GoOnFlag)
	{
		c = drawmenu(14);
		switch (c)
		{
		case 'n':
			INPUT_CR();
			break;
		case 'p':
			INPUT_PCR();
			break;
		case 'm':
			INPUTMAT_CR();
			break;
		case 'M':
			INPUTMAT_PCR();
			break;
		case 'b':
			FINPUTMATBC_CR();
			break;
		case 'f':
			FINPUTMATFILE_CR();
			break;
		case 'v':
			BANDMAT_CR();
			break;
		case 'i':
			IDENTITY_CR();
			break;
		case 'j':
			JORDAN_CR();
			break;
		case 'c':
			COMPANION_CR();
			break;
                case 't':
			ALTER_MAT_CR();
			break;
		case 'T':
			ALTER_MAT_PCR();
			break;
		case 'q':
			Flush();
			GoOnFlag = 0;
			FPRINT_CR(cCR);
			FPRINT_PCR(cPCR);
			FPRINTMAT_CR(cCRM);
			FPRINTMAT_PCR(cPCRM);
			break;
		}
	}
}
