/* cm_ioR.c */

#include <stdio.h>
#include <stdlib.h>
#include "integer.h"
#include "fun.h"

extern MPR *R[];
extern POLYR PR[];
extern MPMATR *RM[];
extern MPMATPR *PRM[];

extern char cR[];
extern char cPR[];
extern char cRM[];
extern char cPRM[];

void FINPUT_R(char *filename)
/*
 * Reads all M0 rational numbers from the specified file into R[].
 * If the file could not be opened, a diagnostic is printed.
 */
{
	int i;
	unsigned int u;
	FILE *infile;

	infile = Fopen_r(filename);
	for (i = 0; i < M0; i++)
	{
		FREEMPR(R[i]);
		R[i] = FINPUTR(infile, &u);
	}
	fclose(infile);
}

void FINPUTMAT_R(char *filename, USI m)
/*
 * rational matrices RM[m],... are read from the specified file.  If the file
 * could not be opened, a diagnostic is printed.
 */
{
	int i;
	unsigned int j, k;
	FILE *infile;

	infile = Fopen_r(filename);
	fscanf(infile, "%u", &k);
	fgetc(infile);
	j = (m + k <= M0) ? m + k: M0;
	for (i = m; i < j; i++)
	{
		FREEMATR(RM[i]);
		RM[i] = FINPUTMATR(infile);
	}
	fclose(infile);
}

void FINPUTMAT_RD(char *filename, USI m)
/*
 * floating point matrices RM[m],... are read from the specified file.
 * If the file could not be opened, a diagnostic is printed.
 */
{
	int i;
	unsigned int j, k;
	FILE *infile;

	infile = Fopen_r(filename);
	fscanf(infile, "%u", &k);
	fgetc(infile);
	j = (m + k <= M0) ? m + k: M0;
	for (i = m; i < j; i++)
	{
		FREEMATR(RM[i]);
		RM[i] = FINPUTMATRD(infile);
	}
	fclose(infile);
}

void FINPUT_PR(char *filename)
/*
 * Reads the M0 rational polynomials from specified file.
 * If the file could not be opened, a diagnostic is printed.
 */
{
	int i;
	FILE *infile;
	infile = Fopen_r(filename);
	for (i = 0; i < M0; i++)
	{
		DELETE_PR(PR[i]);
		PR[i] = INPUT1PR(infile);
	}
	fclose(infile);
	return;
}

void FINPUTMAT_PR(char *filename)
/*
 * M0 rational polynomial matrices are read from the specified file into PRM[].
 * If the file could not be opened, a diagnostic is printed.
 */
{
	FILE *infile;
	unsigned int i;

	infile = Fopen_r(filename);
	for (i = 0; i < M0; i++)
	{
		FREEMATPR(PRM[i]);
		PRM[i] = FINPUTMATPR(infile);
	}
	fclose(infile);
	return;
}

void PRINTE_R()
{
	unsigned int j, l;
	char c;

	c = drawmenu(23);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("R[%u] = ", j);
				PRINTR(R[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("R[%u] = ", j);
		PRINTR(R[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTD_R(USI d)
{
	unsigned int j, l;
	char c;

	c = drawmenu(24);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("R[%u] = ", j);
				PRINTDR(d, R[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("R[%u] = ", j);
		PRINTDR(d, R[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINT_R()
{
	unsigned int d;
	char c;

	c = drawmenu(21);
	switch (c)
	{
	case 'e':
		PRINTE_R();
		break;
	case 'r':
		if (scanf10(&d) == 0)
			break;
		PRINTD_R(d);
		break;
	case 'q':
		Flush();
		break;
	}
}

void PRINT_PR()
{
	unsigned int j, l;
	char c;

	c = drawmenu(18);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("PR[%u] = ", j);
				PRINTPR(PR[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("PR[%u] = ", j);
		PRINTPR(PR[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTMAT_R()
{
	unsigned int j, l;
	char c;

	c = drawmenu(19);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("RM[%u] = \n", j);
				PRINTMATR(0, RM[j]->R - 1, 0, RM[j]->C - 1, RM[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("RM[%u] = \n", j);
		PRINTMATR(0, RM[j]->R - 1, 0, RM[j]->C - 1, RM[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTMATD_R(USI d)
{
	unsigned int j, l;
	char c, filename[50];
	FILE *outfile;

	c = drawmenu(25);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("RM[%u] = \n", j);
				PRINTMATDR(d, 0, RM[j]->R - 1, 0, RM[j]->C - 1, RM[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		printf("print RM[%u] to a file to be named\n", j);
		if(GetYN()){
			printf("enter the output filename:");
			gets(filename);
			outfile = fopen(filename, "w");
			FPRINTMATDR(outfile, d, 0, RM[j]->R - 1, 0, RM[j]->C - 1, RM[j]);
			fclose(outfile);
			printf("RM[%u] sent to output file %s:\n", j, filename);
			GetReturn();
			break;
			   }
		clearscreen();
		printf("RM[%u] = \n", j);
		PRINTMATDR(d, 0, RM[j]->R - 1, 0, RM[j]->C - 1, RM[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTMAT_PR()
{
	unsigned int j, l;
	char c;

	c = drawmenu(20);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("PRM[%u] = \n", j);
				PRINTMATPR(0, PRM[j]->R - 1, 0, PRM[j]->C - 1, PRM[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("PRM[%u] = \n", j);
		PRINTMATPR(0, PRM[j]->R - 1, 0, PRM[j]->C - 1, PRM[j]);
		printf("\n");
		GetReturn();
		break;
	}
}


void FPRINT_R(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		FPRINTR(outfile, R[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void LPRINT_R(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "R[%u] = ", i);
		FPRINTR(outfile, R[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void FPRINTMAT_R(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	fprintf(outfile, "%u\n", M0);
	for (i = 0; i < M0; i++)
		FPRINTMATR(outfile, 0, RM[i]->R - 1, 0, RM[i]->C - 1, RM[i]);
	fclose(outfile);
	return;
}

void LPRINTMAT_R(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "RM[%u] = \n\n", i);
		FPRINTMATR(outfile, 0, RM[i]->R - 1, 0, RM[i]->C - 1, RM[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void LPRINTMATD_R(USI d, char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	fprintf(outfile, "printing matrices to %u decimal places:\n", d);
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "RM[%u] = \n\n", i);
		FPRINTMATDR(outfile, d, 0, RM[i]->R - 1, 0, RM[i]->C - 1, RM[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void FPRINT_PR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		FPRINT1PR(outfile, PR[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void LPRINT_PR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "PR[%u] = ", i);
		FPRINTPR(outfile, PR[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void FPRINTMAT_PR(char *filename)
{
	unsigned i;
	FILE *outfile;

	outfile = fopen(filename, "w");
	for (i = 0; i < M0; i++)
		FPRINTMAT1PR(outfile, 0, PRM[i]->R - 1, 0, PRM[i]->C - 1, PRM[i]);
	fclose(outfile);
}

void LPRINTMAT_PR(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "PRM[%u] = \n\n", i);
		FPRINTMATPR(outfile, 0, PRM[i]->R - 1, 0, PRM[i]->C - 1, PRM[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void PRINTR_()
{
	char c;
	unsigned int GoOnFlag = 1, d;

	while (GoOnFlag)
	{
		c = drawmenu(17);
		switch (c)
		{
		case 'n':
			PRINT_R();
			break;
		case 'p':
			PRINT_PR();
			break;
		case 'm':
			c = drawmenu(22);
			switch (c)
			{
			case 'e':
				PRINTMAT_R();
				break;
			case 'r':
				if (scanf10(&d) == 0)
					break;
				PRINTMATD_R(d);
				break;
			case 'q':
				Flush();
				break;
			}
			break;
		case 'M':
			PRINTMAT_PR();
			break;
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		}
	}
}

void LPRINTR_()
{
	unsigned int d;
	char c;
	unsigned int GoOnFlag = 1;
	char tmpfile[20];

	strcpy(tmpfile,"cmXXXXXX");
	mktemp(tmpfile);
	while (GoOnFlag)
	{
		c = drawmenu(26);
		Flush();
		switch (c)
		{
		case 'n':
			clearscreen();
			LPRINT_R(tmpfile);
			printfile(tmpfile);
			printf("rational numbers sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'p':
			clearscreen();
			LPRINT_PR(tmpfile);
			printfile(tmpfile);
			printf("polynomials sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'm':
			c = drawmenu(22);
			switch (c)
			{
			case 'r':
				if (scanf10(&d) == 0)
					break;
				LPRINTMATD_R(d, tmpfile);
				printfile(tmpfile);
				printf("decimal matrices sent to line-printer.\n");
				GetReturn();
				remove(tmpfile);
				break;
			case 'e':
				Flush();
				clearscreen();
				LPRINTMAT_R(tmpfile);
				printfile(tmpfile);
				printf("rational matrices sent to line-printer.\n");
				GetReturn();
				remove(tmpfile);
				break;
			case 'q':
				Flush();
				break;
			}
			break;
		case 'M':
			clearscreen();
			LPRINTMAT_PR(tmpfile);
			printfile(tmpfile);
			printf("polynomial matrices sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'q':
			GoOnFlag = 0;
			break;
		}
	}
}

void INPUT_R()
{
	unsigned int j, u;
	char str[BUFLEN], *head, *gets();

	if (scanf01(&j) == 0)
		return;
	clearscreen();
	printf("enter R[%u]: ", j);
	FREEMPR(R[j]);
	head = str;
	gets(head);
	R[j] = INPUTSR(&head, &u);
	printf("R[%u] = ", j);
	PRINTR(R[j]);
	printf("\n\n");
	GetReturn();
	return;
}

void INPUT_PR()
{
	unsigned int j, u;

	if (scanf01(&j) == 0)
		return;
	clearscreen();
	printf("enter PR[%u]: ", j);
	DELETE_PR(PR[j]);
	PR[j] = INPUTPR(&u);
	printf("PR[%u] = ", j);
	PRINTPR(PR[j]);
	printf("\n\n");
	GetReturn();
	return;
}

void ALTER_MATR(USI l, USI m)
/*
 * alter matrix *RM[l] to *RM[m].
 */
{
	unsigned int GoOnFlag = 1, Flag, i, j, n, r, *CA, u;
	int e;
	char c, *gets(), *line, *p;
	MPMATR *Temp;

	while (GoOnFlag)
	{
		c = drawmenu(15);
		switch (c)
		{
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		case 'r':
			if (scanf10(&i) == 0)
				break;
			if (i > RM[l]->R)
				printf("i > number of rows of matrix RM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (RM[l]->R == 1)
				printf("matrix RM[%u] has one row\n\n", l);
			else
			{
				clearscreen();
				printf("deleting row %u of RM[%u] =\n", i, l); 
				PRINTMATR(0, RM[l]->R - 1, 0, RM[l]->C - 1, RM[l]);
				Temp = RM[m];
				RM[m] = DELETE_ROWR(i, RM[l]);
				FREEMATR(Temp);
				printf("-> RM[%u] = \n\n", m);
				PRINTMATR(0, RM[m]->R - 1, 0, RM[m]->C - 1, RM[m]);
			}
			GetReturn();
			break;
		case 'c':
			if (scanf10(&j) == 0)
				break;
			if (j > RM[l]->C)
				printf("j > number of columns of matrix RM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else if (RM[l]->C == 1)
				printf("matrix RM[%u] has one column\n\n", l);
			else
			{
				clearscreen();
				printf("deleting column %u of RM[%u] =\n", j, l); 
				PRINTMATR(0, RM[l]->R - 1, 0, RM[l]->C - 1, RM[l]);
				Temp = RM[m];
				RM[m] = DELETE_COLR(j, RM[l]);
				FREEMATR(Temp);
				printf("-> RM[%u] = \n\n", m);
				PRINTMATR(0, RM[m]->R - 1, 0, RM[m]->C - 1, RM[m]);
			}
			GetReturn();
			break;
		case 'a':
			if (scanf20(&i, &j) == 0)
				break;
			if (i > RM[l]->R)
				printf("i > number of rows of matrix RM[%u]\n\n", l);
			else if (j > RM[l]->C)
				printf("j > number of columns of matrix RM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement %u-%u element of matrix RM[%u] :", i, j, l);
				Temp = RM[m];
				RM[m] = COPYMATR(RM[l]);
				FREEMPR(elt(RM[m], i - 1, j - 1));
				elt(RM[m], i - 1, j - 1) = INPUTR(&u);
				if (u == 0)
					fprintf(stderr, "input corrupted\n");
				printf("RM[%u] ->  \n\n", m);
				PRINTMATR(0, RM[m]->R - 1, 0, RM[m]->C - 1, RM[m]);
				FREEMATR(Temp);
			}
			GetReturn();
			break;
		case 'f':
			if (scanf10(&i) == 0)
				break;
			if (i > RM[l]->R)
				printf("i > number of rows of matrix RM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else
			{
				printf("enter the replacement row %u of matrix RM[%u] =\n ", i, l);
				PRINTMATR(0, RM[l]->R - 1, 0, RM[l]->C - 1, RM[l]);
				printf(":");
				Temp = RM[m];
				RM[m] = COPYMATR(RM[l]);
				p = line = Fgets(stdin);
				for (j = 0; j < RM[m]->C; j++)
				{
					FREEMPR(elt(RM[m], i - 1, j));
					elt(RM[m], i - 1, j) = INPUTSR(&p, &u);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				free(line);
				printf("RM[%u] ->  \n\n", m);
				PRINTMATR(0, RM[m]->R - 1, 0, RM[m]->C - 1, RM[m]);
				FREEMATR(Temp);
			}
			GetReturn();
			break;
		case 'g':
			if (scanf10(&j) == 0)
				break;
			if (j > RM[l]->C)
				printf("j > number of columns of matrix RM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement column %u of matrix RM[%u] =\n", j, l);
				PRINTMATR(0, RM[l]->R - 1, 0, RM[l]->C - 1, RM[l]);
				printf(":");
				Temp = RM[m];
				RM[m] = COPYMATR(RM[l]);
				p = line = Fgets(stdin);
				for (i = 0; i < RM[m]->R; i++)
				{
					FREEMPR(elt(RM[m], i, j - 1));
					elt(RM[m], i, j - 1) = INPUTSR(&p, &u);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				free(line);
				printf("RM[%u] ->  \n\n", m);
				PRINTMATR(0, RM[m]->R - 1, 0, RM[m]->C - 1, RM[m]);
				FREEMATR(Temp);
			}
			GetReturn();
			break;
		case 's':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of rows in the submatrix:", RM[l]->R);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= RM[l]->R)
			{
				printf("n >= %u\n", RM[l]->R);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, RM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", RM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a row index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= RM[l]->R)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, RM[l]->R);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = RM[m];
			RM[m] = SUBMATROWSR(RM[l], CA, n);
			FREEMATR(Temp);
			printf("RM[%u] ->  \n\n", m);
			PRINTMATR(0, RM[m]->R - 1, 0, RM[m]->C - 1, RM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		case 't':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of columns in the submatrix:", RM[l]->C);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= RM[l]->C)
			{
				printf("n >= %u\n", RM[l]->C);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, RM[l]->C);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", RM[l]->C);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a column index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= RM[l]->C)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, RM[l]->C);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = RM[m];
			RM[m] = SUBMATCOLUMNSR(RM[l], CA, n);
			FREEMATR(Temp);
			printf("RM[%u] ->  \n\n", m);
			PRINTMATR(0, RM[m]->R - 1, 0, RM[m]->C - 1, RM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		}
	}
}

void ALTER_MAT_R()
{
	unsigned int l, m;

	if (scanf02(&l, &m) == 0)
		return;
	clearscreen();
	printf("altering matrix RM[%u] = \n\n", l);
	PRINTMATR(0, RM[l]->R - 1, 0, RM[l]->C - 1, RM[l]);
	GetReturn();
	ALTER_MATR(l, m);
	return;
}

void ALTER_MATPR(USI l, USI m)
/*
 * alter matrix *PRM[l] to *PRM[m].
 */
{
	unsigned int GoOnFlag = 1, Flag, i, j, n, r, *CA, u;
	int e;
	char c, *gets();
	MPMATPR *Temp;

	while (GoOnFlag)
	{
		c = drawmenu(16);
		switch (c)
		{
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		case 'r':
			if (scanf10(&i) == 0)
				break;
			if (i > PRM[l]->R)
				printf("i > number of rows of matrix PRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (PRM[l]->R == 1)
				printf("matrix PRM[%u] has one row\n\n", l);
			else
			{
				clearscreen();
				printf("deleting row %u of PRM[%u] =\n", i, l); 
				PRINTMATPR(0, PRM[l]->R - 1, 0, PRM[l]->C - 1, PRM[l]);
				Temp = PRM[m];
				PRM[m] = DELETE_ROWPR(i, PRM[l]);
				FREEMATPR(Temp);
				printf("-> PRM[%u] = \n\n", m);
				PRINTMATPR(0, PRM[m]->R - 1, 0, PRM[m]->C - 1, PRM[m]);
			}
			GetReturn();
			break;
		case 'c':
			if (scanf10(&j) == 0)
				break;
			if (j > PRM[l]->C)
				printf("j > number of columns of matrix PRM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else if (PRM[l]->C == 1)
				printf("matrix PRM[%u] has one column\n\n", l);
			else
			{
				clearscreen();
				printf("deleting column %u of PRM[%u] =\n", j, l); 
				PRINTMATPR(0, PRM[l]->R - 1, 0, PRM[l]->C - 1, PRM[l]);
				Temp = PRM[m];
				PRM[m] = DELETE_COLPR(j, PRM[l]);
				FREEMATPR(Temp);
				printf("-> PRM[%u] = \n\n", m);
				PRINTMATPR(0, PRM[m]->R - 1, 0, PRM[m]->C - 1, PRM[m]);
			}
			GetReturn();
			break;
		case 'a':
			if (scanf20(&i, &j) == 0)
				break;
			if (i > PRM[l]->R)
				printf("i > number of rows of matrix PRM[%u]\n\n", l);
			else if (j > PRM[l]->C)
				printf("j > number of columns of matrix PRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				clearscreen();
				printf("enter the replacement %u-%u element of matrix PRM[%u]:", i, j, l);
				Temp = PRM[m];
				PRM[m] = COPYMATPR(PRM[l]);
				DELETE_PR(PRM[m]->V[i - 1][j - 1]);
				PRM[m]->V[i - 1][j - 1] = INPUTPR(&u);
				printf("PRM[%u] ->  \n\n", m);
				PRINTMATPR(0, PRM[m]->R - 1, 0, PRM[m]->C - 1, PRM[m]);
				FREEMATPR(Temp);
			}
			GetReturn();
			break;
		case 'f':
			if (scanf10(&i) == 0)
				break;
			if (i > PRM[l]->R)
				printf("i > number of rows of matrix PRM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else
			{
				printf("enter the replacement row %u of matrix PRM[%u] =\n ", i, l);
				PRINTMATPR(0, PRM[l]->R - 1, 0, PRM[l]->C - 1, PRM[l]);
				printf(":");
				Temp = PRM[m];
				PRM[m] = COPYMATPR(PRM[l]);
				for (j = 0; j < PRM[m]->C; j++)
				{
					printf("Entering polynomial[%u][%u]:\n", i - 1, j);
					DELETE_PR(PRM[m]->V[i - 1][j]);
					PRM[m]->V[i - 1][j] = INPUTPR(&u);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				printf("PRM[%u] ->  \n\n", m);
				PRINTMATPR(0, PRM[m]->R - 1, 0, PRM[m]->C - 1, PRM[m]);
				FREEMATPR(Temp);
			}
			GetReturn();
			break;
		case 'g':
			if (scanf10(&j) == 0)
				break;
			if (j > PRM[l]->C)
				printf("j > number of columns of matrix PRM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement column %u of matrix PRM[%u] =\n", j, l);
				PRINTMATPR(0, PRM[l]->R - 1, 0, PRM[l]->C - 1, PRM[l]);
				printf(":");
				Temp = PRM[m];
				PRM[m] = COPYMATPR(PRM[l]);
				for (i = 0; i < PRM[m]->R; i++)
				{
					printf("Entering polynomial[%u][%u]:\n", i, j - 1);
					DELETE_PR(PRM[m]->V[i][j - 1]);
					PRM[m]->V[i][j - 1] = INPUTPR(&u);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				printf("PRM[%u] ->  \n\n", m);
				PRINTMATPR(0, PRM[m]->R - 1, 0, PRM[m]->C - 1, PRM[m]);
				FREEMATPR(Temp);
			}
			GetReturn();
			break;
		case 's':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of rows in the submatrix:", PRM[l]->R);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= PRM[l]->R)
			{
				printf("n >= %u\n", PRM[l]->R);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, PRM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", PRM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a row index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= PRM[l]->R)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, PRM[l]->R);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = PRM[m];
			PRM[m] = SUBMATROWSPR(PRM[l], CA, n);
			FREEMATPR(Temp);
			printf("PRM[%u] ->  \n\n", m);
			PRINTMATPR(0, PRM[m]->R - 1, 0, PRM[m]->C - 1, PRM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		case 't':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of columns in the submatrix:", PRM[l]->C);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= PRM[l]->C)
			{
				printf("n >= %u\n", PRM[l]->C);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, PRM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", PRM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a column index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= PRM[l]->C)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, PRM[l]->C);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = PRM[m];
			PRM[m] = SUBMATCOLUMNSPR(PRM[l], CA, n);
			FREEMATPR(Temp);
			printf("PRM[%u] ->  \n\n", m);
			PRINTMATPR(0, PRM[m]->R - 1, 0, PRM[m]->C - 1, PRM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		}
	}
}

void ALTER_MAT_PR()
{
	unsigned int l, m;

	if (scanf02(&l, &m) == 0)
		return;
	clearscreen();
	printf("altering matrix PRM[%u] = \n\n", l);
	PRINTMATPR(0, PRM[l]->R - 1, 0, PRM[l]->C - 1, PRM[l]);
	GetReturn();
	ALTER_MATPR(l, m);
	return;
}

void INPUTMAT_R()
{
	unsigned int k;

	if (scanf01(&k) == 0)
		return;
	clearscreen();
	printf("enter RM[%u]:\n\n", k);
	FREEMATR(RM[k]);
	RM[k] = INPUTMATR();
	printf("\n");
	printf("RM[%u] = \n", k);
	PRINTMATR(0, RM[k]->R - 1, 0, RM[k]->C - 1, RM[k]);
	printf("\n");
	printf("Do you wish to alter RM[%u]? :", k);
	if (GetYN())
		ALTER_MATR(k, k);
	return;
}

void INPUTMAT_PR()
{
	unsigned int k;

	if (scanf01(&k) == 0)
		return;
	clearscreen();
	printf("enter PRM[%u]:\n\n", k);
	FREEMATPR(PRM[k]);
	PRM[k] = INPUTMATPR();
	printf("\n");
	printf("PRM[%u] = \n", k);
	PRINTMATPR(0, PRM[k]->R - 1, 0, PRM[k]->C - 1, PRM[k]);
	printf("\n");
	printf("Do you wish to alter PRM[%u]? :", k);
	if (GetYN())
		ALTER_MATPR(k, k);
	return;
}

void FINPUTMATFILE_R()
{
	char filename[50];
	unsigned int m;
	FILE *fp;

	getchar();
	clearscreen();
	printf("enter the name of your matrix file:");
	gets(filename);
	if ((fp = fopen(filename, "r")) == NULL)
	{
		printf("The file `%s' does not exist!\n",filename);
		GetReturn();
	}
	else
	{
		fclose(fp);
		printf("\n");
		printf("To store the matrices at consecutive array positions starting at RM[m],\n\n");
		printf("please enter a non-negative integer m, 0<=m<=%u:  ", M0 - 1);
		if (scanf01(&m) == 0)
			return;
		FINPUTMAT_R(filename, m);
		printf("\n");
		printf("matrices from file %s successfully inputted, starting at RM[%u]:\n", filename, m);
		GetReturn();
	}
	return;
}

void FINPUTMATFILE_RD()
{
	char filename[50];
	unsigned int m;
	FILE *fp;

	getchar();
	clearscreen();
	printf("enter the name of your matrix file:");
	gets(filename);
	if ((fp = fopen(filename, "r")) == NULL)
	{
		printf("The file `%s' does not exist!\n",filename);
		GetReturn();
	}
	else
	{
		fclose(fp);
		printf("\n");
		printf("To store the matrices at consecutive array positions starting at RM[m],\n\n");
		printf("please enter a non-negative integer m, 0<=m<=%u:  ", M0 - 1);
		if (scanf01(&m) == 0)
			return;
		FINPUTMAT_RD(filename, m);
		printf("\n");
		printf("matrices from file %s successfully inputted, starting at RM[%u]:\n", filename, m);
		GetReturn();
	}
	return;
}

void FINPUTMATBC_R()
{
	unsigned int m, n, i, j, k;
	char ns[100], ds[100];
	MPI *Numerator, *Denominator;

	if (scanf01(&k) == 0)
		return;
	clearscreen();
	printf("entering RM[%u]:\n\n", k);
	printf("enter the number of rows and number of columns: ");
	if (scanf20(&m, &n) == 0)
		return;
	if (m == 0 || n == 0)
	{
		printf("one of the integers was zero!\n");
		GetReturn();
		return;
	}
	printf("Enter integer expressions for the numerator and denominator of a[i][j] and a[i][j]:\n\n");
	printf("e.g. i^2 - i*j + i + j^5\n\n");
	printf("enter the numerator of a[i][j]: ");
	scanf("%99[^\n]", ns);
	Flush();
	printf("enter the denominator of a[i][j]: ");
	scanf("%99[^\n]", ds);
	Flush();
	FREEMATR(RM[k]);
	RM[k] = BUILDMATR(m, n);
	for (i = 1; i <= m; i++)
		for (j = 1; j <= n; j++)
		{
			Numerator = Parse(ns, i, j);
			Denominator = Parse(ds, i, j);
			elt(RM[k], i - 1, j - 1) = RATIOI(Numerator, Denominator);
			FREEMPI(Numerator);
			FREEMPI(Denominator);
			
		}
	printf("\n");
	printf("RM[%u] = \n", k);
	PRINTMATR(0, RM[k]->R - 1, 0, RM[k]->C - 1, RM[k]);
	printf("\n");
	printf("Do you wish to alter RM[%d]? : ", k);
	if (GetYN())
		ALTER_MATR(k, k);
	return;
}

void BANDMAT_R()
{
	unsigned int k, m, j, u;
	MPR *A, *B;

	if (scanf21(&k, &m, &j) == 0)
		return;
	clearscreen();
	if (m < 2)
		printf("m < 2\n");
	else if (k > m - 1)
		printf("k >= m\n");
	else
	{
		printf("Enter the diagonal element:");
		A = INPUTR(&u);
		printf("\n");
		printf("Enter the band element:");
		B = INPUTR(&u);
		printf("\n");
		FREEMATR(RM[j]);
		RM[j] = BANDMATR(A, B, k, m);
		printf("RM[%d] = \n\n", j);
		PRINTMATR(0, RM[j]->R - 1, 0, RM[j]->C - 1, RM[j]); 
		FREEMPR(A);
		FREEMPR(B);
	}
	GetReturn();
	return;
}

void IDENTITY_R()
{
	unsigned int m, j;

	if (scanf11(&m, &j) == 0)
		return;
	clearscreen();
	if (m == 0)
		printf("m = 0\n");
	else
	{
		FREEMATR(RM[j]);
		RM[j] = IDENTITYR(m);
		printf("RM[%d] = \n\n", j);
		PRINTMATR(0, RM[j]->R - 1, 0, RM[j]->C - 1, RM[j]); 
	}
	GetReturn();
	return;
}

void JORDAN_R()
{
	unsigned int m, k, l;

	if (scanf12(&m, &k, &l) == 0)
		return;
	clearscreen();
	if (m == 0)
		printf("m = 0\n");
	else
	{
		FREEMATR(RM[l]);
		RM[l] = JORDANR(m, R[k]);
		printf("RM[%u] = \n\n", l);
		PRINTMATR(0, RM[l]->R - 1, 0, RM[l]->C - 1, RM[l]); 
	}
	GetReturn();
	return;
}

void COMPANION_R()
{
	unsigned int j, k;
	MPR *G;
	
	if (scanf02(&j, &k) == 0)
		return;
	clearscreen();
	if (DEGREEPR(PR[j]) <= 0)
		printf("degree of PR[%u] is <= 0\n", j);
	else
	{
		G = LEADPR(PR[j]);
		if (!EQONER(G))
			printf("PR[%u] is not monic\n", j);
		else
		{
			printf("The companion matrix of PR[%u] = ", j);
			PRINTPR(PR[j]);
			printf("\n");
			FREEMATR(RM[k]);
			RM[k] = COMPANIONR(PR[j]);
			printf("-> RM[%u] = \n\n", k);
			PRINTMATR(0, RM[k]->R - 1, 0, RM[k]->C - 1, RM[k]); 
		}
		FREEMPR(G);
	}
	GetReturn();
	return;
}

void INPUTR_()
{
	char c;

	unsigned int GoOnFlag = 1;
	while (GoOnFlag)
	{
		c = drawmenu(14);
		switch (c)
		{
		case 'n':
			INPUT_R();
			break;
		case 'p':
			INPUT_PR();
			break;
		case 'm':
			INPUTMAT_R();
			break;
		case 'M':
			INPUTMAT_PR();
			break;
		case 'b':
			FINPUTMATBC_R();
			break;
		case 'f':
			FINPUTMATFILE_R();
			break;
		case 'd':
			FINPUTMATFILE_RD();
			break;
		case 'v':
			BANDMAT_R();
			break;
		case 'i':
			IDENTITY_R();
			break;
		case 'j':
			JORDAN_R();
			break;
		case 'c':
			COMPANION_R();
			break;
                case 't':
			ALTER_MAT_R();
			break;
		case 'T':
			ALTER_MAT_PR();
			break;
		case 'q':
			Flush();
			GoOnFlag = 0;
			FPRINT_R(cR);
			FPRINTMAT_R(cRM);
			FPRINT_PR(cPR);
			FPRINTMAT_PR(cPRM);
			break;
		}
	}
}
