/* cm_iom.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "integer.h"
#include "fun.h"

extern POLYm Pm[];
extern MPMATm *mM[];
extern MPMATPm *PmM[];

extern char cPm[];
extern char cmM[];
extern char cPmM[];

void FINPUTMAT_m(char *filename, USI m)
/*
 * modular matrices mM[m],... are read from the specified file.  If the file
 * could not be opened, a diagnostic is printed.
 */
{
	int i;
	unsigned int j, k;
	FILE *infile;

	infile = Fopen_r(filename);
	fscanf(infile, "%u", &k);
	fgetc(infile);
	j = (m + k <= M0) ? m + k: M0;
	for (i = m; i < j; i++)
	{
		FREEMATm(mM[i]);
		mM[i] = FINPUTMATm(infile);
	}
	fclose(infile);
}

void FINPUT_Pm(char *filename)
/*
 * Reads the M0 modular polynomials from specified file.
 * If the file could not be opened, a diagnostic is printed.
 */
{
	int i;
	FILE *infile;
	infile = Fopen_r(filename);
	for (i = 0; i < M0; i++)
	{
		DELETE_Pm(Pm[i]);
		Pm[i] = INPUT1Pm(infile);
	}
	fclose(infile);
	return;
}

void FINPUTMAT_Pm(char *filename)
/*
 * The M0 modular polynomial matrices mM[] are read from the specified file.
 * If the file could not be opened, a diagnostic is printed.
 */
{
	FILE *infile;
	unsigned int i;

	infile = Fopen_r(filename);
	for (i = 0; i < M0; i++)
	{
		FREEMATPm(PmM[i]);
		PmM[i] = FINPUTMATPm(infile);
	}
	fclose(infile);
	return;
}

void PRINT_Pm()
{
	unsigned int j, l;
	char c;

	c = drawmenu(18);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("Pm[%u] = ", j);
				PRINTPm(Pm[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("Pm[%u] = ", j);
		PRINTPm(Pm[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTMAT_m()
{
	unsigned int j, l;
	char c;

	c = drawmenu(19);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("mM[%u] = \n", j);
				PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("mM[%u] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void PRINTMAT_Pm()
{
	unsigned int j, l;
	char c;

	c = drawmenu(20);
	switch (c)
	{
	case 'q':
		Flush();
		clearscreen();
		break;
	case 'p':
		Flush();
		l = 1;
		for (j = 0; j < M0; j++)
		{
			if (l)
			{
				clearscreen();
				printf("PmM[%u] = \n", j);
				PRINTMATPm(0, PmM[j]->R - 1, 0, PmM[j]->C - 1, PmM[j]);
				printf("\n");
			}
			else
				break;
			printf("\nCONTINUE? : ");
			l = GetYN();
		}
		break;
	case 'c':
		if (scanf01(&j) == 0)
			break;
		clearscreen();
		printf("PmM[%u] = \n", j);
		PRINTMATPm(0, PmM[j]->R - 1, 0, PmM[j]->C - 1, PmM[j]);
		printf("\n");
		GetReturn();
		break;
	}
}

void FPRINTMAT_m(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	fprintf(outfile, "%u\n", M0);
	for (i = 0; i < M0; i++)
		FPRINTMATm(outfile, 0, mM[i]->R - 1, 0, mM[i]->C - 1, mM[i]);
	fclose(outfile);
	return;
}

void LPRINTMAT_m(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "mM[%u] = \n\n", i);
		FPRINTMATm(outfile, 0, mM[i]->R - 1, 0, mM[i]->C - 1, mM[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void FPRINT_Pm(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		FPRINT1Pm(outfile, Pm[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void LPRINT_Pm(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "Pm[%u] = ", i);
		FPRINTPm(outfile, Pm[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void FPRINTMAT_Pm(char *filename)
{
	unsigned i;
	FILE *outfile;

	outfile = fopen(filename, "w");
	for (i = 0; i < M0; i++)
		FPRINTMAT1Pm(outfile, 0, PmM[i]->R - 1, 0, PmM[i]->C - 1, PmM[i]);
	fclose(outfile);
}

void LPRINTMAT_Pm(char *ptr)
{
	unsigned int i;
	FILE *outfile;

	outfile = fopen(ptr, "w");
	for (i = 0; i < M0; i++)
	{
		fprintf(outfile, "PmM[%u] = \n\n", i);
		FPRINTMATPm(outfile, 0, PmM[i]->R - 1, 0, PmM[i]->C - 1, PmM[i]);
		fprintf(outfile, "\n");
	}
	fclose(outfile);
	return;
}

void PRINTm_()
{
	char c;
	unsigned int GoOnFlag = 1;

	while (GoOnFlag)
	{
		c = drawmenu(17);
		switch (c)
		{
		case 'p':
			PRINT_Pm();
			break;
		case 'm':
			PRINTMAT_m();
			break;
		case 'M':
			PRINTMAT_Pm();
			break;
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		}
	}
}

void LPRINTm_()
{
	char c;
	unsigned int GoOnFlag = 1;
	char tmpfile[20];

	strcpy(tmpfile,"cmXXXXXX");
	mktemp(tmpfile);
	while (GoOnFlag)
	{
		c = drawmenu(21);
		Flush();
		switch (c)
		{
		case 'p':
			clearscreen();
			LPRINT_Pm(tmpfile);
			printfile(tmpfile);
			printf("polynomials sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'm':
			clearscreen();
			LPRINTMAT_m(tmpfile);
			printfile(tmpfile);
			printf("modular matrices sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'M':
			clearscreen();
			LPRINTMAT_Pm(tmpfile);
			printfile(tmpfile);
			printf("polynomial matrices sent to line-printer.\n");
			GetReturn();
			remove(tmpfile);
			break;
		case 'q':
			GoOnFlag = 0;
			break;
		}
	}
}

void INPUT_Pm()
{
	unsigned int j, m, u;

	if (scanf011(&j, &m) == 0)
		return;
	clearscreen();
	printf("enter Pm[%u]: ", j);
	DELETE_Pm(Pm[j]);
	Pm[j] = INPUTPm(&u, m);
	printf("Pm[%u] = ", j);
	PRINTPm(Pm[j]);
	printf("\n\n");
	GetReturn();
	return;
}

void ALTER_MATm(USI l, USI m)
/*
 * alter matrix *mM[l] to *mM[m].
 */
{
	unsigned int GoOnFlag = 1, Flag, i, j, n, r, *CA;
	int e;
	char c, *gets();
	MPMATm *Temp;

	while (GoOnFlag)
	{
		c = drawmenu(15);
		switch (c)
		{
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		case 'r':
			if (scanf10(&i) == 0)
				break;
			if (i > mM[l]->R)
				printf("i > number of rows of matrix mM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (mM[l]->R == 1)
				printf("matrix mM[%u] has one row\n\n", l);
			else
			{
				clearscreen();
				printf("deleting row %u of mM[%u] =\n", i, l); 
				PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
				Temp = mM[m];
				mM[m] = DELETE_ROWm(i, mM[l]);
				FREEMATm(Temp);
				printf("-> mM[%u] = \n\n", m);
				PRINTMATm(0, mM[m]->R - 1, 0, mM[m]->C - 1, mM[m]);
			}
			GetReturn();
			break;
		case 'c':
			if (scanf10(&j) == 0)
				break;
			if (j > mM[l]->C)
				printf("j > number of columns of matrix mM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else if (mM[l]->C == 1)
				printf("matrix mM[%u] has one column\n\n", l);
			else
			{
				clearscreen();
				printf("deleting column %u of mM[%u] =\n", j, l); 
				PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
				Temp = mM[m];
				mM[m] = DELETE_COLm(j, mM[l]);
				FREEMATm(Temp);
				printf("-> mM[%u] = \n\n", m);
				PRINTMATm(0, mM[m]->R - 1, 0, mM[m]->C - 1, mM[m]);
			}
			GetReturn();
			break;
		case 'a':
			if (scanf20(&i, &j) == 0)
				break;
			if (i > mM[l]->R)
				printf("i > number of rows of matrix mM[%u]\n\n", l);
			else if (j > mM[l]->C)
				printf("j > number of columns of matrix mM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement %u-%u element of matrix mM[%u] :", i, j, l);
				Temp = mM[m];
				mM[m] = COPYMATm(mM[l]);
				if (scanf10(&(mM[m]->V[i - 1][j - 1])) == 0)
					break;
				printf("mM[%u] ->  \n\n", m);
				PRINTMATm(0, mM[m]->R - 1, 0, mM[m]->C - 1, mM[m]);
				FREEMATm(Temp);
			}
			GetReturn();
			break;
		case 'f':
			if (scanf10(&i) == 0)
				break;
			if (i > mM[l]->R)
				printf("i > number of rows of matrix mM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else
			{
				printf("enter the replacement row %u of matrix mM[%u] =\n", i, l);
				PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
				printf(":");
				Temp = mM[m];
				mM[m] = COPYMATm(mM[l]);
				for (j = 0; j < mM[m]->C; j++)
				{
					if (scanf("%u", &(mM[m]->V[i - 1][j])) != 1)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				Flush();
				printf("mM[%u] ->  \n\n", m);
				PRINTMATm(0, mM[m]->R - 1, 0, mM[m]->C - 1, mM[m]);
				FREEMATm(Temp);
			}
			GetReturn();
			break;
		case 'g':
			if (scanf10(&j) == 0)
				break;
			if (j > mM[l]->C)
				printf("j > number of columns of matrix mM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement column %u of matrix mM[%u] = \n", j, l);
				PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
				printf(":");
				Temp = mM[m];
				mM[m] = COPYMATm(mM[l]);
				for (i = 0; i < mM[m]->R; i++)
				{
					if (scanf("%u", &(mM[m]->V[i][j - 1])) != 1)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				Flush();
				printf("mM[%u] ->  \n\n", m);
				PRINTMATm(0, mM[m]->R - 1, 0, mM[m]->C - 1, mM[m]);
				FREEMATm(Temp);
			}
			GetReturn();
			break;
		case 's':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of rows in the submatrix:", mM[l]->R);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= mM[l]->R)
			{
				printf("n >= %u\n", mM[l]->R);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, mM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", mM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a row index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= mM[l]->R)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, mM[l]->R);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = mM[m];
			mM[m] = SUBMATROWSm(mM[l], CA, n);
			FREEMATm(Temp);
			printf("mM[%u] ->  \n\n", m);
			PRINTMATm(0, mM[m]->R - 1, 0, mM[m]->C - 1, mM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		case 't':
			Flag = 1;
			printf("enter n (1 <= n < %u), the number of columns in the submatrix:", mM[l]->C);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= mM[l]->C)
			{
				printf("n >= %u\n", mM[l]->C);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, mM[l]->C);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", mM[l]->C);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a column index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= mM[l]->C)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, mM[l]->C);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = mM[m];
			mM[m] = SUBMATCOLUMNSm(mM[l], CA, n);
			FREEMATm(Temp);
			printf("mM[%u] ->  \n\n", m);
			PRINTMATm(0, mM[m]->R - 1, 0, mM[m]->C - 1, mM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		}
	}
}

void ALTER_MAT_m()
{
	unsigned int l, m;

	if (scanf02(&l, &m) == 0)
		return;
	clearscreen();
	printf("altering matrix mM[%u] = \n\n", l);
	PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	GetReturn();
	ALTER_MATm(l, m);
	return;
}

void ALTER_MATPm(USI l, USI m)
/*
 * alter matrix *PmM[l] to *PmM[m].
 */
{
	unsigned int GoOnFlag = 1, Flag, i, j, n, r, *CA, u, M;
	int e;
	char c, *gets();
	MPMATPm *Temp;

	while (GoOnFlag)
	{
		c = drawmenu(16);
		switch (c)
		{
		case 'q':
			Flush();
			GoOnFlag = 0;
			break;
		case 'r':
			if (scanf20(&i, &M) == 0)
				break;
			if (i > PmM[l]->R)
				printf("i > number of rows of matrix PmM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (PmM[l]->R == 1)
				printf("matrix PmM[%u] has one row\n\n", l);
			else
			{
				clearscreen();
				printf("deleting row %u of PmM[%u] =\n", i, l); 
				PRINTMATPm(0, PmM[l]->R - 1, 0, PmM[l]->C - 1, PmM[l]);
				Temp = PmM[m];
				PmM[m] = DELETE_ROWPm(i, PmM[l], M);
				FREEMATPm(Temp);
				printf("-> PmM[%u] = \n\n", m);
				PRINTMATPm(0, PmM[m]->R - 1, 0, PmM[m]->C - 1, PmM[m]);
			}
			GetReturn();
			break;
		case 'c':
			if (scanf20(&j, &M) == 0)
				break;
			if (j > PmM[l]->C)
				printf("j > number of columns of matrix PmM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else if (PmM[l]->C == 1)
				printf("matrix PmM[%u] has one column\n\n", l);
			else
			{
				clearscreen();
				printf("deleting column %u of PmM[%u] =\n", j, l); 
				PRINTMATPm(0, PmM[l]->R - 1, 0, PmM[l]->C - 1, PmM[l]);
				Temp = PmM[m];
				PmM[m] = DELETE_COLPm(j, PmM[l], M);
				FREEMATPm(Temp);
				printf("-> PmM[%u] = \n\n", m);
				PRINTMATPm(0, PmM[m]->R - 1, 0, PmM[m]->C - 1, PmM[m]);
			}
			GetReturn();
			break;
		case 'a':
			if (scanf30(&i, &j, &M) == 0)
				break;
			if (i > PmM[l]->R)
				printf("i > number of rows of matrix PmM[%u]\n\n", l);
			else if (j > PmM[l]->C)
				printf("j > number of columns of matrix PmM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				clearscreen();
				printf("enter the replacement %u-%u element of matrix PmM[%u]:", i, j, l);
				Temp = PmM[m];
				PmM[m] = COPYMATPm(PmM[l], M);
				DELETE_Pm(PmM[m]->V[i - 1][j - 1]);
				PmM[m]->V[i - 1][j - 1] = INPUTPm(&u, M);
				printf("PmM[%u] ->  \n\n", m);
				PRINTMATPm(0, PmM[m]->R - 1, 0, PmM[m]->C - 1, PmM[m]);
				FREEMATPm(Temp);
			}
			GetReturn();
			break;
		case 'f':
			if (scanf20(&i, &M) == 0)
				break;
			if (i > PmM[l]->R)
				printf("i > number of rows of matrix PmM[%u]\n\n", l);
			else if (i == 0)
				printf("i = 0\n");
			else
			{
				printf("enter the replacement row %u of matrix PmM[%u] =\n ", i, l);
				PRINTMATPm(0, PmM[l]->R - 1, 0, PmM[l]->C - 1, PmM[l]);
				printf(":");
				Temp = PmM[m];
				PmM[m] = COPYMATPm(PmM[l], M);
				for (j = 0; j < PmM[m]->C; j++)
				{
					printf("Entering polynomial[%u][%u]:\n", i - 1, j);
					DELETE_Pm(PmM[m]->V[i - 1][j]);
					PmM[m]->V[i - 1][j] = INPUTPm(&u, M);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				printf("PmM[%u] ->  \n\n", m);
				PRINTMATPm(0, PmM[m]->R - 1, 0, PmM[m]->C - 1, PmM[m]);
				FREEMATPm(Temp);
			}
			GetReturn();
			break;
		case 'g':
			if (scanf20(&j, &M) == 0)
				break;
			if (j > PmM[l]->C)
				printf("j > number of columns of matrix PmM[%u]\n\n", l);
			else if (j == 0)
				printf("j = 0\n");
			else
			{
				printf("enter the replacement column %u of matrix PmM[%u] =\n", j, l);
				PRINTMATPm(0, PmM[l]->R - 1, 0, PmM[l]->C - 1, PmM[l]);
				printf(":");
				Temp = PmM[m];
				PmM[m] = COPYMATPm(PmM[l], M);
				for (i = 0; i < PmM[m]->R; i++)
				{
					printf("Entering polynomial[%u][%u]:\n", i, j - 1);
					DELETE_Pm(PmM[m]->V[i][j - 1]);
					PmM[m]->V[i][j - 1] = INPUTPm(&u, M);
					if (u == 0)
					{
						fprintf(stderr, "matrix input interrupted prematurely\n");
						break;
					}
				}
				printf("PmM[%u] ->  \n\n", m);
				PRINTMATPm(0, PmM[m]->R - 1, 0, PmM[m]->C - 1, PmM[m]);
				FREEMATPm(Temp);
			}
			GetReturn();
			break;
		case 's':
			Flag = 1;
			if (scanf10(&M) == 0)
				break;
			printf("enter n (1 <= n < %u), the number of rows in the submatrix:", PmM[l]->R);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= PmM[l]->R)
			{
				printf("n >= %u\n", PmM[l]->R);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, PmM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", PmM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a row index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= PmM[l]->R)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, PmM[l]->R);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = PmM[m];
			PmM[m] = SUBMATROWSPm(PmM[l], CA, n, M);
			FREEMATPm(Temp);
			printf("PmM[%u] ->  \n\n", m);
			PRINTMATPm(0, PmM[m]->R - 1, 0, PmM[m]->C - 1, PmM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		case 't':
			Flag = 1;
			if (scanf10(&M) == 0)
				break;
			printf("enter n (1 <= n < %u), the number of columns in the submatrix:", PmM[l]->C);
			if (scanf10(&n) == 0)
				break;
			if (n == 0)
			{
				printf("n = 0\n");
				GetReturn();
				break;
			}
			if (n >= PmM[l]->C)
			{
				printf("n >= %u\n", PmM[l]->C);
				GetReturn();
				break;
			}
			e = 0;
			if (n > 1)
				printf("enter %u array integers i, 1 <= i <= %u) :", n, PmM[l]->R);
			else
				printf("enter an array integer i, 1 <= i <= %u) :", PmM[l]->R);
			CA = (unsigned int *)mmalloc(n * sizeof(unsigned int));
			for (i = 0; i < n; i++)
			{
				e += scanf("%u", &r);
				if (r == 0)
				{
					printf("you have entered 0 as a column index\n");
					Flag = 0;
					break;
				}
				for (j = 0; j < i; j++)
				{
					if (r - 1 == CA[j])
					{
						printf("you have entered the number %u twice\n", r);
						Flag = 0;
						break;
					}
				}
				if (Flag == 0)
					break;
				CA[i] = r - 1;
			}
			Flush();
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				GetReturn();	
				break;
			}
			if (e != n)
			{
				printf("\n\ninput error : number of successfully entered integers = %d\n", e);	
				GetReturn();
				break;
			}
			for (i = 0; i < n; i++)
			{
				if (CA[i] >= PmM[l]->C)
				{
					printf("\n\ninput error : %u-th array integer > %u\n", i + 1, PmM[l]->C);
					Flag = 0;
					GetReturn();
					break;
				}
			}
			if (Flag == 0)
			{
				ffree((char *)CA, n * sizeof(unsigned int));
				break;
			}
			Temp = PmM[m];
			PmM[m] = SUBMATCOLUMNSPm(PmM[l], CA, n, M);
			FREEMATPm(Temp);
			printf("PmM[%u] ->  \n\n", m);
			PRINTMATPm(0, PmM[m]->R - 1, 0, PmM[m]->C - 1, PmM[m]);
			GetReturn();
			ffree((char *)CA, n * sizeof(unsigned int));
			break;
		}
	}
}

void ALTER_MAT_Pm()
{
	unsigned int l, m;

	if (scanf02(&l, &m) == 0)
		return;
	clearscreen();
	printf("altering matrix PmM[%u] = \n\n", l);
	PRINTMATPm(0, PmM[l]->R - 1, 0, PmM[l]->C - 1, PmM[l]);
	GetReturn();
	ALTER_MATPm(l, m);
	return;
}

void INPUTMAT_m()
{
	unsigned int k, m, u;

	if (scanf011(&k, &m) == 0)
		return;
	clearscreen();
	printf("enter mM[%u]:\n\n", k);
	FREEMATm(mM[k]);
	mM[k] = INPUTMATm(&u, m);
	if (u == 0)
		return;
	printf("\n");
	printf("mM[%u] = \n", k);
	PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	printf("\n");
	printf("Do you wish to alter mM[%u]? :", k);
	if (GetYN())
		ALTER_MATm(k, k);
	return;
}

void INPUTMAT_Pm()
{
	unsigned int k, m, u;

	if (scanf011(&k, &m) == 0)
		return;
	clearscreen();
	printf("enter PmM[%u]:\n\n", k);
	FREEMATPm(PmM[k]);
	PmM[k] = INPUTMATPm(&u, m);
	if (u == 0)
		return;
	printf("\n");
	printf("PmM[%u] = \n", k);
	PRINTMATPm(0, PmM[k]->R - 1, 0, PmM[k]->C - 1, PmM[k]);
	printf("\n");
	printf("Do you wish to alter PmM[%u]? :", k);
	if (GetYN())
		ALTER_MATPm(k, k);
	return;
}

void FINPUTMATFILE_m()
{
	char filename[50];
	unsigned int m;
	FILE *fp;

	getchar();
	clearscreen();
	printf("enter the name of your matrix file: ");
	gets(filename);
	if ((fp = fopen(filename, "r")) == NULL)
	{
		printf("The file `%s' does not exist!\n",filename);
		GetReturn();
	}
	else
	{
		fclose(fp);
		printf("\n");
		printf("To store the matrices at consecutive array positions starting at mM[m],\n\n");
		printf("please enter a non-negative integer m, 0<=m<=%u:  ", M0 - 1);
		if (scanf01(&m) == 0)
			return;
		FINPUTMAT_m(filename, m);
		printf("\n");
		printf("matrices from file %s successfully inputted, starting at mM[%u]:\n", filename, m);
		GetReturn();
	}
	return;
}

void FINPUTMATBC_m()
{
	unsigned int m, n, i, j, k, z;
	char ns[100];
	MPI *NUMBER;

	if (scanf011(&k, &z) == 0)
		return;
	clearscreen();
	printf("entering mM[%u]:\n\n", k);
	printf("enter the number of rows and number of columns: ");
	if (scanf20(&m, &n) == 0)
		return;
	if (m == 0 || n == 0)
	{
		printf("one of the integers was zero!\n");
		GetReturn();
		return;
	}
	printf("Enter an integer expression  for a[i][j]:\n\n");
	printf("e.g. i^2 - i*j + i + j^5\n\n");
	printf("enter the expression for the residue a[i][j]: ");
	scanf("%99[^\n]", ns);
	Flush();
	FREEMATm(mM[k]);
	mM[k] = BUILDMATm(m, n);
	for (i = 1; i <= m; i++)
		for (j = 1; j <= n; j++)
		{
			NUMBER = Parse(ns, i, j);
			mM[k]->V[i - 1][j - 1] = MOD_(NUMBER, z);
			FREEMPI(NUMBER);
		}
	printf("\n");
	printf("mM[%u] = \n", k);
	PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	printf("\n");
	printf("Do you wish to alter mM[%d]? : ", k);
	if (GetYN())
		ALTER_MATm(k, k);
	return;
}

void BANDMAT_m()
{
	unsigned int a, b, k, m, j;

	if (scanf21(&k, &m, &j) == 0)
		return;
	clearscreen();
	if (m < 2)
		printf("m < 2\n");
	else if (k > m - 1)
		printf("k >= m\n");
	else
	{
		printf("Enter the diagonal element:");
		if (scanf10(&a) == 0)
			return;
		printf("\n");
		printf("Enter the band element:");
		if (scanf10(&b) == 0)
			return;
		printf("\n");
		FREEMATm(mM[j]);
		mM[j] = BANDMATm(a, b, k, m);
		printf("Rm[%d] = \n\n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]); 
	}
	GetReturn();
	return;
}

void IDENTITY_m()
{
	unsigned int m, j;

	if (scanf11(&m, &j) == 0)
		return;
	clearscreen();
	if (m == 0)
		printf("m = 0\n");
	else
	{
		FREEMATm(mM[j]);
		mM[j] = IDENTITYm(m);
		printf("mM[%d] = \n\n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]); 
	}
	GetReturn();
	return;
}

void JORDAN_m()
{
	unsigned int m, l, k;

	if (scanf21(&m, &k, &l) == 0)
		return;
	clearscreen();
	if (m == 0)
		printf("m = 0\n");
	else
	{
		FREEMATm(mM[l]);
		mM[l] = JORDANm(m, k);
		printf("mM[%u] = \n\n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]); 
	}
	GetReturn();
	return;
}

void COMPANION_m()
{
	unsigned int g, j, k, m;
	
	if (scanf021(&j, &k, &m) == 0)
		return;
	clearscreen();
	if (DEGREEPm(Pm[j]) <= 0)
		printf("degree of Pm[%u] is <= 0\n", j);
	else
	{
		g = Pm[j]->COEF;
		if (g != 1)
			printf("Pm[%u] is not monic\n", j);
		else
		{
		printf("The companion matrix of Pm[%u] = ", j);
		PRINTPm(Pm[j]);
		printf("\n");
		FREEMATm(mM[k]);
		mM[k] = COMPANIONm(Pm[j], m);
		printf("-> mM[%u] = \n\n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]); 
		}
	}
	GetReturn();
	return;
}

void INPUTm_()
{
	char c;

	unsigned int GoOnFlag = 1;
	while (GoOnFlag)
	{
		c = drawmenu(14);
		switch (c)
		{
		case 'p':
			INPUT_Pm();
			break;
		case 'm':
			INPUTMAT_m();
			break;
		case 'M':
			INPUTMAT_Pm();
			break;
		case 'b':
			FINPUTMATBC_m();
			break;
		case 'f':
			FINPUTMATFILE_m();
			break;
		case 'v':
			BANDMAT_m();
			break;
		case 'i':
			IDENTITY_m();
			break;
		case 'j':
			JORDAN_m();
			break;
		case 'c':
			COMPANION_m();
			break;
                case 't':
			ALTER_MAT_m();
			break;
		case 'T':
			ALTER_MAT_Pm();
			break;
		case 'q':
			Flush();
			GoOnFlag = 0;
			FPRINT_Pm(cPm);
			FPRINTMAT_m(cmM);
			FPRINTMAT_Pm(cPmM);
			break;
		}
	}
}
