
/* cm_mM1.c */

/* modular arithmetic matrix routines used in cmm.c */

#include <stdio.h>
#include "integer.h"
#include "fun.h"

extern MPMATm *mM[];
extern POLYm Pm[];

extern char cmM[];

void ADDMAT_m()
{
	unsigned int j, k, l, m;
	MPMATm *Temp;

	if (scanf031(&j, &k, &l, &m) == 0)
		return;
	if (mM[j]->R != mM[k]->R || mM[j]->C != mM[k]->C)
		printf("matrices mM[%u] and mM[%u] have incompatible sizes\n", j, k);
	else
	{
		printf("mM[%u] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("+ mM[%u] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		Temp = mM[l];
		mM[l] = ADDMATm(mM[j], mM[k], m);
		FREEMATm(Temp);
		printf("(mod %u) ->\n", m);
		printf("mM[%u] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void SUBMAT_m()
{
	unsigned int j, k, l, m;
	MPMATm *Temp;

	if (scanf031(&j, &k, &l, &m) == 0)
		return;
	if (mM[j]->R != mM[k]->R || mM[j]->C != mM[k]->C)
		printf("matrices mM[%u] and mM[%u] have incompatible sizes\n", j, k);
	else
	{
		printf("mM[%u] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("- mM[%u] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		Temp = mM[l];
		mM[l] = SUBMATm(mM[j], mM[k], m);
		FREEMATm(Temp);
		printf("(mod %u) ->\n", m);
		printf("mM[%u] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void SCALARMAT_m()
{
	unsigned int j, k, l, m;
	MPMATm *Temp;

	if (scanf121(&j, &k, &l, &m) == 0)
		return;
	printf("%u times ", j);
	printf("mM[%u] = \n", k);
	PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	Temp = mM[l];
	mM[l] = SCALARMATm(mM[k], j, m);
	FREEMATm(Temp);
	printf("->\n");
	printf("mM[%u] = \n", l);
	PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	GetReturn();
	return;
}

void TRANSPOSE_m()
{
	unsigned int j, k;
	MPMATm *Temp;

	if (scanf02(&j, &k) == 0)
		return;
	printf("The transpose of mM[%u] = \n", j);
	PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
	Temp = mM[k];
	mM[k] = TRANSPOSEm(mM[j]);
	FREEMATm(Temp);
	printf("->\n");
	printf("mM[%u] = \n", k);
	PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	GetReturn();
	return;
}

void POWERMAT_m()
{
	unsigned int j, k, n, m;
	MPMATm *Temp;

	if (scanf121(&n, &j, &k, &m) == 0)
		return;
	if (mM[j]->R != mM[j]->C)
		printf("Matrix mM[%u] is not square\n", j);
	else
	{
		printf("The power %u of mM[%u] = \n", n, j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		Temp = mM[k];
		mM[k] = POWERMATm(mM[j], n, m);
		FREEMATm(Temp);
		printf("(mod %u) ->\n", m);
		printf("mM[%u] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	}
	GetReturn();
	return;
}

void DIAG_m()
{
	unsigned int j, k, l, m;
	MPMATm *Temp;

	if (scanf121(&j, &k, &l, &m) == 0)
		return;
	if (mM[j]->R != mM[j]->C)
		printf("Matrix mM[%u] is not square\n", j);
	else
	{
		printf("mM[%u] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf(" - %u * I(%u)\n", k, mM[j]->R);
		Temp = mM[l];
		mM[l] = DIAGm(mM[j], k, m);
		FREEMATm(Temp);
		printf("(mod %u) -> \n", m);
		printf("mM[%u] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void MULTMAT_m()
{
	unsigned int j, k, l, m;
	MPMATm *Temp;

	if (scanf031(&j, &k, &l, &m) == 0)
		return;
	if (mM[j]->C != mM[k]->R)
		printf("matrices mM[%u] and mM[%u] have incompatible sizes\n", j, k);
	else
	{
		printf("mM[%u] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("* mM[%u] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		Temp = mM[l];
		mM[l] = MULTMATm(mM[j], mM[k], m);
		FREEMATm(Temp);
		printf("(mod %u) ->\n", m);
		printf("mM[%u] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void MINUSMAT_m()
{
	unsigned int j, k, m;
	MPMATm *Temp;

	if (scanf021(&j, &k, &m) == 0)
		return;
	printf("- mM[%d] = \n", j);
	PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
	Temp = mM[k];
	mM[k] = MINUSMATm(mM[j], m);
	FREEMATm(Temp);
	printf("(mod %u) ->\n", m);
	printf("mM[%d] = \n", k);
	PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	GetReturn();
	return;
}

void VALMAT_m()
{
	unsigned int j, k, l, m;
	MPMATm *Temp;

	if (scanf031(&j, &k, &l, &m) == 0)
		return;
	if (mM[j]->R != mM[j]->C)
		printf("matrix mM[%u] is not square\n", j);
	else
	{
		printf("Pm[%u] = ", j);
		PRINTPm(Pm[j]);
		printf("\nevaluated at\nmM[%u] (mod %u) =\n", k, m);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		Temp = mM[l];
		mM[l] = VALMATm(Pm[j], mM[k], m);
		FREEMATm(Temp);
		printf("\n->\nmM[%u] =\n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void MATRIX_ARITH_m()
{
	char c;
	int GoOnFlag = 1;

	while (GoOnFlag)
	{
		c = drawmenu(7);
		switch (c)
		{
		case 'a':
			ADDMAT_m();
			break;
		case 's':
			SUBMAT_m();
			break;
		case 'f':
			SCALARMAT_m();
			break;
		case '*':
			TRANSPOSE_m();
			break;
		case 'e':
			POWERMAT_m();
			break;
		case 't':
			MULTMAT_m();
			break;
		case 'm':
			MINUSMAT_m();
			break;
		case 'z':
			DIAG_m();
			break;
		case 'v':
			VALMAT_m();
			break;
		case 'p':
			PRINTm_();
			break;
		case 'x':
			INPUTm_();
			break;
		case 'q':
			FPRINTMAT_m(cmM);
			GoOnFlag = 0;
			break;
		}
	}
}

void JOIN_COLUMNS_m()
{
	unsigned int j, k, l;
	MPMATm *Temp;

	if (scanf03(&j, &k, &l) == 0)
		return;
	if (mM[j]->R != mM[k]->R)
		printf("matrices mM[%u] and mM[%u] have row numbers\n", j, k);
	else
	{
		printf("column-join mM[%u] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("to mM[%u] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		Temp = mM[l];
		mM[l] = JCOLSMATm(mM[j], mM[k]);
		FREEMATm(Temp);
		printf("->\n");
		printf("mM[%u] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void JOIN_ROWS_m()
{
	unsigned int j, k, l;
	MPMATm *Temp;

	if (scanf03(&j, &k, &l) == 0)
		return;
	if (mM[j]->C != mM[k]->C)
		printf("matrices mM[%u] and mM[%u] have column numbers\n", j, k);
	else
	{
		printf("row-join mM[%u] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("to mM[%u] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		Temp = mM[l];
		mM[l] = JROWSMATm(mM[j], mM[k]);
		FREEMATm(Temp);
		printf("->\n");
		printf("mM[%u] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void JCOLSIDENTITY_m()
{
	unsigned int j, k;
	MPMATm *Temp;

	if (scanf02(&j, &k) == 0)
		return;
	printf("join mM[%u] = \n", j);
	PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
	printf("to I[%u]\n", mM[j]->R);
	Temp = mM[k];
	mM[k] = JCOLSIDENTITYm(mM[j]);
        FREEMATm(Temp);
	printf("->\n");
	printf("mM[%u] = \n", k);
	PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	GetReturn();
	return;
}

void KRONECKER_m()
{
	unsigned int j, k, l, m;
	MPMATm *Temp;

	if (scanf031(&j, &k, &l, &m) == 0)
		return;
	printf("mM[%d] = \n", j);
	PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
	printf("KRONECKER (mod %u)  mM[%d] = \n", m, k);
	PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	Temp = mM[l];
	mM[l] = KRONECKERm(mM[j], mM[k], m);
	FREEMATm(Temp);
	printf("->\n");
	printf("mM[%d] = \n", l);
	PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	GetReturn();
	return;
}

void DIRECT_SUM_m()
{
	unsigned int j, k, l;
	MPMATm *Temp;

	if (scanf03(&j, &k, &l) == 0)
		return;
	printf("mM[%d] = \n", j);
	PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
	printf("DIRECT_SUM mM[%d] = \n", k);
	PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	Temp = mM[l];
	mM[l] = DIRECT_SUMm(mM[j], mM[k]);
	FREEMATm(Temp);
	printf("->\n");
	printf("mM[%d] = \n", l);
	PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	GetReturn();
	return;
}
 
void UNWRAPMAT_m()
{
	unsigned int j, k;
	MPMATm *Temp;

	if (scanf02(&j, &k) == 0)
		return;
	printf("unwrap mM[%d] = \n", j);
	PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
	Temp = mM[k];
	mM[k] = UNWRAPMATm(mM[j]);
	FREEMATm(Temp);
	printf("done\n");
	GetReturn();
	return;
}

void JOINMAT_m()
{
	char c;
	int GoOnFlag = 1;

	while (GoOnFlag)
	{
		c = drawmenu(8);
		switch (c)
		{
		case 'c':
			JOIN_COLUMNS_m();
			break;
		case 'r':
			JOIN_ROWS_m();
			break;
		case 'i':
			JCOLSIDENTITY_m();
			break;
		case 'k':
			KRONECKER_m();
			break;
		case 'd':
			DIRECT_SUM_m();
			break;
		case 'u':
			UNWRAPMAT_m();
			break;
		case 'q':
			FPRINTMAT_m(cmM);
			GoOnFlag = 0;
			break;
		}	
	}
}

void SWAP_ROWSm_()
{
	unsigned int j, k, p, q;
	MPMATm *Temp;

	if (scanf22(&p, &q, &j, &k) == 0)
		return;
	if (p < 1 || p > mM[j]->R)
		printf("p < 1 or > number of rows of mM[%u]\n", j);
	else if (q < 1 || q > mM[j]->R)
		printf("q < 1 or > number of rows of mM[%u]\n", j);
	else
	{
		printf("mM[%d] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("->\n");
		Temp = mM[k];
		mM[k] = SWAP_ROWSm(p - 1, q - 1, mM[j]);
		FREEMATm(Temp);
		printf("mM[%d] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	}
	GetReturn();
	return;
}

void SWAP_COLSm_()
{
	unsigned int j, k, p, q;
	MPMATm *Temp;

	if (scanf22(&p, &q, &j, &k) == 0)
		return;
	if (p < 1 || p > mM[j]->C)
		printf("p < 1 or > number of columns of mM[%u]\n", j);
	else if (q < 1 || q > mM[j]->C)
		printf("q < 1 or > number of columns of mM[%u]\n", j);
	else
	{
		printf("mM[%d] = \n", j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("->\n");
		Temp = mM[k];
		mM[k] = SWAP_COLSm(p - 1, q - 1, mM[j]);
		FREEMATm(Temp);
		printf("mM[%d] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	}
	GetReturn();
	return;
}

void SCALAR_ROWm_()
{
	unsigned int j, k, l, p, m;
	MPMATm *Temp;

	if (scanf221(&j, &p, &k, &l, &m) == 0)
		return;
	if (p < 1 || p > mM[k]->R)
		printf("p < 1 or > number of rows of mM[%u]\n", k);
	else
	{
		printf("mM[%u] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		printf("->\n");
		Temp = mM[l];
		mM[l] = SCALAR_ROWm(p - 1, j, mM[k], m);
		FREEMATm(Temp);
		printf("mM[%u] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void SCALAR_COLm_()
{
	unsigned int j, k, l, p, m;
	MPMATm *Temp;

	if (scanf221(&j, &p, &k, &l, &m) == 0)
		return;
	if (p < 1 || p > mM[k]->C)
		printf("p < 1 or > number of columns of mM[%u]\n", k);
	else
	{
		printf("mM[%u] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		printf("->\n");
		Temp = mM[l];
		mM[l] = SCALAR_COLm(p - 1, j, mM[k], m);
		FREEMATm(Temp);
		printf("mM[%u] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void ADD_MULT_ROWm_()
{
	unsigned int j, k, l, p, q, m;
	MPMATm *Temp;

	if (scanf321(&j, &p, &q, &k, &l, &m) == 0)
		return;
	if (q < 1 || q > mM[k]->R)
		printf("q < 1 or > number of rows of mM[%u]\n", k);
	else if (p < 1 || p > mM[k]->R)
		printf("p < 1 or > number of rows of mM[%u]\n", k);
	else
	{
		printf("mM[%d] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		printf("->\n");
		Temp = mM[l];
		mM[l] = ADD_MULT_ROWm(p - 1, q - 1, j, mM[k], m);
		FREEMATm(Temp);
		printf("mM[%d] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void ADD_MULT_COLm_()
{
	unsigned int j, k, l, p, q, m;
	MPMATm *Temp;

	if (scanf321(&j, &p, &q, &k, &l, &m) == 0)
		return;
	if (q < 1 || q > mM[k]->C)
		printf("q < 1 or > number of columns of mM[%u]\n", k);
	else if (p < 1 || p > mM[k]->C)
		printf("p < 1 or > number of columns of mM[%u]\n", k);
	else
	{
		printf("mM[%d] = \n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
		printf("->\n");
		Temp = mM[l];
		mM[l] = ADD_MULT_COLm(p - 1, q - 1, j, mM[k], m);
		FREEMATm(Temp);
		printf("mM[%d] = \n", l);
		PRINTMATm(0, mM[l]->R - 1, 0, mM[l]->C - 1, mM[l]);
	}
	GetReturn();
	return;
}

void OPS_m()
{
	int GoOnFlag = 1;
        char c;

	while (GoOnFlag)
	{
		c = drawmenu(9);
		switch (c)
		{
		case 'i':
			SWAP_ROWSm_();
			break;
		case 'j':
			SWAP_COLSm_();
			break;
		case 'm':
			SCALAR_ROWm_();
			break;
		case 'n':
			SCALAR_COLm_();
			break;
		case 'a':
			ADD_MULT_ROWm_();
			break;
		case 'b':
			ADD_MULT_COLm_();
			break;
		case 'x':
			INPUTm_();
			break;
		case 'q':
			FPRINTMAT_m(cmM);
			GoOnFlag = 0;
			break;
		}
	} 
}

void Extractm_()
{
	char c;
	int GoOnFlag = 1;
	while (GoOnFlag)
	{
		c = drawmenu(10);
		switch (c)
		{
		case 'r':
			ExtractRow_m();
			break;
		case 'c':
			ExtractCol_m();
			break;
		case 'q':
			FPRINTMAT_m(cmM);
			GoOnFlag = 0;
			break;
		}
	}
}

void ExtractRow_m()
{
	unsigned int j, k, q;
	MPMATm *Temp;

	if (scanf12(&q, &j, &k) == 0)
		return;
	if (q < 1 || q > mM[j]->R)
		printf("q < 1 or > number of rows of mM[%u]\n", j);
	else
	{
		Temp = mM[k];
		mM[k] = ROWVECTORm(mM[j], q - 1);
		FREEMATm(Temp);
		printf("%u-th row vector of mM[%d] =\n", q, j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("->\nmM[%d] =\n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	}
	GetReturn();
	return;
}

void ExtractCol_m()
{
	unsigned int j, k, q;
	MPMATm *Temp;

	if (scanf12(&q, &j, &k) == 0)
		return;
	if (q < 1 || q > mM[j]->C)
		printf("q < 1 or > number of columns of mM[%u]\n", j);
	else
	{
		Temp = mM[k];
		mM[k] = ColVectorm(mM[j], q - 1);
		FREEMATm(Temp);
		printf("%u-th column vector of mM[%d] =\n", q, j);
		PRINTMATm(0, mM[j]->R - 1, 0, mM[j]->C - 1, mM[j]);
		printf("->\nmM[%d] =\n", k);
		PRINTMATm(0, mM[k]->R - 1, 0, mM[k]->C - 1, mM[k]);
	}
	GetReturn();
	return;
}
