/* i4R.c */
/* matrix operations using MPR's */

#include <stdio.h>
#include "integer.h"
#include "fun.h"

MPMATR *SWAP_ROWSR(USI p, USI q, MPMATR *Mptr)
/*
 * interchange rows p and q of *Mptr.
 */
{
	unsigned int j;
	MPR *Temp;
	MPMATR *Nptr;

	Nptr = COPYMATR(Mptr);
	for (j = 0; j <= Nptr->C - 1; j++)
	{
		Temp = elt(Nptr, p, j);
		elt(Nptr, p, j) = elt(Nptr, q, j);
		elt(Nptr, q, j) = Temp;
	}
	return (Nptr);
}

MPMATR *SCALAR_ROWR(USI p, MPR *Aptr, MPMATR *Mptr)
/*
 * multiplying the pth row of *Mptr by *Aptr.
 */
{
	unsigned int j;
	MPR *Temp;
	MPMATR *Nptr;

	Nptr = COPYMATR(Mptr);
	for (j = 0; j <= Nptr->C - 1; j++)
	{
		Temp = elt(Nptr, p, j);
		elt(Nptr, p, j) = MULTR(Temp, Aptr);
		FREEMPR(Temp);
	}
	return (Nptr);
}

MPMATR *ADD_MULT_ROWR(USI p, USI q, MPR *Aptr, MPMATR *Mptr)
/*
 * adding *Aptr times row p to row q of *Mptr.
 */
{
	unsigned int j;
	MPR *X, *Temp;
	MPMATR *Nptr;

	Nptr = COPYMATR(Mptr);
	for (j = 0; j <= Nptr->C - 1; j++)
	{
		X = MULTR(elt(Nptr, p, j), Aptr);
		Temp = elt(Nptr, q, j);
		elt(Nptr, q, j) = ADDR(Temp, X);
		FREEMPR(X);
		FREEMPR(Temp);
	}
	return (Nptr);
}

MPMATR *SWAP_COLSR(USI p, USI q, MPMATR *Mptr)
/*
 * interchange columns p and q of *Mptr.
 */
{
	unsigned int i;
	MPR *Temp;
	MPMATR *Nptr;

	Nptr = COPYMATR(Mptr);
	for (i = 0; i <= Nptr->R - 1; i++)
	{
		Temp = elt(Nptr, i, p);
		elt(Nptr, i, p) = elt(Nptr, i, q);
		elt(Nptr, i, q) = Temp;
	}
	return (Nptr);
}

MPMATR *SCALAR_COLR(USI p, MPR *Aptr, MPMATR *Mptr)
/*
 * multiplying the pth column of *Mptr by *Aptr.
 */
{
	unsigned int i;
	MPR *Temp;
	MPMATR *Nptr;

	Nptr = COPYMATR(Mptr);
	for (i = 0; i <= Nptr->R - 1; i++)
	{
		Temp = elt(Nptr, i, p);
		elt(Nptr, i, p) = MULTR(Temp, Aptr);
		FREEMPR(Temp);
	}
	return(Nptr);
}

MPMATR *ADD_MULT_COLR(USI p, USI q, MPR *Aptr, MPMATR *Mptr)
/*
 * adding *Aptr times column p to column q of *Mptr.
 */
{
	unsigned int i;
	MPR *X, *Temp;
	MPMATR *Nptr;

	Nptr = COPYMATR(Mptr);
	for (i = 0; i <= Nptr->R - 1; i++)
	{
		X = MULTR(elt(Nptr, i, p), Aptr);
		Temp = elt(Nptr, i, q);
		elt(Nptr, i, q) = ADDR(Temp, X);
		FREEMPR(X);
		FREEMPR(Temp);
	}
	return (Nptr);
}

MPMATR *TRANSPOSER(MPMATR *Mptr)
/* 
 * returns the transpose of *Mptr.
 */
{
	MPMATR *Nptr;
	unsigned int i, j;

	Nptr = BUILDMATR(Mptr->C, Mptr->R);
	for (j = 0; j <= Nptr->R - 1; j++)
		for (i = 0; i <= Nptr->C - 1; i++)
			elt(Nptr, j, i) = COPYR(elt(Mptr, i, j));
	return (Nptr);
}

MPMATR *DELETE_ROWR(USI r, MPMATR *Mptr)
/*
 * deletes row r of *Mptr.
 */
{
	unsigned int i, j;
	MPMATR *Nptr;

	r--;
	Nptr = BUILDMATR(Mptr->R - 1, Mptr->C);
	for (i = 0; i <= Nptr->R - 1; i++)
	{
		if (i < r)
		{
			for (j = 0; j <= Nptr->C - 1; j++)
				elt(Nptr, i, j) = COPYR(elt(Mptr, i, j));
		}
		else
		{
			for (j = 0; j <= Nptr->C - 1; j++)
				elt(Nptr, i, j) = COPYR(elt(Mptr, i + 1, j));
		}
	}
	return (Nptr);
}

MPMATR *DELETE_COLR(USI c, MPMATR *Mptr)
/*
 * deletes  column c of *Mptr.
 */
{
	unsigned int i, j;
	MPMATR *Nptr;

	c--;
	Nptr = BUILDMATR(Mptr->R, Mptr->C - 1);
	for (j = 0; j <= Nptr->C - 1; j++)
	{
		if (j < c)
		{
			for (i = 0; i <= Nptr->R - 1; i++)
				elt(Nptr, i, j) = COPYR(elt(Mptr, i, j));
		}
		else
		{
			for (i = 0; i <= Nptr->R - 1; i++)
				elt(Nptr, i, j) = COPYR(elt(Mptr, i, j + 1));
		}
	}
	return (Nptr);
}

MPMATR *KRONECKERR(MPMATR *Mptr, MPMATR *Nptr)
/*
 * returns the Kronecker product of *Mptr and *Nptr.
 */
{
	unsigned int i1, i2, j1, j2, m1, m2, n1, n2;
	MPMATR *Qptr;

	m1 = Mptr->R;
	m2 = Nptr->R;
	n1 = Mptr->C;
	n2 = Nptr->C;
	Qptr = BUILDMATR(m1 * m2, n1 * n2);
	for (i1 = 0; i1 <= m1 - 1; i1++)
	{
		for (j1 = 0; j1 <= n1 - 1; j1++)
		{
			for (i2 = 0; i2 <= m2 - 1; i2++)
			{
				for (j2 = 0; j2 <= n2 - 1; j2++)
					elt(Qptr, m2 * i1 + i2, n2 * j1 + j2) = 				MULTR(elt(Mptr, i1, j1), elt(Nptr, i2, j2));
			}
		}
	}
	return(Qptr);
}

MPMATR *DIRECT_SUMR(MPMATR *Mptr, MPMATR *Nptr)
/*
 * returns the direct sum of *Mptr and *Nptr.
 */
{
	unsigned int i, j, m1, m2, n1, n2;
	MPMATR *Qptr;
	
	m1 = Mptr->R;
	m2 = Nptr->R;
	n1 = Mptr->C;
	n2 = Nptr->C;
	Qptr = BUILDMATR(m1 + m2, n1 + n2);
	for (i = 0; i <= m1 + m2 - 1; i++)
	{
		for (j = 0; j <= n1 + n2 - 1; j++)
		{
			if (i <= m1 - 1 && j <= n1 - 1)
				elt(Qptr, i, j) =  COPYR(elt(Mptr, i, j));
			else if (m1 <= i && n1 <= j)
				elt(Qptr, i, j) = COPYR(elt(Nptr, i - m1, j - n1));
			else
				elt(Qptr, i, j) = ZEROR();
		}
	}
	return (Qptr);
}

MPMATR *JORDANR(USI n, MPR *Aptr)
/*
 * returns the elementary Jordan matrix of size n, diagonal elt. = *Aptr.
 */
{
	unsigned int i, j;
	MPMATR *Mptr;
	
	Mptr = BUILDMATR(n, n);
	for (i = 0; i <= n - 1; i++)
	{
		for (j = 0; j <= n - 1; j++)
		{
			if (i == j)
				elt(Mptr, i, j) = COPYR(Aptr);
			else if (i == j + 1)
				elt(Mptr, i, j) = ONER();
			else
				elt(Mptr, i, j) = ZEROR();
		}
	}
	return (Mptr);
}

MPMATR *ZEROMNR(USI m, USI n)
/*
 * returns the zero  m x n matrix.
 */
{
	unsigned int i, j;
	MPMATR *Mptr;

	Mptr = BUILDMATR(m, n);
	for (i = 0; i <= m - 1; i++)
	{
		for (j = 0; j <= n - 1; j++)
			elt(Mptr, i, j) = ZEROR();
	}
	return (Mptr);
}

MPMATR *BANDMATR(MPR *Aptr, MPR *Bptr, USI k, USI n)
/*
 * Constructs the n x n band matrix with *Aptr on the diagonal and *Bptr
 * on the k sub and super diagonals. Here n >= 2 and 1 <= k <= n - 1.
 */
{
	unsigned int i, j;
	MPMATR *Mptr;

	Mptr = ZEROMNR(n, n);
	for (i = 0; i < n; i++)
	{
		FREEMPR(elt(Mptr, i, i));
		elt(Mptr, i, i) = COPYR(Aptr);
	}
	for (j = 1; j <= k; j++)
	{
		for (i = 0; i <= n - 1 - j; i++)
		{
			FREEMPR(elt(Mptr, i, i + j));
			elt(Mptr, i, i + j) = COPYR(Bptr);
			FREEMPR(elt(Mptr, i + j, i));
			elt(Mptr, i + j, i) = COPYR(Bptr);
		}
	}
	return (Mptr);
}

MPMATR *IDENTITYR(USI n)
/*
 * returns the identity matrix of size n.
 */
{
	unsigned int i, j;
	MPMATR *Mptr;

	Mptr = BUILDMATR(n, n);
	for (i = 0; i <= n - 1; i++)
	{
		for (j = 0; j <= n - 1; j++)
		{
			if ( i == j)
				elt(Mptr, i, j) = ONER();
			else
				elt(Mptr, i, j) = ZEROR();
		}
	}
	return (Mptr);
}

MPMATR *SUBMATROWSR(MPMATR *Mptr, USI *Cptr, USI n)
/*
 * Cptr is an array of n (distinct) non-negative integers, n < Mptr->R.
 * Output: the submatrix formed from rows Cptr[0],...,Cptr[n-1].
 */
{
	unsigned int i, j;
	MPMATR *Nptr;

	Nptr = BUILDMATR(n, Mptr->C);
	for (i = 0; i < n; i++)
		for (j = 0; j < Mptr->C; j++)
			elt(Nptr, i, j) = COPYR(elt(Mptr, Cptr[i], j));
	return (Nptr);
}

MPMATR *SUBMATCOLUMNSR(MPMATR *Mptr, USI *Cptr, USI n)
/*
 * Cptr is an array of n (distinct) non-negative integers, n < Mptr->R.
 * Output: the submatrix formed from columns Cptr[0],...,Cptr[n-1].
 */
{
	unsigned int i, j;
	MPMATR *Nptr;

	Nptr = BUILDMATR(Mptr->R, n);
	for (j = 0; j < n; j++)
		for (i = 0; i < Mptr->R; i++)
			elt(Nptr, i, j) = COPYR(elt(Mptr, i, Cptr[j]));
	return (Nptr);
}
