/* i5CR.c */
/* programs for doing arithmetic with MPCR's. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "integer.h"
#include "fun.h"

MPCR *BUILDMPCR( )
/*
 * mallocs space for an MPCR.
 */
{
	MPCR *Mptr;

	Mptr = (MPCR *)mmalloc(sizeof(MPCR));
	return (Mptr);
}

void FREEMPCR(MPCR *Mptr)
/*
 * deallocates the space alloted for the MPCR Mptr.
 */
{
	if (Mptr == NULL)
		return;
	FREEMPR(Mptr->R);
	FREEMPR(Mptr->I);
	ffree ((char *)Mptr, sizeof(MPCR));
	return;
}

MPCR *COPYCR(MPCR *Aptr)
/*
 * returns a pointer to a copy of *Aptr.
 */
{
	MPCR *Bptr;

	Bptr = BUILDMPCR();
	Bptr->R = COPYR(Aptr->R);
	Bptr->I = COPYR(Aptr->I);
	return (Bptr);
}

unsigned int EQUALCR(MPCR *Aptr, MPCR *Bptr)
/*
 * returns 1 if *Aptr = *Bptr, 0 otherwise.
 */
{
	if (EQUALR(Aptr->R, Bptr->R) * EQUALR(Aptr->I, Bptr->I))
		return (1);
	else
		return (0);
}

MPCR *MINUSCR(MPCR *Aptr)
/*
 * *Bptr = -(*Aptr).
 */
{
	MPCR *Bptr;

	Bptr = BUILDMPCR();
	Bptr->R = MINUSR(Aptr->R);
	Bptr->I = MINUSR(Aptr->I);
	return (Bptr);
}

MPCR *ZEROCR()
/*
 * Returns the MPCR ZERO.
 */
{
	MPCR *Eptr;

	Eptr = BUILDMPCR();
	Eptr->R = ZEROR();
	Eptr->I = ZEROR();
	return Eptr;
}

MPCR *ONECR()
/*
 * Returns the MPCR ONE.
 */
{
	MPCR *Eptr;

	Eptr = BUILDMPCR();
	Eptr->R = ONER();
	Eptr->I = ZEROR();
	return Eptr;
}

MPCR *MINUS_ONECR()
/*
 * Returns the MPCR MINUS_ONE.
 */
{
	MPCR *Eptr;

	Eptr = BUILDMPCR();
	Eptr->R = MINUS_ONER();
	Eptr->I = ZEROR();
	return Eptr;
}

MPCR *INPUTCR(char **ptr, USI *uptr)
/*
 * inputs the complex rational *Mptr.
 */
{
	MPR *TempR;
	MPCR *Mptr;
	char t;
	
	while (**ptr == ' ')
		(*ptr)++;
	if ((**ptr != '-') && (**ptr != 'i') && (**ptr < '0' || **ptr > '9'))
	{
		printf("inside INPUTCR: error1: illegal character %c entered:\n", **ptr);
		*uptr = 0;
		return ZEROCR();
	}
	t = **ptr;
	Mptr = BUILDMPCR();
	if (t == '-')
	{
		(*ptr)++;
		while (**ptr == ' ')
			(*ptr)++;
		if (**ptr == 'i')
		{
			(*ptr)++;
			Mptr->R = ZEROR();
			Mptr->I = MINUS_ONER();
			if (**ptr != ' ' && **ptr != '\n' && **ptr != '\0')
			{
				printf("inside INPUTCR: error2: illegal character %c entered:\n", **ptr);
				*uptr = 0;
				return (Mptr);
			}
			*uptr = 1;
			return (Mptr);
		}
		else
		{
			Mptr->R = INPUTSR(ptr, uptr);
			if (*uptr == 0)
			{
				Mptr->I = ZEROR();
				return (Mptr);
			}
			if (((Mptr->R)->N)->S != 0)
			{
				TempR = Mptr->R;
				Mptr->R = MINUSR(Mptr->R);
				FREEMPR(TempR);
			}
		}
	}
	else if (t == 'i')
	{
		(*ptr)++;
		Mptr->R = ZEROR();
		Mptr->I = ONER();
		if (**ptr != ' ' && **ptr != '\n' && **ptr != '\0')
		{
			printf("inside INPUTCR: error3: illegal character %c entered:\n", **ptr);
			*uptr = 0;
			return (Mptr);
		}
		*uptr = 1;
		return (Mptr);
	}
	else
	{
		Mptr->R = INPUTSR(ptr, uptr);
		if (*uptr == 0)
		{
			Mptr->I = ZEROR();
			*uptr = 0;
			return (Mptr);
		}
	}
	if (**ptr == 'i')
	{
		(*ptr)++;
		Mptr->I = Mptr->R;
		Mptr->R = ZEROR();
		if (**ptr != ' ' && **ptr != '\n' && **ptr != '\0')
		{
			printf("inside INPUTCR: error4: illegal character %c entered:\n", **ptr);
			*uptr = 0;
			return (Mptr);
		}
		*uptr = 1;
		return (Mptr);
	}
	else if (**ptr == '\0')
	{
		Mptr->I = ZEROR();
		*uptr = 1;
		return (Mptr);
	}
	else if (**ptr == ' ')
	{
		while (**ptr == ' ')
			(*ptr)++;
		if (**ptr != '\0' && **ptr != 'i'  && **ptr != '+' && **ptr != '-' && (**ptr < '0' || **ptr > '9'))
		{
			Mptr->I = ZEROR();
			printf("inside INPUTCR: error5: illegal character %c entered:\n", **ptr);
			*uptr = 0;
			return (Mptr);
		}
		if (**ptr == '\0')
		{
			Mptr->I = ZEROR();
			*uptr = 1;
			return (Mptr);
		}
		else if (**ptr == '-' || **ptr == '+')
		{
			(*ptr)++;
			if (**ptr != ' ')
			{
				if (**ptr != 'i' && (**ptr < '0' || **ptr > '9'))
				{
					Mptr->I = ZEROR();
					printf("inside INPUTCR: error6: illegal character %c entered:\n", **ptr);
					*uptr = 0;
					return (Mptr);
				}
				else
				{
					(*ptr)--;
					Mptr->I = ZEROR();
					*uptr = 1;
					return (Mptr);
				}
			}
			else 
				(*ptr)--;
		}
		else if (((**ptr >= '0') && (**ptr <= '9')) || **ptr == 'i')
		{
			(*ptr)--;
			Mptr->I = ZEROR();
			*uptr = 1;
			return (Mptr);
		}
	}
	/* **ptr == '+' or **ptr == '-' (meaningful addition or subtraction). */
	t = **ptr;
	(*ptr)++;
	while (**ptr == ' ')
		(*ptr)++;
	if (**ptr == 'i')
	{
		if (t == '+')
		{
			(*ptr)++;
			Mptr->I = ONER();
			*uptr = 1;
			return (Mptr);
		}
		else 
		{
			(*ptr)++;
			Mptr->I = MINUS_ONER();
			*uptr = 1;
			return (Mptr);
		}
	}
	else
	{
		Mptr->I = INPUTSR(ptr, uptr);
		if (*uptr == 0)
		{
			*uptr = 0;
			return (Mptr);
		}
		if (t == '-')
			((Mptr->I)->N)->S = -1;
		(*ptr)++;
	}
	*uptr = 1;
	return (Mptr);
}

MPMATCR *FINPUTMATCR(FILE *infile)
{
	MPMATCR *Temp;
	unsigned int u, i, j, m, n;
	char *line, *p;

	if (infile == stdin)
		printf("Enter the number of rows and number of columns\n");

	if (fscanf(infile, "%u%u", &m, &n) != 2 || m == 0 || n == 0)
	{
		fprintf(stderr, "corrupt input: returning the 1 x 1 zero matrix\n");
		if (infile != stdin)
			exit (1);
		return ZEROMNCR(1, 1);
	}
	fgetc(infile);
	Temp = ZEROMNCR(m, n);
	for (i = 0; i < m; i++)
	{
		if (infile == stdin)
			printf("Enter row %u:\n", i + 1);
		p = line = Fgets(infile);
		for (j = 0; j < n; j++)
		{
			FREEMPCR(elt(Temp, i, j));
			elt(Temp, i, j) = INPUTCR(&p, &u);
			if (u == 0)
			{
				free(line);
				fprintf(stderr, "matrix input interrupted prematurely\n");
				if (infile != stdin)
					exit (1);
				GetReturn();
				return Temp;
			}
		}
		free(line);
	}
	return Temp;
}

MPMATCR *INPUTMATCR()
/*
 * Input a matrix of complex rationals from the user with prompting.
 */
{
	return FINPUTMATCR(stdin);
}

void FPRINTCR(FILE *outfile, MPCR *Mptr)
/*
 * the MPCR *Mptr is printed to *outfile in decimal notation. 
 */
{
	int s;

	if (((Mptr->R)->N)->S != 0)
		FPRINTR(outfile, Mptr->R);
	else
	{
		if (EQMINUSONER(Mptr->I))
		{
			fprintf(outfile, "-i");
			return;
		}
		else if (EQONER(Mptr->I))
		{
			fprintf(outfile, "i");
			return;
		}
		else if (((Mptr->I)->N)->S == 0)
		{
			fprintf(outfile, "0");
			return;
		}
		else
		{
			FPRINTR(outfile, Mptr->I);
			fprintf(outfile, "i");
			return;
		}
	}
	if (((Mptr->I)->N)->S == 0)
		return;
	else
	{
		s = ((Mptr->I)->N)->S;
		if (s == -1)
		{
			fprintf(outfile, " - ");
			((Mptr->I)->N)->S = 1;
		}
		else 
			fprintf(outfile, " + ");
		if (!EQONER(Mptr->I))
			FPRINTR(outfile, Mptr->I);
		if (s == -1)
			((Mptr->I)->N)->S = -1;
		fprintf(outfile, "i");
		return;
	}
}

void PRINTCR(MPCR *Mptr)
/*
 * the MPCR *Mptr is printed to stdout in decimal notation. 
 */
{
	FPRINTCR(stdout, Mptr);
	return;
}

void SPRINTCR(char *buffer, MPCR *Mptr)
/*
 * the MPCR *Mptr is printed to buffer in decimal notation. 
 */
{
	int s;

	if (((Mptr->R)->N)->S != 0)
	{
		SPRINTR(buffer, Mptr->R);
		buffer += strlen(buffer);
	}
	else
	{
		if (EQMINUSONER(Mptr->I))
		{
			sprintf(buffer, "-i");
			return;
		}
		else if (EQONER(Mptr->I))
		{
			sprintf(buffer, "i");
			return;
		}
		else if (((Mptr->I)->N)->S == 0)
		{
			sprintf(buffer, "0");
			return;
		}
		else
		{
			SPRINTR(buffer, Mptr->I);
			buffer += strlen(buffer);
			sprintf(buffer, "i");
			return;
		}
	}
	if (((Mptr->I)->N)->S == 0)
		return;
	else
	{
		s = ((Mptr->I)->N)->S;
		if (s == -1)
		{
			sprintf(buffer, " - ");
			buffer += strlen(buffer);
			((Mptr->I)->N)->S = 1;
		}
		else 
		{
			sprintf(buffer, " + ");
			buffer += strlen(buffer);
		}
		if (!EQONER(Mptr->I))
		{
			SPRINTR(buffer, Mptr->I);
			buffer += strlen(buffer);
		}
		if (s == -1)
			((Mptr->I)->N)->S = -1;
		sprintf(buffer, "i");
		return;
	}
}

unsigned int LENGTHCR(MPCR *Aptr)
/*
 * length of a complex rational.
 */
{
	if (((Aptr->I)->N)->S == 0)
		return (LENGTHR(Aptr->R));
	else 
	{
		if (((Aptr->R)->N)->S == 0)
		{
			if (EQONER(Aptr->I))
			{
				return (1);
			}
			else if (EQMINUSONER(Aptr->I))
			{
				return (2);
			}
			else
				return (1 + LENGTHR(Aptr->I));
		}
		else
		{
			if (EQONER(Aptr->I))
				return (4 + LENGTHR(Aptr->R));
			else if (EQMINUSONER(Aptr->I))
				return (4 + LENGTHR(Aptr->R));
			else if (((Aptr->I)->N)->S == 1)
				return (4 + LENGTHR(Aptr->I) + LENGTHR(Aptr->R));
			else 
				return (3 + LENGTHR(Aptr->I) + LENGTHR(Aptr->R));
		}
	}
}

void FPRINTMATCR(FILE *outfile, USI i1, USI i2, USI j1, USI j2, MPMATCR *Mptr)
/*
 * Modified 8/1/93 using Peter Adams' improvement from, August 1992.
 */
{
	char **str, *buff;
	unsigned int tmp, i, j, nrow, ncol, len, nstr = 0;
	int *colwidth;
	unsigned int ct;
	
	nrow = i2 - i1 + 1;
	ncol = j2 - j1 + 1;
	str = (char **)mmalloc(nrow * ncol * sizeof(char *));
	colwidth=(int *)mmalloc(ncol*sizeof(int));

	for(i=0; i<ncol; i++)
	  colwidth[i]=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
			tmp = 1 + LENGTHCR(elt(Mptr, i, j));
			buff = (char *)mmalloc(tmp * sizeof(char));
			SPRINTCR(buff, elt(Mptr, i, j));
		    	if ((len = strlen(buff)) > colwidth[j-j1])
				colwidth[j-j1] = len;
			str[nstr++] = strcpy((char *)mmalloc(len + 1), buff);
			ffree(buff, tmp * sizeof(char));
		}
	}
	if (outfile != stdout)
		fprintf(outfile, "%u %u\n", nrow, ncol);
	ct=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
	    		fprintf(outfile, "%*s ", colwidth[j-j1], str[ct]);
	    		ffree(str[ct], (unsigned int) (strlen(str[ct]) + 1) * sizeof(char));
	    		if ((ct % ncol) == (ncol - 1))
				fprintf(outfile, "\n");
	    		ct++;
		}
	}
	ffree((char *) str, nrow * ncol * sizeof(char *));
	ffree((char *) colwidth, ncol*sizeof(int));
	return;
}

void PRINTMATCR(USI i1, USI i2, USI j1, USI j2, MPMATCR *Mptr)
/*
 * prints *Mptr from rows i1 to i2 and cols j1 to j2.
 */
{
	unsigned int i, j;

	i = i2 - i1 + 1;
	j = j2 - j1 + 1;
	if (MAX((int)i, (int)j) > 20)
	{
		printf("(The number of rows or columns to be printed exceeds 20;\n");
		printf("there is no point in printing this matrix on the screen.)\n");
		return;
	}
	FPRINTMATCR(stdout, i1, i2, j1, j2, Mptr);
	return;
}

void FPRINTDCR(FILE *outfile, USI d, MPCR *Mptr)
/*
 * prints *Mptr truncated to d (>= 1) decimal places to outfile.
 */
{
	int s;
	unsigned int i;

	if (EQZEROCR(Mptr))
	{
		fprintf(outfile, "0.");
		for (i = 0; i <= d - 1; i++)
			fprintf(outfile, "0");
		return;
	}
	if (((Mptr->R)->N)->S)
		FPRINTDR(outfile, d, Mptr->R);
	if (((Mptr->I)->N)->S)
	{
		s = ((Mptr->R)->N)->S;
		if (s == -1)
		{
			fprintf(outfile, " - ");
			((Mptr->I)->N)->S = 1;
		}
		else
			fprintf(outfile, " + ");
		FPRINTDR(outfile, d, Mptr->I);
		if (s == -1)
			((Mptr->I)->N)->S = -1;
		fprintf(outfile, "i");
	}
	return;
}

void PRINTDCR(USI d, MPCR *Mptr)
/*
 * prints *Mptr truncated to d (>= 1) decimal places to stdout.
 */
{
	FPRINTDCR(stdout, d, Mptr);
	return;
}

void SPRINTDCR(char *buffer, USI d, MPCR *Mptr)
/*
 * prints *Mptr truncated to d (>= 1) decimal places to outfile.
 */
{
	int s;
	unsigned int i;

	if (EQZEROCR(Mptr))
	{
		sprintf(buffer, "0.");
		buffer += 2;
		for (i = 0; i <= d - 1; i++)
		{
			sprintf(buffer, "0");
			buffer++;
		}
		return;
	}
	if (((Mptr->R)->N)->S)
	{
		SPRINTDR(buffer, d, Mptr->R);
		buffer += strlen(buffer);
	}
	if (((Mptr->I)->N)->S)
	{
		s = ((Mptr->R)->N)->S;
		if (s == -1)
		{
			sprintf(buffer, " - ");
			((Mptr->I)->N)->S = 1;
		}
		else
			sprintf(buffer, " + ");
		buffer += 3;
		SPRINTDR(buffer, d, Mptr->I);
		buffer += strlen(buffer);
		sprintf(buffer, "i");
		if (s == -1)
			((Mptr->I)->N)->S = -1;
	}
	return;
}

unsigned int LENGTHDCR(USI d, MPCR *Aptr)
/*
 * d-length of a complex rational.
 */
{
	if (((Aptr->I)->N)->S == 0)
		return (LENGTHDR(d, Aptr->R));
	else 
	{
		if (((Aptr->R)->N)->S == 0)
			return (1 + LENGTHDR(d, Aptr->I));
		else
		{
			if (((Aptr->I)->N)->S == 1)
				return (4 + LENGTHDR(d, Aptr->I) + LENGTHDR(d, Aptr->R));
			else 
				return (3 + LENGTHDR(d, Aptr->I) + LENGTHDR(d, Aptr->R));
		}
	}
}

void FPRINTMATDCR(FILE *outfile, USI d, USI i1, USI i2, USI j1, USI j2, MPMATCR *Mptr)
/*
 * Modified 8/1/93 using Peter Adams' improvement from, August 1992.
 * prints  *Mptr  with elements truncated to d (>= 1) decimal places to outfile.
 */
{
	char **str, *buff;
	unsigned int tmp, i, j, nrow, ncol, len, nstr = 0;
	int *colwidth;
	unsigned int ct;
	
	nrow = i2 - i1 + 1;
	ncol = j2 - j1 + 1;
	str = (char **)mmalloc(nrow * ncol * sizeof(char *));
	colwidth=(int *)mmalloc(ncol*sizeof(int));

	for(i=0; i<ncol; i++)
	  colwidth[i]=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
			tmp = 1 + LENGTHDCR(d, elt(Mptr, i, j));
			buff = (char *)mmalloc(tmp * sizeof(char));
			SPRINTDCR(buff, d, elt(Mptr, i, j));
		    	if ((len = strlen(buff)) > colwidth[j-j1])
				colwidth[j-j1] = len;
			str[nstr++] = strcpy((char *)mmalloc(len + 1), buff);
			ffree(buff, tmp * sizeof(char));
		}
	}
	if (outfile != stdout)
		fprintf(outfile, "%u %u\n", nrow, ncol);
	ct=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
	    		fprintf(outfile, "%*s ", colwidth[j-j1], str[ct]);
	    		ffree(str[ct], (unsigned int) (strlen(str[ct]) + 1) * sizeof(char));
	    		if ((ct % ncol) == (ncol - 1))
				fprintf(outfile, "\n");
	    		ct++;
		}
	}
	ffree((char *) str, nrow * ncol * sizeof(char *));
	ffree((char *) colwidth, ncol*sizeof(int));
	return;
}

void PRINTMATDCR(USI d, USI i1, USI i2, USI j1, USI j2, MPMATCR *Mptr)
/*
 * prints  *Mptr  with elements truncated to d (>= 1) decimal places to stdout.
 */
{
	unsigned int i, j;

	i = i2 - i1 + 1;
	j = j2 - j1 + 1;
	if (MAX((int)i, (int)j) > 20)
	{
		printf("(The number of rows or columns to be printed exceeds 20;\n");
		printf("there is no point in printing this matrix on the screen.)\n");
		return;
	}
	FPRINTMATDCR(stdout, d, i1, i2, j1, j2, Mptr);
	return;
}

MPCR *ADDCR(MPCR *Aptr, MPCR *Bptr)
/*
 * *Cptr = *Aptr + *Bptr.
 */
{
	MPCR *Cptr;

	Cptr = BUILDMPCR();
	Cptr->R = ADDR(Aptr->R, Bptr->R);
	Cptr->I = ADDR(Aptr->I, Bptr->I);
	return (Cptr);
}

MPCR *MULTCR(MPCR *Aptr, MPCR *Bptr)
/*
 * *Cptr = *Aptr * *Bptr.
 */
{
	MPR *U, *V, *F, *G;
	MPCR *Cptr;

	Cptr = BUILDMPCR();
	U = MULTR(Aptr->R, Bptr->R);
	V = MULTR(Aptr->I, Bptr->I);
	F = MULTR(Aptr->R, Bptr->I);
	G = MULTR(Aptr->I, Bptr->R);
	Cptr->R = SUBR(U, V);
	Cptr->I = ADDR(F, G);
	FREEMPR(U);
	FREEMPR(V);
	FREEMPR(F);
	FREEMPR(G);
	return (Cptr);
}

MPCR *POWERCR(MPCR *Aptr, USI n)
/*
 *  *Eptr = (*Aptr) ^ n, where 0 <= n < R0 * R0.
 */
{
	MPCR *B, *TempCR, *Eptr;

	if (n == 0)
		return ONECR();
	B = COPYCR(Aptr);
	Eptr = ONECR();
	while (1)
        {
		if (n & 1)
	        {
			TempCR = Eptr;
			Eptr = MULTCR(Eptr, B);
			FREEMPCR(TempCR);
			if (n == 1)
			{
				FREEMPCR(B);	
				return (Eptr);
			}
		}
		TempCR = B;
		B = MULTCR(B, B);
		FREEMPCR(TempCR);
		n = n >> 1;
	}
}

MPCR *SUBCR(MPCR *Aptr, MPCR *Bptr)
/*
 * *Cptr = *Aptr - *Bptr.
 */
{
	MPCR *E, *Cptr;

	E = MINUSCR(Bptr);
	Cptr = ADDCR(Aptr, E);
	FREEMPCR(E);
	return (Cptr);
}

MPCR *RATIOCI(MPCI *Aptr, MPCI *Bptr)
/*
 * the ratio *Aptr / *Bptr of complex MPCI's is expressed as an MPCR.
 */
{
	MPI *U, *V, *W, *X, *Y, *TempI;
	MPCR *Cptr;

	Cptr = BUILDMPCR();
	U = MULTI(Aptr->R, Bptr->R);
	V = MULTI(Aptr->I, Bptr->I);
	X = MULTI(Aptr->R, Bptr->I);
	Y = MULTI(Aptr->I, Bptr->R);
	TempI = U;
	U = ADDI(U, V);
	FREEMPI(TempI);
	TempI = Y;
	Y = SUBI(Y, X);
	FREEMPI(TempI);
	TempI = V;
	V = MULTI(Bptr->R, Bptr->R);
	FREEMPI(TempI);
	TempI = X;
	X = MULTI(Bptr->I, Bptr->I);
	FREEMPI(TempI);
	W = ADDI(V, X);
	Cptr->R = RATIOI(U, W);
	Cptr->I = RATIOI(Y, W);
	FREEMPI(U);
	FREEMPI(V);
	FREEMPI(W);
	FREEMPI(X);
	FREEMPI(Y);
	return (Cptr);
}

MPCR *RATIOCR(MPCR *Aptr, MPCR *Bptr)
/*
 * the ratio *Aptr / *Bptr of complex MPCR's is expressed as an MPCR.
 */
{
	MPR *U, *V, *W, *X, *Y, *TempR;
	MPCR *Cptr;

	Cptr = BUILDMPCR();
	U = MULTR(Aptr->R, Bptr->R);
	V = MULTR(Aptr->I, Bptr->I);
	X = MULTR(Aptr->R, Bptr->I);
	Y = MULTR(Aptr->I, Bptr->R);
	TempR = U;
	U = ADDR(U, V);
	FREEMPR(TempR);
	TempR = Y;
	Y = SUBR(Y, X);
	FREEMPR(TempR);
	TempR = V;
	V = MULTR(Bptr->R, Bptr->R);
	FREEMPR(TempR);
	TempR = X;
	X = MULTR(Bptr->I, Bptr->I);
	FREEMPR(TempR);
	W = ADDR(V, X);
	Cptr->R = RATIOR(U, W);
	Cptr->I = RATIOR(Y, W);
	FREEMPR(U);
	FREEMPR(V);
	FREEMPR(W);
	FREEMPR(X);
	FREEMPR(Y);
	return (Cptr);
}

MPCR *INVERSECR(MPCR *Aptr)
/*
 * *Bptr = 1 / *Aptr.
 */
{
	MPR *U, *V, *W, *Z;
	MPCR *Bptr;

	Bptr = BUILDMPCR();
	U = MULTR(Aptr->R, Aptr->R);
	V = MULTR(Aptr->I, Aptr->I);
	W = ADDR(U, V);
	Bptr->R = RATIOR(Aptr->R, W);
	Z = MINUSR(Aptr->I);
	Bptr->I = RATIOR(Z, W);
	FREEMPR(U);
	FREEMPR(V);
	FREEMPR(W);
	FREEMPR(Z);
	return (Bptr);
}

MPCR *CONJCR(MPCR *Aptr)
/*
 * *Bptr = complex conjugate of *Aptr.
 */
{
	MPCR *Bptr;

	Bptr = BUILDMPCR();
	Bptr->R = COPYR(Aptr->R);
	Bptr->I = MINUSR(Aptr->I);
	return (Bptr);
}
