/* i5m.c */
/* modular arithmetic */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "integer.h"
#include "fun.h"

extern unsigned int reciprocal[][Z0];
void DUMPMPI(MPI *Mptr)
{
	unsigned int i;

	printf("Mptr->S = %d, Mptr->D = %u\n", Mptr->S, Mptr->D);
	for (i = 0; i <= Mptr->D; i++)
		printf("Mptr->V[%u] = %u\n", i, Mptr->V[i]);
	return;
}

MPI *ADDM(MPI *Aptr, MPI *Bptr, MPI *Mptr)
/*
 * *Cptr = (*Aptr + *Bptr) mod (*Mptr). 
 * Here 0 <= *Aptr, *Bptr, *Cptr < *Mptr. 
 */
{
	MPI *Cptr, *TempI;

	Cptr = ADD0I(Aptr, Bptr);
	if (RSV(Cptr, Mptr) >= 0)
	{
		TempI = Cptr;
		Cptr = SUB0I(Cptr, Mptr);
		FREEMPI(TempI);
	}
	return (Cptr);
}

unsigned int ADDm(USI a, USI b, USI m)
/*
 * returns a + b mod m, if m > 0, a + b in GF(4) if m = 0.
 * here 0 <= a, b < m < R0. 
 */
{
	unsigned int c;

	if (a == 0)
		return b;
	if (b == 0)
		return a;
	if (m)
	{
		c = a + b;
		if (c >= m)
			c -= m;
		return c;
	}
	else
	{
		if (a == 1 && b == 1)
			return 0;
		else if (a == 1 && b == 2)
			return 3;
		else if (a == 1 && b == 3)
			return 2;
		else if (a == 2 && b == 1)
			return 3;
		else if (a == 2 && b == 2)
			return 0;
		else if (a == 2 && b == 3)
			return 1;
		else if (a == 3 && b == 1)
			return 3;
		else if (a == 3 && b == 2)
			return 1;
		else /* (a == 3 && b == 3)*/
			return 0;
	}
}

MPI *MINUSM(MPI *Aptr, MPI *Mptr)
/*
 * *Bptr = -(*Aptr) mod (*Mptr). 
 * Here 0 <= *Aptr, *Bptr < *Mptr. 
 */
{

	if (Aptr->S == 0)
		return ZEROI();
	else
		return SUB0I(Mptr, Aptr);
}

unsigned int MINUSm(USI a, USI m)
/*
 * returns -a mod m if m > 0, -a  = a in GF(4) if m = 0.
 * here 0 <= a < m <R0.
 */
{
	if (m)
	{
		if (a == 0)
			return 0;
		else
			return (m - a);
	}
	else
		return a;
}

MPI *SUBM(MPI *Aptr, MPI *Bptr, MPI *Mptr)
/*
 * *Cptr = (*Aptr - *Bptr) mod (*Mptr). 
 * Here 0 <= *Aptr, *Bptr, *Cptr < *Mptr.
 */
{
	MPI *Cptr, *TempI;

	Cptr = MINUSM(Bptr, Mptr);
	TempI = Cptr;
	Cptr = ADDM(Aptr, Cptr, Mptr);
	FREEMPI(TempI);
	return (Cptr);
}

unsigned int SUBm(USI a, USI b, USI m)
/*
 * returns a - b mod m if m > 0, a - b = a + b in GF(4) if m = 0.
 * here 0 <= a, b < m < R0.
 */
{
	unsigned int c;
	
	if (m)
	{
		c = MINUSm(b, m);
		return (ADDm(a, c, m));
	}
	else
		return ADDm(a, b, 0);
}

MPI *MULTM(MPI *Aptr, MPI *Bptr, MPI *Mptr)
/*
 * *Cptr = (*Aptr * *Bptr) mod (*Mptr).
 * Here 0 <= *Aptr, *Bptr, *Cptr < *Mptr.
 */
{
	MPI *TempI, *Cptr;

	Cptr = MULTI(Aptr, Bptr);
	TempI = Cptr;
	Cptr = MOD0(Cptr, Mptr);
	FREEMPI(TempI);
	return (Cptr);
}

unsigned int MULTm(USI a, USI b, USI m)
/*
 * returns a * b mod m  if m > 0, a * b in GF(4) if m = 0.
 * here 0 <= a, b < m < R0.
 */
{
	if (a == 0 || b == 0)
		return 0;
	if (m)
		return ((a * b) % m);
	else
	{
		if (a == 1)
			return b;
		else if (b == 1)
			return a;
		else if (a == 2 && b == 2)
			return 3;
		else if (a == 2 && b == 3)
			return 1;
		else if (a == 3 && b == 2)
			return 1;
		else /* (a == 3 && b == 3) */
			return 2;
	}
}

MPI *INVERSEM(MPI *Nptr, MPI *Mptr)
/*
 * here gcd(*Nptr, *Mptr) = 1, 1 <= *Nptr < *Mptr. 
 * *Kptr satisfies 1 <= *Kptr < *Mptr with (*Kptr) * (*Nptr) = 1  mod *Mptr.
 */
{
	MPI *A, *B, *C, *H, *L, *Q, *Kptr;
	MPI *TempI;

	if (EQONEI(Nptr))
		return ONEI();
	A = COPYI(Mptr);
	B = COPYI(Nptr);
	C = MOD(A, B);
	Kptr = ONEI();
	L = ZEROI();
	while (C->S > 0)
	{
		Q = INT0(A, B);
		FREEMPI(A);
		A = B;
		B = C;
		C = MOD(A, B);
		TempI = Q;
		Q = MULTM(Q, Kptr, Mptr);
		FREEMPI(TempI);
		H = SUBM(L, Q, Mptr);
		FREEMPI(Q);
		TempI = L;
		L = Kptr;
		FREEMPI(TempI);
		Kptr = H;
	}
	FREEMPI(L);
	FREEMPI(A);
	FREEMPI(B);
	FREEMPI(C);
	if (Kptr->S == -1)
	{
		Kptr->S = -(Kptr->S);
		TempI = Kptr;
		Kptr = SUB0I(Mptr, Kptr);
		FREEMPI(TempI);
	}
	return (Kptr);
}

unsigned int GCDm(USI m, USI n)
/*
 * returns the gcd of m and n.
 */
{
	unsigned int c;

	if (n == 0)
		return m;
        c = m % n;
        while (c)
	{
		m = n;
                n = c;
                c = m % n;
        }
	return n;
}   

unsigned int INVERSEm(USI n, USI m)
/*
 * here gcd(n, m) = 1, 1 <= n < m < R0. 
 * returns k, 1 <= k < m with n * k congruent to 1 mod m if m > 0,
 * returns n^{-1} in GF(4) if m = 0.
 */
{
	unsigned int a, b, c, h, k, l, q, t;

	if (n == 1)
		return (1);
	if (m)
	{
		if (n == m - 1)
			return (m - 1);
		if (m <= Z0 + 1)
		{
			t = reciprocal[m - 2][n - 1];
			return t;
		}
		a = m;
		b = n;
		c = a % b;	
		l = 0;
		k = 1;
		while (c)
		{
			q = a / b;
			a = b;
			b = c;
			c = a % b;
			q = MULTm(q , k, m);
			h = SUBm(l, q, m);
			l = k;
			k = h;
		}
		return (k);
	}
	else
	{
		if (n == 2)
			return 3;
		else /* n == 3 */
			return 2;
	}
}

MPI *DIVM(MPI *Aptr, MPI *Bptr, MPI *Mptr)
/*
 * *Cptr = (*Aptr / *Bptr) mod (*Mptr). 
 * Here 0 <= *Aptr, *Bptr, *Cptr < *Mptr, gcd(*Bptr, *Mptr) = 1.
 */
{
	MPI *K, *Cptr;

	K = INVERSEM(Bptr, Mptr);
	Cptr = MULTM(Aptr, K, Mptr);
	FREEMPI(K);
	return (Cptr);
}

unsigned int DIVm(USI a, USI b, USI m)
/*
 * returns a / b mod m if m > 0, a / b in GF(4) if m = 0.
 * here 0 <= a, b < m < R0, gcd(b, m) = 1 if m > 0, b != 0 if m = 0.
 */
{
	unsigned int k;

	k = INVERSEm(b, m);
	return (MULTm(a, k, m));
}

unsigned int POWERm(USI a, MPI *Bptr, USI m)
/*
 * returns a^ *Bptr mod m.
 * here 0 <= a < m < R0 and 0 <= *Bptr.
 */
{
	unsigned int x, z;
	MPI *Y, *TempI;
	
	x = a;
	Y = COPYI(Bptr);
	z = 1;
	while (Y->S)
        {
		while (!(Y->V[0] & 1))
		{
			TempI = Y;
			Y = INT0_(Y, 2);
			FREEMPI(TempI);
			x = MULTm(x, x, m);
		} 
		TempI = Y;
		Y = SUB0_I(Y, 1);
		FREEMPI(TempI);
		z = MULTm(z, x, m);
	}
	FREEMPI(Y);
	return (z);
}

unsigned int POWER_m(USI a, USI y, USI m)
/*
 * returns a^ y mod m. 
 * Here 0 <= a < m < R0 and 0 <= y is an unsigned int.
 */ 
{
	unsigned int x, z;
	
	x = a;
	z = 1;
	while (y)
        {
		while (y % 2 == 0)
	        {
			y = y / 2;
			x = MULTm(x, x, m);
		} 
		y = y - 1;
		z = MULTm(z, x, m);
	}
	return (z);
}

MPI *MPOWER(MPI *Aptr, MPI *Bptr, MPI *Cptr)
/*
 * *Aptr, *Bptr, *Cptr, *Eptr  are MPI'S, *Cptr > 0, *Bptr >= 0,
 * Eptr = (*Aptr)^ *Bptr (mod *Cptr).
 */ 
{
	MPI *X, *Y, *Z, *TempI;
	
	X = COPYI(Aptr);
	Y = COPYI(Bptr);
	Z = ONEI();
	while (Y->S)
        {
		while (!(Y->V[0] & 1))
	        {
			TempI = Y;
			Y = INT0_(Y, 2);
			FREEMPI(TempI);
			TempI = X;
			X = MULTI(X, X);
			FREEMPI(TempI);
			TempI = X;
			X = MOD0(X, Cptr);
			FREEMPI(TempI);
		} 
		TempI = Y;
		Y = SUB0_I(Y, 1);
		FREEMPI(TempI);
		TempI = Z;
		Z = MULTI(Z, X);
		FREEMPI(TempI);
		TempI = Z;
		Z = MOD(Z, Cptr);
		FREEMPI(TempI);
	}
	FREEMPI(X);
	FREEMPI(Y);
	return (Z);
}

MPI *MPOWER_M(MPI *Aptr, USI b, MPI *Cptr)
/*
 * *Aptr, *Cptr, *Eptr  are MPI'S, *Cptr > 0, b an unsigned int.
 * *Eptr = (*Aptr)^ b (mod *Cptr). 
 */ 
{
	MPI *X, *Z, *TempI;
	unsigned int y;
	
	X = COPYI(Aptr);
	y = b;
	Z = ONEI();
	while (y)
        {
		while (y % 2 == 0)
	        {
			y = y / 2;
			TempI = X;
			X = MULTI(X, X);
			FREEMPI(TempI);
			TempI = X;
			X = MOD0(X, Cptr);
			FREEMPI(TempI);
		} 
		y = y - 1;
		TempI = Z;
		Z = MULTI(Z, X);
		FREEMPI(TempI);
		TempI = Z;
		Z = MOD(Z, Cptr);
		FREEMPI(TempI);
	}
	FREEMPI(X);
	return (Z);
}

MPI *MPOWER_(int a, MPI *Bptr, MPI *Cptr)
/*
 * 0 < |a| < R0. 
 *  *Cptr, *Eptr  are MPI'S, *Cptr > 0, *Bptr >= 0,
 * *Eptr = a^ *Bptr (mod *Cptr). 
 */ 
{
	MPI *X, *Y, *Z, *TempI;
	
	X = BUILDMPI(1);
	X->S = a > 0 ? 1 : -1;
	X->D = 0;
	X->V[0] = a > 0 ? a : -a;
	Y = COPYI(Bptr);
	Z = ONEI();
	while (Y->S)
        {
		while (!(Y->V[0] & 1))
	        {
			TempI = Y;
			Y = INT0_(Y, 2);
			FREEMPI(TempI);
			TempI = X;
			X = MULTI(X, X);
			FREEMPI(TempI);
			TempI = X;
			X = MOD0(X, Cptr);
			FREEMPI(TempI);
		} 
		TempI = Y;
		Y = SUB0_I(Y, 1);
		FREEMPI(TempI);
		TempI = Z;
		Z = MULTI(Z, X);
		FREEMPI(TempI);
		TempI = Z;
		Z = MOD(Z, Cptr);
		FREEMPI(TempI);
	}
	FREEMPI(X);
	FREEMPI(Y);
	return (Z);
}

MPI *BINOMIAL(USI n, USI m)
/*
 * returns n choose m, where n >= m are unsigned integers.
 */
{
	unsigned int j;
	MPI *G, *TempI, *Aptr;
	
	Aptr = ONEI();
	for (j = 1; j <= m; j++)
	{
		G = CHANGE(n - j + 1);
		TempI = Aptr;
		Aptr = MULTI(G, Aptr);
		FREEMPI(G);
		FREEMPI(TempI);
		TempI = Aptr;
		Aptr = INT0_(Aptr, j);	
		FREEMPI(TempI);
	}
	return (Aptr);
}

unsigned int LENGTHm(USI n)
/*
 * returns the number of decimal digits in the unsigned int n.
 */
{
	unsigned int i = 0;
	
	do
	{
		n = n / 10;
		i++;
	} while (n);
	return (i);
}

MPMATm *FINPUTMATm(FILE *infile)
/*
 * inputting a matrix of unsigned ints from a file.
 */
{
	unsigned int i, j, m, n;
	MPMATm *Mptr;

	if (fscanf(infile, "%u%u", &m, &n) != 2 || m == 0 || n == 0)
	{
		fprintf(stderr, "corrupt input: returning the 1 x 1 zero matrix\n");
		if (infile != stdin)
			exit (1);
		return ZEROMNm(1, 1);
	}
	fgetc(infile);
	Mptr = ZEROMNm(m, n);
	for (i = 0; i <= m - 1; i++) 
        {
		for (j = 0; j <= n - 1; j++) 
			if (fscanf(infile, "%u", &(Mptr->V[i][j])) != 1)
			{
				fprintf(stderr, "matrix input interrupted prematurely\n");
				if (infile != stdin)
					exit (1);
				GetReturn();
				return Mptr;
			}
	}
	return (Mptr);
}

MPMATm *FINPUTMATFILEm_m()
{
	char filename[50];
	MPMATm *Mptr;
	FILE *infile;

	printf("enter the name of your matrix file:");
	gets(filename);
	infile = Fopen_r(filename);
	if (infile == NULL)
	{
		printf("The file `%s' does not exist!\n",filename);
		GetReturn();
		Mptr = NULL;
	}
	else
	{
		Mptr = FINPUTMATm(infile);
		fclose(infile);
		printf("matrices from file %s successfully inputted\n", filename);
		GetReturn();
	}
	return (Mptr);
}

MPMATm *INPUTMATm(USI *uptr, USI z)
/*
 * inputting a matrix of unsigned ints from stdin.
 * *uptr is 1 for successful entry, 0 otherwise.
 */
{
	unsigned int i, j, m, n;
	int e;
	MPMATm *Mptr;

	printf("Enter the number of rows and number of columns\n");
	e = scanf("%u%u", &m, &n);
	if (e != 2 || m == 0 || n == 0)
	{
		fprintf(stderr, "corrupt input: returning the 1 x 1 zero matrix\n");
		*uptr = 0;
		GetReturn();
		getchar();
		return ZEROMNm(1, 1);
	}
	getchar();
	Mptr = ZEROMNm(m, n);
	for (i = 0; i <= m - 1; i++) 
        {
		printf("Enter row %u: all numbers < %u: \n", i + 1, z);
		for (j = 0; j <= n - 1; j++) 
		{
			e = scanf("%u", &(Mptr->V[i][j])); 
			if (e != 1)
			{
				fprintf(stderr, "matrix input interrupted prematurely\n");
				*uptr = 0;
				GetReturn();
				getchar();
				return ZEROMNm(1, 1);
			}
			if (Mptr->V[i][j] >= z)
			{
				fprintf(stderr, "matrix entry >= %u\n", z);
				fprintf(stderr, "matrix input interrupted prematurely\n");
				*uptr = 1;
				GetReturn();
				getchar();
				Mptr->V[i][j] = 0;
				return (Mptr);
			}
		}
	}
	*uptr = 1;
	return (Mptr);
}

void FPRINTMATm(FILE *outfile, USI i1, USI i2, USI j1, USI j2, MPMATm *Mptr)
/*
 * Modified 8/1/93 using Peter Adams' improvement from, August 1992.
 */
{
	char **str, *buff;
	unsigned int tmp, i, j, nrow, ncol, len, nstr = 0;
	int *colwidth;
	unsigned int ct;
	
	nrow = i2 - i1 + 1;
	ncol = j2 - j1 + 1;
	str = (char **)mmalloc(nrow * ncol * sizeof(char *));
	colwidth=(int *)mmalloc(ncol*sizeof(int));

	for(i=0; i<ncol; i++)
	  colwidth[i]=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
			tmp = 1 + LENGTHm(Mptr->V[i][j]);
			buff = (char *)mmalloc(tmp * sizeof(char));
			sprintf(buff, "%u", Mptr->V[i][j]);
		    	if ((len = strlen(buff)) > colwidth[j-j1])
				colwidth[j-j1] = len;
			str[nstr++] = strcpy((char *)mmalloc(len + 1), buff);
			ffree(buff, tmp * sizeof(char));
		}
	}
	if (outfile != stdout)
		fprintf(outfile, "%u %u\n", nrow, ncol);
	ct=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
	    		fprintf(outfile, "%*s ", colwidth[j-j1], str[ct]);
	    		ffree(str[ct], (unsigned int) (strlen(str[ct]) + 1) * sizeof(char));
	    		if ((ct % ncol) == (ncol - 1))
				fprintf(outfile, "\n");
	    		ct++;
		}
	}
	ffree((char *) str, nrow * ncol * sizeof(char *));
	ffree((char *) colwidth, ncol*sizeof(int));
	return;
}

void PRINTMATm(USI i1, USI i2, USI j1, USI j2, MPMATm *Mptr)
/*
 * prints *Mptr from rows i1 to i2 and cols j1 to j2.
 */
{
	unsigned int i, j;

	i = i2 - i1 + 1;
	j = j2 - j1 + 1;
	if (MAX((int)i, (int)j) > 20)
	{
		printf("(The number of rows or columns to be printed exceeds 20;\n");
		printf("there is no point in printing this matrix on the screen.)\n");
		return;
	}
	FPRINTMATm(stdout, i1, i2, j1, j2, Mptr);
	return;
}

unsigned int RUSSm(USI a, USI b, USI c, USI p)
/*
 *  input: unsigned ints a, b, c, p, with p > 0.
 * output: a + b * c (mod p).
 * Russian Peasant multiplication algorithm. Uses the identities
 * RUSSm(a, b, 2 * c, p) = RUSSm(a, 2 * b, c, p),
 * RUSSm(a, b, c + 1, p) = RUSSm(a + b, b, c, p).
 * If a, b, c and 2 * p are less than R0 * R0,
 * so is RUSSm(a, b, c, p). From H. Luneburg, "On the Rational Normal Form of
 * Endomorphisms", B.I. WissenSchaftsverlag, Mannheim/Wien/Zurich, 1987,
 * pp 18-19.
 */
{
	unsigned int t;

	a = a % p;
	b = b % p;
	c = c % p;
	while (c)
	{
		while (!(c & 1))
		{
			c = c >> 1;
			b = b << 1;
			b = b % p;
		}
		c--;
		t = a + b;
		a = (t < p) ? t : t - p;
	}
	return a;
}

unsigned int RANDOMm(USI x)
/*
 * input: unsigned int x, output:a "random number" a * x + c (mod m).
 * a = 1001, m = R0 = 65536, c = 65;
 * From H. Luneburg, "On the Rational Normal Form of Endomorphisms",
 * B.I. WissenSchaftsverlag, Mannheim/Wien/Zurich, 1987.
 * See Knuth Vol 2, Theorem A, p. 16.
 */
{
	unsigned int a, c, m;
		
	m = R0; a = 1001; c = 65;
	return RUSSm(c, a, x, m);
}
