/* i6R.c */
/* matrix operations using MPR's */

#include <stdio.h>
#include <stdlib.h>
#include "integer.h"
#include "fun.h"

MPMATR *BUILDMATR(USI m, USI n)
/*
 * Allocates space for an m x n matrix of MPR's.
 */
{
	MPMATR *N;

	N=(MPMATR *)mmalloc(sizeof(MPMATR));
	N->R = m;
	N->C = n;
	N->V = (MPR **)mmalloc(m * n * sizeof(MPR *));
	return (N);
}

MPMATR *COPYMATR(MPMATR *Mptr)
/*
 * a replacement for the assignment *Nptr = *Mptr.
 */
{
	unsigned int i, t;
	MPMATR *Nptr;

	t = Mptr->R * Mptr->C;
	Nptr = BUILDMATR(Mptr->R, Mptr->C);
	for (i = 0; i < t; i++)
			Nptr->V[i] = COPYR(Mptr->V[i]);
	return (Nptr);
}

void FREEMATR(MPMATR *Mptr)
/*
 * frees the storage allocated to the two-dimensional array Mptr->V.
 */
{
	unsigned int i, t;

	if (Mptr == NULL)
		return;
	t = Mptr->R * Mptr->C;
	for (i = 0; i < t; i++)
		FREEMPR(Mptr->V[i]);
	ffree((char *)(Mptr->V), t * sizeof(MPR *));
	ffree((char *)Mptr, sizeof(MPMATR));
	return;
}

MPMATR *UNWRAPMATR(MPMATR *Mptr)
/*
 * unwraps the matrix *Mptr into a long column vector.
 */
{
	unsigned int p, k;
	MPMATR *Nptr;

	p = Mptr->R * Mptr->C;
	Nptr = BUILDMATR(p, 1);
	for (k = 0; k <= p - 1; k++)
	elt(Nptr, k, 0) = COPYR(elt(Mptr, k /( Mptr->C), k % (Mptr->C)));
	return (Nptr);
}

unsigned int ZEROTESTR(MPMATR *Mptr)
/*
 * returns 1 if *Mptr is the ZERO matrix, 0 otherwise.
 */
{
	unsigned int i, j;

	for (i = 0; i < Mptr->R; i++)
	{
		for (j = 0; j < Mptr->C; j++)
		{
			if (!EQZEROR(elt(Mptr, i, j)))
				return (0);
		}
	}
	return (1);
}

unsigned int PIVOTR(USI i, USI j, MPMATR *Mptr)
/*
 * here 0 <= i <= j <= Mptr->R - 1.
 * returns the row index k, with least k (i <= k <= Mptr->R - 1), such that 
 * Mptr->V[k][j] is non-zero; returns Mptr->R otherwise.
 */
{
	unsigned int k;
	
	k = i;
	while ((k <= Mptr->R - 1) && (elt(Mptr, k, j)->N)->S == 0)
		k++;
	return (k);
}

void TRANSR(USI p, USI q, MPMATR *Mptr)
/*
 * 0 <= p < q <= Mprt->R - 1.
 * Mptr->V[i][j] = 0 for Mptr->R - 1 >= i >= p and 0 <= j < q 
 * transposes the p-th and q-th rows of *Mptr.
 */
{
	MPR *X;
	unsigned int j;

	for (j = p; j <= Mptr->C - 1; j++) 
	{
		X = elt(Mptr, p, j);
		elt(Mptr, p, j) = elt(Mptr, q, j);
		elt(Mptr, q, j) = X;
	}
	return;
}

void NORMR(USI p, USI j, MPMATR *Mptr)
/*
 * divides the p-th row by Mptr->V[p][j].
 * 0 <= p <= Mprt->R - 1.
 * 0 <= j <= Mprt->C - 1.
 * Mptr->V[p][k] = 0 if k < j.
 */
{
	MPR *X, *TempR;
	unsigned int k;

	X = COPYR(elt(Mptr, p, j));
	TempR = elt(Mptr, p, j);
	elt(Mptr, p, j) = ONER();
	FREEMPR(TempR);
	for (k = j + 1; k <= Mptr->C - 1; k++) 
	{
		TempR = elt(Mptr, p, k);
		elt(Mptr, p, k) = RATIOR(TempR, X);
		FREEMPR(TempR);
	}
	FREEMPR(X);
	return;
}

void SUBMR(USI p, USI q, MPR *Aptr, USI j, MPMATR *Mptr)
/*
 * subtracts *Aptr times the p-th row of *Mptr from the q-th,
 * for columns Mptr->C - 1 >= k > j.
 * 0 <= p, q <= Mprt->R - 1.
 * Mptr->V[p][k] = 0 if k < j, Mptr->V[p][j] = 1.
 */
{
	MPR *Temp1R, *Temp2R;
	unsigned int k;

	for (k = j + 1; k <= Mptr->C - 1; k++) 
	{
		Temp1R = MULTR(Aptr, elt(Mptr, p, k));
		Temp2R = elt(Mptr, q, k);
		elt(Mptr, q, k) = SUBR(Temp2R, Temp1R);
		FREEMPR(Temp2R);
		FREEMPR(Temp1R);
	}
	return;
}

MPR *TRACER(MPMATR *Mptr)
/*
 * *Aptr is the trace of *Mptr.
 */
{
	unsigned int i;
	MPR *Aptr, *Temp;

	Aptr = ZEROR();
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		Temp = Aptr;
		Aptr = ADDR(Aptr, elt(Mptr, i, i));
		FREEMPR(Temp);
	}
	return (Aptr);
}

MPMATR *SCALARMATR(MPMATR *Mptr, MPR *Aptr)
/*
 * returns (*Aptr) * (*Mptr).
 */
{
	unsigned int i, j;
	MPMATR *Nptr;

	Nptr = BUILDMATR(Mptr->R, Mptr->C);
	for (i = 0; i <= Nptr->R - 1; i++)
	{
		for (j = 0; j <= Nptr->C - 1; j++)
			elt(Nptr, i, j) = MULTR(elt(Mptr, i, j), Aptr);
	}
	return (Nptr);
}

MPMATR *MINUSMATR(MPMATR *Mptr)
/*
 * returns -(*Mptr).
 */
{
	unsigned int i, j;
	MPMATR *Nptr;

	Nptr = BUILDMATR(Mptr->R, Mptr->C);
	for (i = 0; i <= Nptr->R - 1; i++)
	{
		for (j = 0; j <= Nptr->C - 1; j++)
			elt(Nptr, i, j) = MINUSR(elt(Mptr, i, j));
	}
	return (Nptr);
}

MPMATR *DIAGR(MPMATR *Mptr, MPR *Aptr)
/*
 * returns *Mptr - (*Aptr) * (Unit Matrix).
 */
{
	unsigned int i;
	MPR *Temp;
	MPMATR *Nptr;

	Nptr = COPYMATR(Mptr);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
			Temp = elt(Nptr, i, i);
			elt(Nptr, i, i) = SUBR(elt(Mptr, i, i), Aptr);
			FREEMPR(Temp);
	}
	return (Nptr);
}

MPMATR *ADDMATR(MPMATR *Mptr, MPMATR *Nptr)
/*
 * returns *Mptr + *Nptr.
 */
{
	unsigned int i, j;
	MPMATR *Lptr;

	Lptr = BUILDMATR(Mptr->R, Mptr->C);
	for (i = 0; i <= Lptr->R - 1; i++)
		for (j = 0; j <= Lptr->C - 1; j++)
			elt(Lptr, i, j) = ADDR(elt(Mptr, i, j), elt(Nptr, i, j));
	return (Lptr);
}

MPMATR *SUBMATR(MPMATR *Mptr, MPMATR *Nptr)
/*
 * returns *Mptr - *Nptr.
 */
{
	unsigned int i, j;
	MPMATR *Lptr;

	Lptr = BUILDMATR(Mptr->R, Mptr->C);
	for (i = 0; i <= Lptr->R - 1; i++)
		for (j = 0; j <= Lptr->C - 1; j++)
			elt(Lptr, i, j) = SUBR(elt(Mptr, i, j), elt(Nptr, i, j));
	return (Lptr);
}

MPMATR *MULTMATR(MPMATR *Mptr, MPMATR *Nptr)
/*
 * Here Mptr->C = Nptr->R. 
 * returns (*Mptr) * (*Nptr).
 */
{
	MPR *X, *Y, *Temp;
	unsigned int i, j, k;
	MPMATR *Lptr;

	Lptr = BUILDMATR(Mptr->R, Nptr->C);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		for (j = 0; j <= Nptr->C - 1; j++)
		{
			X = ZEROR();
			for (k = 0; k <= Mptr->C - 1; k++)
			{
				Y = MULTR(elt(Mptr, i, k), elt(Nptr, k, j));
				Temp = X;
				X = ADDR(X, Y);
				FREEMPR(Temp);
				FREEMPR(Y);
			}
			elt(Lptr, i, j) = X;
		}
	}
	return (Lptr);
}

MPMATR *POWERMATR(MPMATR *Aptr, USI n)
/*
 * returns (*Aptr) ^ n, where 0 <= n < R0 * R0. 
 */
{
	MPMATR *Temp, *B, *Eptr;

	Eptr = IDENTITYR(Aptr->R);
	if (n == 0)
		return (Eptr);
	B = COPYMATR(Aptr);
	while (1)
        {
		if (n & 1) 
		{
			Temp = Eptr;
			Eptr = MULTMATR(Temp, B);
			FREEMATR(Temp);
			if (n == 1)
			{
				FREEMATR(B);
				return (Eptr);
			}
		}
		Temp = B;
		B = MULTMATR(Temp, Temp);
		FREEMATR(Temp);
		n = n >> 1;
	}
}

MPMATR *JCOLSMATR(MPMATR *Mptr, MPMATR *Nptr)
/*
 * returns [ *Mptr | *Nptr]
 */
{
	unsigned int i, j;
	MPMATR *Lptr;

	Lptr = BUILDMATR(Mptr->R, Mptr->C + Nptr->C);
	for (i = 0; i <= Lptr->R - 1; i++)
	{
		for (j = 0; j <= Lptr->C - 1; j++)
		{
			if (j <= Mptr->C - 1)
				elt(Lptr, i, j) = COPYR(elt(Mptr, i, j));
			else
				elt(Lptr, i, j) = COPYR(elt(Nptr, i, j - Mptr->C));
		}
	}
	return (Lptr);
}

MPMATR *JROWSMATR(MPMATR *Mptr, MPMATR *Nptr)
/*
 *         [ *Mptr]
 * returns [------]
 *         [ *Nptr]
 */
{
	unsigned int i, j;
	MPMATR *Lptr;

	Lptr = BUILDMATR(Mptr->R + Nptr->R, Mptr->C);
	for (i = 0; i <= Lptr->R - 1; i++)
	{
		if (i <= Mptr->R - 1)
		{
			for (j = 0; j <= Lptr->C - 1; j++)
				elt(Lptr, i, j) = COPYR(elt(Mptr, i, j));
		}
		else
		{
			for (j = 0; j <=  Lptr->C - 1; j++)
				elt(Lptr, i, j) = COPYR(elt(Nptr, i - Mptr->R, j));
		}
	}
	return (Lptr);
}

MPMATR *JCOLSIDENTITYR(MPMATR *Mptr)
/*
 * forms the partioned matrix [*Mptr | I]
 */
{
	MPMATR *Temp, *Nptr;

	Temp = IDENTITYR(Mptr->R);
	Nptr = JCOLSMATR(Mptr, Temp);
	FREEMATR(Temp);
	return Nptr;
}

MPMATR *GAUSS_JORDANR(MPMATR *Mptr, USI *rptr, USI *Cptr[])
/*
 * returns the reduced row-echelon form of *Mptr. 
 * *rptr = rank *Mptr and *Cptr[0],...,*Cptr[rank-1] are the columns in which 
 * the leading entries 1 occur.
 */
{
	MPMATR *Nptr;
	unsigned int i, j, k, m, n, p, r, *A;
	MPR *TempR;

	m = Mptr->R;
	n = Mptr->C;
	A = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	for (i = 0; i <= n - 1; i++)
		A[i] = 1;
	Nptr = COPYMATR(Mptr);
	i = 0;
	j = 0;

	*Cptr = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	while (1) 
	{
		while ((j <= n - 1) && (PIVOTR(i, j, Nptr) == m))
			j++;
		if (j == n)
			break;
		else 
		{
			p = PIVOTR(i, j, Nptr);
			if (p > i)
				TRANSR(i, p, Nptr);
			if (!EQONER(elt(Nptr, i, j)))
				NORMR(i, j, Nptr);
			*(*Cptr + i) = j;
			A[j] = 0;
			for (k = 0; k <= m - 1; k++) 
			{
				if (k != i)
				{
					if ((elt(Nptr, k, j)->N)->S)
					{
						SUBMR(i, k, elt(Nptr, k, j), j, Nptr);
						TempR = elt(Nptr, k, j);
						elt(Nptr, k, j) = ZEROR();
						FREEMPR(TempR);
					}
				}
			}
			if ((i == m - 1) || (j == n - 1))
			{
				i++;
				break;
			}
			i++;
			j++;
		}
	}
	*rptr = r = i;
	for (j = r; j <= n - 1; j++)
	{
		for (i = 0; i <= n - 1; i++)
		{
			if (A[i])
			{
				*(*Cptr + j) = i;
				A[i] = 0;
				break;
			}
		}
	}
	ffree((char *)A, n * sizeof(unsigned int));
	/*for (k = 0; k <= n - 1; k++)
		printf("C[%u] = %u\n", k, C[k]);*/
	return (Nptr);
 }

void EQUATIONSR(MPMATR *Mptr)
/*
 * solution of a system of linear equations with MPR coefficients,
 * where *Mptr is the augmented matrix.
 */
{
	MPMATR *N;
	MPR *H;
	unsigned int f, i, j, n, r, s, *C;

	n = Mptr->C - 1;
	if (n == 0)
	{
		printf("augmented matrix has only one column!\n");
		return;
	}
	N = GAUSS_JORDANR(Mptr, &r, &C);
	s = n - r;
	if (r)
	{
		if (C[r - 1] == n)
		{
			printf("the system has no solution\n");
			ffree((char *)C, (n + 1) * sizeof(unsigned int));
			FREEMATR(N);
			return;
		}
		for (i = 0; i <= r - 1; i++)
		{
			printf("x[%u] = ", 1 + C[i]);
			if (s == 0)
			{
				PRINTR(elt(N, i, C[n]));
				printf("\n");
				continue;
			}
			if ((elt(N, i, C[n])->N)->S)
			{
				f = 0;
				PRINTR(elt(N, i, C[n]));
			}
			else if ((elt(N, i, C[n])->N)->S == 0)
				f = 1;
			for (j = r; j <= n - 1; j++)
			{

				H =  MINUSR(elt(N, i, C[j]));
				if ((H->N)->S == 0)
				{
					FREEMPR(H);
					continue;
				}
				else if ((H->N)->S == 1)
				{
					if (f == 0)
						printf(" + ");
					f = 0;
				}
				else if ((H->N)->S == -1)
				{
					if (f == 1)
						printf(" -");
					else
						printf(" - ");
					if (!EQMINUSONER(H))
						(H->N)->S = -((H->N)->S);
					f = 0;
				}
				if (!EQONEI(H->N) && !EQMINUSONEI(H->N))
					PRINTI(H->N);
				printf("x[%u]", 1 + C[j]);
				if (!EQONEI(H->D))
				{
					printf("/");
					PRINTI(H->D);
				}
				FREEMPR(H);
			}
			if (f)
				printf("%u", 0);
			printf("\n");
		}
	}
	if (s)
	{
		printf("Here ");
		for (j = r; j <= n - 1; j++)
		{
			if (j != n - 1)
				printf("x[%u], ", 1 + C[j]);
			else
				printf("x[%u] ", 1 + C[j]);
		}
		if (s == 1)
			printf("is arbitrary.\n");
		else
			printf("are arbitrary.\n");
	}
	ffree((char *)C, (n + 1) * sizeof(unsigned int));
	FREEMATR(N);
	return;
}

MPMATR *NULL_SPACER(MPMATR *Mptr)
/*
 * returns a matrix whose columns form a basis for N(*Mptr),
 * otherwise returns NULL.
 */
{
	MPMATR *N, *Lptr;
	unsigned int  i, j, n, *C, r;

	n = Mptr->C;
	N = GAUSS_JORDANR(Mptr, &r, &C);
	if (r == n)
	{
		ffree((char *)C, n * sizeof(unsigned int));
		FREEMATR(N);
		return (NULL);
	}
	else
	{
		Lptr = BUILDMATR(n, n - r);
		if (r)
		{
			for (i = 0; i <= r - 1; i++)
			{
				for (j = 0; j <= n - r - 1; j++)
					elt(Lptr, C[i], j) = MINUSR(elt(N, i, C[r + j]));
			}
		}
		FREEMATR(N);
		for (i = r; i <= n - 1; i++)
		{
			for (j = 0; j <= n - r - 1; j++)
			{
				if (j == i - r)
					elt(Lptr, C[i], j) = ONER();
				else
					elt(Lptr, C[i], j) = ZEROR();
			}
		}
		ffree((char *)C, n * sizeof(unsigned int));
		return (Lptr);
	}
}

MPMATR *COLUMN_SPACER(MPMATR *Mptr)
/*
 * returns a matrix whose columns form a basis for C(*Mptr).
 * otherwise returns NULL.
 */
{
	MPMATR *N, *Lptr;
	unsigned int  i, j, m, n, *C, r;

	m = Mptr->R;
	n = Mptr->C;
	N = GAUSS_JORDANR(Mptr, &r, &C);
	FREEMATR(N);
	if (r == 0)
	{
		ffree((char *)C, n * sizeof(unsigned int));
		return (NULL);
	}
	else
	{
		Lptr = BUILDMATR(m, r);
		for (i = 0; i <= m - 1; i++)
		{
			for (j = 0; j <= r - 1; j++)
				elt(Lptr, i, j) = COPYR(elt(Mptr, i, C[j]));
		}
		ffree((char *)C, n * sizeof(unsigned int));
		return (Lptr);
	}
}

MPMATR *EQUATIONS1R(MPMATR *Mptr)
/*
 * solution of a system of linear equations with MPR coefficients, where 
 * *Mptr is the augmented matrix. To be used when there is no solution, in
 * which case NULL is returned, or when there is a unique solution, in which
 * case the column vector (x[0],...x[n-1]) is returned.
 */
{
	MPMATR *N, *Nptr;
	unsigned int i, n, r, *C;

	n = Mptr->C - 1;
	N = GAUSS_JORDANR(Mptr, &r, &C);
	Nptr = NULL;
	if (C[r - 1] != n)
	{
		Nptr = BUILDMATR(n, 1);
		for (i = 0; i <= n - 1; i++)
			elt(Nptr, i, 0) = COPYR(elt(N, i, C[n]));
	}
	ffree((char *)C, (n + 1) * sizeof(unsigned int));
	FREEMATR(N);
	return Nptr;
}

MPMATR *CHOLESKYR(MPMATR *A)
/*
 * Input: A positive definite matrix A.
 * Output: The Cholesky decomposition of A.
 * See U.Finke and M. Pohst, "Improved methods for calculating vectors of
 * short length in a lattice, including a complexity analysis." Math. Comp.
 * 44, 463-471, 1985
 */
{
	MPMATR *Q;
	unsigned int i, j, k, l, m;
	MPR *temp1, *temp2;

	m = A->R;
	Q = ZEROMNR(m, m);
	for (i = 0; i < m; i++)
	{
		for (j = i; j < m; j++)
		{
			FREEMPR(elt(Q, i, j));
			elt(Q, i, j) = COPYR(elt(A, i, j));
		}
	}
	for (i = 0; i < m - 1; i++)
	{
		for (j = i + 1; j < m; j++)
		{
			FREEMPR(elt(Q, j, i));
			elt(Q, j, i) = COPYR(elt(Q, i, j));
			temp1 = elt(Q, i, j);
			elt(Q, i, j) = RATIOR(elt(Q, i, j), elt(Q, i, i));
			FREEMPR(temp1);
		}
		for (k = i + 1; k < m; k++)
		{
			for (l = k; l < m; l++)
			{
				temp1 = elt(Q, k, l);
				temp2 = MULTR(elt(Q, k, i), elt(Q, i, l));
				elt(Q, k, l) = SUBR(temp1, temp2);
				FREEMPR(temp1);
				FREEMPR(temp2);
			}
		}
	}
/*
	for (j = 0; j < m - 1; j++)
	{
		for (i = j + 1; i < m; i++)
		{
			FREEMPR(elt(Q, i, j));
			elt(Q, i, j) = ZEROR();
		}
	}
*/
	return (Q);
}

MPR *INTROOT(MPR *Z, MPR *U)
/*
 * Returns [sqrt(Z)+U]. First ANSWER = [sqrt(Z)] + [U]. One then
 * tests if Z < ([sqrt(Z)] + 1 -{U})^2. If this does not hold, ANSWER++.
 */
{
	int t;
	MPR *X, *Y, *ANSWER, *R, *S, *T1, *T2, *ONE, *Temp;

	ONE = ONER();
	if (EQZEROR(Z))
		return (INTR(U));
	X = MTHROOTR(Z, 2, 0);
	Y = INTR(U);
	ANSWER = ADDR(X, Y);
	R = SUBR(U, Y);
	FREEMPR(Y);
	S = SUBR(ONE, R);
	FREEMPR(R);
	T1 = ADDR(X, S);
	FREEMPR(S);
	FREEMPR(X);
	T2 = MULTR(T1, T1);
	FREEMPR(T1);
	t = COMPARER(T2, Z);
	FREEMPR(T2);
	if (t <= 0)
	{
		Temp = ANSWER;
		ANSWER = ADDR(ANSWER, ONE);
		FREEMPR(Temp);
	}
	FREEMPR(ONE);
	return (ANSWER);
}

int COMPARER(MPR *Aptr, MPR *Bptr)
/*
 *				    1 if *Aptr > *Bptr
 *            Returns               0 if *Aptr = *Bptr
 *       			   -1 if *Aptr < *Bptr
 */
{
	MPI *T1, *T2;
	int t;

	T1 = MULTI(Aptr->N, Bptr->D);
	T2 = MULTI(Aptr->D, Bptr->N);
	t = COMPAREI(T1, T2);
	FREEMPI(T1);
	FREEMPI(T2);
	return (t);
}
