/* i7CR.c */
#include <stdio.h>
#include "integer.h"
#include "fun.h"

MPMATCR *ROWVECTORCR(MPMATCR *Mptr, USI i)
/*
 * returns the i-th row vector of *Mptr.
 */
{
	MPMATCR *Temp;
	unsigned int j;
	
	Temp = BUILDMATCR(1, Mptr->C);
	for (j = 0; j < Mptr->C; j++)
		elt(Temp, 0, j) = COPYCR(elt(Mptr, i, j));
	return (Temp);
}

MPMATCR *ColVectorCR(MPMATCR *Mptr, USI j)
/*
 * returns the j-th column vector of *Mptr.
 */
{
	MPMATCR *Temp;
	unsigned i;
	Temp = BUILDMATCR(Mptr->R, 1);
	for (i = 0; i < Mptr->R; i++)
		elt(Temp, i, 0) = COPYCR(elt(Mptr, i, j));
	return Temp;
}

MPCR *DOTCR(MPMATCR *Mptr, MPMATCR *Nptr)
/*
 * *Aptr is the dot product of row/column vectors *Mptr and *Nptr.
 */
{
	unsigned int i, j;
	MPCR *H, *TempCR, *Aptr;
	
	Aptr = ZEROCR();
	for (i = 0; i <= Mptr->R - 1; i++)
		for (j = 0; j <= Mptr->C - 1; j++)
		{
			H = CONJCR(elt(Mptr, i, j));
			TempCR = H;
			H = MULTCR(elt(Nptr, i, j), H);
			FREEMPCR(TempCR);
			TempCR = Aptr;
			Aptr = ADDCR(Aptr, H);
			FREEMPCR(TempCR);
		}
	return (Aptr);
}

MPMATCR *GRAM_SCHMIDTCR(MPMATCR *Mptr)
/*
 * *Mptr is a non-zero matrix. The left to right basis is found for the
 * column-space. The Gram-Schmidt process is then applied to get an orthogonal
 * basis for the column-space of *Mptr.
 */
{
	MPCR *G, *H, *I, *TempCR;
	MPMATCR *M1, *R1, *R2, *Tmp;
	unsigned int i, j, k, n, r;

	n = Mptr->R;
	M1 = COLUMN_SPACECR(Mptr);
	r = M1->C;
	for (j = 1; j <= r - 1; j++)
	{
		R1 = ColVectorCR(M1, j);
		for (k = 0; k < j; k++)
		{
			R2 = ColVectorCR(M1, k);
			G = DOTCR(R2, R1);
			H = DOTCR(R2, R2);
			TempCR = G;
			G = MINUSCR(G);
			FREEMPCR(TempCR);
			I = RATIOCR(G, H);
			Tmp = R2;
			R2 = SCALARMATCR(Tmp, I);
			FREEMATCR(Tmp);
			Tmp = R1;
			R1 = ADDMATCR(Tmp, R2);
			FREEMATCR(R2);
			FREEMATCR(Tmp);
			FREEMPCR(G);
			FREEMPCR(H);
			FREEMPCR(I);
		}
		for (i = 0; i <= n - 1; i++)
			elt(M1, i, j) = COPYCR(elt(R1, i, 0));
		FREEMATCR(R1);
	}
	return (M1);
}

MPCR *DOTLCR(MPMATCR *Mptr)
/*
 * calculates the square of the length of *Mptr.
 */
{
	return DOTCR(Mptr, Mptr);
}
