/* p1R.c */

#include <stdio.h>
#include <stdlib.h>
#include "integer.h"
#include "fun.h"

MPMATPR *BUILDMATPR(USI m, USI n)
/*
 * Allocates space for an m x n matrix of POLYR's.
 */
{
	unsigned int i;
	MPMATPR *N;

	N=(MPMATPR *)mmalloc(sizeof(MPMATPR));
	N->R = m;
	N->C = n;
	N->V = (POLYR **)mmalloc(m * sizeof(POLYR *));
	for (i = 0; i <= m - 1; i++)
		*(N->V + i) = (POLYR *)mmalloc(n * sizeof(POLYR));
	return (N);
}

MPMATPR *COPYMATPR(MPMATPR *Mptr)
/*
 * a replacement for the assignment *Nptr = *Mptr. 
 */
{
	unsigned int i, j;
	MPMATPR *Nptr;

	Nptr = BUILDMATPR(Mptr->R, Mptr->C);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		for (j = 0; j <= Mptr->C - 1; j++)
			Nptr->V[i][j] = COPYPR(Mptr->V[i][j]);
	}
	return (Nptr);
}

void FREEMATPR(MPMATPR *Mptr)
/*
 * frees the storage allocated to the two-dimensional array Mptr->V.
 */
{
	unsigned int i, j;

	if (Mptr == NULL)
		return;
	for (i = 0; i < Mptr->R; i++)
	{
		for (j = 0; j < Mptr->C; j++)
			DELETE_PR(Mptr->V[i][j]);
		ffree((char *)(*(Mptr->V + i)), Mptr->C * sizeof(POLYR));
	}
	ffree((char *)(Mptr->V), Mptr->R * sizeof(POLYR *));
	ffree((char *)Mptr, sizeof(MPMATPR));
}

MPMATPR *INPUTMATPR()
/*
 * inputting a matrix of POLYR'S.
 */
{
	unsigned int u, i, j, m, n;
	MPMATPR *Mptr;

	printf("Enter the number of rows and number of columns\n");
	if (scanf("%u%u", &m, &n) != 2 || m == 0 || n == 0)
	{
		fprintf(stderr, "corrupt input: returning the 1 x 1 zero matrix\n");
		return ZEROMNPR(1, 1);
	}
	getchar();
	Mptr = ZEROMNPR(m, n);
	for (i = 0; i <= m - 1; i++) 
        {
		printf("Enter row %u:\n", i + 1);
		for (j = 0; j <= n - 1; j++) 
		{
			printf("Entering polynomial[%u][%u]:\n", i + 1, j + 1);
			Mptr->V[i][j] = INPUTPR(&u);
			if (u == 0)
			{
				printf("matrix input prematurely interrupted\n");
				GetReturn();
				return Mptr;
			}
			printf("Polynomial[%u][%u] = ", i + 1, j + 1);
			PRINTPR(Mptr->V[i][j]);
			printf("\n");
		}
	}
	return (Mptr);
}

POLYR *SMITHPR(MPMATPR *Mptr, MPMATPR **Pptr, MPMATPR **Qptr, USI *ptr)
/*
 * returns the invariant factors of *Mptr.
 * **Pptr and **Qptr are invertible matrices such that
 * **Pptr x *Mptr x **Qptr is the Smith canonical form *Nptr below.
 * *ptr is the number of invariant factors.
 * See Rings, Modules and Linear Algebra by B. Hartley and T. O. Hawkes,
 * Chapman and Hall, 1970.
 */
{
	unsigned int i, k, l, m, n, p, q, t, z, isok;
	MPR *G, *TempR;
	POLYR Q, *N, Tmp;
	MPMATPR *Nptr, *Temp;
	
	m = Mptr->R;
	n = Mptr->C;
	Nptr = COPYMATPR(Mptr);
	*Pptr = IDENTITYPR(m);
	*Qptr = IDENTITYPR(n);
	z = MIN((int) m - 1, (int) n - 1);
	N = (POLYR *)mmalloc((z + 1) * sizeof(POLYR));
	for (i = 0; i <= z; i++)
	{
		ZEROTESTPR(0, Nptr, &p, &q);
		if (p == Nptr->R && q == Nptr->C)
			break;
		if (p > 0)
		{
			Nptr = SWAP_ROWSPR1(0, p, Nptr);
			*Pptr = SWAP_ROWSPR1(i, p + i, *Pptr);
		}
		if (q > 0)
		{
			Nptr = SWAP_COLSPR1(0, q, Nptr);
			*Qptr = SWAP_COLSPR1(i, q + i, *Qptr);
		}
		/* Now Nptr->V[0][0] != NULL. */
	while (1)
	{
		while (1)
		{
			isok = 1;
			l = ROWSEEKPR(0, Nptr);
			if (l == Nptr->C)
				break;
			else
			{
				/* Case 1, p. 112. */
				Q = DIVPR(Nptr->V[0][l], Nptr->V[0][0]);
				Nptr = COLSUBPR1(0, l, Q, Nptr); 
				*Qptr = COLSUBPR1(i, l + i, Q, *Qptr); 
				DELETE_PR(Q);
				Nptr = SWAP_COLSPR1(0, l, Nptr); 
				*Qptr = SWAP_COLSPR1(i, l + i, *Qptr); 
			}
/* Now all elts to the right of Nptr->V[0][0] are divisible by it. */
		while (1)
		{
			k = COLSEEKPR(0, Nptr);
			if (k == Nptr->R)
				break;
			else
			{
				isok = 0;
				/* Case 2, p. 112. */
				Q = DIVPR(Nptr->V[k][0], Nptr->V[0][0]);
				Nptr = ROWSUBPR1(0, k, Q, Nptr);
				*Pptr = ROWSUBPR1(i, k + i, Q, *Pptr);
				DELETE_PR(Q);
				Nptr = SWAP_ROWSPR1(0, k, Nptr); 
				*Pptr = SWAP_ROWSPR1(i, k + i, *Pptr); 
				/* This reduces the degree of Nptr->V[0][0]. */
			}
		}
		if (isok)
			break;
	}
/* Now (Case 3, p.112) Nptr->V[0][0] divides each element to its right as well as below it.*/
		for (l = 1; l < Nptr->C; l++)
		{
			if (Nptr->V[0][l] != NULL)
			{
				Q = DIVPR(Nptr->V[0][l], Nptr->V[0][0]);
				Nptr = COLSUBPR1(0, l, Q, Nptr); 
				*Qptr = COLSUBPR1(i, l + i, Q, *Qptr); 
				DELETE_PR(Q);
			}
		}
		for (k = 1; k < Nptr->R; k++)
		{
			if ( Nptr->V[k][0] != NULL)
			{
				Q = DIVPR(Nptr->V[k][0], Nptr->V[0][0]);
				*Pptr = ROWSUBPR1(i, k + i, Q, *Pptr);
				DELETE_PR(Q);
				DELETE_PR(Nptr->V[k][0]);
				Nptr->V[k][0] = NULL;
			}
		}
		/* Now each element to the right of and below 
		Nptr->V[0][0] is zero. */
		k = MATSEEKPR(0, Nptr);
		if (k != Nptr->R)
		{
			for (t = 0; t < Nptr->C; t++)
			{
				if (Nptr->V[k][t] != NULL)
				{
					Tmp = Nptr->V[0][t];
					Nptr->V[0][t] = ADDPR(Nptr->V[0][t], Nptr->V[k][t]);
					DELETE_PR(Tmp);
				}
			}
			for (t = 0; t < m; t++)
			{
				if ((*Pptr)->V[k+i][t] != NULL) 
				{
					Tmp = (*Pptr)->V[i][t];
					(*Pptr)->V[i][t] = ADDPR((*Pptr)->V[i][t], (*Pptr)->V[k+i][t]);
					DELETE_PR(Tmp);
				}
			}
		}
		else
			break;
	}
/* Now Nptr->V[0][0] divides all elements Nptr->V[k][l], k,l > 0. */
		G = LEADPR(Nptr->V[0][0]);
		TempR = G;
		G = INVERSER(G);
		FREEMPR(TempR);
		Temp = *Pptr;
		*Pptr = SCALAR_ROWPR(i, G, Temp);
		FREEMATPR(Temp);
		FREEMPR(G);
		Tmp = Nptr->V[0][0];
		Nptr->V[0][0] = MONICPR(Nptr->V[0][0]);
		DELETE_PR(Tmp);
		N[i] = COPYPR(Nptr->V[0][0]);
		printf("\n");
		if (i < z)
		{
			Temp = Nptr;
			Nptr = DELETE_ROWPR(1, Nptr);
			FREEMATPR(Temp);
			Temp = Nptr;
			Nptr = DELETE_COLPR(1, Nptr);
			FREEMATPR(Temp);
		}
	}
	FREEMATPR(Nptr);
	*ptr = i++;
	return (N);
}

POLYR *SMITH1PR(MPMATPR *Mptr, USI *ptr)
/*
 * returns the invariant factors of *Mptr.
 * See Rings, Modules and Linear Algebra by B. Hartley and T. O. Hawkes,
 * Chapman and Hall, 1970. 						   
 */
{
	unsigned int i, k, l, m, n, p, q, t, z;
	POLYR Q, *N, Tmp;
	MPMATPR *Nptr, *Temp;
	
	m = Mptr->R;
	n = Mptr->C;
	Nptr = COPYMATPR(Mptr);
	z = MIN((int) m - 1, (int) n - 1);
	N = (POLYR *)mmalloc((z + 1) * sizeof(POLYR));
	for (i = 0; i <= z; i++)
	{
		ZEROTESTPR(0, Nptr, &p, &q);
		if (p == Nptr->R && q == Nptr->C)
			break;
		if (p > 0)
			Nptr = SWAP_ROWSPR1(0, p, Nptr);
		if (q > 0)
			Nptr = SWAP_COLSPR1(0, q, Nptr);
		/* Now Nptr->V[0][0] != NULL. */
	while (1)
	{
		while (1)
		{
			l = ROWSEEKPR(0, Nptr);
			if (l == Nptr->C)
				break;
			else
			{
				/* Case 1, p. 112. */
				Q = DIVPR(Nptr->V[0][l], Nptr->V[0][0]);
				Nptr = COLSUBPR1(0, l, Q, Nptr); 
				DELETE_PR(Q);
				Nptr = SWAP_COLSPR1(0, l, Nptr); 
			}
		}
/* Now all elts to the right of Nptr->V[0][0] are divisible by it. */
		while (1)
		{
			k = COLSEEKPR(0, Nptr);
			if (k == Nptr->R)
				break;
			else
			{
				/* Case 2, p. 112. */
				Q = DIVPR(Nptr->V[k][0], Nptr->V[0][0]);
				Nptr = ROWSUBPR1(0, k, Q, Nptr);
				DELETE_PR(Q);
				Nptr = SWAP_ROWSPR1(0, k, Nptr); 
				/* This reduces the degree of Nptr->V[0][0]. */
			}
		}
/* Now (Case 3, p.112) Nptr->V[0][0] divides each element to its right as well as below it.*/
		for (l = 1; l < Nptr->C; l++)
		{
			if (Nptr->V[0][l] != NULL)
			{
				Q = DIVPR(Nptr->V[0][l], Nptr->V[0][0]);
				Nptr = COLSUBPR1(0, l, Q, Nptr); 
				DELETE_PR(Q);
			}
		}
		for (k = 1; k < Nptr->R; k++)
		{
			if ( Nptr->V[k][0] != NULL)
			{
				DELETE_PR(Nptr->V[k][0]);
				Nptr->V[k][0] = NULL;
			}
		}
		/* Now each element to the right of and below 
		Nptr->V[0][0] is zero. */
		k = MATSEEKPR(0, Nptr);
		if (k != Nptr->R)
		{
			for (t = 0; t < Nptr->C; t++)
			{
				if (Nptr->V[k][t] != NULL)
				{
					Tmp = Nptr->V[0][t];
					Nptr->V[0][t] = ADDPR(Nptr->V[0][t], Nptr->V[k][t]);
					DELETE_PR(Tmp);
				}
			}
		}
		else
		{
			/* Now Nptr->V[0][0] divides all elements
			Nptr->V[k][l], k,l > 0. */
			Tmp = MONICPR(Nptr->V[0][0]);
			DELETE_PR(Nptr->V[0][0]);
			Nptr->V[0][0] = Tmp;
			N[i] = COPYPR(Nptr->V[0][0]);
			break;
		}
	}
		if (i < z)
		{
			Temp = Nptr;
			Nptr = DELETE_ROWPR(1, Nptr);
			FREEMATPR(Temp);
			Temp = Nptr;
			Nptr = DELETE_COLPR(1, Nptr);
			FREEMATPR(Temp);
		}
	}
	FREEMATPR(Nptr);
	*ptr = i++;
	return (N);
}
