/* p1m.c */

#include <stdio.h>
#include <stdlib.h>
#include "integer.h"
#include "fun.h"

MPMATPm *BUILDMATPm(USI m, USI n)
/*
 * Allocates space for an m x n matrix of POLYm's.
 */
{
	unsigned int i;
	MPMATPm *N;

	N=(MPMATPm *)mmalloc(sizeof(MPMATPm));
	N->R = m;
	N->C = n;
	N->V = (POLYm **)mmalloc(m * sizeof(POLYm *));
	for (i = 0; i <= m - 1; i++)
		*(N->V + i) = (POLYm *)mmalloc(n * sizeof(POLYm));
	return (N);
}

MPMATPm *COPYMATPm(MPMATPm *Mptr, USI m)
/*
 * a replacement for the assignment *Nptr = *Mptr. 
 */
{
	unsigned int i, j;
	MPMATPm *Nptr;

	Nptr = BUILDMATPm(Mptr->R, Mptr->C);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		for (j = 0; j <= Mptr->C - 1; j++)
			Nptr->V[i][j] = COPYPm(Mptr->V[i][j], m);
	}
	return (Nptr);
}

void FREEMATPm(MPMATPm *Mptr)
/*
 * frees the storage allocated to the two-dimensional array Mptr->V.
 */
{
	unsigned int i, j;

	if (Mptr == NULL)
		return;
	for (i = 0; i < Mptr->R; i++)
	{
		for (j = 0; j < Mptr->C; j++)
			DELETE_Pm(Mptr->V[i][j]);
		ffree((char *)(*(Mptr->V + i)), Mptr->C * sizeof(POLYm));
	}
	ffree((char *)(Mptr->V), Mptr->R * sizeof(POLYm *));
	ffree((char *)Mptr, sizeof(MPMATPm));
}

MPMATPm *ZEROMNPm(USI m, USI n)
/*
 * returns the zero polynomial matrix of size m x n.
 */
{
	unsigned int i, j;
	MPMATPm *Mptr;

	Mptr = BUILDMATPm(m, n);
	for (i = 0; i <= m - 1; i++)
	{
		for (j = 0; j <= n - 1; j++)
			Mptr->V[i][i] =  NULL;
	}
	return (Mptr);
}

MPMATPm *INPUTMATPm(USI *uptr, USI M)
/*
 * inputting a matrix of POLYm'S.
 * *uptr is 1 for successful entry, 0 otherwise.
 */
{
	unsigned int i, j, m, n, u;
	MPMATPm *Mptr;

	printf("Enter the number of rows and number of columns\n");
	if (scanf("%u%u", &m, &n) != 2 || m == 0 || n == 0)
	{
		fprintf(stderr, "corrupt input: returning the 1 x 1 zero matrix\n");
		*uptr = 0;
		GetReturn();
		getchar();
		return ZEROMNPm(1, 1);
	}
	getchar();
	*uptr = 1;
	Mptr = ZEROMNPm(m, n);
	for (i = 0; i <= m - 1; i++) 
        {
		printf("Enter row %u:\n", i + 1);
		for (j = 0; j <= n - 1; j++) 
		{
			printf("Entering polynomial[%u][%u]:\n", i + 1, j + 1);
			Mptr->V[i][j] = INPUTPm(&u, M);
			if (u == 0)
			{
				printf("matrix input prematurely interrupted.\n");
				GetReturn();
				return (Mptr);
			}
			printf("Polynomial[%u][%u] = ", i + 1, j + 1);
			PRINTPm(Mptr->V[i][j]);
			printf("\n");
		}
	}
	return (Mptr);
}

POLYm *SMITHPm(MPMATPm *Mptr, MPMATPm **Pptr, MPMATPm **Qptr, USI *ptr, USI M)
/*
 * returns the invariant factors of *Mptr.
 * **Pptr and **Qptr are invertible matrices such that 
 * **Pptr x *Mptr x **Qptr is the Smith canonical form *Nptr below.
 * *ptr is the number of invariant factors.
 * See Rings, Modules and Linear Algebra by B. Hartley and T. O. Hawkes.
 * Chapman and Hall, 1970.
 */
{
	unsigned int g, i, k, l, m, n, p, q, t, z;
	POLYm Q, *N, Tmp;
	MPMATPm *Nptr, *Temp;
	
	m = Mptr->R;
	n = Mptr->C;
	Nptr = COPYMATPm(Mptr, M);
	*Pptr = IDENTITYPm(m);
	*Qptr = IDENTITYPm(n);
	z = MIN((int) m - 1, (int) n - 1);
	N = (POLYm *)mmalloc((z + 1) * sizeof(POLYm));
	for (i = 0; i <= z; i++)
	{
		ZEROTESTPm(0, Nptr, &p, &q);
		if (p == Nptr->R && q == Nptr->C)
			break;
		if (p > 0)
		{
			Nptr = SWAP_ROWSPm1(0, p, Nptr, M);
			*Pptr = SWAP_ROWSPm1(i, p + i, *Pptr, M);
		}
		if (q > 0)
		{
			Nptr = SWAP_COLSPm1(0, q, Nptr, M);
			*Qptr = SWAP_COLSPm1(i, q + i, *Qptr, M);
		}
		/* Now Nptr->V[0][0] != NULL. */
	while (1)
	{
		while (1)
		{
			l = ROWSEEKPm(0, Nptr, M);
			if (l == Nptr->C)
				break;
			else
			{
				/* Case 1, p. 112. */
				Q = DIVPm(Nptr->V[0][l], Nptr->V[0][0], M);
				Nptr = COLSUBPm1(0, l, Q, Nptr, M); 
				*Qptr = COLSUBPm1(i, l + i, Q, *Qptr, M); 
				DELETE_Pm(Q);
				Nptr = SWAP_COLSPm1(0, l, Nptr, M); 
				*Qptr = SWAP_COLSPm1(i, l + i, *Qptr, M); 
			}
		}
/* Now all elts to the right of Nptr->V[0][0] are divisible by it. */
		while (1)
		{
			k = COLSEEKPm(0, Nptr, M);
			if (k == Nptr->R)
				break;
			else
			{
				/* Case 2, p. 112. */
				Q = DIVPm(Nptr->V[k][0], Nptr->V[0][0], M);
				Nptr = ROWSUBPm1(0, k, Q, Nptr, M);
				*Pptr = ROWSUBPm1(i, k + i, Q, *Pptr, M);
				DELETE_Pm(Q);
				Nptr = SWAP_ROWSPm1(0, k, Nptr, M); 
				*Pptr = SWAP_ROWSPm1(i, k + i, *Pptr, M); 
				/* This reduces the degree of Nptr->V[0][0]. */
			}
		}
/* Now (Case 3, p.112) Nptr->V[0][0] divides each element to its right as well as below it.*/
		for (l = 1; l < Nptr->C; l++)
		{
			if (Nptr->V[0][l] != NULL)
			{
				Q = DIVPm(Nptr->V[0][l], Nptr->V[0][0], M);
				Nptr = COLSUBPm1(0, l, Q, Nptr, M); 
				*Qptr = COLSUBPm1(i, l + i, Q, *Qptr, M); 
				DELETE_Pm(Q);
			}
		}
		for (k = 1; k < Nptr->R; k++)
		{
			if ( Nptr->V[k][0] != NULL)
			{
				Q = DIVPm(Nptr->V[k][0], Nptr->V[0][0], M);
				*Pptr = ROWSUBPm1(i, k + i, Q, *Pptr, M);
				DELETE_Pm(Q);
				DELETE_Pm(Nptr->V[k][0]);
				Nptr->V[k][0] = NULL;
			}
		}
		/* Now each element to the right of and below 
		Nptr->V[0][0] is zero. */
		k = MATSEEKPm(0, Nptr, M);
		if (k != Nptr->R)
		{
			for (t = 0; t < Nptr->C; t++)
			{
				if (Nptr->V[k][t] != NULL)
				{
					Tmp = Nptr->V[0][t];
					Nptr->V[0][t] = ADDPm(Nptr->V[0][t], Nptr->V[k][t], M);
					DELETE_Pm(Tmp);
				}
			}
			for (t = 0; t < m; t++)
			{
				if ((*Pptr)->V[k+i][t] != NULL)
				{
					Tmp = (*Pptr)->V[i][t];
					(*Pptr)->V[i][t] = ADDPm((*Pptr)->V[i][t], (*Pptr)->V[k+i][t], M);
					DELETE_Pm(Tmp);
				}
			}
		}
		else
		{
/* Now Nptr->V[0][0] divides all elements Nptr->V[k][l], k,l > 0. */
			g = Nptr->V[0][0]->COEF;
			g = INVERSEm(g, M);
			Temp = *Pptr;
			*Pptr = SCALAR_ROWPm(i, g, Temp, M);
			FREEMATPm(Temp);
			Tmp = MONICPm(Nptr->V[0][0], M);
			DELETE_Pm(Nptr->V[0][0]);
			Nptr->V[0][0] = Tmp;
			N[i] = COPYPm(Nptr->V[0][0], M);
			break;
		}
	}
		if (i < z)
		{
			Temp = Nptr;
			Nptr = DELETE_ROWPm(1, Nptr, M);
			FREEMATPm(Temp);
			Temp = Nptr;
			Nptr = DELETE_COLPm(1, Nptr, M);
			FREEMATPm(Temp);
		}
	}
	FREEMATPm(Nptr);
	*ptr = i++;
	return (N);
}

POLYm *SMITH1Pm(MPMATPm *Mptr, USI *ptr, USI M)
/*
 * returns the invariant factors of *Mptr.
 * See Rings, Modules and Linear Algebra by B. Hartley and T. O. Hawkes,
 * Chapman and Hall, 1970.
 */
{
	unsigned int i, k, l, m, n, p, q, t, z;
	POLYm Q, *N, Tmp;
	MPMATPm *Nptr, *Temp;

	m = Mptr->R;
	n = Mptr->C;
	Nptr = COPYMATPm(Mptr, M);
	z = MIN((int) m - 1, (int) n - 1);
	N = (POLYm *)mmalloc((z + 1) * sizeof(POLYm));
	for (i = 0; i <= z; i++)
	{
		ZEROTESTPm(0, Nptr, &p, &q);
		if (p == Nptr->R && q == Nptr->C)
			break;
		if (p > 0)
			Nptr = SWAP_ROWSPm1(0, p, Nptr, M);
		if (q > 0)
			Nptr = SWAP_COLSPm1(0, q, Nptr, M);
		/* Now Nptr->V[0][0] != NULL. */
	while (1)
	{
		while (1)
		{
			l = ROWSEEKPm(0, Nptr, M);
			if (l == Nptr->C)
				break;
			else
			{
				/* Case 1, p. 112. */
				Q = DIVPm(Nptr->V[0][l], Nptr->V[0][0], M);
				Nptr = COLSUBPm1(0, l, Q, Nptr, M); 
				DELETE_Pm(Q);
				Nptr = SWAP_COLSPm1(0, l, Nptr, M); 
			}
		}
/* Now all elts to the right of Nptr->V[0][0] are divisible by it. */
		while (1)
		{
			k = COLSEEKPm(0, Nptr, M);
			if (k == Nptr->R)
				break;
			else
			{
				/* Case 2, p. 112. */
				Q = DIVPm(Nptr->V[k][0], Nptr->V[0][0], M);
				Nptr = ROWSUBPm1(0, k, Q, Nptr, M);
				DELETE_Pm(Q);
				Nptr = SWAP_ROWSPm1(0, k, Nptr, M); 
				/* This reduces the degree of Nptr->V[0][0]. */
			}
		}
/* Now (Case 3, p.112) Nptr->V[0][0] divides each element to its right as well as below it.*/
		for (l = 1; l < Nptr->C; l++)
		{
			if (Nptr->V[0][l] != NULL)
			{
				Q = DIVPm(Nptr->V[0][l], Nptr->V[0][0], M);
				Nptr = COLSUBPm1(0, l, Q, Nptr, M); 
				DELETE_Pm(Q);
			}
		}
		for (k = 1; k < Nptr->R; k++)
		{
			if ( Nptr->V[k][0] != NULL)
			{
				DELETE_Pm(Nptr->V[k][0]);
				Nptr->V[k][0] = NULL;
			}
		}
		/* Now each element to the right of and below 
		Nptr->V[0][0] is zero. */
		k = MATSEEKPm(0, Nptr, M);
		if (k != Nptr->R)
		{
			for (t = 0; t < Nptr->C; t++)
			{
				if (Nptr->V[k][t] != NULL)
				{
					Tmp = Nptr->V[0][t];
					Nptr->V[0][t] = ADDPm(Nptr->V[0][t], Nptr->V[k][t], M);
					DELETE_Pm(Tmp);
				}
			}
		}
		else
		{
			/* Now Nptr->V[0][0] divides all elements
			Nptr->V[k][l], k,l > 0. */
			Tmp = MONICPm(Nptr->V[0][0], M);
			DELETE_Pm(Nptr->V[0][0]);
			Nptr->V[0][0] = Tmp;
			N[i] = COPYPm(Nptr->V[0][0], M);
			break;
		}
	}
		if (i < z)
		{
			Temp = Nptr;
			Nptr = DELETE_ROWPm(1, Nptr, M);
			FREEMATPm(Temp);
			Temp = Nptr;
			Nptr = DELETE_COLPm(1, Nptr, M);
			FREEMATPm(Temp);
		}
	}
	FREEMATPm(Nptr);
	*ptr = i++;
	return (N);
}

void GCDKNUTHPm(POLYm Pptr, POLYm Qptr, POLYm *Uptr, POLYm *Vptr, POLYm *Wptr, USI p)
/*
 * *Uptr = GCDPm(Pptr, Qptr); *Vptr = Pptr/(*Uptr); *Wptr = Qptr/(*Uptr) mod p.
 * See Knuth Vol.2, page 440.
 */
{
	*Uptr = GCDPm(Pptr, Qptr, p);
	*Vptr = DIVPm(Pptr, *Uptr, p);
	*Wptr = DIVPm(Qptr, *Uptr, p);
	return;
}

POLYm *SQUAREFREEYUNPm(POLYm Pptr, USI *C[], USI *eptr, POLYm *Zptr, USI p)
/* The algorithm of D.Y.Y. Yun. See Knuth Vol. 2, Solution to problem 34, 
 * page 631.
 * The polynomials U[0],...,U[*eptr-1] are returned.
 * If Pptr = g(x^p), a default U[0] = 1 is returned.
 * *Zptr = NULL if squarefree decomposition is attained.
 * In which case the decomposition has factors U[0],...,U[*eptr-1],
 * with corresponding multiplicities C[0],...,C[*eptr-1].
 * Otherwise equals the polynomial z(x) of Knuth Vol. 2, Solution to problem 36
 * page 632. 
 * Then z(x) can be used to find the squarefree decomposition recursively.
 */
{
	unsigned int i, j, m, sum;
	POLYm *U, U1, V1, t, Temp, TempU, TempV, TempW, TempVV, TempWW;
	POLYm temp, temp1, prod, onepm, CURSOR;
	
	m = (unsigned int)(Pptr->DEG);
	*C = (unsigned int *)mmalloc(m * sizeof(unsigned int));
	U = (POLYm *)mmalloc(m * sizeof(POLYm));
	U1 = DERIVPm(Pptr, p);
	GCDKNUTHPm(Pptr, U1, &t, &TempV, &TempW, p);
	if (t->DEG == 0) /* Pptr is squarefree */
	{
		U[0] = COPYPm(TempV, p);
		(*C)[0] = 1;
		*eptr = 1;
		DELETE_Pm(TempV);
		DELETE_Pm(TempW);
		DELETE_Pm(U1);
		DELETE_Pm(t);
		*Zptr = NULL;
		return U;
	}
	DELETE_Pm(U1);
	i = 1;
	j = 0;
	while (1)
	{
		V1 = DERIVPm(TempV, p);
		Temp = SUBPm(TempW, V1, p);
		DELETE_Pm(V1);
		DELETE_Pm(TempW);
		if (Temp == NULL)
			break;
		GCDKNUTHPm(TempV, Temp, &TempU, &TempVV, &TempWW, p);
		DELETE_Pm(Temp);
		DELETE_Pm(TempV);
		if (TempU->DEG > 0)
		{
			U[j] = COPYPm(TempU, p);
			(*C)[j] = i;
			j++;
		}
		DELETE_Pm(TempU);
		i++;
		TempV = COPYPm(TempVV, p);
		DELETE_Pm(TempVV);
		TempW = COPYPm(TempWW, p);
		DELETE_Pm(TempWW);
	} 
	U[j] = MONICPm(TempV, p);
	DELETE_Pm(TempV);
	(*C)[j] = i;
	*eptr = j + 1;
	if (p > m)
	{
		*Zptr = NULL;
		DELETE_Pm(t);
		return U;
	}
	sum = 0;
	for (i = 0; i <= j; i++)
		sum += (*C)[i] * (unsigned int)(U[i]->DEG);
	if (sum == m)
	{
		*Zptr = NULL;
		DELETE_Pm(t);
		return U;
	}
	else
	{
		onepm = CONSTANTPm(1);
		prod = COPYPm(onepm, p);
		for (i = 0; i <= j; i++)
		{
			if ( (*C)[i] >= 2 && (*C)[i] <= p - 1)
			{
				temp = POWERPm(U[i], (*C)[i] - 1, p);
				temp1 = prod;
				prod = MULTPm(temp1, temp, p);
				DELETE_Pm(temp);
				DELETE_Pm(temp1);
			}
		}
		*Zptr = DIVPm(t, prod, p);
		CURSOR = *Zptr;
		while (CURSOR != NULL)
		{
			CURSOR->DEG = CURSOR->DEG / p;
			CURSOR = CURSOR->NEXT;
		}
		DELETE_Pm(t);
		DELETE_Pm(prod);
		DELETE_Pm(onepm);
		return U;
	}
}

POLYm *YUNSQUAREFREEPm(POLYm Pptr, USI *C[], USI *eptr, USI p)
/* The algorithm of D.Y.Y. Yun. See Knuth Vol. 2, Solution to problem 34
 * and 36, pages 631, 632.
 * The squarefree decomposition has factors U[0],...,U[*eptr-1],
 * with corresponding multiplicities C[0],...,C[*eptr-1].
 */
{
	unsigned int *D, *DW, *DWW, i, j, k, l, m, n, e, count, pos, flag;
	POLYm *U, **W, *V, *Temp, Temp0, Temp1, Z, g, u, v, temp, temp1, temp2;

	n = (unsigned int)(Pptr->DEG);
	*C = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	U = (POLYm *)mmalloc(n * sizeof(POLYm));
	W = (POLYm **)mmalloc(n * sizeof(POLYm *));
	DW = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	DWW = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	count = 0;
	i = 0;
	Temp0 = COPYPm(Pptr, p);
	while (1)
	{
		m = (unsigned int)(Temp0->DEG);
		V = SQUAREFREEYUNPm(Temp0, &D, &e, &Z, p);
		if (Z == NULL)
		{
			if (count == 0)
			{
				for (j = 0; j <= e - 1; j++)
				{
					(*C)[j] = D[j];
					U[j] = COPYPm(V[j], p);			
					DELETE_Pm(V[j]);
				}
				*eptr = e;
				DELETE_Pm(Temp0);
				ffree((char *)W, n * sizeof(POLYm *));
				ffree((char *)V, m * sizeof(POLYm));
				ffree((char *)D, n * sizeof(unsigned int));
				ffree((char *)DW, n * sizeof(unsigned int));
				ffree((char *)DWW, n * sizeof(unsigned int));
				return U;
			}
			else
			{
				W[i] =  V;
				DW[i] =  e;
				DWW[i] =  m;
				i++;
				DELETE_Pm(Temp0);
				ffree((char *)D, m * sizeof(unsigned int));
				break;
			}
		}
		else if (e == 1 && V[0]->DEG == 0)
		{
			DELETE_Pm(V[0]);
			ffree((char *)V, m * sizeof(POLYm));
		}
		else
		{
			W[i] =  V;
			DW[i] =  e;
			DWW[i] =  m;
			i++;
		}
		Temp1 = Temp0;
		Temp0 = Z;
		DELETE_Pm(Temp1);
		ffree((char *)D, m * sizeof(unsigned int));
		count++;
	}
	i--;
	flag = 0;
	for (l = i; l >= 1; l--)
	{
		flag = 1;
		Temp = W[l - 1];
		W[l - 1] = (POLYm *)mmalloc(n * sizeof(POLYm));
		pos = 0;
		for (k = 0; k <= DW[l] - 1; k++)
		{
			for (j = 0; j <= DW[l - 1] - 1; j++)
			{
				GCDKNUTHPm(Temp[j], W[l][k], &g, &u, &v, p);
				temp = Temp[j];
				Temp[j] = COPYPm(u, p);
				DELETE_Pm(temp);
				DELETE_Pm(u);
				if(g->DEG > 0)
				{
					W[l - 1][pos] = COPYPm(g, p);
					pos++;
				}
				temp = W[l][k];
				W[l][k] = COPYPm(v, p);
				DELETE_Pm(temp);
				DELETE_Pm(g);
				DELETE_Pm(v);
			}
			if (W[l][k]->DEG > 0)
			{
				W[l - 1][pos] = COPYPm(W[l][k], p);
				pos++;
			}
			DELETE_Pm(W[l][k]);
		}
		for (j = 0; j <= DW[l - 1] - 1; j++)
		{
			if (Temp[j]->DEG > 0)
			{
				W[l - 1][pos] = COPYPm(Temp[j], p);
				pos++;
			}
			DELETE_Pm(Temp[j]);
		}
		ffree((char *)Temp, DWW[l - 1] * sizeof(POLYm));
		ffree((char *)W[l], DWW[l] * sizeof(POLYm));
		DW[l - 1] = pos;
		DWW[l - 1] = n;
	}
	*eptr = DW[0];
	Temp = W[0];
	for (j = 0; j <= DW[0] - 1; j++)
	{
		U[j] = COPYPm(Temp[j], p);
		DELETE_Pm(Temp[j]);
	}
	if (flag == 0)
		ffree((char *)Temp, DWW[0] * sizeof(POLYm));
	else
		ffree((char *)Temp, n * sizeof(POLYm));
	ffree((char *)DW, n * sizeof(unsigned int));
	ffree((char *)DWW, n * sizeof(unsigned int));
	ffree((char *)W, n * sizeof(POLYm *));
	temp = MONICPm(Pptr, p);
	for (j = 0; j <= *eptr - 1; j++)
	{
		i = 0;
		while (temp->DEG != 0)
		{
			temp2 = MODPm(temp, U[j], p);
			if (temp2 != NULL)
			{
				DELETE_Pm(temp2);
				break;
			}
			temp1 = temp;
			temp = DIVPm(temp1, U[j], p);
			DELETE_Pm(temp1);
			i++;
		}
		(*C)[j] = i;	
	}
	DELETE_Pm(temp);
	return U;
}

POLYm LEVERRIER_m(MPMATm *Mptr, MPMATm **Nptr, USI *dptr, USI z)
/*
 * The Faddeeva/Leverrier method for finding the characteristic polynomial,
 * adjoint and determinant of a matrix mod z.
 * See "Computational methods in linear algebra", by V. N. Faddeeva, Dover
 * 1959. Here *Pptr[i] is the coefficient of x^(n - 1 - i) in ch(*Mptr).
 */
{
	POLYm Pptr, CURSOR;
	POLYI PptrI;
	unsigned int g, i, j, n;
	MPMATm *Qptr, *M;
	MPMATI *Temp1I, *Temp2I;
	MPI *DETI;

	g = 0; /* to keep the compiler quiet */
	n = Mptr->R;
	if (z <= n)
	{
		Temp1I = BUILDMATI(n, n);
		for (i = 0; i <= n - 1; i++)
		{
			for (j = 0; j <= n - 1; j++)
				elt(Temp1I, i, j) = CHANGE(Mptr->V[i][j]);
		}
		PptrI = LEVERRIER_I(Temp1I, &Temp2I, &DETI);
		FREEMATI(Temp1I);
		*Nptr = BUILDMATm(n, n);
		for (i = 0; i <= n - 1; i++)
		{
			for (j = 0; j <= n - 1; j++)
				(*Nptr)->V[i][j] = MOD_(elt(Temp2I, i, j), z);	
		}
		FREEMATI(Temp2I);
		Pptr = CHANGETOPm(PptrI, z);
		DELETE_PI(PptrI);
		*dptr = MOD_(DETI, z);
		FREEMPI(DETI);
		return (Pptr);
	}
	Qptr = COPYMATm(Mptr);
	Pptr = NULL;
	for (i = 0; i <= n - 1; i++)
	{
		g = TRACEm(Qptr, z);
		g = DIVm(g, i + 1, z);
		PINSERTPm((int)(n - 1 - i), g, &Pptr, 0, z);
		M = DIAGm(Qptr, g, z);
		FREEMATm(Qptr);
		if (i == n - 2)
		{
			if (i % 2 == 0)
				*Nptr = MINUSMATm(M, z);
			else
				*Nptr = COPYMATm(M);
		}  
		Qptr =	MULTMATm(Mptr, M, z);
		FREEMATm(M);
	}
	FREEMATm(Qptr);
	PINSERTPm((int)i, 1, &Pptr, 0, z);
	PURGEPm(&Pptr);
	*dptr = g;
	if (i % 2 == 0)
		*dptr = MINUSm(*dptr, z);
	CURSOR = Pptr->NEXT;
	while (CURSOR != NULL)
	{
		CURSOR->COEF = MINUSm(CURSOR->COEF, z);
		CURSOR = CURSOR->NEXT;
	}
	return (Pptr);
}

MPMATm *BERLEKAMPm(POLYm Pptr, USI z)
/*
 * returns the matrix [q_{jk}] where x^{zk}=sigma q_{jk}x^j (mod Pptr). 
 * Here Pptr is squarefree.
 */
{
	MPMATm *Nptr;
	unsigned int n, k, q;
	POLYm P, CURSOR, temp;

	n = (unsigned int)(Pptr->DEG);
	if (z == 0)
		q = 4;
	else
		q = z;
	Nptr = ZEROMNm(n, n);
	Nptr->V[0][0] = 1;
	P = CREATEm();
	P->DEG = q;
	P->COEF = 1;
	P->NEXT = NULL;
	for (k = 1; k <= n - 1; k++)
	{
		temp = P;
		P = MODPm(temp, Pptr, z);
		DELETE_Pm(temp);
		CURSOR = P;
		while (CURSOR != NULL)
		{
			Nptr->V[CURSOR->DEG][k] = CURSOR->COEF;
			CURSOR->DEG = CURSOR->DEG + q;
			CURSOR = CURSOR->NEXT;
		}
	}
	DELETE_Pm(P);
	return Nptr;
}

MPMATm *BERLEKAMP1m(POLYm Pptr, USI z)
/*
 * returns the matrix [q_{jk}] where x^{zk}=sigma q_{jk}x^j (mod Pptr). 
 * Here Pptr is squarefree.
 */
{
	MPMATm *Nptr;
	unsigned int m, n, k, q, t;
	POLYm P, Q, CURSOR, temp;

	n = (unsigned int)(Pptr->DEG);
	if (z == 0)
		q = 4;
	else
		q = z;
	Nptr = ZEROMNm(n, n);
	Nptr->V[0][0] = 1;
	P = CREATEm();
	P->DEG = q;
	P->COEF = 1;
	P->NEXT = NULL;
	temp = P;
	P = MODPm(temp, Pptr, z);
	DELETE_Pm(temp);
	t = 0;
	m = (n - 1) / q;
	for (k = 0; k <= m; k++)
	{
		Nptr->V[t][k] = 1;
		t += q;
	}
	Q = CREATEm();
	Q->DEG = t;
	Q->COEF = 1;
	Q->NEXT = NULL;
	for (k = m + 1; k <= n - 1; k++)
	{
		temp = Q;
		Q = MODPm(temp, Pptr, z);
		DELETE_Pm(temp);
		CURSOR = Q;
		while (CURSOR != NULL)
		{
			Nptr->V[CURSOR->DEG][k] = CURSOR->COEF;
			CURSOR = CURSOR->NEXT;
		}
		temp = Q;
		Q = MULTPm(temp, P, z);
		DELETE_Pm(temp);
	}
	DELETE_Pm(P);
	DELETE_Pm(Q);
	return Nptr;
}

POLYm NULL_SPACEBm(MPMATm *Mptr, USI z)
/*
 * returns NULL if *Mptr is the Berlekamp matrix of an irreducible polynomial
 * mod z. Otherwise returns a polynomial corresponding to the second basis
 * vector of the null-space of the Berlekamp matrix.
 */
{
	MPMATm *N;
	POLYm w;
	unsigned int i, k, n, *C, r, *V;

	n = Mptr->C;
	N = GAUSS_JORDANm(Mptr, &r, &C, z);
	if (n - r == 1) 
	{
		FREEMATm(N);
		ffree((char *)C, n * sizeof(unsigned int));
		return NULL;
	}
	V = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	if (r)
	{
		for (i = 0; i <= r - 1; i++)
			V[C[i]] = MINUSm(N->V[i][C[r + 1]], z);
	}
	FREEMATm(N);
	for (i = r; i <= n - 1; i++)
	{
		{
			if (i - r == 1)
				V[C[i]] = 1;
			else
				V[C[i]] = 0;
		}
	}
	w = NULL;
	for (k = 0; k <= n - 1; k++)
		PINSERTPm((int)k, V[k], &w, 0, z);
	PURGEPm(&w);
	ffree((char *)C, n * sizeof(unsigned int));
	ffree((char *)V, n * sizeof(unsigned int));
	return w;
}

POLYm FACTOR_BERLEKAMPm(POLYm Pptr, USI z)
/*
 * returns a non--trivial factor of a reducible squarefree polynomial Pptr mod z,
 * using P. Camion's method, but returns MONICPm(Pptr, z) if Pptr is irreducible
 * mod z.
 */
{
	unsigned int d, n, i, l;
	POLYm output, w, ipm, Temp, Temp1, Temp2, Temp3, Temp5, Temp6;
	POLYm Temp7, onepm, twopm;
	MPMATm *MATm1, *MATm2, *MATm3;
	
	n = (unsigned int)(Pptr->DEG);
	MATm1 = BERLEKAMP1m(Pptr, z);
	MATm2 = IDENTITYm(n);
	MATm3 = SUBMATm(MATm1, MATm2, z);
	FREEMATm(MATm1);
	FREEMATm(MATm2);
	w = NULL_SPACEBm(MATm3, z);
	FREEMATm(MATm3);
	if (w == NULL)
		return (MONICPm(Pptr, z));
	if (z > 2)
	{
		d = (z - 1) / 2;
		onepm = CONSTANTPm(1);
		for (i = 0; i <= z - 1; i++)
		{
			ipm = CONSTANTPm(i);
			Temp1 = ADDPm(w, ipm, z);
			DELETE_Pm(ipm);
			Temp2 = POWER0Pm(Temp1, d, Pptr, z);
			DELETE_Pm(Temp1);
			Temp3 = ADDPm(Temp2, onepm, z);
			Temp5 = MULTPm(Temp3, Temp2, z);
			DELETE_Pm(Temp2);
			DELETE_Pm(Temp3);
			Temp6 = MODPm(Temp5, Pptr, z);
			DELETE_Pm(Temp5);
			if (Temp6 == NULL || Temp6->DEG == 0)
			{
				DELETE_Pm(Temp6);
				continue;
			}
			l = INVERSEm(2, z);
			Temp7 = SCALARPm(l, Temp6, z);
			DELETE_Pm(Temp6);
			output = GCDPm(Temp7, Pptr, z);
			DELETE_Pm(Temp7);
			DELETE_Pm(w);
			DELETE_Pm(onepm);
			return output;
		}
	}
	else
	{
		onepm = CONSTANTPm(1);
		twopm = CONSTANTPm(2);
		for (i = 0; i <= 1; i++)
		{
			Temp = w;
			w = ADDPm(Temp, onepm, 0);
			DELETE_Pm(Temp);
			Temp2 = POWER0Pm(w, 3, Pptr, z);
			if (Temp2->DEG >0)
			{
				Temp = GCDPm(Temp2, Pptr, z);
				DELETE_Pm(w);
				DELETE_Pm(onepm);
				DELETE_Pm(twopm);
				DELETE_Pm(Temp2);
				return Temp;
			}
			DELETE_Pm(Temp2);
			Temp = COPYPm(w, 0);
			Temp1 = ADDPm(Temp, twopm, 0);
			DELETE_Pm(Temp);
			Temp2 = POWER0Pm(Temp1, 3, Pptr, 0);
			DELETE_Pm(Temp1);
			if (Temp2->DEG > 0)
			{
				Temp = GCDPm(Temp2, Pptr, z);
				DELETE_Pm(Temp2);
				DELETE_Pm(onepm);
				DELETE_Pm(twopm);
				DELETE_Pm(w);
				return Temp;
			}
			DELETE_Pm(Temp2);
		}
	}
	return (NULL); /* to keep the compiler quiet */
}

POLYm IRRED_FACTORPm(POLYm Uptr, USI p)
/*
 * input: Uptr is a reducible polynomial mod p which  is squarefree.
 * output: a monic irreducible factor of Uptr mod p using the P. Camion method,
 * Annals of Discrete Mathematics 17 (1983) 149-157.
 */
{
	int n;
	POLYm V, Temp;

	V = MONICPm(Uptr, p);
	while (1)
	{
		n = V->DEG;
		Temp = V;
		V = FACTOR_BERLEKAMPm(Temp, p);
		if (V->DEG == n)
		{
			DELETE_Pm(Temp);
			return V;
		}
		DELETE_Pm(Temp);
	}
}

POLYm *IRRED_FACTOR1Pm(POLYm Uptr, USI *nptr, USI p)
/*
 * input: Uptr is a reducible polynomial mod p which  is squarefree.
 * output: the *nptr monic irreducible factors of Uptr mod p using the 
 * P. Camion method, Annals of Discrete Mathematics 17 (1983) 149-157.
 */
{
	int c = 1, i = 0, n;
	unsigned int m;
	POLYm *Q, *F, V, Temp;

	m = (unsigned int)(Uptr->DEG);
	Q = (POLYm *)mmalloc(m * sizeof(POLYm));	
	F = (POLYm *)mmalloc(m * sizeof(POLYm));	
	Q[0] = COPYPm(Uptr, p);
	c = 1;/* c = number of Q[i] in the queue. */
	while (c)
	{
		n = Q[c - 1]->DEG;
		Temp = Q[c - 1];
		V = FACTOR_BERLEKAMPm(Temp, p);
		if (V->DEG == n)
		{
			F[i] = COPYPm(V, p); 
			c--;
			i++;
		}
		else
		{
			Q[c - 1] = COPYPm(V, p);
			Q[c] = DIVPm(Temp, V, p);
			c++;
		}
		DELETE_Pm(Temp);
		DELETE_Pm(V);
	}
	*nptr = i;
	ffree((char *)Q, m * sizeof(POLYm));
	return F;
}

POLYm *FACTORSPm(POLYm Pptr, USI *leadptr, USI *C[], USI *nptr, USI p)
/*
* Factors a polynomial over Z_p[x] using the P. Camion method,
* Annals of Discrete Mathematics 17 (1983) 149-157, using the 
* squarefree decomposition mentioned in Knuth, solution to problem 36, p. 632.
* Returns the leading coefficient of Pptr and factorization into irreducibles
* *Q[0],...,*Q[*nptr - 1] are the essentially distinct irreducibles,
* C[0],...,C[*nptr - 1] are the corresponding multiplicities.
*/
{
	unsigned int n, i, j = 0, k, t, *D, number;
	POLYm *SQ, temp, *Q, *P;

	n = (unsigned int)(Pptr->DEG);
	*leadptr = Pptr->COEF;
	temp = MONICPm(Pptr, p);
	Q = (POLYm *)mmalloc(n * sizeof(POLYm));
	*C = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	SQ = YUNSQUAREFREEPm(temp, &D, &t, p);
	DELETE_Pm(temp);
	for (i = 0; i < t; i++)
	{
		P = IRRED_FACTOR1Pm(SQ[i], &number, p);
		for (k = 0; k <= number - 1; k++)
		{
			(*C)[j + k] = D[i];	
			Q[j + k] = P[k];
		}
		ffree((char *)P, (unsigned int)(SQ[i]->DEG) * sizeof(POLYm));
		j += number;
		DELETE_Pm(SQ[i]);
	}
	*nptr = j;
	ffree((char *)D, n * sizeof(unsigned int));
	ffree((char *)SQ, n * sizeof(POLYm));
	return Q;
}
 
unsigned int TESTIRREDm(POLYm Pptr, USI p)
/*
 * returns 1 if Pptr is irreducible (mod p), 0 otherwise.
 */
{
	unsigned int n;
	POLYm P, Q, Temp;
	MPMATm *MATm1, *MATm2, *MATm3;
	
	Q = DERIVPm(Pptr, p);
	Temp = GCDPm(Pptr, Q, p);
	if (Temp->DEG > 0)
	{
		DELETE_Pm(Q);
		DELETE_Pm(Temp);
		return 0;
	}
	/* Pptr is squarefree, so we can apply BERLEKAMP1m. */
	DELETE_Pm(Temp);
	DELETE_Pm(Q);
	n = (unsigned int)(Pptr->DEG);
	MATm1 = BERLEKAMP1m(Pptr, p);
	MATm2 = IDENTITYm(n);
	MATm3 = SUBMATm(MATm1, MATm2, p);
	FREEMATm(MATm1);
	FREEMATm(MATm2);
	P = NULL_SPACEBm(MATm3, p);
	FREEMATm(MATm3);
	if (P == NULL)
		return 1;
	else
	{
		DELETE_Pm(P);
		return 0;
	}
}

POLYm IRRPm(USI n, USI p)
/* This program computes for a given prime p and integer n >= 2,
 * an monic irreducible polynomial of degree n over GF(p).
 * Method: Pick at random a monic polynomial of degree n over GF(p),
 * test whether it is irreducible; if not pick another.
 * From H. Luneburg, "On the Rational Normal Form of Endomorphisms",
 * B.I. WissenSchaftsverlag, Mannheim/Wien/Zurich, 1987, pp. 146-149.
 * 18/3/93. Restrict search to x^n + f(x), where deg(f) = binary(n).
 */
{
	POLYm test;
	unsigned int SUCCESS, m, t, testnumber = 0, x, k = 1, z;
	int i;

 	m = R0;
	t = R0 - 5;
	
	z = n;
	while(z)
	{
		z = z / 2;
		k++;
	}
	while (1)
	{
		do 
		{
			t = RANDOMm(t);
			x = (p * t) / m;
		}
		while (x == 0);
		test = NULL;
		PINSERTPm(0, x, &test, 0, p);
		for (i = 1; i <= k; i++)
		{
			t = RANDOMm(t);
			PINSERTPm(i, (p * t)/ m, &test, 0, p);
		}
		PINSERTPm(n, 1, &test, 0, p);
		PURGEPm(&test);
		SUCCESS = TESTIRREDm(test, p);
		testnumber++;
		printf("testnumber = %u\n", testnumber);
		if (SUCCESS)
		{
			printf("\7\7\n");
			return test;
		}
		else
			DELETE_Pm(test);
	}
}

POLYm MCWORTERm(MPMATm *Mptr, USI p)
/*
 * Returns the characteristic polynomial of *Mptr, a square matrix over Z_p.
 * The Method of W.A. McWorter, Mathematics Magazine, Vol 56, No. 3, May 1983,
 * 168-175 is used.
 */
{
	unsigned int e, j, k, n, jj, t;
	MPMATm *M, *R, *X, *Y, *Z, *T, *W;
	POLYm Q, Pptr, Temp;

	k = 0; jj = 0; n = Mptr->R;
	Z = IDENTITYm(n);
	M = ColVectorm(Z, k);
	Pptr = CONSTANTPm(1);
	while (1)
	{
		j = 0;
		while (1)
		{
			j++;
			X = ColVectorm(M, M->C - 1);
			Y = MULTMATm(Mptr, X, p);
			T = M;
			M = JCOLSMATm(T, Y);
			FREEMATm(T);
			FREEMATm(X);
			FREEMATm(Y);
			R = EQUATIONS1m(M, p);
			if (R != NULL)
				break;
		}
		jj += j;
		Q = NULL;
		for (t = 0; t <= j; t++)
		{
			e = (t == 0) ? 1 : MINUSm(R->V[jj - t][0], p);
			PINSERTPm((int)(j - t), e, &Q, 0, p);
		}
		PURGEPm(&Q);
		FREEMATm(R);
		Temp = Pptr;
		Pptr = MULTPm(Temp, Q, p);
		DELETE_Pm(Temp);
		DELETE_Pm(Q);
		if (jj == n)
			break;
		T = M;
		M = DELETE_COLm(M->C, T);
		FREEMATm(T);
		while (1)
		{
			k++;
			X = ColVectorm(Z, k);
			W = JCOLSMATm(M, X);
			R = EQUATIONS1m(W, p);
			FREEMATm(W);
			if (R == NULL)
			{
				T = M;
				M = JCOLSMATm(T, X);
				FREEMATm(T);
				FREEMATm(X);
				break;
			}
			FREEMATm(X);
			FREEMATm(R);
		}
	}
	FREEMATm(Z);
	FREEMATm(M);
	return Pptr;
}
