/* p2CR.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "integer.h"
#include "fun.h"

void ZEROTESTPCR(USI i, MPMATPCR *Mptr, USI *p, USI *q)
/*
 * returns the least indices *p >= i, *q >= i such that Mptr->V[*p][*q] != 0.
 * otherwise *p = Mptr->R and *q = Mptr->C.
 */
{
	unsigned int k, l;
	
	
	for (k = i; k <= Mptr->R - 1; k++)
	{
		for (l = i; l <= Mptr->C - 1; l++)
		{
			if (Mptr->V[k][l] != NULL)
			{
				*p = k;
				*q = l;
				return;
			}
		}
	}
	*p = Mptr->R;
	*q = Mptr->C;
	return;
}

unsigned int MATSEEKPCR(USI i, MPMATPCR *Mptr)
/*
 * returns the least row index k, i < k <= Mptr->R - 1, such that 
 * Mptr->V[k][l] is not divisible by Mptr->V[i][i], for some l,
 * i < l <= Mptr->C - 1; otherwise returns Mptr->R.
 */
{
	unsigned int k, l;
	POLYCR P;
	
	for (k = i + 1; k <= Mptr->R - 1; k++)
	{
		for (l = i + 1; l <= Mptr->C - 1; l++)
		{
			if ((P = MODPCR(Mptr->V[k][l], Mptr->V[i][i])) != NULL)
			{
				DELETE_PCR(P);
				return (k);
			}
		}
	}
	return (k);
}

unsigned int ROWSEEKPCR(USI i, MPMATPCR *Mptr)
/*
 * returns the least integer l > i such that Mptr->V[i][l] is not divisible
 * by Mptr->V[i][i], otherwise returns Mptr->C.
 */
{
	unsigned int l;
	POLYCR P;

	for (l = i + 1; l <= Mptr->C - 1; l++)
	{
		if ((P = MODPCR(Mptr->V[i][l], Mptr->V[i][i])) != NULL)
		{
			DELETE_PCR(P);
			return (l);
		}
	}
	return (l);
}

unsigned int COLSEEKPCR(USI i, MPMATPCR *Mptr)
/*
 * returns the least integer k > i such that Mptr->V[k][i] is not divisible
 * by Mptr->V[i][i], otherwise returns Mptr->C.
 */
{
	unsigned int k;
	POLYCR P;

	for (k = i + 1; k <= Mptr->R - 1; k++)
	{
		if ((P = MODPCR(Mptr->V[k][i], Mptr->V[i][i])) != NULL)
		{
			DELETE_PCR(P);
			return (k);
		}
	}
	return (k);
}

MPMATPCR *ROWSUBPCR(USI p, USI q, POLYCR Aptr, MPMATPCR *Mptr)
/*
 * subtracts *Aptr times the p-th row of *Mptr from the q-th.
 * 0 <= p, q <= Mprt->R - 1.
 */
{
	POLYCR Y, Temp;
	unsigned int l;
	MPMATPCR *Nptr;
	Nptr = COPYMATPCR(Mptr);
	if (Aptr != NULL)
	{
		for (l = 0; l <= Nptr->C - 1; l++) 
		{
			Y = MULTPCR(Aptr, Nptr->V[p][l]);
			Temp = Nptr->V[q][l];
			Nptr->V[q][l] = SUBPCR(Nptr->V[q][l], Y);
			DELETE_PCR(Y);
			DELETE_PCR(Temp);
		}
	}
	return (Nptr);
}

MPMATPCR *COLSUBPCR(USI p, USI q, POLYCR Aptr, MPMATPCR *Mptr)
/*
 * subtracts *Aptr times the p-th column of *Mptr from the q-th.
 * 0 <= p, q <= Mprt->C - 1.
 */
{
	POLYCR Y, Temp;
	unsigned int k;
	MPMATPCR *Nptr;

	Nptr = COPYMATPCR(Mptr);
	if (Aptr != NULL)
	{
		for (k = 0; k <= Nptr->R - 1; k++) 
		{
			Y = MULTPCR(Aptr, Nptr->V[k][p]);
			Temp = Nptr->V[k][q];
			Nptr->V[k][q] = SUBPCR(Nptr->V[k][q], Y);
			DELETE_PCR(Y);
			DELETE_PCR(Temp);
		}
	}
	return (Nptr);
}

MPMATPCR *ROWSUBPCR1(USI p, USI q, POLYCR Aptr, MPMATPCR *Mptr)
/*
 * subtracts *Aptr times the p-th row of *Mptr from the q-th.
 * 0 <= p, q <= Mprt->R - 1.
 */
{
	POLYCR Y, Temp;
	unsigned int l;
 
	if (Aptr != NULL)
	{
		for (l = 0; l <= Mptr->C - 1; l++) 
		{
			if (Mptr->V[p][l] != NULL)
			{
				Y = MULTPCR(Aptr, Mptr->V[p][l]);
				Temp = Mptr->V[q][l];
				Mptr->V[q][l] = SUBPCR(Mptr->V[q][l], Y);
				DELETE_PCR(Y);
				DELETE_PCR(Temp);
			}
		}
	}
	return (Mptr);
}

MPMATPCR *COLSUBPCR1(USI p, USI q, POLYCR Aptr, MPMATPCR *Mptr)
/*
 * subtracts *Aptr times the p-th column of *Mptr from the q-th.
 * 0 <= p, q <= Mprt->C - 1.
 */
{
	POLYCR Y, Temp;
	unsigned int k;

	if (Aptr != NULL)
	{
		for (k = 0; k <= Mptr->R - 1; k++) 
		{
			if (Mptr->V[k][p] != NULL)
			{
				Y = MULTPCR(Aptr, Mptr->V[k][p]);
				Temp = Mptr->V[k][q];
				Mptr->V[k][q] = SUBPCR(Mptr->V[k][q], Y);
				DELETE_PCR(Y);
				DELETE_PCR(Temp);
			}
		}
	}
	return (Mptr);
}

MPMATPCR *ADD_MULT_ROWPCR(USI p, USI q, POLYCR Aptr, MPMATPCR *Mptr)
/*
 * adds *Aptr times the p-th row of *Mptr to the q-th.
 * 0 <= p, q <= Mprt->R - 1.
 */
{
	POLYCR Y, Tmp;
	unsigned int l;
	MPMATPCR *Nptr;

	Nptr = COPYMATPCR(Mptr);
	if (Aptr != NULL)
	{
		for (l = 0; l <= Nptr->C - 1; l++) 
		{
			Y = MULTPCR(Aptr, Nptr->V[p][l]);
			Tmp = Nptr->V[q][l];
			Nptr->V[q][l] = ADDPCR(Tmp, Y);
			DELETE_PCR(Y);
			DELETE_PCR(Tmp);
		}
	}
	return (Nptr);
}

MPMATPCR *ADD_MULT_COLPCR(USI p, USI q, POLYCR Aptr, MPMATPCR *Mptr)
/*
 * adds *Aptr times the p-th column of *Mptr to the q-th.
 * 0 <= p, q <= Mprt->R - 1.
 */
{
	POLYCR Y, Tmp;
	unsigned int l;
	MPMATPCR *Nptr;

	Nptr = COPYMATPCR(Mptr);
	if (Aptr != NULL)
	{
		for (l = 0; l <= Nptr->R - 1; l++) 
		{
			Y = MULTPCR(Aptr, Nptr->V[l][p]);
			Tmp = Nptr->V[l][q];
			Nptr->V[l][q] = ADDPCR(Tmp, Y);
			DELETE_PCR(Y);
			DELETE_PCR(Tmp);
		}
	}
	return (Nptr);
}

MPMATPCR *SWAP_ROWSPCR(USI p, USI q, MPMATPCR *Mptr)
/*
 * 0 <= p < q <= Mprt->R - 1.
 * transposes the p-th and q-th rows of *Mptr.
 */
{
	POLYCR X;
	unsigned int l;
	MPMATPCR *Nptr;

	Nptr = COPYMATPCR(Mptr);
	for (l = 0; l <= Nptr->C - 1; l++) 
	{
		X = Nptr->V[p][l];
		Nptr->V[p][l] = Nptr->V[q][l];
		Nptr->V[q][l] = X;
	}
	return (Nptr);
}

MPMATPCR *SWAP_COLSPCR(USI p, USI q, MPMATPCR *Mptr)
/*
 * 0 <= p < q <= Mprt->C - 1.
 * transposes the p-th and q-th columns of *Mptr.
 */
{
	POLYCR Temp;
	unsigned int k;
	MPMATPCR *Nptr;

	Nptr = COPYMATPCR(Mptr);
	for (k = 0; k <= Nptr->R - 1; k++)
        {
		Temp = Nptr->V[k][p];
		Nptr->V[k][p] = Nptr->V[k][q];
		Nptr->V[k][q] = Temp;
	}
	return (Nptr);
}

MPMATPCR *SWAP_ROWSPCR1(USI p, USI q, MPMATPCR *Mptr)
/*
 * 0 <= p < q <= Mprt->R - 1.
 * transposes the p-th and q-th rows of *Mptr.
 */
{
	POLYCR X;
	unsigned int l;

	for (l = 0; l <= Mptr->C - 1; l++) 
	{
		X = Mptr->V[p][l];
		Mptr->V[p][l] = Mptr->V[q][l];
		Mptr->V[q][l] = X;
	}
	return (Mptr);
}

MPMATPCR *SWAP_COLSPCR1(USI p, USI q, MPMATPCR *Mptr)
/*
 * 0 <= p < q <= Mprt->C - 1.
 * transposes the p-th and q-th columns of *Mptr.
 */
{
	POLYCR Temp;
	unsigned int k;

	for (k = 0; k <= Mptr->R - 1; k++)
        {
		Temp = Mptr->V[k][p];
		Mptr->V[k][p] = Mptr->V[k][q];
		Mptr->V[k][q] = Temp;
	}
	return (Mptr);
}

MPMATPCR *DIAGXPCR(MPMATCR *Mptr)
/*
 * returns the polynomial matrix xI - *Mptr.
 */
{
	unsigned int i, j, m;
	MPMATPCR *Nptr;
	MPCR *G;

	m = Mptr->R;
	Nptr = BUILDMATPCR(m, m);
	for (i = 0; i <= m - 1; i++)
	{
		for (j = 0; j <= m - 1; j++)
		{
			if (i == j)
			{
				G = MINUSCR(elt(Mptr, i, i));
				Nptr->V[i][i] = LINEARPCR(G);
			}
			else
			{
				G = MINUSCR(elt(Mptr, i, j));
				Nptr->V[i][j] = CONSTANTPCR(G);
			}
			FREEMPCR(G);
		}
	}
	return (Nptr);
}

MPMATPCR *IDENTITYPCR(USI n)
/*
 * returns the identity polynomial matrix of size n.
 */
{
	unsigned int i, j;
	MPMATPCR *Mptr;
	MPCR *TempCR;

	Mptr = BUILDMATPCR(n, n);
	for (i = 0; i <= n - 1; i++)
	{
		for (j = 0; j <= n - 1; j++)
		{
			if ( i == j)
			{
				TempCR = ONECR();
				Mptr->V[i][i] =  CONSTANTPCR(TempCR);
				FREEMPCR(TempCR);
			}
			else
				Mptr->V[i][j] =  NULL;
		}
	}
	return (Mptr);
}

MPMATPCR *ZEROMNPCR(USI m, USI n)
/*
 * returns the zero polynomial matrix of size m x n.
 */
{
	unsigned int i, j;
	MPMATPCR *Mptr;

	Mptr = BUILDMATPCR(m, n);
	for (i = 0; i <= m - 1; i++)
	{
		for (j = 0; j <= n - 1; j++)
			Mptr->V[i][i] =  NULL;
	}
	return (Mptr);
}

MPMATPCR *SCALAR_ROWPCR(USI p, MPCR *Aptr, MPMATPCR *Mptr)
/*
 * multiplies row p by the scalar *Aptr.
 */
{
	unsigned int l;
	MPMATPCR *Nptr;
	POLYCR Tmp;

	Nptr = COPYMATPCR(Mptr);
	for (l = 0; l <= Nptr->C - 1; l++)
	{
		Tmp = Nptr->V[p][l];
		Nptr->V[p][l] = SCALARPCR(Aptr, Tmp);
		DELETE_PCR(Tmp);
	}
	return (Nptr);
}

MPMATPCR *SCALAR_COLPCR(USI p, MPCR *Aptr, MPMATPCR *Mptr)
/*
 * multiplies column p by the scalar *Aptr.
 */
{
	unsigned int l;
	MPMATPCR *Nptr;
	POLYCR Tmp;

	Nptr = COPYMATPCR(Mptr);
	for (l = 0; l <= Nptr->R - 1; l++)
	{
		Tmp = Nptr->V[l][p];
		Nptr->V[l][p] = SCALARPCR(Aptr, Tmp);
		DELETE_PCR(Tmp);
	}
	return (Nptr);
}

POLYCR INPUT1PCR(FILE *infile)
/*
 * Inputs a polynomial from a file created by FPRINT1PCR below.
 */
{
	unsigned g, i, u;
	char *line, *p;
	POLYCR Nptr;
	MPI *G;
	MPCR *TempCR;

	fscanf(infile, "%u", &g);
	p = line = Fgets(infile);
	Nptr = NULL;
	if (g)
	{
		for (i = 0; i <= g - 1; i++)
		{
			G = INPUTSI(&p, &u);
		        TempCR = INPUTCR(&p, &u); 
			PINSERTPCR((int)(G->V[0]), TempCR, &Nptr, 0);
			FREEMPI(G);
			FREEMPCR(TempCR);
		}
	}
	free(line);
	return Nptr;
}

void FPRINT1PCR(FILE *outfile, POLYCR Pptr)
{
	unsigned int l;
	POLYCR CURSOR;

	l = CARDPCR(Pptr);
	fprintf(outfile, "%u ", l);
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		fprintf(outfile, "%u ", CURSOR->DEG);
		FPRINTCR(outfile, CURSOR->COEF);
		fprintf(outfile, " ");
		CURSOR = CURSOR->NEXT;
	}
}

unsigned int LENGTHPCR(POLYCR Pptr)
{
	POLYCR CURSOR;
	unsigned int d = 0;
	int k, l;

	if (Pptr == NULL)
	{
		d++;
		return (d);
	}
	else
	{
		CURSOR = Pptr;
		k = DEGREEPCR(Pptr);
		while (CURSOR != NULL)
		{
		if ((((CURSOR->COEF)->R)->N)->S && (((CURSOR->COEF)->I)->N)->S)
		{
			if (CURSOR->DEG != k)
				d++;
			d++;
			d += LENGTHCR(CURSOR->COEF);
			d++;
		}
		if ((((CURSOR->COEF)->R)->N)->S && (((CURSOR->COEF)->I)->N)->S == 0)
		{
			l = (((CURSOR->COEF)->R)->N)->S;
			if (l == -1)
			{
				(((CURSOR->COEF)->R)->N)->S = 1;
				d += 2;
			}
			else
			{
				if (CURSOR->DEG != k)
					d += 2;
			}
			if (CURSOR->DEG == 0 ||(!EQONER((CURSOR->COEF)->R)))
				d += LENGTHR((CURSOR->COEF)->R);
			if (l == -1)
				(((CURSOR->COEF)->R)->N)->S = -1;
		}
		if ((((CURSOR->COEF)->R)->N)->S == 0 && (((CURSOR->COEF)->I)->N)->S)
		{
			l = (((CURSOR->COEF)->I)->N)->S;
			if (l == -1)
			{
				(((CURSOR->COEF)->I)->N)->S = 1;
				d += 2;
			}
			else
			{
				if (CURSOR->DEG != k)
					d += 2;
			}
			if (CURSOR->DEG == 0 ||(!EQONER((CURSOR->COEF)->I)))
				d += LENGTHCR(CURSOR->COEF);
			if (EQONER((CURSOR->COEF)->I) && CURSOR->DEG)
				d++;
			if (l == -1)
				(((CURSOR->COEF)->I)->N)->S = -1;
		}
			if (CURSOR->DEG == 1)
				d++;
			if (CURSOR->DEG > 1)
				d += 2 + CURSOR->DEG;
			if (CURSOR->NEXT != NULL)
				d++;
			CURSOR = CURSOR->NEXT;
		}
		return (d);
	}
}

void FPRINTPCR(FILE *outfile, POLYCR Pptr)
{
	POLYCR CURSOR;
	int k, l;

	if (Pptr == NULL)
	{
		fprintf(outfile , "0");
		return;
	}
	else
	{
		CURSOR = Pptr;
		k = DEGREEPCR(Pptr);
		while (CURSOR != NULL)
		{
			if ((((CURSOR->COEF)->R)->N)->S && (((CURSOR->COEF)->I)->N)->S)
			{
				if (CURSOR->DEG != k)
					fprintf(outfile, "+ ");
				fprintf(outfile, "(");
				FPRINTCR(outfile, CURSOR->COEF);
				fprintf(outfile, ")");
			}
			if ((((CURSOR->COEF)->R)->N)->S && (((CURSOR->COEF)->I)->N)->S == 0)
			{
				l = (((CURSOR->COEF)->R)->N)->S;
				if (l == -1)
				{
					(((CURSOR->COEF)->R)->N)->S = 1;
					fprintf(outfile, "- ");
				}
				else
				{
					if (CURSOR->DEG != k)
						fprintf(outfile, "+ ");
				}
				if (CURSOR->DEG == 0 ||(!EQONER((CURSOR->COEF)->R)))
					FPRINTR(outfile, (CURSOR->COEF)->R);
				if (l == -1)
					(((CURSOR->COEF)->R)->N)->S = -1;
			}
			if ((((CURSOR->COEF)->R)->N)->S == 0 && (((CURSOR->COEF)->I)->N)->S)
			{
				l = (((CURSOR->COEF)->I)->N)->S;
				if (l == -1)
				{
					(((CURSOR->COEF)->I)->N)->S = 1;
					fprintf(outfile, "- ");
				}
				else
				{
					if (CURSOR->DEG != k)
						fprintf(outfile, "+ ");
				}
				if (CURSOR->DEG == 0 ||(!EQONER((CURSOR->COEF)->I)))
					FPRINTCR(outfile, CURSOR->COEF);
				if (EQONER((CURSOR->COEF)->I) && CURSOR->DEG)
					fprintf(outfile, "i");
				if (l == -1)
					(((CURSOR->COEF)->I)->N)->S = -1;
			}
			if (CURSOR->DEG == 1)
				fprintf(outfile, "x");
			if (CURSOR->DEG > 1)
				fprintf(outfile, "x^%d", CURSOR->DEG);
			if (CURSOR->NEXT != NULL)
				fprintf(outfile, " ");
			CURSOR = CURSOR->NEXT;
		}
		return;
	}
}

void SPRINTPCR(char *buffer, POLYCR Pptr)
{
	POLYCR CURSOR;
	int k, l;

	if (Pptr == NULL)
	{
		sprintf(buffer, "0");
		return;
	}
	else
	{
		CURSOR = Pptr;
		k = DEGREEPCR(Pptr);
		while (CURSOR != NULL)
		{
			if ((((CURSOR->COEF)->R)->N)->S && (((CURSOR->COEF)->I)->N)->S)
			{
				if (CURSOR->DEG != k)
				{
					sprintf(buffer, "+ ");
					buffer += 2;
				}
				sprintf(buffer, "(");
				buffer++;
				SPRINTCR(buffer, CURSOR->COEF);
				buffer += strlen(buffer);
				sprintf(buffer, ")");
				buffer++;
			}
			if ((((CURSOR->COEF)->R)->N)->S && (((CURSOR->COEF)->I)->N)->S == 0)
			{
				l = (((CURSOR->COEF)->R)->N)->S;
				if (l == -1)
				{
					(((CURSOR->COEF)->R)->N)->S = 1;
					sprintf(buffer, "- ");
					buffer += 2;
				}
				else
				{
					if (CURSOR->DEG != k)
					{
						sprintf(buffer, "+ ");
						buffer += 2;
					}
				}
				if (CURSOR->DEG == 0 ||(!EQONER((CURSOR->COEF)->R)))
				{
					SPRINTR(buffer, (CURSOR->COEF)->R);
					buffer += strlen(buffer);
				}
				if (l == -1)
					(((CURSOR->COEF)->R)->N)->S = -1;
			}
			if ((((CURSOR->COEF)->R)->N)->S == 0 && (((CURSOR->COEF)->I)->N)->S)
			{
				l = (((CURSOR->COEF)->I)->N)->S;
				if (l == -1)
				{
					(((CURSOR->COEF)->I)->N)->S = 1;
					sprintf(buffer, "- ");
					buffer += 2;
				}
				else
				{
					if (CURSOR->DEG != k)
					{
						sprintf(buffer, "+ ");
						buffer += 2;
					}
				}
				if (CURSOR->DEG == 0 ||(!EQONER((CURSOR->COEF)->I)))
				{
					SPRINTCR(buffer, CURSOR->COEF);
					buffer += strlen(buffer);
				}
				if (EQONER((CURSOR->COEF)->I) && CURSOR->DEG)
				{
					sprintf(buffer, "i");
					buffer++;
				}
				if (l == -1)
					(((CURSOR->COEF)->I)->N)->S = -1;
			}
			if (CURSOR->DEG == 1)
			{
				sprintf(buffer, "x");
				buffer++;
			}
			if (CURSOR->DEG > 1)
			{
				sprintf(buffer, "x^%d", CURSOR->DEG);
				buffer += strlen(buffer);
			}
			if (CURSOR->NEXT != NULL)
			{
				sprintf(buffer, " ");
				buffer++;
			}
			CURSOR = CURSOR->NEXT;
		}
		return;
	}
}

void FPRINTMAT1PCR(FILE *outfile, USI i1, USI i2, USI j1, USI j2, MPMATPCR *Mptr)
{
	unsigned i, j;
	
	fprintf(outfile, "%u %u\n", i2 - i1 + 1, j2 - j1 + 1);
	for (i = i1; i <= i2; i++)
		for (j = j1; j <= j2; j++)
		{
			FPRINT1PCR(outfile, Mptr->V[i][j]);
			fputc('\n', outfile);
		}
}

void FPRINTMATPCR(FILE *outfile, USI i1, USI i2, USI j1, USI j2, MPMATPCR *Mptr)
/*
 * Modified 8/1/93 using Peter Adams' improvement from, August 1992.
 */
{
	char **str, *buff;
	unsigned int tmp, i, j, nrow, ncol, len, nstr = 0;
	int *colwidth;
	unsigned int ct;
	
	nrow = i2 - i1 + 1;
	ncol = j2 - j1 + 1;
	str = (char **)mmalloc(nrow * ncol * sizeof(char *));
	colwidth=(int *)mmalloc(ncol*sizeof(int));

	for(i=0; i<ncol; i++)
	  colwidth[i]=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
			tmp = 1 + LENGTHPCR(Mptr->V[i][j]);
			buff = (char *)mmalloc(tmp * sizeof(char));
			SPRINTPCR(buff, Mptr->V[i][j]);
		    	if ((len = strlen(buff)) > colwidth[j-j1])
				colwidth[j-j1] = len;
			str[nstr++] = strcpy((char *)mmalloc(len + 1), buff);
			ffree(buff, tmp * sizeof(char));
		}
	}
	if (outfile != stdout)
		fprintf(outfile, "%u %u\n", nrow, ncol);
	ct=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
	    		fprintf(outfile, "%*s ", colwidth[j-j1], str[ct]);
	    		ffree(str[ct], (unsigned int) (strlen(str[ct]) + 1) * sizeof(char));
	    		if ((ct % ncol) == (ncol - 1))
				fprintf(outfile, "\n");
	    		ct++;
		}
	}
	ffree((char *) str, nrow * ncol * sizeof(char *));
	ffree((char *) colwidth, ncol*sizeof(int));
	return;
}

void PRINTMATPCR(USI i1, USI i2, USI j1, USI j2, MPMATPCR *Mptr)
{
	unsigned int i, j;

	i = i2 - i1 + 1;
	j = j2 - j1 + 1;
	if (MAX((int)i, (int)j) > 20)
	{
		printf("(The number of rows or columns to be printed exceeds 20;\n");
		printf("there is no point in printing this matrix on the screen.)\n");
		return;
	}
	FPRINTMATPCR(stdout, i1, i2, j1, j2, Mptr);
}

MPMATPCR *FINPUTMATPCR(FILE *infile)
{
	unsigned m, n, i, j;
	MPMATPCR *Temp;

	fscanf(infile, "%u%u\n", &m, &n);
	Temp = BUILDMATPCR(m, n);
	for (i = 0; i < m; i++)
		for (j = 0; j < n; j++)
			Temp->V[i][j] = INPUT1PCR(infile);
	return Temp;
}
