/* p2R.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "integer.h"
#include "fun.h"

void ZEROTESTPR(USI i, MPMATPR *Mptr, USI *p, USI *q)
/*
 * returns the least indices *p >= i, *q >= i such that Mptr->V[*p][*q] != 0.
 * otherwise *p = Mptr->R and *q = Mptr->C.
 */
{
	unsigned int k, l;
	
	
	for (k = i; k <= Mptr->R - 1; k++)
	{
		for (l = i; l <= Mptr->C - 1; l++)
		{
			if (Mptr->V[k][l] != NULL)
			{
				*p = k;
				*q = l;
				return;
			}
		}
	}
	*p = Mptr->R;
	*q = Mptr->C;
	return;
}

unsigned int MATSEEKPR(USI i, MPMATPR *Mptr)
/*
 * returns the least row index k, i < k <= Mptr->R - 1, such that 
 * Mptr->V[k][l] is not divisible by Mptr->V[i][i], for some l,
 * i < l <= Mptr->C - 1; otherwise returns Mptr->R.
 */
{
	unsigned int k, l;
	POLYR P;
	
	for (k = i + 1; k <= Mptr->R - 1; k++)
	{
		for (l = i + 1; l <= Mptr->C - 1; l++)
		{
			if ((P = MODPR(Mptr->V[k][l], Mptr->V[i][i])) != NULL)
			{
				DELETE_PR(P);
				return (k);
			}
		}
	}
	return (k);
}

unsigned int ROWSEEKPR(USI i, MPMATPR *Mptr)
/*
 * returns the least integer l > i such that Mptr->V[i][l] is not divisible
 * by Mptr->V[i][i], otherwise returns Mptr->C.
 */
{
	unsigned int l;
	POLYR P;

	for (l = i + 1; l <= Mptr->C - 1; l++)
	{
		if ((P = MODPR(Mptr->V[i][l], Mptr->V[i][i])) != NULL)
		{
			DELETE_PR(P);
			return (l);
		}
	}
	return (l);
}

unsigned int COLSEEKPR(USI i, MPMATPR *Mptr)
/*
 * returns the least integer k > i such that Mptr->V[k][i] is not divisible
 * by Mptr->V[i][i], otherwise returns Mptr->C.
 */
{
	unsigned int k;
	POLYR P;

	for (k = i + 1; k <= Mptr->R - 1; k++)
	{
		if ((P = MODPR(Mptr->V[k][i], Mptr->V[i][i])) != NULL)
		{
			DELETE_PR(P);
			return (k);
		}
	}
	return (k);
}

MPMATPR *ROWSUBPR(USI p, USI q, POLYR Aptr, MPMATPR *Mptr)
/*
 * subtracts *Aptr times the p-th row of *Mptr from the q-th.
 * 0 <= p, q <= Mprt->R - 1.
 */
{
	POLYR Y, Temp;
	unsigned int l;
	MPMATPR *Nptr;
	Nptr = COPYMATPR(Mptr);
	if (Aptr != NULL)
	{
		for (l = 0; l <= Nptr->C - 1; l++) 
		{
			Y = MULTPR(Aptr, Nptr->V[p][l]);
			Temp = Nptr->V[q][l];
			Nptr->V[q][l] = SUBPR(Nptr->V[q][l], Y);
			DELETE_PR(Y);
			DELETE_PR(Temp);
		}
	}
	return (Nptr);
}

MPMATPR *COLSUBPR(USI p, USI q, POLYR Aptr, MPMATPR *Mptr)
/*
 * subtracts *Aptr times the p-th column of *Mptr from the q-th.
 * 0 <= p, q <= Mprt->C - 1.
 */
{
	POLYR Y, Temp;
	unsigned int k;
	MPMATPR *Nptr;

	Nptr = COPYMATPR(Mptr);
	if (Aptr != NULL)
	{
		for (k = 0; k <= Nptr->R - 1; k++) 
		{
			Y = MULTPR(Aptr, Nptr->V[k][p]);
			Temp = Nptr->V[k][q];
			Nptr->V[k][q] = SUBPR(Nptr->V[k][q], Y);
			DELETE_PR(Y);
			DELETE_PR(Temp);
		}
	}
	return (Nptr);
}

MPMATPR *ROWSUBPR1(USI p, USI q, POLYR Aptr, MPMATPR *Mptr)
/*
 * subtracts *Aptr times the p-th row of *Mptr from the q-th.
 * 0 <= p, q <= Mprt->R - 1.
 */
{
	POLYR Y, Temp;
	unsigned int l;
 
	if (Aptr != NULL)
	{
		for (l = 0; l <= Mptr->C - 1; l++) 
		{
			if (Mptr->V[p][l] != NULL)
			{
				Y = MULTPR(Aptr, Mptr->V[p][l]);
				Temp = Mptr->V[q][l];
				Mptr->V[q][l] = SUBPR(Mptr->V[q][l], Y);
				DELETE_PR(Y);
				DELETE_PR(Temp);
			}
		}
	}
	return (Mptr);
}

MPMATPR *COLSUBPR1(USI p, USI q, POLYR Aptr, MPMATPR *Mptr)
/*
 * subtracts *Aptr times the p-th column of *Mptr from the q-th.
 * 0 <= p, q <= Mprt->C - 1.
 */
{
	POLYR Y, Temp;
	unsigned int k;

	if (Aptr != NULL)
	{
		for (k = 0; k <= Mptr->R - 1; k++) 
		{
			if (Mptr->V[k][p] != NULL)
			{
				Y = MULTPR(Aptr, Mptr->V[k][p]);
				Temp = Mptr->V[k][q];
				Mptr->V[k][q] = SUBPR(Mptr->V[k][q], Y);
				DELETE_PR(Y);
				DELETE_PR(Temp);
			}
		}
	}
	return (Mptr);
}

MPMATPR *ADD_MULT_ROWPR(USI p, USI q, POLYR Aptr, MPMATPR *Mptr)
/*
 * adds *Aptr times the p-th row of *Mptr to the q-th.
 * 0 <= p, q <= Mprt->R - 1.
 */
{
	POLYR Y, Tmp;
	unsigned int l;
	MPMATPR *Nptr;

	Nptr = COPYMATPR(Mptr);
	if (Aptr != NULL)
	{
		for (l = 0; l <= Nptr->C - 1; l++) 
		{
			Y = MULTPR(Aptr, Nptr->V[p][l]);
			Tmp = Nptr->V[q][l];
			Nptr->V[q][l] = ADDPR(Tmp, Y);
			DELETE_PR(Y);
			DELETE_PR(Tmp);
		}
	}
	return (Nptr);
}

MPMATPR *ADD_MULT_COLPR(USI p, USI q, POLYR Aptr, MPMATPR *Mptr)
/*
 * adds *Aptr times the p-th column of *Mptr to the q-th.
 * 0 <= p, q <= Mprt->R - 1.
 */
{
	POLYR Y, Tmp;
	unsigned int l;
	MPMATPR *Nptr;

	Nptr = COPYMATPR(Mptr);
	if (Aptr != NULL)
	{
		for (l = 0; l <= Nptr->R - 1; l++) 
		{
			Y = MULTPR(Aptr, Nptr->V[l][p]);
			Tmp = Nptr->V[l][q];
			Nptr->V[l][q] = ADDPR(Tmp, Y);
			DELETE_PR(Y);
			DELETE_PR(Tmp);
		}
	}
	return (Nptr);
}

MPMATPR *SWAP_ROWSPR(USI p, USI q, MPMATPR *Mptr)
/*
 * 0 <= p < q <= Mprt->R - 1.
 * transposes the p-th and q-th rows of *Mptr.
 */
{
	POLYR X;
	unsigned int l;
	MPMATPR *Nptr;

	Nptr = COPYMATPR(Mptr);
	for (l = 0; l <= Nptr->C - 1; l++) 
	{
		X = Nptr->V[p][l];
		Nptr->V[p][l] = Nptr->V[q][l];
		Nptr->V[q][l] = X;
	}
	return (Nptr);
}

MPMATPR *SWAP_COLSPR(USI p, USI q, MPMATPR *Mptr)
/*
 * 0 <= p < q <= Mprt->C - 1.
 * transposes the p-th and q-th columns of *Mptr.
 */
{
	POLYR Temp;
	unsigned int k;
	MPMATPR *Nptr;

	Nptr = COPYMATPR(Mptr);
	for (k = 0; k <= Nptr->R - 1; k++)
        {
		Temp = Nptr->V[k][p];
		Nptr->V[k][p] = Nptr->V[k][q];
		Nptr->V[k][q] = Temp;
	}
	return (Nptr);
}

MPMATPR *SWAP_ROWSPR1(USI p, USI q, MPMATPR *Mptr)
/*
 * 0 <= p < q <= Mprt->R - 1.
 * transposes the p-th and q-th rows of *Mptr.
 */
{
	POLYR X;
	unsigned int l;

	for (l = 0; l <= Mptr->C - 1; l++) 
	{
		X = Mptr->V[p][l];
		Mptr->V[p][l] = Mptr->V[q][l];
		Mptr->V[q][l] = X;
	}
	return (Mptr);
}

MPMATPR *SWAP_COLSPR1(USI p, USI q, MPMATPR *Mptr)
/*
 * 0 <= p < q <= Mprt->C - 1.
 * transposes the p-th and q-th columns of *Mptr.
 */
{
	POLYR Temp;
	unsigned int k;

	for (k = 0; k <= Mptr->R - 1; k++)
        {
		Temp = Mptr->V[k][p];
		Mptr->V[k][p] = Mptr->V[k][q];
		Mptr->V[k][q] = Temp;
	}
	return (Mptr);
}

MPMATPR *DIAGXPR(MPMATR *Mptr)
/*
 * returns the polynomial matrix xI - *Mptr.
 */
{
	unsigned int i, j, m;
	MPMATPR *Nptr;
	MPR *G;

	m = Mptr->R;
	Nptr = BUILDMATPR(m, m);
	for (i = 0; i <= m - 1; i++)
	{
		for (j = 0; j <= m - 1; j++)
		{
			if (i == j)
			{
				G = MINUSR(elt(Mptr, i, i));
				Nptr->V[i][i] = LINEARPR(G);
			}
			else
			{
				G = MINUSR(elt(Mptr, i, j));
				Nptr->V[i][j] = CONSTANTPR(G);
			}
			FREEMPR(G);
		}
	}
	return (Nptr);
}

MPMATPR *IDENTITYPR(USI n)
/*
 * returns the identity polynomial matrix of size n.
 */
{
	unsigned int i, j;
	MPMATPR *Mptr;
	MPR *TempR;

	Mptr = BUILDMATPR(n, n);
	for (i = 0; i <= n - 1; i++)
	{
		for (j = 0; j <= n - 1; j++)
		{
			if ( i == j)
			{
				TempR = ONER();
				Mptr->V[i][i] =  CONSTANTPR(TempR);
				FREEMPR(TempR);
			}
			else
				Mptr->V[i][j] =  NULL;
		}
	}
	return (Mptr);
}

MPMATPR *ZEROMNPR(USI m, USI n)
/*
 * returns the zero polynomial matrix of size m x n.
 */
{
	unsigned int i, j;
	MPMATPR *Mptr;

	Mptr = BUILDMATPR(m, n);
	for (i = 0; i <= m - 1; i++)
	{
		for (j = 0; j <= n - 1; j++)
			Mptr->V[i][i] =  NULL;
	}
	return (Mptr);
}

MPMATPR *SCALAR_ROWPR(USI p, MPR *Aptr, MPMATPR *Mptr)
/*
 * multiplies row p by the scalar *Aptr.
 */
{
	unsigned int l;
	MPMATPR *Nptr;
	POLYR Tmp;

	Nptr = COPYMATPR(Mptr);
	for (l = 0; l <= Nptr->C - 1; l++)
	{
		Tmp = Nptr->V[p][l];
		Nptr->V[p][l] = SCALARPR(Aptr, Tmp);
		DELETE_PR(Tmp);
	}
	return (Nptr);
}

MPMATPR *SCALAR_COLPR(USI p, MPR *Aptr, MPMATPR *Mptr)
/*
 * multiplies column p by the scalar *Aptr.
 */
{
	unsigned int l;
	MPMATPR *Nptr;
	POLYR Tmp;

	Nptr = COPYMATPR(Mptr);
	for (l = 0; l <= Nptr->R - 1; l++)
	{
		Tmp = Nptr->V[l][p];
		Nptr->V[l][p] = SCALARPR(Aptr, Tmp);
		DELETE_PR(Tmp);
	}
	return (Nptr);
}

POLYR INPUT1PR(FILE *infile)
/*
 * Inputs a polynomial from a file created by FPRINT1PR below.
 */
{
	unsigned NoTerms, i, u;
	int k;
	POLYR Temp;
	MPR *TempR;

	fscanf(infile, "%u", &NoTerms);
	Temp = NULL;
	if (NoTerms)
	{
		for (i = 0; i < NoTerms; i++)
		{
			fscanf(infile, "%d", &k);
			TempR = FINPUTR(infile, &u);
			PINSERTPR(k, TempR, &Temp, 0);
			FREEMPR(TempR);
		}
	}
	return Temp;
}

void FPRINT1PR(FILE *outfile, POLYR Pptr)
{
	unsigned int l;
	POLYR CURSOR;

	l = CARDPR(Pptr);
	fprintf(outfile, "%u ", l);
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		fprintf(outfile, "%u ", CURSOR->DEG);
		FPRINTR(outfile, CURSOR->COEF);
		fprintf(outfile, " ");
		CURSOR = CURSOR->NEXT;
	}
}

unsigned int LENGTHPR(POLYR Pptr)
{
	POLYR CURSOR;
	unsigned int d = 0;
	int k, l;

	if (Pptr == NULL)
	{
		d++;
		return (d);
	}
	else
	{
		CURSOR = Pptr;
		k = DEGREEPR(Pptr);
		while (CURSOR != NULL)
		{
			l = ((CURSOR->COEF)->N)->S;
			if (l == -1)
			{
				((CURSOR->COEF)->N)->S = 1;
				d += 2;
			}
			else
			{
				if (CURSOR->DEG != k)
					d += 2;
			}
			if (CURSOR->DEG == 0 ||(!EQONER(CURSOR->COEF)))
				d += LENGTHR(CURSOR->COEF);
			if (l == -1)
				((CURSOR->COEF)->N)->S = -1;
			if (CURSOR->DEG == 1)
				d++;
			if (CURSOR->DEG > 1)
				d += 2 + CURSOR->DEG;
			if (CURSOR->NEXT != NULL)
				d++;
			CURSOR = CURSOR->NEXT;
		}
		return (d);
	}
}

void FPRINTPR(FILE *outfile, POLYR Pptr)
{
	POLYR CURSOR;
	int k, l;

	if (Pptr == NULL)
	{
		fprintf(outfile, "0");
		return;
	}
	else
	{
		CURSOR = Pptr;
		k = DEGREEPR(Pptr);
		while (CURSOR != NULL)
		{
			l = ((CURSOR->COEF)->N)->S;
			if (l == -1)
			{
				((CURSOR->COEF)->N)->S = 1;
				fprintf(outfile, "- ");
			}
			else
			{
				if (CURSOR->DEG != k)
					fprintf(outfile, "+ ");
			}
			if (CURSOR->DEG == 0 ||(!EQONER(CURSOR->COEF)))
				FPRINTR(outfile, CURSOR->COEF);
			if (l == -1)
				((CURSOR->COEF)->N)->S = -1;
			if (CURSOR->DEG == 1)
				fprintf(outfile, "x");
			if (CURSOR->DEG > 1)
				fprintf(outfile, "x^%d", CURSOR->DEG);
			if (CURSOR->NEXT != NULL)
				fprintf(outfile, " ");
			CURSOR = CURSOR->NEXT;
		}
		return;
	}
}

void SPRINTPR(char *buffer, POLYR Pptr)
{
	POLYR CURSOR;
	int k, l;

	if (Pptr == NULL)
	{
		sprintf(buffer, "0");
		return;
	}
	else
	{
		CURSOR = Pptr;
		k = DEGREEPR(Pptr);
		while (CURSOR != NULL)
		{
			l = ((CURSOR->COEF)->N)->S;
			if (l == -1)
			{
				((CURSOR->COEF)->N)->S = 1;
				sprintf(buffer, "- ");
				buffer += 2;
			}
			else
			{
				if (CURSOR->DEG != k)
				{
					sprintf(buffer, "+ ");
					buffer += 2;
				}
			}
			if (CURSOR->DEG == 0 ||(!EQONER(CURSOR->COEF)))
			{
				SPRINTR(buffer, CURSOR->COEF);
				buffer += strlen(buffer);
			}
			if (l == -1)
				((CURSOR->COEF)->N)->S = -1;
			if (CURSOR->DEG == 1)
			{
				sprintf(buffer, "x");
				buffer++;
			}
			if (CURSOR->DEG > 1)
			{
				sprintf(buffer, "x^%d", CURSOR->DEG);
				buffer += strlen(buffer);
			}
			if (CURSOR->NEXT != NULL)
			{
				sprintf(buffer, " ");
				buffer++;
			}
			CURSOR = CURSOR->NEXT;
		}
		return;
	}
}

void FPRINTMATPR(FILE *outfile, USI i1, USI i2, USI j1, USI j2, MPMATPR *Mptr)
/*
 * Modified 8/1/93 using Peter Adams' improvement from, August 1992.
 */
{
	char **str, *buff;
	unsigned int tmp, i, j, nrow, ncol, len, nstr = 0;
	int *colwidth;
	unsigned int ct;
	
	nrow = i2 - i1 + 1;
	ncol = j2 - j1 + 1;
	str = (char **)mmalloc(nrow * ncol * sizeof(char *));
	colwidth=(int *)mmalloc(ncol*sizeof(int));

	for(i=0; i<ncol; i++)
	  colwidth[i]=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
			tmp = 1 + LENGTHPR(Mptr->V[i][j]);
			buff = (char *)mmalloc(tmp * sizeof(char));
			SPRINTPR(buff, Mptr->V[i][j]);
		    	if ((len = strlen(buff)) > colwidth[j-j1])
				colwidth[j-j1] = len;
			str[nstr++] = strcpy((char *)mmalloc(len + 1), buff);
			ffree(buff, tmp * sizeof(char));
		}
	}
	if (outfile != stdout)
		fprintf(outfile, "%u %u\n", nrow, ncol);
	ct=0;
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
	    		fprintf(outfile, "%*s ", colwidth[j-j1], str[ct]);
	    		ffree(str[ct], (unsigned int) (strlen(str[ct]) + 1) * sizeof(char));
	    		if ((ct % ncol) == (ncol - 1))
				fprintf(outfile, "\n");
	    		ct++;
		}
	}
	ffree((char *) str, nrow * ncol * sizeof(char *));
	ffree((char *) colwidth, ncol*sizeof(int));
	return;
}

void PRINTMATPR(USI i1, USI i2, USI j1, USI j2, MPMATPR *Mptr)
{
	unsigned int i, j;

	i = i2 - i1 + 1;
	j = j2 - j1 + 1;
	if (MAX((int)i, (int)j) > 20)
	{
		printf("(The number of rows or columns to be printed exceeds 20;\n");
		printf("there is no point in printing this matrix on the screen.)\n");
		return;
	}
	FPRINTMATPR(stdout, i1, i2, j1, j2, Mptr);
}

void FPRINTMAT1PR(FILE *outfile, USI i1, USI i2, USI j1, USI j2, MPMATPR *Mptr)
{
	unsigned i, j;

	fprintf(outfile, "%u %u\n", i2 - i1 + 1, j2 - j1 + 1);
	for (i = i1; i <= i2; i++)
	{
		for (j = j1; j <= j2; j++)
		{
			FPRINT1PR(outfile, Mptr->V[i][j]);
			fputc('\n', outfile);
		}
	}
}

MPMATPR *FINPUTMATPR(FILE *infile)
{
	unsigned i, j, m, n;
	MPMATPR *Temp;

	fscanf(infile, "%u%u\n", &m, &n);
	Temp = BUILDMATPR(m, n);
	for (i = 0; i < m; i++)
		for (j = 0; j < n; j++)
			Temp->V[i][j] = INPUT1PR(infile);
	return Temp;
}
