/* p3CR.c */

#include <stdio.h>
#include <stdlib.h>
#include "integer.h"
#include "fun.h"

POLYCR EUCLIDPCR(POLYCR Pptr, POLYCR Qptr, POLYCR *Hptr, POLYCR *Kptr)
/*
 * gcd(Pptr, Qptr) = Hptr * Pptr + Kptr * Qptr.
 */
{
	POLYCR Q, A, B, C, H1, H2, K1, K2, L1, L2, Tmp, Tmp1, Tmp2;
	MPCR *G, *TempCR;

	if (Qptr == NULL)
	{
		if (Pptr != NULL)
		{
			G = LEADPCR(Pptr);
			TempCR = G;
			G = INVERSECR(G);
			FREEMPCR(TempCR);
			*Hptr = CONSTANTPCR(G);
			*Kptr = NULL;
			FREEMPCR(G);
			return (MONICPCR(Pptr));
		}
		else
		{
			*Hptr = NULL;
			*Kptr = NULL;
			return (NULL);
		}
	}
	A = COPYPCR(Pptr);
	B = COPYPCR(Qptr);
	C = MODPCR(A, B);
	if (C == NULL)
	{
		*Hptr = NULL;
		G = LEADPCR(Qptr);
		TempCR = G;
		G = INVERSECR(G);
		FREEMPCR(TempCR);
		*Kptr = CONSTANTPCR(G);
		DELETE_PCR(A);
		DELETE_PCR(B);
		FREEMPCR(G);
		return (MONICPCR(Qptr));
	}
	TempCR = ONECR();
	L1 = CONSTANTPCR(TempCR);
	K1 = NULL;
	L2 = NULL;
	K2 = CONSTANTPCR(TempCR);
	FREEMPCR(TempCR);
	while (C != NULL)
	{
		Q = DIVPCR(A, B);
		DELETE_PCR(A);
		A = B;
		B = C;
		C = MODPCR(A, B);
		Tmp1 = MULTPCR(Q, K1);
		Tmp2 = MULTPCR(Q, K2);
		DELETE_PCR(Q);
		H1 = SUBPCR(L1, Tmp1);
		H2 = SUBPCR(L2, Tmp2);
		DELETE_PCR(Tmp1);
		DELETE_PCR(Tmp2);
		Tmp = L1;
		L1 = K1;
		DELETE_PCR(Tmp);
		Tmp = L2;
		L2 = K2;
		DELETE_PCR(Tmp);
		K1 = COPYPCR(H1);
		K2 = COPYPCR(H2);
		DELETE_PCR(H1);
		DELETE_PCR(H2);
	}
	G = LEADPCR(B);
	TempCR = G;
	G = INVERSECR(G);
	FREEMPCR(TempCR);
	*Hptr = SCALARPCR(G, K1);
	*Kptr = SCALARPCR(G, K2);
	FREEMPCR(G);
	DELETE_PCR(K1);
	DELETE_PCR(K2);
	DELETE_PCR(L1);
	DELETE_PCR(L2);
	DELETE_PCR(A);
	DELETE_PCR(C);
	Tmp = MONICPCR(B);
	DELETE_PCR(B);
	return (Tmp);
}

MPMATPCR *MULTMATPCR(MPMATPCR *Mptr, MPMATPCR *Nptr)
/*
 * Here Mptr->C = Nptr->R
 * returns (*Mptr) * (*Nptr)
 */
{
	POLYCR X, Y, Tmp;
	MPMATPCR *Qptr;
	unsigned int i, j, k;

	Qptr = BUILDMATPCR(Mptr->R, Nptr->C);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		for (j = 0; j <= Nptr->C - 1; j++)
		{
			X = NULL;
			for (k = 0; k <= Mptr->C - 1; k++)
			{
				Y = MULTPCR(Mptr->V[i][k], Nptr->V[k][j]);
				Tmp = X;
				X = ADDPCR(Tmp, Y);
				DELETE_PCR(Tmp);
				DELETE_PCR(Y);
			}
			Qptr->V[i][j] = X;
		}
	}
	return (Qptr);
}

POLYCR TRACEPCR(MPMATPCR *Mptr)
/*
 * returns the trace of *Mptr
 */
{
	POLYCR A, Tmp;
	unsigned int i;

	A = NULL;
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		Tmp = A;
		A = ADDPCR(Tmp, Mptr->V[i][i]);
		DELETE_PCR(Tmp);
	}
	return (A);
}

MPMATPCR *MINUSMATPCR(MPMATPCR *Mptr)
/*
 * returns -(*Mptr)
 */
{
	unsigned int i, j;
	MPMATPCR *Nptr;
	MPCR *TempCR;

	TempCR = MINUS_ONECR();
	Nptr = BUILDMATPCR(Mptr->R, Mptr->C);
	for (i = 0; i <= Nptr->R - 1; i++)
	{
		for (j = 0; j <= Nptr->C - 1; j++)
			Nptr->V[i][j] = SCALARPCR(TempCR, Mptr->V[i][j]);
	}
	FREEMPCR(TempCR);
	return (Nptr);
}

MPMATPCR *DIAGPCR(MPMATPCR *Mptr, POLYCR Aptr)
/*
 * returns *Mptr - Aptr * (Unit Matrix)
 */
{
	unsigned int i;
	POLYCR Tmp;
	MPMATPCR *Nptr;

	Nptr = COPYMATPCR(Mptr);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		Tmp = Nptr->V[i][i];
		Nptr->V[i][i] = SUBPCR(Tmp, Aptr);
		DELETE_PCR(Tmp);
	}
	return (Nptr);
}

POLYCR LEVERRIER_PCR(MPMATPCR *Mptr, MPMATPCR **Nptr)
/*
 * The Faddeeva/Leverrier method for finding the
 * adjoint and determinant,  applied to a polynomial matrix *Mptr.
 * returns the determinant while *Nptr is the adjoint.
 * See "Computational methods in linear algebra", by V. N. Faddeeva, Dover
 * 1959.
 */
{
	unsigned int i;
	MPMATPCR *Qptr, *M;
	POLYCR P, D, Tmp;
	MPI *G;
	MPCR *H, *TempCR;

	D = NULL;
	Qptr = COPYMATPCR(Mptr);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		P = TRACEPCR(Qptr);
		G = CHANGE(i + 1);
		H = BUILDMPCR();
		H->R = BUILDMPR();
		(H->R)->N = ONEI();
		(H->R)->D = G;
		H->I = ZEROR();
		Tmp = P;
		P = SCALARPCR(H, Tmp);
		FREEMPCR(H);
		DELETE_PCR(Tmp);
		M = DIAGPCR(Qptr, P);
		FREEMATPCR(Qptr);
		if (i == Mptr->R - 1)
			D = COPYPCR(P);
		DELETE_PCR(P);
		if (i == Mptr->R - 2)
		{
			if (i % 2 == 0)
			{
				*Nptr = MINUSMATPCR(M);
			}
			else
			{
				*Nptr = COPYMATPCR(M);
			}
		}  
		Qptr =	MULTMATPCR(Mptr, M);
		FREEMATPCR(M);
	}
	FREEMATPCR(Qptr);
	if (i % 2 == 0)
	{
		Tmp = D;
		TempCR = MINUS_ONECR();
		D = SCALARPCR(TempCR, Tmp);
		FREEMPCR(TempCR);
		DELETE_PCR(Tmp);
	}
	return (D);
}

POLYCR EUCLIDSPCR(POLYCR Pptr[], POLYCR *Hptr[], USI n)
/*
 * returns the gcd of polynomials Pptr[0],...,Pptr[n - 1]
 * and expresses it as Hptr[0] * Pptr[0] + ... Hptr[n - 1] * Pptr[n - 1].
 */
{
	unsigned int i;
	POLYCR B, H, K, L, Tmp;
	MPCR *TempCR;

	*Hptr = (POLYCR *)mmalloc(n * sizeof(POLYCR));
	for (i = 0; i <= n - 1; i++)
	{
		*(*Hptr + i) = COPYPCR(Pptr[i]);
	}
	for (i = 1; i <= n - 1; i++)
	{
		Tmp = *(*Hptr + i);
		*(*Hptr + i) = GCDPCR(Tmp, *(*Hptr + (i - 1)));
		DELETE_PCR(Tmp);
	}
	Tmp = *(*Hptr + (n - 1));
	B = COPYPCR(Tmp);
	DELETE_PCR(Tmp);
	*(*Hptr + (n - 1)) = COPYPCR(Pptr[n - 1]);
	TempCR = ONECR();
	L = CONSTANTPCR(TempCR);
	FREEMPCR(TempCR);
	for (i = n - 1; i >= 1; i--)
	{
		Tmp = EUCLIDPCR(*(*Hptr + i), *(*Hptr + (i - 1)), &H, &K);
		DELETE_PCR(Tmp);
		DELETE_PCR(*(*Hptr + i));
		*(*Hptr + i) = MULTPCR(H, L);
		DELETE_PCR(H);
		if (i == 1)
			break;
		Tmp = L;
		L = MULTPCR(Tmp, K);
		DELETE_PCR(K);
		DELETE_PCR(Tmp);
		DELETE_PCR(*(*Hptr + (i - 1)));
		*(*Hptr + (i - 1)) = COPYPCR(Pptr[i - 1]);
	}
	DELETE_PCR(**Hptr);
	**Hptr  = MULTPCR(L, K);
	DELETE_PCR(L);
	DELETE_PCR(K);
	return (B);
}

MPMATCR *VALMATCR(POLYCR Pptr, MPMATCR *Mptr)
/*
 * Evaluates the polynomial Pptr at the square matrix *Mptr.
 */
{
	unsigned int i, n;
	int k;
	MPMATCR *Aptr, *Temp1, *Temp, *ID;
	POLYCR CURSOR;
	MPCR *TempCR;

	n = Mptr->R;
	if (Pptr == NULL)
		return ZEROMNCR(n, n);
	else
	{
		ID = IDENTITYCR(n);
		Aptr = SCALARMATCR(ID, Pptr->COEF);
		FREEMATCR(ID);
	}
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		if (CURSOR->NEXT == NULL)
			k = DEGREEPCR(CURSOR);
		else
			k = DEGREEPCR(CURSOR) - DEGREEPCR(CURSOR->NEXT);
		Temp1 = POWERMATCR(Mptr, (unsigned int)k);
		Temp = Aptr;
		Aptr = MULTMATCR(Temp1, Temp);
		FREEMATCR(Temp);
		FREEMATCR(Temp1);
		CURSOR = CURSOR->NEXT;
		if (CURSOR != NULL)
		{
			for (i = 0; i < n; i++)
			{
				TempCR = elt(Aptr, i, i);
				elt(Aptr, i, i) = ADDCR(elt(Aptr, i, i), CURSOR->COEF);
				FREEMPCR(TempCR);
			}
		}
	}
	return Aptr;
}
