/* p3I.c */

#include <stdio.h>
#include "integer.h"
#include "fun.h"

MPMATPI *MULTMATPI(MPMATPI *Mptr, MPMATPI *Nptr)
/*
 * Here Mptr->C = Nptr->R
 * returns (*Mptr) * (*Nptr)
 */
{
	POLYI X, Y, Tmp;
	MPMATPI *Qptr;
	unsigned int i, j, k;

	Qptr = BUILDMATPI(Mptr->R, Nptr->C);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		for (j = 0; j <= Nptr->C - 1; j++)
		{
			X = NULL;
			for (k = 0; k <= Mptr->C - 1; k++)
			{
				Y = MULTPI(Mptr->V[i][k], Nptr->V[k][j]);
				Tmp = X;
				X = ADDPI(Tmp, Y);
				DELETE_PI(Tmp);
				DELETE_PI(Y);
			}
			Qptr->V[i][j] = X;
		}
	}
	return (Qptr);
}

POLYI TRACEPI(MPMATPI *Mptr)
/*
 * returns the trace of *Mptr
 */
{
	POLYI A, Tmp;
	unsigned int i;

	A = NULL;
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		Tmp = A;
		A = ADDPI(Tmp, Mptr->V[i][i]);
		DELETE_PI(Tmp);
	}
	return (A);
}

MPMATPI *MINUSMATPI(MPMATPI *Mptr)
/*
 * returns -(*Mptr)
 */
{
	unsigned int i, j;
	MPMATPI *Nptr;
	MPI *TempI;

	Nptr = BUILDMATPI(Mptr->R, Mptr->C);
	TempI = MINUS_ONEI();
	for (i = 0; i <= Nptr->R - 1; i++)
	{
		for (j = 0; j <= Nptr->C - 1; j++)
			Nptr->V[i][j] = SCALARPI(TempI, Mptr->V[i][j]);
	}
	FREEMPI(TempI);
	return (Nptr);
}

MPMATPI *DIAGPI(MPMATPI *Mptr, POLYI Aptr)
/*
 * returns *Mptr - Aptr * (Unit Matrix)
 */
{
	unsigned int i;
	POLYI Tmp;
	MPMATPI *Nptr;

	Nptr = COPYMATPI(Mptr);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		Tmp = Nptr->V[i][i];
		Nptr->V[i][i] = SUBPI(Tmp, Aptr);
		DELETE_PI(Tmp);
	}
	return (Nptr);
}

POLYI LEVERRIER_PI(MPMATPI *Mptr, MPMATPI **Nptr)
/*
 * The Faddeeva/Leverrier method for finding the
 * adjoint and determinant,  applied to a polynomial matrix *Mptr.
 * returns the determinant while *Nptr is the adjoint.
 * See "Computational methods in linear algebra", by V. N. Faddeeva, Dover
 * 1959.
 */
{
	unsigned int i;
	MPMATPI *Qptr, *M;
	POLYI P, D, Tmp, CURSOR;
	MPI *G, *TempI;

	D = NULL;
	Qptr = COPYMATPI(Mptr);
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		P = TRACEPI(Qptr);
		G = CHANGE(i + 1);
		CURSOR = P;
		while (CURSOR != NULL)
		{
			TempI = CURSOR->COEF;
			CURSOR->COEF = INT(CURSOR->COEF, G);
			FREEMPI(TempI);
			CURSOR = CURSOR->NEXT;
		}
		FREEMPI(G);
		M = DIAGPI(Qptr, P);
		FREEMATPI(Qptr);
		if (i == Mptr->R - 1)
			D = COPYPI(P);
		DELETE_PI(P);
		if (i == Mptr->R - 2)
		{
			if (i % 2 == 0)
				*Nptr = MINUSMATPI(M);
			else
				*Nptr = COPYMATPI(M);
		}  
		Qptr =	MULTMATPI(Mptr, M);
		FREEMATPI(M);
	}
	FREEMATPI(Qptr);
	if (i % 2 == 0)
	{
		TempI = MINUS_ONEI();
		Tmp = D;
		D = SCALARPI(TempI, Tmp);
		FREEMPI(TempI);
		DELETE_PI(Tmp);
	}
	return (D);
}
