/* p4CI.c */

/* matrix operations using POLYCI's */

#include <stdio.h>
#include "integer.h"
#include "fun.h"

POLYCI LEVERRIER_CI(MPMATCI *Mptr, MPMATCI **Nptr, MPCI **Dptr)
/*
 * The Faddeeva/Leverrier method for finding the characteristic polynomial,
 * adjoint and determinant of a matrix of complex integers.
 * "Computational methods in linear algebra", by V.N. Faddeeva, Dover 1959 .
 */
{
	POLYCI Pptr, CURSOR;
	unsigned int i, n;
	MPMATCI *N, *M;
	MPCI *G, *TempCI;
	MPI *TempI;

	G = NULL;
	n = Mptr->R;
	N = COPYMATCI(Mptr);
	Pptr = NULL;
	for (i = 0; i <= n - 1; i++)
	{
		G = TRACECI(N);
		TempI = G->R;
		G->R = INT_(G->R, i + 1);
		FREEMPI(TempI);
		TempI = G->I;
		G->I = INT_(G->I, i + 1);
		FREEMPI(TempI);
		PINSERTPCI((int)(n - 1 - i), G, &Pptr, 0);
		M = DIAGCI(N, G);
		if (i != n - 1)
			FREEMPCI(G);
		FREEMATCI(N);
		if (i == n - 2)
		{
			if (i % 2 == 0)
				*Nptr = MINUSMATCI(M);
			else
				*Nptr = COPYMATCI(M);
		}
		N = MULTMATCI(Mptr, M);
		FREEMATCI(M);
	}
	FREEMATCI(N);
	TempCI = ONECI();
	PINSERTPCI((int)i, TempCI, &Pptr, 0);
	FREEMPCI(TempCI);
	PURGEPCI(&Pptr);
	*Dptr = G;
	if (i % 2 == 0)
	{
		TempCI = *Dptr;
		*Dptr = MINUSCI(*Dptr);
		FREEMPCI(TempCI);
	}
	CURSOR = Pptr->NEXT;
	while (CURSOR != NULL)
	{
		TempCI = CURSOR->COEF;
		CURSOR->COEF = MINUSCI(CURSOR->COEF);
		FREEMPCI(TempCI);
		CURSOR = CURSOR->NEXT;
	}
	return (Pptr);
}
