/* p4I.c */

/* matrix operations using POLYI's */

#include <stdio.h>
#include "integer.h"
#include "fun.h"

MPMATPI *DELETE_ROWPI(USI r, MPMATPI *Mptr)
/*
 * deletes row r of *Mptr.
 */
{
	unsigned int i, j;
	MPMATPI *Nptr;

	r--;
	Nptr = BUILDMATPI(Mptr->R - 1, Mptr->C);
	for (i = 0; i < Nptr->R; i++)
	{
		if (i < r)
		{
			for (j = 0; j < Nptr->C; j++)
				Nptr->V[i][j] = COPYPI(Mptr->V[i][j]);	
		}
		else
		{
			for (j = 0; j < Nptr->C; j++)
				Nptr->V[i][j] = COPYPI(Mptr->V[i + 1][j]);	
		}
	}
	return (Nptr);
}

MPMATPI *DELETE_COLPI(USI c, MPMATPI *Mptr)
/*
 * deletes  column c of *Mptr.
 */
{
	unsigned int i, j;
	MPMATPI *Nptr;

	c--;
	Nptr = BUILDMATPI(Mptr->R, Mptr->C - 1);
	for (j = 0; j < Nptr->C; j++)
	{
		if (j < c)
		{
			for (i = 0; i < Nptr->R; i++)
				Nptr->V[i][j] = COPYPI(Mptr->V[i][j]);	
		}
		else
		{
			for (i = 0; i < Nptr->R; i++)
				Nptr->V[i][j] = COPYPI(Mptr->V[i][j + 1]);	
		}
	}
	return (Nptr);
}

MPMATI *COMPANIONI(POLYI Pptr)
/*
 * returns the companion matrix corresponding to the polynomial Pptr.
 */
{
	unsigned int j, n;
	MPMATI *Mptr;
	int i;
	POLYI CURSOR;

	n = DEGREEPI(Pptr);
	CURSOR = Pptr;
	Mptr = BUILDMATI(n, n);
	CURSOR = CURSOR->NEXT;
	for (i = n - 1; i >= 0; i--)
	{
		for (j = 0; j <= n - 1; j++)
		{
			if ( j == n - 1)
			{
				if (CURSOR != NULL)
				{
					if (CURSOR->DEG == i)
					{
						elt(Mptr, i, j) = COPYI(CURSOR->COEF);
						CURSOR = CURSOR->NEXT;
					}
					else
						elt(Mptr, i, j) = ZEROI();
				}
				else
					elt(Mptr, i, j) = ZEROI();
			}
			else if (i == j + 1)
				elt(Mptr, i, j) = ONEI();
			else
				elt(Mptr, i, j) = ZEROI();
		}
	}
	return (Mptr);
}

POLYI COLITOPOLYI(MPMATI *Mptr)
/*
 * converts a column vector (Mptr->V[0][0], ... , Mptr->V[n - 1][0]) of 
 * MPI's to a POLYI
 */
{
	unsigned int i;
	POLYI Pptr;

	Pptr = NULL;
	for (i = 0; i <= Mptr->R - 1; i++)
	{
		if ((Mptr->V[i][0]).S != 0)
			PINSERTPI((int)i, elt(Mptr, i, 0), &Pptr,  0);
	}
	return (Pptr);
}

MPMATI *POLYITOCOLI(POLYI Pptr)
/*
 * creates column matrix whose elements are the coefficients of Pptr
 */
{
	MPMATI *Nptr;
	POLYI CURSOR;
	unsigned int m, i;
	MPI *TempI;

	m = 1 + (unsigned int)DEGREEPI(Pptr);
	Nptr = BUILDMATI(m, 1);
	for (i = 0; i < m; i++)
		elt(Nptr, i, 0) = ZEROI();
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		i = (unsigned int)(CURSOR->DEG);
		TempI = elt(Nptr, i, 0);
		elt(Nptr, i, 0) = COPYI(CURSOR->COEF);
		FREEMPI(TempI);
		CURSOR = CURSOR->NEXT;
	}
	return (Nptr);
}

MPMATI *FORM_LATTICE(POLYI Pptr, MPI *Mptr, int m)
/*
 * creates the matrix whose columns are the basis defined in LLL p.536.
 */
{
	unsigned int i, j, k, l;
	MPMATI *Nptr, *Bptr;
	MPI *TempI;
	
	k = (unsigned int)m;
	l = (unsigned int)DEGREEPI(Pptr);
	Nptr = ZEROMNI(k + 1, k + 1);
	for (i = 0; i <= l - 1; i++)
	{
		TempI = elt(Nptr, i, i);
		elt(Nptr, i, i) = COPYI(Mptr);
		FREEMPI(TempI);
	}
	Bptr = POLYITOCOLI(Pptr);
	for (j = l; j <= k; j++)
	{
		for (i = j - l; i <= j; i++)
		{
			TempI = elt(Nptr, i, j);
			elt(Nptr, i, j) = COPYI(elt(Bptr, i - j + l, 0));
			FREEMPI(TempI);
		}
	}
	FREEMATI(Bptr);
	return (Nptr);
}

POLYI LEVERRIER_I(MPMATI *Mptr, MPMATI **Nptr, MPI **Dptr)
/*
 * The Faddeeva/Leverrier method for finding the characteristic polynomial,
 * adjoint and determinant of a matrix.
 * See "Computational methods in linear algebra", by V. N. Faddeeva, Dover
 * 1959. Here *Pptr[i] is the coefficient of x^(n - 1 - i) in ch(*Mptr).
 */
{
	POLYI Pptr, CURSOR;
	unsigned int i, n;
	MPMATI *Qptr, *M;
	MPI *G, *TempI;

	G = NULL;
	n = Mptr->R;
	Qptr = COPYMATI(Mptr);
	Pptr = NULL;
	for (i = 0; i <= n - 1; i++)
	{
		G = TRACEI(Qptr);
		TempI = G;
		G = INT_(G, i + 1);
		FREEMPI(TempI);
		PINSERTPI((int)(n - 1 - i), G, &Pptr, 0);
		M = DIAGI(Qptr, G);
		FREEMATI(Qptr);
		if ( i != n - 1)
			FREEMPI(G);
		if (i == n - 2)
		{
			if (i % 2 == 0)
				*Nptr = MINUSMATI(M);
			else
				*Nptr = COPYMATI(M);
		}  
		Qptr =	MULTMATI(Mptr, M);
		FREEMATI(M);
	}
	FREEMATI(Qptr);
	TempI = ONEI();
	PINSERTPI((int)i, TempI, &Pptr, 0);
	FREEMPI(TempI);
	PURGEPI(&Pptr);
	*Dptr = G;
	if (i % 2 == 0)
		(*Dptr)->S = -((*Dptr)->S);
	CURSOR = Pptr->NEXT;
	while (CURSOR != NULL)
	{
		(CURSOR->COEF)->S = -(CURSOR->COEF)->S;
		CURSOR = CURSOR->NEXT;
	}
	return (Pptr);
}
