/* p5.c */

#include <stdio.h>
#include <stdlib.h>
#include "integer.h"
#include "fun.h"

extern unsigned int PRIME[];

POLYI EUCLIDSPM(POLYI Pptr[], POLYI *Hptr[], USI n, MPI *Mptr)
/*
 * returns the gcd of polynomials Pptr[0],...,Pptr[n - 1]  mod *Mptr
 * and expresses it as Hptr[0] * Pptr[0] + ... Hptr[n - 1] * Pptr[n - 1].
 */
{
	unsigned int i;
	POLYI B, H, K, L, Tmp;
	MPI *TempI;

	*Hptr = (POLYI *)mmalloc(n * sizeof(POLYI));
	for (i = 0; i <= n - 1; i++)
	{
		*(*Hptr + i) = COPYPI(Pptr[i]);
	}
	for (i = 1; i <= n - 1; i++)
	{
		Tmp = *(*Hptr + i);
		*(*Hptr + i) = GCDPM(Tmp, *(*Hptr + (i - 1)), Mptr);
		DELETE_PI(Tmp);
	}
	Tmp = *(*Hptr + (n - 1));
	B = COPYPI(Tmp);
	DELETE_PI(Tmp);
	*(*Hptr + (n - 1)) = COPYPI(Pptr[n - 1]);
	TempI = ONEI();
	L = CONSTANTPM(TempI);
	FREEMPI(TempI);
	for (i = n - 1; i >= 1; i--)
	{
		Tmp = EUCLIDPM(*(*Hptr + i), *(*Hptr + (i - 1)), &H, &K, Mptr);
		DELETE_PI(Tmp);
		DELETE_PI(*(*Hptr + i));
		*(*Hptr + i) = MULTPM(H, L, Mptr);
		DELETE_PI(H);
		if (i == 1)
			break;
		Tmp = L;
		L = MULTPM(Tmp, K, Mptr);
		DELETE_PI(K);
		DELETE_PI(Tmp);
		DELETE_PI(*(*Hptr + (i - 1)));
		*(*Hptr + (i - 1)) = COPYPI(Pptr[i - 1]);
	}
	DELETE_PI(**Hptr);
	**Hptr  = MULTPM(L, K, Mptr);
	DELETE_PI(L);
	DELETE_PI(K);
	return (B);
}

POLYm EUCLIDSPm(POLYm Pptr[], POLYm *Hptr[], USI n, USI m)
/*
 * returns the gcd of polynomials Pptr[0],...,Pptr[n - 1]  mod m
 * and expresses it as Hptr[0] * Pptr[0] + ... Hptr[n - 1] * Pptr[n - 1].
 */
{
	unsigned int i;
	POLYm B, H, K, L, Tmp;

	*Hptr = (POLYm *)mmalloc(n * sizeof(POLYm));
	for (i = 0; i <= n - 1; i++)
		*(*Hptr + i) = COPYPm(Pptr[i], m);
	for (i = 1; i <= n - 1; i++)
	{
		Tmp = *(*Hptr + i);
		*(*Hptr + i) = GCDPm(Tmp, *(*Hptr + (i - 1)), m);
		DELETE_Pm(Tmp);
	}
	Tmp = *(*Hptr + (n - 1));
	B = COPYPm(Tmp, m);
	DELETE_Pm(Tmp);
	*(*Hptr + (n - 1)) = COPYPm(Pptr[n - 1], m);
	L = CONSTANTPm(1);
	for (i = n - 1; i >= 1; i--)
	{
		printf("i = %u\n", i);
		Tmp = EUCLIDPm(*(*Hptr + i), *(*Hptr + (i - 1)), &H, &K, m);
		DELETE_Pm(Tmp);
		DELETE_Pm(*(*Hptr + i));
		*(*Hptr + i) = MULTPm(H, L, m);
		DELETE_Pm(H);
		if (i == 1)
			break;
		Tmp = L;
		L = MULTPm(Tmp, K, m);
		DELETE_Pm(K);
		DELETE_Pm(Tmp);
		DELETE_Pm(*(*Hptr + (i - 1)));
		*(*Hptr + (i - 1)) = COPYPm(Pptr[i - 1], m);
	}
	DELETE_Pm(**Hptr);
	**Hptr  = MULTPm(L, K, m);
	DELETE_Pm(L);
	DELETE_Pm(K);
	return (B);
}

POLYm *COMPFACTORPm(POLYm Pptr, USI m, USI *nptr)
/*
 * returns the number *nptr of irreducible factors of  the squarefree polym 
 * Pptr mod m, the factors  being returned as p1[0],...,p1[n - 1].
 */
{
	POLYm *p1, Temp, Temp1;
	unsigned int n, j = 0;

	n = (unsigned int)(Pptr->DEG);
	Temp = COPYPm(Pptr, m);
	p1 = (POLYm *)mmalloc(n * sizeof(POLYm));
	while (DEGREEPm(Temp) != 0)
	{
		p1[j] = IRRED_FACTORPm(Temp, m);
		Temp1 = Temp;
		Temp = DIVPm(Temp1, p1[j], m);
		DELETE_Pm(Temp1);
		j++;
	}
	DELETE_Pm(Temp);
	*nptr = j;
	return p1;
}

POLYI AMULTPM(POLYI Pptr[], USI n, MPI *Mptr)
/*
 * returns the product of the n POLYm's Pptr[0],...,Pptr[n - 1]. 
 */
{
	POLYI B, Temp;	
	unsigned int i;
	
	B = COPYPI(Pptr[0]);
	for (i = 1; i <= n - 1; i++)
	{
		Temp = B;
		B = MULTPM(Temp, Pptr[i], Mptr);
		DELETE_PI(Temp);	
	}
	return B;
}

POLYm AMULTPm(POLYm Pptr[], USI n, USI m)
/*
 * returns the product of the n POLYm's Pptr[0],...,Pptr[n - 1]  mod m. 
 */
{
	POLYm B, Temp;	
	unsigned int i;
	
	B = COPYPm(Pptr[0], m);
	for (i = 1; i <= n - 1; i++)
	{
		Temp = B;
		B = MULTPm(Temp, Pptr[i], m);
		DELETE_Pm(Temp);	
	}
	return B;
}

POLYI *PARTPRODPM(POLYI Pptr[], USI n, MPI *Mptr)
/*
 * returns the n partial products P[i] = prod_{j=1,j!=i}^{n}Pptr[j]. 
 */
{
	POLYI *H, *P;
	unsigned int i, j;

	H = (POLYI *)mmalloc((n - 1)* sizeof(POLYI));
	P = (POLYI *)mmalloc(n * sizeof(POLYI));
	for (i = 0; i <= n - 1; i++)
	{
		for (j = 0; j <= n - 2; j++)
		{
			if (j < i)
				H[j] = COPYPI(Pptr[j]);
			if (j >= i)
				H[j] = COPYPI(Pptr[j + 1]);
		}
		P[i] = AMULTPM(H, n - 1, Mptr);
		for (j = 0; j <= n - 2; j++)
			DELETE_PI(H[j]);
	}
	ffree((char *)H, (n - 1) * sizeof(POLYI));
	return(P);
}

POLYm *PARTPRODPm(POLYm Pptr[], USI n, USI m)
/*
 * returns the n partial products P[i] = prod_{j=1,j!=i}^{n}Pptr[j].
 */
{
	POLYm *H, *P;
	unsigned int i, j;

	H = (POLYm *)mmalloc((n - 1)* sizeof(POLYm));
	P = (POLYm *)mmalloc(n * sizeof(POLYm));
	for (i = 0; i <= n - 1; i++)
	{
		for (j = 0; j <= n - 2; j++)
		{
			if (j < i)
				H[j] = COPYPm(Pptr[j], m);
			if (j >= i)
				H[j] = COPYPm(Pptr[j + 1], m);
		}
		P[i] = AMULTPm(H, n - 1, m);
		for (j = 0; j <= n - 2; j++)
			DELETE_Pm(H[j]);
	}
	ffree((char *)H, (n - 1) * sizeof(POLYm));
	return(P);
}

POLYm *PARTFRACPm(POLYm Rptr[], USI n, USI m)
/*
 * Gives the partial fraction decomposition:
 * 1/Rptr[0]*...Rptr[n-1]=A[0]/Rptr[0]+...+A[n-1]/Rptr[n-1].
 * Here the input polynomials are squarefree and pairwise relatively prime.
 * See H.T. Kung and D.M. Tong, Siam J. on Computing, 6, 1977, 582-593.
 */
{
	POLYm R, *A, *D, *PP, Temp, K;
	unsigned int i;

	A = (POLYm *)mmalloc(n * sizeof(POLYm));
	D = (POLYm *)mmalloc(n * sizeof(POLYm));
	PP = PARTPRODPm(Rptr, n, m);
	R = NULL;
	for (i = 0; i < n; i++)
	{
		Temp = R;
		R = ADDPm(R, PP[i], m);
		DELETE_Pm(PP[i]);
		DELETE_Pm(Temp);
	}	
	for (i = 0; i < n; i++)
	{
		D[i] = MODPm(R, Rptr[i], m); 
		if (D[i]->DEG == 0)
		{
			A[i] = CONSTANTPm(INVERSEm(D[i]->COEF, m));
			DELETE_Pm(D[i]);
		}
		else
		{
			Temp = EUCLIDPm(D[i], Rptr[i], &A[i], &K, m);
			DELETE_Pm(D[i]);
			DELETE_Pm(Temp);
			DELETE_Pm(K);
		}
	}
	DELETE_Pm(R);
	ffree((char *)D, n * sizeof(POLYm));
	ffree((char *)PP, n * sizeof(POLYm));
	return (A);
}
MPI *AHENSELPM(POLYI Fptr, POLYI uptr[], POLYI vptr[], POLYI *Uptr[], POLYI *Vptr[], USI n, MPI *Qptr)
/*
 * input: polynomials mod *Qptr : uptr[0],..., uptr[n-1],
 * vptr[0],...,vptr[n-1], with Fptr a polynomial with MPI coefficients;
 * gcd(*Qptr, LEADPI(Fptr)) = 1;
 * LEADPI(Fptr) = LEADPI(uptr[0]) mod *Qptr; uptr[i] monic for i >= 1;
 * Fptr = uptr[0]***uptr[n - 1] mod *Qptr.
 * DEGREEPI(vptr[i]) < DEGREEPI(uptr[i]) for all i;
 * u'ptr[0]*vptr[0]+...+u'ptr[n-1]*vptr[n-1] = 1 mod *Qptr;
 * output: polynomials mod *Qptr^2=*Mptr : Uptr[0],..., Uptr[n-1],
 * Vptr[0],...,Vptr[n-1], with LEADPI(Fptr) = LEADPI(Uptr[0]) mod *Qptr^2;
 * Uptr[i] monic for i>=1; U'ptr[0]*Vptr[0]+...+U'ptr[n-1]*Vptr[n-1] = 1
 * mod *Qptr^2; DEGREEPI(Vptr[i]) < DEGREEPI(Uptr[i]) for all i;
 * Fptr = Uptr[0]***Uptr[n - 1] mod *Mptr.
 * Uptr[i] = uptr[i] mod *Qptr, Vptr[i] = vptr[i] mod *Qptr for all i.
 *
 * The algorithm is the quadratic lift version of `Hensel lifting' and is 
 * due to Zassenhaus. See E. Kaltofen, Factorization of polynomials, pp. 95-113,
 * Computing Supplement 4, Springer 1982.
 */
{
	POLYI B, w1, f, t, *H, CURSOR, one, b, Temp, Temp1, Temp0;
	MPI *Mptr, *TempI;
	unsigned int i;

	
	*Uptr = (POLYI *)mmalloc(n * sizeof(POLYI));
	*Vptr = (POLYI *)mmalloc(n * sizeof(POLYI));
	Mptr = MULTI(Qptr, Qptr);
	/* Step H1 */
	Temp0 = COPYPI(uptr[0]);
	TempI = Temp0->COEF;
	Temp0->COEF = MOD(Fptr->COEF, Mptr);
	FREEMPI(TempI);
	Temp1 = COPYPI(Temp0);
	for (i = 1; i <= n - 1; i++)
	{
		Temp = Temp1;
		Temp1 = MULTPM(Temp, uptr[i], Mptr);
		DELETE_PI(Temp);	
	}
	f = CHANGETOPM(Fptr, Mptr);
	t = SUBPM(f, Temp1, Mptr);
	/* DEGREEPI(t) < DEGREEPI(Fptr) */
	DELETE_PI(Temp1);
	DELETE_PI(f);
	CURSOR = t;
	/* the coefficients of t are divisible by *Qptr */
	while (CURSOR != NULL)
	{
		TempI = CURSOR->COEF;
		CURSOR->COEF = INT0(CURSOR->COEF, Qptr); 
		FREEMPI(TempI);
		CURSOR = CURSOR->NEXT;
	}
	Temp1 = MULTPM(t, vptr[0], Qptr);
	w1 = MODPM(Temp1, uptr[0], Qptr);
	DELETE_PI(Temp1);
	Temp1 = SCALARPM(Qptr, w1, Mptr);
	DELETE_PI(w1);
	*(*Uptr + 0) = ADDPM(Temp0, Temp1, Mptr);
	DELETE_PI(Temp1);
	DELETE_PI(Temp0);
	for (i = 1; i <= n - 1; i++)
	{
		Temp1 = MULTPM(t, vptr[i], Qptr);
		w1 = MODPM(Temp1, uptr[i], Qptr);
		DELETE_PI(Temp1);
		Temp1 = SCALARPM(Qptr, w1, Mptr);
		DELETE_PI(w1);
		*(*Uptr + i) = ADDPM(uptr[i], Temp1, Mptr);
		DELETE_PI(Temp1);
	}
	/* t = w1[0]*u'ptr[0]+...+w1[n-1]*u'ptr[n-1] mod *Qptr */
	DELETE_PI(t);
	/* Step H2 */
	H = PARTPRODPM(*Uptr, n, Mptr);
	B = MULTPM(H[0], vptr[0], Mptr);
	for (i = 1; i <= n - 1; i++)
	{
		Temp = B;
		Temp1 = MULTPM(H[i], vptr[i], Mptr);
		B = ADDPM(Temp, Temp1, Mptr);
		DELETE_PI(Temp);	
		DELETE_PI(Temp1);	
	}
	for (i = 0; i <= n - 1; i++)
		DELETE_PI(H[i]);
	ffree((char *)H, n * sizeof(POLYI));
	TempI = ONEI();
	one = CONSTANTPM(TempI);
	FREEMPI(TempI);
	b = SUBPM(one, B, Mptr);
	/* DEGREEPI(b) < DEGREEPI(Fptr) */
	DELETE_PI(one);
	DELETE_PI(B);
	CURSOR = b;
	/* the coefficients of b are divisible by *Qptr */
	while (CURSOR != NULL)
	{
		TempI = CURSOR->COEF;
		CURSOR->COEF = INT0(CURSOR->COEF, Qptr); 
		FREEMPI(TempI);
		CURSOR = CURSOR->NEXT;
	}
	for (i = 0; i <= n - 1; i++)
	{
		Temp1 = MULTPM(b, vptr[i], Qptr);
		w1 = MODPM(Temp1, uptr[i], Qptr);
		DELETE_PI(Temp1);
		Temp1 = SCALARPM(Qptr, w1, Mptr);
		DELETE_PI(w1);
		*(*Vptr + i) = ADDPM(vptr[i], Temp1, Mptr);
		DELETE_PI(Temp1);
	}
	DELETE_PI(b);
	/* 1 = Vptr[0]*U'ptr[0]+...+Vptr[n-1]*U'ptr[n-1] mod *Qptr */
	return (Mptr);
}

MPI *AHENSEL1PM(POLYI Fptr, POLYI uptr[], POLYI vptr[], POLYI *Uptr[], USI n, MPI *Qptr, USI prime)
/*
 * input: polynomials mod *Qptr : uptr[0],..., uptr[n-1],
 * vptr[0],...,vptr[n-1], with Fptr a polynomial with MPI coefficients;
 * gcd(*Qptr, LEADPI(Fptr)) = 1;
 * Fptr = uptr[0]***uptr[n - 1] mod *Qptr.
 * LEADPI(Fptr) = LEADPI(uptr[0]) mod *Qptr; uptr[i] monic for i >= 1;
 * DEGREEPI(vptr[i]) < DEGREEPI(uptr[i]) for all i;
 * u'ptr[0]*vptr[0]+...+u'ptr[n-1]*vptr[n-1] = 1 mod prime;
 * output: polynomials mod *Qptr * prime = *Mptr : Uptr[0],..., Uptr[n-1],
 * LEADPI(Fptr) = LEADPI(Uptr[0]) mod *Mptr; *Uptr[i] monic for i>=1;
 * Fptr = Uptr[0]***Uptr[n - 1] mod *Mptr.
 * U'ptr[0]*vptr[0]+...+U'ptr[n-1]*vptr[n-1] = 1 mod prime.
 * Uptr[i] = uptr[i] mod prime for all i.
 *
 * The algorithm is the linear lift version of `Hensel lifting' and is 
 * due to Zassenhaus. See E. Kaltofen, Factorization of polynomials, pp. 95-113,
 * Computing Supplement 4, Springer 1982.
 */
{
	POLYI w1, f, t, CURSOR, Temp, Temp1, Temp0;
	MPI *Mptr, *TempI;
	unsigned int i;

	
	*Uptr = (POLYI *)mmalloc(n * sizeof(POLYI));
	Mptr = MULT_I(Qptr, (int)prime);
	/* Step H1 */
	Temp0 = COPYPI(uptr[0]);
	TempI = Temp0->COEF;
	Temp0->COEF = MOD(Fptr->COEF, Mptr);
	FREEMPI(TempI);
	Temp1 = COPYPI(Temp0);
	for (i = 1; i <= n - 1; i++)
	{
		Temp = Temp1;
		Temp1 = MULTPM(Temp, uptr[i], Mptr);
		DELETE_PI(Temp);	
	}
	f = CHANGETOPM(Fptr, Mptr);
	t = SUBPM(f, Temp1, Mptr);
	/* DEGREEPI(t) < DEGREEPI(Fptr) */
	DELETE_PI(Temp1);
	DELETE_PI(f);
	CURSOR = t;
	/* the coefficients of t are divisible by *Qptr */
	while (CURSOR != NULL)
	{
		TempI = CURSOR->COEF;
		CURSOR->COEF = INT0(CURSOR->COEF, Qptr); 
		FREEMPI(TempI);
		CURSOR = CURSOR->NEXT;
	}
	Temp1 = MULTPM(t, vptr[0], Qptr);
	w1 = MODPM(Temp1, uptr[0], Qptr);
	DELETE_PI(Temp1);
	Temp1 = SCALARPM(Qptr, w1, Mptr);
	DELETE_PI(w1);
	*(*Uptr + 0) = ADDPM(Temp0, Temp1, Mptr);
	DELETE_PI(Temp1);
	DELETE_PI(Temp0);
	for (i = 1; i <= n - 1; i++)
	{
		Temp1 = MULTPM(t, vptr[i], Qptr);
		w1 = MODPM(Temp1, uptr[i], Qptr);
		DELETE_PI(Temp1);
		Temp1 = SCALARPM(Qptr, w1, Mptr);
		DELETE_PI(w1);
		*(*Uptr + i) = ADDPM(uptr[i], Temp1, Mptr);
		DELETE_PI(Temp1);
	}
	/* t = w1[0]*u'ptr[0]+...+w1[n-1]*u'ptr[n-1] mod *Qptr */
	DELETE_PI(t);
	return (Mptr);
}

void AHENSELPm(POLYI Fptr, POLYm uptr[], POLYm vptr[], POLYm *Uptr[], POLYm *Vptr[], USI n, USI q, USI *m)
/*
 * input: polynomials mod q : uptr[0],..., uptr[n-1],
 * vptr[0],...,vptr[n-1], with Fptr a polynomial with coefficients mod m;
 * gcd(*Qptr, LEADPI(Fptr)) = 1;
 * LEADPm(Fptr) = LEADPm(uptr[0]) mod q; uptr[i] monic for i >= 1;
 * DEGREEPm(vptr[i]) < DEGREEPm(uptr[i]) for all i;
 * u'ptr[0]*vptr[0]+...+u'ptr[n-1]*vptr[n-1] = 1 mod q;
 * output: polynomials mod q * q = m : Uptr[0],..., Uptr[n-1],
 * Vptr[0],...,Vptr[n-1], with LEADPm(Fptr) = LEADPm(Uptr[0]) mod m;
 * Uptr[i] monic for i>=1; U'ptr[0]*Vptr[0]+...+U'ptr[n-1]*Vptr[n-1] = 1
 * mod m; DEGREEPm(Vptr[i]) < DEGREEPm(Uptr[i]) for all i;
 * Uptr[i] = uptr[i] mod q, Vptr[i] = vptr[i] mod q for all i; 
 * m < R0.
 *
 * The algorithm is the quadratic lift version of `Hensel lifting' and is 
 * due to Zassenhaus. See E. Kaltofen, Factorization of polynomials, pp. 95-113,
 * Computing Supplement 4, Springer 1982.
 */
{
	POLYm B, w1, f, t, *H, CURSOR, one, b, Temp, Temp1, Temp0;
	unsigned int i;

	
	*Uptr = (POLYm *)mmalloc(n * sizeof(POLYm));
	*Vptr = (POLYm *)mmalloc(n * sizeof(POLYm));
	*m = q * q;
	/* Step H1 */
	Temp0 = COPYPm(uptr[0], *m);
	Temp0->COEF = MOD_(Fptr->COEF, *m);
	Temp1 = COPYPm(Temp0, *m);
	for (i = 1; i <= n - 1; i++)
	{
		Temp = Temp1;
		Temp1 = MULTPm(Temp, uptr[i], *m);
		DELETE_Pm(Temp);	
	}
	f = CHANGETOPm(Fptr, *m);
	t = SUBPm(f, Temp1, *m);
	/* DEGREEPm(t) < DEGREEPm(Fptr) */
	DELETE_Pm(Temp1);
	DELETE_Pm(f);
	CURSOR = t;
	/* the coefficients of t are divisible by q */
	while (CURSOR != NULL)
	{
		CURSOR->COEF = CURSOR->COEF / q;
		CURSOR = CURSOR->NEXT;
	}
	Temp1 = MULTPm(t, vptr[0], q);
	w1 = MODPm(Temp1, uptr[0], q);
	DELETE_Pm(Temp1);
	Temp1 = SCALARPm(q, w1, *m);
	DELETE_Pm(w1);
	*(*Uptr + 0) = ADDPm(Temp0, Temp1, *m);
	DELETE_Pm(Temp1);
	DELETE_Pm(Temp0);
	for (i = 1; i <= n - 1; i++)
	{
		Temp1 = MULTPm(t, vptr[i], q);
		w1 = MODPm(Temp1, uptr[i], q);
		DELETE_Pm(Temp1);
		Temp1 = SCALARPm(q, w1, *m);
		DELETE_Pm(w1);
		*(*Uptr + i) = ADDPm(uptr[i], Temp1, *m);
		DELETE_Pm(Temp1);
	}
	/* t = w1[0]*u'ptr[0]+...+w1[n-1]*u'ptr[n-1] mod q */
	DELETE_Pm(t);
	/* Step H2 */
	H = PARTPRODPm(*Uptr, n, *m);
	B = MULTPm(H[0], vptr[0], *m);
	for (i = 1; i <= n - 1; i++)
	{
		Temp = B;
		Temp1 = MULTPm(H[i], vptr[i], *m);
		B = ADDPm(Temp, Temp1, *m);
		DELETE_Pm(Temp);	
		DELETE_Pm(Temp1);	
	}
	for (i = 0; i <= n - 1; i++)
		DELETE_Pm(H[i]);
	ffree((char *)H, n * sizeof(POLYm));
	one = CONSTANTPm(1);
	b = SUBPm(one, B, *m);
	/* DEGREEPm(b) < DEGREEPm(Fptr) */
	DELETE_Pm(one);
	DELETE_Pm(B);
	CURSOR = b;
	/* the coefficients of b are divisible by q */
	while (CURSOR != NULL)
	{
		CURSOR->COEF = CURSOR->COEF / q;
		CURSOR = CURSOR->NEXT;
	}
	for (i = 0; i <= n - 1; i++)
	{
		Temp1 = MULTPm(b, vptr[i], q);
		w1 = MODPm(Temp1, uptr[i], q);
		DELETE_Pm(Temp1);
		Temp1 = SCALARPm(q, w1, *m);
		DELETE_Pm(w1);
		*(*Vptr + i) = ADDPm(vptr[i], Temp1, *m);
		DELETE_Pm(Temp1);
	}
	DELETE_Pm(b);
	/* 1 = Vptr[0]*U'ptr[0]+...+Vptr[n-1]*U'ptr[n-1] mod q */
	return;
}

POLYI BALANCEPM(POLYI Pptr, MPI *Mptr)
/*
 * Balances the coefficients of the POLYI Pptr. 
 */
{
	POLYI CURSOR, P;
	MPI *H, *TempI;

	H = INT0_(Mptr, 2);
	P = COPYPI(Pptr);
	CURSOR = P;
	while (CURSOR != NULL)
	{
		if (RSV(CURSOR->COEF, H) == 1)
		{
			TempI = CURSOR->COEF;
			CURSOR->COEF = SUBI(CURSOR->COEF, Mptr);
			FREEMPI(TempI);
		}
		CURSOR = CURSOR->NEXT;
	}
	FREEMPI(H);
	return P;
}

POLYI *KFACTORSPI(POLYI Pptr, USI *nptr)
/*
 * The Kaltofen way of factorizing Pptr. 
 * Returns the distinct irreducible factors of a squarefree POLYI and their
 * number *nptr.  See E. Kaltofen, Factorization of polynomials, pp. 95-113,
 * Computing Supplement 4, Springer 1982.
 * See D.R. Musser, Multivariate Polynomial Factorization,
 * JACM,22,1975,291-308; On the Efficiency of a Polynomial Irreducibility
 * Test, JACM 25, 271-282.
 */
{
	MPI *A, *E, *L, *Q, *M, *M1, *M2, *M3, *R, *R1, *R2;
	MPI *TempI, *TempONEI;
	int m1;
	unsigned int *C, nmod, d, i, j, k, l, m, n, q, r, t, prime, u, v, z;
	unsigned int *Tmp1, *Tmp2, flag, no = 5;
	unsigned int *D, length, *number, *prime_, number0, lead;
	POLYI deriv, P, *FACTORS, B, Temp, *H1, *K1, *U1, *V1;
	POLYm p, p1, temp, *h1, *k1, *u1, *v1;

	flag = 0; /* to keep the compiler quiet */
	D = NULL;
	t = (unsigned int)DEGREEPI(Pptr);
	L = LENGTHSQPI(Pptr);	
	deriv = DERIVPI(Pptr);
	A = SUBRESULTANT(Pptr, deriv);
	DELETE_PI(deriv);	
	for (i = 0; i < Y0; i++)
	{
		if (MOD_(A, PRIME[i]))
		{
			p = CHANGETOPm(Pptr, PRIME[i]);
			D = DEGREESETPm(p, &length, &number0, PRIME[i]);
			DELETE_Pm(p);
			break;
		}
	}
	if (i == Y0)
	{
		printf("Input polynomial f has the unlikely property that Res(f,f')\
 is divisible by the first %u primes\n", Y0);
		exit (1);
	}
	FACTORS = (POLYI *)mmalloc(t * sizeof(POLYI));
	if (TESTARRAY0(D, length) == 0)
	{
		printf("Polynomial is irreducible mod %u:\n", PRIME[i]);
		*nptr = 1;
		FACTORS[0] = COPYPI(Pptr);
		FREEMPI(L);
		FREEMPI(A);
		ffree((char *)D, length * sizeof(unsigned int));
		return FACTORS;
	}
	number =(unsigned int *)mmalloc(no * sizeof(unsigned int));
	prime_ = (unsigned int *)mmalloc(no * sizeof(unsigned int));
	prime_[0] = PRIME[i];
	number[0] = number0;
	m = 1;
	i++;
	for (j = i; j < Y0; j++)
	{
		if (MOD_(A, PRIME[j]))
		{
			p1 = CHANGETOPm(Pptr, PRIME[j]);
			Tmp1 = DEGREESETPm(p1, &length, &number[m], PRIME[j]);
			prime_[m] = PRIME[j];
			DELETE_Pm(p1);
			Tmp2 = D;
			D = INTERSECTION(Tmp1, Tmp2, length);
			ffree((char *)Tmp1, length * sizeof(unsigned int));
			ffree((char *)Tmp2, length * sizeof(unsigned int));
			if (TESTARRAY0(D, length) == 0)
			{
				printf("By the Degree_Compatibility Test,\n");
				printf("input polynomial is irreducible over Z[x]:\n");
				*nptr = 1;
				FACTORS[0] = COPYPI(Pptr);
				ffree((char *)D, length * sizeof(unsigned int));
				ffree((char *)prime_, no * sizeof(unsigned int));
				ffree((char *)number, no * sizeof(unsigned int));
				FREEMPI(L);
				FREEMPI(A);
				return FACTORS;
			}
			m++;
		}
		if (m == no)
		break;	
	}
	FREEMPI(A);
	m--;
	m1 = (int)m;
	z = number[m1];
	prime = prime_[m1];
	for (m1 = no - 1; m1 >= 1; m1--)
		if (number[m1 - 1] <= z)
		{
			z = number[m1 - 1];
			prime = prime_[m1 - 1];
		}
	printf("factorizing mod %u :\n", prime);
	ffree((char *)prime_, no * sizeof(unsigned int));
	ffree((char *)number, no * sizeof(unsigned int));
	p = CHANGETOPm(Pptr, prime);
	h1 = FACTORSPm(p, &lead, &C, &nmod, prime);
	if (nmod >= 2 * T0)
	{
		fprintf(stderr, "too many factors mod %u\n", prime);
		exit(1);
	}
	ffree((char *)C, t * sizeof(unsigned int));
	printf("number of irreducible factors mod %u = %u\n", prime, nmod);
	DELETE_Pm(p);
	P = COPYPI(Pptr);
	R = LEADPI(P);
	temp = h1[0];
	h1[0] = SCALARPm(MOD0_(R, prime), temp, prime);
	DELETE_Pm(temp);
	printf("doing PARTFRACPm :\n");
	k1 = PARTFRACPm(h1, nmod, prime);
	printf("finished PARTFRACPm :\n");
	M1 = BINOMIAL(2 * t, t);
	M2 = MULT_I(M1, 4);
	FREEMPI(M1);
	M3 = MULTI(M2, L);
	FREEMPI(M2);
	FREEMPI(L);
	TempI = M3;
	M3 = BABY_MTHROOT(M3, 2);
	FREEMPI(TempI);
	u = 0;
	q = prime;
	Q = CHANGE(prime);
	printf("Q = ");
	PRINTI(Q);
	printf("\n");
	M = MULTI(Q, Q);
	while (RSV(Q, M3) <= 0 && M->D == 0)
	{
		printf("doing POLYm Hensel-Zassenhaus quadratic lift:\n");
		AHENSELPm(Pptr, h1, k1, &u1, &v1, nmod, q, &m);
		for (i = 0; i <= nmod - 1; i++)
		{
			temp = h1[i];
			h1[i] = u1[i];
			DELETE_Pm(temp);
			temp = k1[i];
			k1[i] = v1[i];
			DELETE_Pm(temp);
		}
		ffree((char *)u1, nmod * sizeof(POLYm));
		ffree((char *)v1, nmod * sizeof(POLYm));
		q = m;
		FREEMPI(Q);
		Q = CHANGE(q);
		printf("Q = ");
		PRINTI(Q);
		printf("\n");
		FREEMPI(M);
		M = MULTI(Q, Q);
	}
	printf("finished POLYm Hensel-Zassenhaus quadratic lift:\n");
	H1 = (POLYI *)mmalloc(nmod * sizeof(POLYI));
	K1 = (POLYI *)mmalloc(nmod * sizeof(POLYI));
	for (i = 0; i <= nmod - 1; i++)
	{
		H1[i] = CHANGETOPI(h1[i]);
		K1[i] = CHANGETOPI(k1[i]);
		DELETE_Pm(h1[i]);
		DELETE_Pm(k1[i]);
	}
	ffree((char *)h1, t * sizeof(POLYm));
	ffree((char *)k1, nmod * sizeof(POLYm));
	while (RSV(M, M3) <= 0)
	{
		flag = 1;
		printf("doing POLYI Hensel-Zassenhaus quadratic lift:\n");
		FREEMPI(M);
		M = AHENSELPM(Pptr, H1, K1, &U1, &V1, nmod, Q);
		for (i = 0; i <= nmod - 1; i++)
		{
			Temp = H1[i];
			H1[i] = U1[i];
			DELETE_PI(Temp);
			Temp = K1[i];
			K1[i] = V1[i];
			DELETE_PI(Temp);
		}
		ffree((char *)U1, nmod * sizeof(POLYI));
		ffree((char *)V1, nmod * sizeof(POLYI));
		FREEMPI(Q);
		Q = M;
		printf("Q = ");
		PRINTI(Q);
		printf("\n");
		M = MULTI(Q, Q);
	}
	if (flag)
	{
		printf("finished POLYI Hensel-Zassenhaus quadratic lift:\n");
		flag = 0;
	}
	while (RSV(Q, M3) <= 0)
	{
		printf("doing Hensel-Zassenhaus linear lift:\n");
		flag = 1;
		FREEMPI(M);
		M = AHENSEL1PM(Pptr, H1, K1, &U1, nmod, Q, prime);
		for (i = 0; i <= nmod - 1; i++)
		{
			Temp = H1[i];
			H1[i] = U1[i];
			DELETE_PI(Temp);
		}
		ffree((char *)U1, nmod * sizeof(POLYI));
		FREEMPI(Q);
		Q = COPYI(M);
		printf("Q = ");
		PRINTI(Q);
		printf("\n");
	}
	FREEMPI(M3);
	FREEMPI(M);
	if (flag)
	{
		printf("finished Hensel-Zassenhaus linear lift:\n");
		flag = 0;
	}
	for (i = 0; i <= nmod - 1; i++)
		DELETE_PI(K1[i]);
	ffree((char *)K1, nmod * sizeof(POLYI));
	Temp = H1[0];
	H1[0] = MONICPM(Temp, Q);
	DELETE_PI(Temp);
	C = (unsigned int *)mmalloc(nmod * sizeof(unsigned int));
	n = nmod;
	l = 0;
	TempONEI = ONEI();
	while (n)
	{
		E = POWER_I(2, n);
		m = CONVERTI(E);
		FREEMPI(E);
		v = (unsigned int)DEGREEPI(P) / 2;
		j = 1;
		while (j <= m - 1)
		{
			k = 0;
			z = 0;
			u = j;
			B = CONSTANTPM(TempONEI);
			d = 0;
			for (i = 0; i <= n - 1; i++)
				C[i] = 0;
			while (z <= n - 1)
			{
				if (u % 2)
				{
					d += (unsigned int)DEGREEPI(H1[z]);
					if (d > v)
						break;
					Temp = B;
					B = MULTPM(Temp, H1[z], Q);
					DELETE_PI(Temp);
					C[z] = 1;
					k++;
				}
				u = u / 2;
				z++;
			}
			if (d > v)
			{
				DELETE_PI(B);
				j++;
				d = 0;
				if (j == m)
				{
					FACTORS[l] = P;
					for (i = 0; i <= n - 1; i++)
						DELETE_PI(H1[i]);
					n = 0;
					l++;
					break;
				}
				continue;
			}
			if (D[d])
			{
				Temp = B;
				B = SCALARPM(R, Temp, Q);
				DELETE_PI(Temp);
				Temp = B;
				B = BALANCEPM(Temp, Q);
				DELETE_PI(Temp);
				Temp = B;
				B = PRIMITIVEPI(Temp);
				DELETE_PI(Temp);
				R1 = VAL0PI(P);
				R2 = VAL0PI(B);
				TempI = R2;
				R2 = ABSI(R2);
				FREEMPI(TempI);
				if (R2->S != 0)
				{
					TempI = R2;
					R2 = MOD(R1, R2);
					FREEMPI(TempI);
				}
				FREEMPI(R1);
				if (R2->S == 0)
				{
					Temp = MODPI(P, B);
					if (Temp == NULL)
					{
						FACTORS[l] = B;
						Temp = P;
						P = DIVPI(Temp, FACTORS[l]);
						DELETE_PI(Temp);
						Temp = P;
						P = PRIMITIVEPI(Temp);
						DELETE_PI(Temp);
						TempI = R;
						R = LEADPI(P);
						FREEMPI(TempI);
						for (i = 0; i <= n - 1; i++)
						{
							if (C[i])
								DELETE_PI(H1[i]);
						}
						r = 0;
						i = 0;
						if (n != k)
						{
						  for (i = 0; i <= n - k - 1; i++)
						  {
						    while (r <= n - 1)
						    {
						      if (!C[r])
						      {
							H1[i] = H1[r];
							r++;
							break;
						      }
						      r++;
						    }
						  }
						}
						n = n - k;
						l++;
						FREEMPI(R2);
						break;
					}
					else
						DELETE_PI(Temp);
				}
				FREEMPI(R2);
			}
			DELETE_PI(B);
			j++;
		} 
	}
	FREEMPI(Q);
	FREEMPI(R);
	FREEMPI(TempONEI);
	printf("factorization finished:\n");
	*nptr = l;
	ffree((char *)C, nmod * sizeof(unsigned int));
	ffree((char *)D, length * sizeof(unsigned int));
	ffree((char *)H1, nmod * sizeof(POLYI));
	return FACTORS;
}

POLYm *GOLOMBPm(POLYm Pptr, USI *C[], USI *nptr, USI p)
/*
 * Input: POLYm Pptr mod p. 
 * Output: d[0],...d[*npt - 1], where d[i] is the product of all the monic
 * irreducibles of degree C[i] dividing Pptr.
 */
{
	unsigned int i = 0, j = 1, m;
	int n;
	POLYm U, *d, h, x, Temp;

	U = MONICPm(Pptr, p);
	x = LINEARPm(0, p);
	m = (unsigned int)DEGREEPm(Pptr);
	*C = (unsigned int *)mmalloc(m * sizeof(unsigned int));
	d = (POLYm *)mmalloc(m * sizeof(POLYm));
	h = COPYPm(x, p);
	while (j <= (unsigned int)DEGREEPm(U) / 2)
	{
		Temp = h;
		h = POWER0Pm(Temp, p, U, p);
		DELETE_Pm(Temp);
		Temp = SUBPm(h, x, p);
		d[i] = GCDPm(Temp, U, p);
		DELETE_Pm(Temp);
		n = DEGREEPm(d[i]);
		if (n != 0)
		{
			Temp = U;
			U = DIVPm(Temp, d[i], p);
			DELETE_Pm(Temp);
			Temp = h;	
			h = MODPm(Temp, U, p);
			DELETE_Pm(Temp);
			(*C)[i] = (unsigned int)DEGREEPm(d[i]);
			i++;
		}
		else
			DELETE_Pm(d[i]);
		j++;
	}
	if (DEGREEPm(U) != 0)
	{
		(*C)[i] = (unsigned int)DEGREEPm(U);
		d[i] = COPYPm(U, p);
		i++;
	}
	*nptr = i;
	DELETE_Pm(x);
	DELETE_Pm(h);
	DELETE_Pm(U);
	return d;
}

POLYm *SQUARE_FREE_GOLOMBPm(POLYm Pptr, USI *C[], USI *degree[], USI *nptr, 
USI p)
/*
 * Input: POLYm Pptr mod p, not necessarily squarefree, as in GOLOMBPm().
 * Output: d[0],...d[*npt - 1], where d[i] is the product of all the monic 
 * irreducibles of the same degree degree[i] and dividing Pptr to degree C[i].
 * A finer decomposition than the squarefree decomposition.
 */
{
	unsigned int i = 0, j = 1, k,  m, t;
	int n;
	POLYm U, *d, h, x, Temp, Temp1, Temp2, Temp3, D, di;

	U = MONICPm(Pptr, p);
	x = LINEARPm(0, p);
	m = (unsigned int)DEGREEPm(Pptr);
	*C = (unsigned int *)mmalloc(m * sizeof(unsigned int));
	*degree = (unsigned int *)mmalloc(m * sizeof(unsigned int));
	d = (POLYm *)mmalloc(m * sizeof(POLYm));
	h = COPYPm(x, p);
	while (j <= (unsigned int)DEGREEPm(U) / 2)
	{
		Temp = h;
		h = POWER0Pm(Temp, p, U, p);
		DELETE_Pm(Temp);
		Temp = SUBPm(h, x, p);
		di = GCDPm(Temp, U, p);
		n = DEGREEPm(di);
		if (n != 0)
		{
			k = 0;
			D = COPYPm(di, p);
			while (D->DEG > 0)
			{
				k++;
				t = (unsigned int)(D->DEG);
				Temp3 = D;
				Temp1 = U;
				U = DIVPm(Temp1, D, p);
				DELETE_Pm(Temp1);
				D = GCDPm(Temp, U, p);
				if (D->DEG < t)
				{
					d[i] = DIVPm(Temp3, D, p);
					(*degree)[i] = j;
					(*C)[i] =  k;
					i++;
				}
				DELETE_Pm(Temp3);
			}
			DELETE_Pm(D);
			Temp2 = h;	
			h = MODPm(Temp2, U, p);
			DELETE_Pm(Temp2);
		}
		DELETE_Pm(di);
		DELETE_Pm(Temp);
		j++;
	}
	if (DEGREEPm(U) != 0)/* U is irreducible  here*/
	{
		(*C)[i] = 1;
		d[i] = COPYPm(U, p);
		(*degree)[i] = (unsigned int)(U->DEG);
		i++;
	}
	*nptr = i;
	DELETE_Pm(x);
	DELETE_Pm(h);
	DELETE_Pm(U);
	return d;
}

unsigned int *SHIFT(USI D[], USI n, USI d)
/*
 * Input: D[0],...D[n - 1] is a bit string, 0 < d < n.
 * Output: D shifted d bits.
 */
{
	unsigned int *C, i;

	C = (unsigned int *)mmalloc(n * sizeof (unsigned int));
	for (i = 0; i <= d - 1; i++)
		C[i] = 0;
	for (i = d; i <= n - 1; i++)
		C[i] = D[i - d];
	return C;
}

unsigned int *UNION(USI D1[], USI D2[], USI n)
/*
 * Returns the bit union of strings D1 and D2, each of length n.
 */
{
	unsigned int *C, i;

	C = (unsigned int *)mmalloc(n * sizeof (unsigned int));
	for (i = 0; i <= n - 1; i++)
	{
		if (D1[i] + D2[i])
			C[i] = 1;
		else
			C[i] = 0;
	}
	return C;
}

unsigned int *INTERSECTION(USI D1[], USI D2[], USI n)
/*
 * Returns the bit intersection of strings D1 and D2, each of length n.
 */
{
	unsigned int *C, i;

	C = (unsigned int *)mmalloc(n * sizeof (unsigned int));
	for (i = 0; i <= n - 1; i++)
	{
		if (D1[i] * D2[i])
			C[i] = 1;
		else
			C[i] = 0;
	}
	return C;
}

unsigned int *DEGREESETPm(POLYm Pptr, USI *nptr, USI *dptr,  USI p)
/*
 * The degree set E of factors of a squarefree polynomial f over Z is contained
 * in each degree set E[p] mod p. If the intersection of E[p] over a small set 
 * of primes consists of 0 and deg(f), the F is irreducible over Z.
 * e.g. if deg(f)=10 and f has irreducible factors of degrees 2,2,6 mod 2, then
 * E[2]={0,2,4,6,8,10}; if f has irreducible factors of degrees 3,3,4 mod 3,
 * then E[3]={0,3,4,6,7,10}; if f has irreducible factors of degrees 2,3,5 mod 5
 * then E[3]={0,2,3,5,7,8,10}. Then E[2],E[3],E[5] intersect in {0,10} and so
 * f is irreducible over Z.
 * Input: POLYm Pptr mod p. Output: *nptr = 1 + n, n = DEGREEPm(Pptr) / 2.
 * Note: f|Pptr => (Pptr/f)|Pptr.
 *  D[0],...D[n], the degrees of all monic factors of Pptr of degree <= n
 *  represented as a bit string of length  1 + n.
 * *dptr = number of irreducible factors of Pptr.
 * See D.R. Musser, Multivariate Polynomial Factorization,
 *  JACM,22,1975,291-308; On the Efficiency of a Polynomial Irreducibility
 * Test, JACM 25, 271-282.
 */
{
	unsigned int i, j = 1, m, t, repeats, *C, *C1, *C2;
	int a, n;
	POLYm U, d, h, x, Temp;

	*dptr = 0;
	U = MONICPm(Pptr, p);
	x = LINEARPm(0, p);
	m = (unsigned int)DEGREEPm(Pptr);
	t =  1 + (m / 2);
	*nptr = t;
	C = (unsigned int *)mmalloc(t * sizeof(unsigned int));
	C[0] = 1;/* trivial factor 1 */
	for (i = 1; i <= t - 1; i++)
		C[i] = 0;
	h = COPYPm(x, p);
	while (j <= (unsigned int)DEGREEPm(U) / 2)
	{
		Temp = h;
		h = POWER0Pm(Temp, p, U, p);
		DELETE_Pm(Temp);
		Temp = SUBPm(h, x, p);
		d = GCDPm(Temp, U, p);
		DELETE_Pm(Temp);
		printf("d=");PRINTPm(d);printf("\n");
		n = DEGREEPm(d);
		if (n != 0)
		{
			Temp = U;
			U = DIVPm(Temp, d, p);
			DELETE_Pm(Temp);
			Temp = h;	
			h = MODPm(Temp, U, p);
			DELETE_Pm(Temp);
			repeats = (unsigned int)DEGREEPm(d) / j;
			for (i = 0; i <= repeats - 1; i++)
			{
				C1 = C;
				C = SHIFT(C1, t, j);
				C2 = C;
				C = UNION(C1, C2, t);
				ffree((char *)C1, t * sizeof(unsigned int));
				ffree((char *)C2, t * sizeof(unsigned int));
			}
			*dptr += repeats;
		}
		DELETE_Pm(d);
		j++;
	}
	a = DEGREEPm(U);
	if (a) /* U is irreducible */
	{
		(*dptr)++;
		C[a] = 1;
	}
	DELETE_Pm(x);
	DELETE_Pm(h);
	DELETE_Pm(U);
	return C;
}

unsigned int TESTARRAY0(USI D[], USI n)
/*
 * returns 1 if d != 0 for some i, 1 <= i <= n - 1; 0 otherwise.
 */
{
	unsigned int i;

	for (i = 1; i <= n - 1; i++)
		if (D[i])
			return 1;
	return 0;
}

POLYI *KALTOFENPI(POLYI Pptr, MPI **CONTENT, USI **C, USI *nptr)
/*
 * Factors a polynomial over Z[x} using Kaltofen-Musser algorithm.
 * Returns the content of Pptr and the factorization into irreducibles of
 * the primitive part of Pptr.
 * *Q[0],...,*Q[*nptr - 1] are the essentially distinct irreducibles,
 * C[0],...,C[*nptr - 1] are the corresponding multiplicities.
 */
{
	unsigned int k, m, n, i, j = 0, t, *D;
	POLYI P, *SQ, *Q, *K;

	n = (unsigned int)DEGREEPI(Pptr);
	*CONTENT = CONTENTPI(Pptr);
	P = PRIMITIVEPI(Pptr);
	Q = (POLYI *)mmalloc(n * sizeof(POLYI));	
	*C = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	SQ = SQUAREFREEPI(P, &D, &t);
	DELETE_PI(P);
	for (i = 0; i < t; i++)
	{
		K = KFACTORSPI(SQ[i], &m);
		for (k = 0; k <= m - 1; k++)
		{
			Q[j] = K[k];
			(*C)[j] = D[i];	
			j++;
		}
		ffree((char *)K, (unsigned int)(SQ[i]->DEG) * sizeof(POLYI));
		DELETE_PI(SQ[i]);
	}
	*nptr = j;
	ffree((char *)D, n * sizeof(unsigned int));
	ffree((char *)SQ, n * sizeof(POLYI));
	return (Q);
}

POLYR *KALTOFENPR(POLYR Pptr, USI **C, USI *nptr)
/*
 * Factors a polynomial over Q[x] using the Kaltofen-Musser algorithm.
 * Returns the factorization of Pptr into monic irreducibles.
 * *Q[0],...,*Q[*nptr - 1] are the essentially distinct irreducibles,
 * C[0],...,C[*nptr - 1] are the corresponding multiplicities.
 */
{
	unsigned int k, m, n, i, j = 0, t, *D;
	POLYI *K, P, *SQ, TempI1;
	POLYR TempR1, *Q;

	n = (unsigned int)DEGREEPR(Pptr);
	TempI1 = CLEARDENOMPI(Pptr);
	P = PRIMITIVEPI(TempI1);
	DELETE_PI(TempI1);
	Q = (POLYR *)mmalloc(n * sizeof(POLYR));	
	*C = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	SQ = SQUAREFREEPI(P, &D, &t);
	DELETE_PI(P);
	for (i = 0; i < t; i++)
	{
		printf("Factorizing ");
		PRINTPI(SQ[i]); printf(" :\n");
		K = KFACTORSPI(SQ[i], &m);
		for (k = 0; k <= m - 1; k++)
		{
			TempI1 = K[k];
			TempR1 = COPYPI_TO_PR(TempI1);
			DELETE_PI(TempI1);
			Q[j] = MONICPR(TempR1);
			DELETE_PR(TempR1);
			(*C)[j] = D[i];	
			j++;
		}
		ffree((char *)K, (unsigned int)(SQ[i]->DEG) * sizeof(POLYI));
		DELETE_PI(SQ[i]);
	}
	*nptr = j;
	ffree((char *)D, n * sizeof(unsigned int));
	ffree((char *)SQ, n * sizeof(POLYI));
	return (Q);
}
