/* pCR.c */

#include <stdio.h>
#include <stdlib.h>
#include "integer.h"
#include "fun.h"

int DEGREEPCR(POLYCR Pptr)
/*
 * returns the degree of the polynomial Pptr.
 */
{
	if (Pptr == NULL)
		return (-1);
	else
		return (Pptr->DEG);
	
}

MPCR *LEADPCR(POLYCR Pptr)
/*
 * *Aptr is the  leading coefficient of the polynomial Pptr.
 */
{
	if (Pptr == NULL)
		return ZEROCR();
	else
		return COPYCR(Pptr->COEF);
}

void DELETE_PCR(POLYCR Pptr)
/*
 * returns back to the system storage dynamically allocated to the linked list
 * which Pptr points to.
 */
{
	POLYCR Qtmp, orig;
	
	if(!Pptr) return;
	orig = Pptr;
	while (Pptr != NULL)
	{
 		Qtmp = Pptr->NEXT;
		FREEMPCR(Pptr->COEF);
#ifdef DEBUG
		SAVEPCR(Pptr);
#endif
		Pptr= Qtmp;
	}
#ifndef DEBUG
	SAVEPCR(orig);
#endif
}

void PINSERTPCR(int DEG, MPCR *COEF, POLYCR *Pptr, USI OPT)
/*
 * inserts a copy of a single term *Qptr into polynomial *Pptr.
 * It has two options that control the insertion in case a term of *Pptr already
 * has the degree of *Qptr, If OPT = 0, then the term is replaced; if OPT = 1,
 * then the coefficient of *Qptr is added to that of the corresponding term of
 *  *Pptr.
 */
{
	POLYCR CURSOR, Ttmp;
	MPCR *TempCR;

	if (*Pptr == NULL)
	{
		*Pptr = CREATECR();
		(*Pptr)->DEG = DEG;
		(*Pptr)->COEF = COPYCR(COEF);
		(*Pptr)->NEXT = NULL;
		return;
	}
	else
	{
		CURSOR = *Pptr;
		while ((CURSOR->DEG > DEG) && (CURSOR->NEXT != NULL))
			CURSOR = CURSOR->NEXT;
		if (CURSOR->DEG < DEG)
		{
			/* Qptr inserted before CURSOR */
			Ttmp = CREATECR();
			Ttmp->DEG = CURSOR->DEG;
			Ttmp->COEF = CURSOR->COEF;
			Ttmp->NEXT = CURSOR->NEXT;
			CURSOR->DEG = DEG;
			CURSOR->COEF = COPYCR(COEF);
			CURSOR->NEXT = Ttmp;
			return;
		}
		else if (CURSOR->DEG == DEG)
		{
			if (OPT == 0)
				CURSOR->COEF = COPYCR(COEF);
			else
			{
				TempCR = CURSOR->COEF;
				CURSOR->COEF = ADDCR(CURSOR->COEF, COEF);
				FREEMPCR(TempCR);
			}
			return;
		}
		else
		{
			/* CURSOR->DEG > Qptr->DEG and CURSOR->NEXT === NULL */
			/* Qptr inserted at end */
			Ttmp = CREATECR();
			Ttmp->DEG = DEG;
			Ttmp->COEF = COPYCR(COEF);
			Ttmp->NEXT = NULL;
			CURSOR->NEXT = Ttmp;
			return;
		}
	}
}

void PURGEPCR(POLYCR *Pptr)
/*
 * gets rid of any zero coefficients in the polynomial Pptr.
 */
{
	POLYCR R1tmp, R2tmp;

	if (*Pptr != NULL)
	{
		R1tmp = *Pptr;
		R2tmp = (*Pptr)->NEXT;
		while (R2tmp != NULL)
		{
			/* purge non-leading terms */
			if (EQZEROCR(R2tmp->COEF))
			{
				R1tmp->NEXT = R2tmp->NEXT;
				R2tmp->NEXT = NULL;
				DELETE_PCR(R2tmp);
				R2tmp = R1tmp->NEXT;
			}
			else
			{
				R1tmp = R2tmp;
				R2tmp = R2tmp->NEXT;
			}
		}
		if (EQZEROCR((*Pptr)->COEF))
		{
			/* purge leading term */
			R1tmp = *Pptr;
			*Pptr = (*Pptr)->NEXT;
			R1tmp->NEXT = NULL;
			DELETE_PCR(R1tmp);
		}
	}
	return;
}

POLYCR INPUTPCR(USI *ptr)
/*
 * Inputting a polynomial Pptr from the terminal.
 * *ptr = 0 indicates some garbage input and a default POLYCR as output. 
 */
{
	POLYCR Pptr;
	int k;
	char *line, *p;
	MPCR *TempCR;
	
	Pptr = NULL;
	*ptr = 1;
	do
	{
		printf("Enter the degree of new term, (-1 to terminate): ");
		if (scanfd10(&k) == 0)
		{
			*ptr = 0;
			k = -1;
		}
		if (k >= 0)
		{
			printf("Enter the coefficient of x^%d: ", k);
			p = line = Gets();
		        TempCR = INPUTCR(&p, ptr); 
			PINSERTPCR(k, TempCR, &Pptr, 0);
			FREEMPCR(TempCR);
			if (*ptr == 0)
				k = -1;
			free(line);
		}
	}
	while (k >= 0);
	PURGEPCR(&Pptr);
	return Pptr;
}

POLYCR COPYPCR(POLYCR Pptr)
/*
 * makes a copy of the polynomial Pptr.
 */
{
	POLYCR Rtmp, Sptr;

	Sptr = NULL;
	Rtmp = Pptr;
	while (Rtmp != NULL)
	{
		PINSERTPCR(Rtmp->DEG, Rtmp->COEF, &Sptr, 0);
		Rtmp = Rtmp->NEXT;
	}
	return (Sptr);
}
 
void PRINTPCR(POLYCR Pptr)
/*
 * outputs the terms of the polynomial Pptr.
 */
{
	FPRINTPCR(stdout, Pptr);
	return;
}

POLYCR SCALARPCR(MPCR *Cptr, POLYCR Pptr)
/*
 * multiplying the polynomial Pptr by the MPCR *Cptr.
 */
{
	POLYCR Sptr, Rtmp;
	MPCR *Temp2CR;

	if (Pptr == NULL || EQZEROCR(Cptr))
		Sptr = NULL;
	else
	{
		Sptr = COPYPCR(Pptr);
		Rtmp = Sptr;
		while (Rtmp != NULL)
		{
			Temp2CR = Rtmp->COEF;
			Rtmp->COEF = MULTCR(Rtmp->COEF, Cptr);
			FREEMPCR(Temp2CR);
			Rtmp = Rtmp->NEXT;
		}
	}
	return (Sptr);
}

POLYCR ADDPCR(POLYCR Pptr, POLYCR Qptr)
/*
 * returns the sum of two polynomials.
 */
{
	POLYCR CURSOR, Sptr;

		Sptr = COPYPCR(Pptr);
		CURSOR = Qptr;
		while (CURSOR != NULL)
		{
			PINSERTPCR(CURSOR->DEG, CURSOR->COEF, &Sptr, 1);
			CURSOR = CURSOR->NEXT;
		}
		PURGEPCR(&Sptr);
		return (Sptr);
}

POLYCR SUBPCR(POLYCR Pptr, POLYCR Qptr)
/*
 * returns the difference Pptr - Qptr of two polynomials.
 */
{
	POLYCR CURSOR, Sptr;
	MPCR *TempCR;

	Sptr = COPYPCR(Pptr);
	if (Qptr == NULL)
		return (Sptr);
	CURSOR = Qptr;
	while (CURSOR != NULL)
	{
		TempCR = MINUSCR(CURSOR->COEF);
		PINSERTPCR(CURSOR->DEG, TempCR, &Sptr, 1);
		FREEMPCR(TempCR);
		CURSOR = CURSOR->NEXT;
	}
	PURGEPCR(&Sptr);
	return (Sptr);
}

POLYCR MULTPCR(POLYCR Pptr, POLYCR Qptr)
/*
 * returns the product of two polynomials.
 */
{
	POLYCR Sptr, CURSOR_P, CURSOR_Q;
	MPCR *TempCR;
	int k;

	if (Pptr == NULL || Qptr == NULL)
		return (NULL);
	Sptr = NULL;
	CURSOR_P = Pptr;
	while (CURSOR_P != NULL)
	{
		CURSOR_Q = Qptr;
		while (CURSOR_Q != NULL)
		{
			k = CURSOR_P->DEG + CURSOR_Q->DEG;
			TempCR = MULTCR(CURSOR_P->COEF, CURSOR_Q->COEF);
			PINSERTPCR(k, TempCR, &Sptr, 1);
			FREEMPCR(TempCR);
			CURSOR_Q = CURSOR_Q->NEXT;
		}
		CURSOR_P = CURSOR_P->NEXT;
	}
	PURGEPCR(&Sptr);
	return (Sptr);
}

MPCR *VALPCR(POLYCR Pptr, MPCR *Cptr)
/*
 * Evaluating the polynomial Pptr at the MPCR *Cptr.
 */
{
	int k;
	MPCR *B, *Aptr, *TempCR;
	POLYCR CURSOR;

	if (Pptr == NULL)
		return ZEROCR();
	else
		Aptr = COPYCR(Pptr->COEF);
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		if (CURSOR->NEXT == NULL)
			k = DEGREEPCR(CURSOR);
		else
			k = DEGREEPCR(CURSOR) - DEGREEPCR(CURSOR->NEXT);
		B = POWERCR(Cptr, (unsigned int)k);
		TempCR = Aptr;
		Aptr = MULTCR(Aptr, B);
		FREEMPCR(B);
		FREEMPCR(TempCR);
		CURSOR = CURSOR->NEXT;
		if (CURSOR != NULL)
		{
			TempCR = Aptr;
			Aptr = ADDCR(Aptr, CURSOR->COEF);
			FREEMPCR(TempCR);
		}
	}
	return (Aptr);
}

POLYCR DIVPCR(POLYCR Fptr, POLYCR Gptr)
/*
 * division of polynomials: see Knuth, Vol 2, p.402 
 * and H. Flanders, Scientific Pascal, p.510.
 */
{
	POLYCR Htmp, CURSOR, Qptr, Temp, F;
	int j, d;
	MPCR *A, *G, *TempCR;

	Qptr = NULL;
	d = DEGREEPCR(Gptr);
	j = DEGREEPCR(Fptr) - d;
	if (j < 0)
		return (Qptr);
	else
	{
		F = COPYPCR(Fptr);
		G = LEADPCR(Gptr);
		while ((j = DEGREEPCR(F)) >= d)
		{
			j = j - d;
			A = LEADPCR(F);
			TempCR = A;
			A = RATIOCR(A, G);
			FREEMPCR(TempCR);
			Htmp = COPYPCR(Gptr);
			CURSOR = Htmp;
			do
			{
				CURSOR->DEG = j + CURSOR->DEG;
				TempCR = CURSOR->COEF;
				CURSOR->COEF = MINUSCR(CURSOR->COEF);
				FREEMPCR(TempCR);
				TempCR = CURSOR->COEF;
				CURSOR->COEF = MULTCR(CURSOR->COEF, A);
				FREEMPCR(TempCR);
				CURSOR = CURSOR->NEXT;
			}
			while (CURSOR != NULL);
			Temp = ADDPCR(F, Htmp);
			DELETE_PCR(Htmp);
			DELETE_PCR(F);
			F = Temp;
			PINSERTPCR(j, A, &Qptr, 0);
			FREEMPCR(A);
		}
		FREEMPCR(G);
		DELETE_PCR(F);
		return (Qptr);
	}
}

POLYCR MODPCR(POLYCR Fptr, POLYCR Gptr)
/*
 * division of polynomials: see Knuth, Vol 2, p.402 
 * and H. Flanders, Scientific Pascal, p.510.
 */
{
	POLYCR Htmp, CURSOR, Temp, F;
	int j, d;
	MPCR *A, *G, *TempCR;

	d = DEGREEPCR(Gptr);
	j = DEGREEPCR(Fptr) - d;
	F = COPYPCR(Fptr);
	if (j < 0)
		return (F);
	else
	{
		G = LEADPCR(Gptr);
		while ((j = DEGREEPCR(F)) >= d)
		{
			j = j - d;
			A = LEADPCR(F);
			TempCR = A;
			A = RATIOCR(A, G);
			FREEMPCR(TempCR);
			Htmp = COPYPCR(Gptr);
			CURSOR = Htmp;
			do
			{
				CURSOR->DEG = j + CURSOR->DEG;
				TempCR = CURSOR->COEF;
				CURSOR->COEF = MINUSCR(CURSOR->COEF);
				FREEMPCR(TempCR);
				TempCR = CURSOR->COEF;
				CURSOR->COEF = MULTCR(CURSOR->COEF, A);
				FREEMPCR(TempCR);
				CURSOR = CURSOR->NEXT;
			}
			while (CURSOR != NULL);
			Temp = ADDPCR(F, Htmp);
			DELETE_PCR(F);
			F = Temp;
			DELETE_PCR(Htmp);
			FREEMPCR(A);
		}
		FREEMPCR(G);
		return (F);
	}
}

POLYCR MONICPCR(POLYCR Pptr)
/*
 * converts Pptr to a monic polynomial by multiplying it by 1/LEADPCR(Pptr).
 */
{
	MPCR *G, *TempCR;
	POLYCR P;

	G = LEADPCR(Pptr);
	TempCR = G;
	G = INVERSECR(G);
	FREEMPCR(TempCR);
	P = SCALARPCR(G, Pptr);
	FREEMPCR(G);
	return (P);
}

POLYCR DERIVPCR(POLYCR Pptr)
/*
 * returns the derivative of a polynomial.
 */
{
	POLYCR CURSOR, Sptr;
	MPI *G, *TempI;
	MPCR *F, *TempCR;

	if (DEGREEPCR(Pptr) == 0)
		return (NULL);
	else
	{
		TempI = ONEI();
		Sptr = COPYPCR(Pptr);
  		CURSOR = Sptr;
		while (CURSOR != NULL)
		{
			F = BUILDMPCR();
			G = CHANGE((unsigned int) CURSOR->DEG);
			F->R = RATIOI(G, TempI);
			F->I = ZEROR();
			FREEMPI(G);
			TempCR = CURSOR->COEF;
			CURSOR->COEF = MULTCR(CURSOR->COEF, F);
			FREEMPCR(F);
			FREEMPCR(TempCR);
			CURSOR->DEG = CURSOR->DEG - 1;
			if (CURSOR->NEXT == NULL)
				break;
			else if ((CURSOR->NEXT)->DEG == 0)
			{
				DELETE_PCR(CURSOR->NEXT);
				CURSOR->NEXT = NULL;
				break;
			}
			else
				CURSOR = CURSOR->NEXT;
		}
		FREEMPI(TempI);
		return (Sptr);
	}
}

POLYCR GCDPCR(POLYCR Pptr, POLYCR Qptr)
/*
 * returns the gcd of polynomials Pptr and Qptr.
 * see Knuth, Vol 2, p.403-408 and H. Flanders, Scientific Pascal, p.510.
 */
{
	POLYCR Ptmp, Qtmp, Rtmp;
	MPCR *TempCR;
	int j;
	
	if (Qptr == NULL)
	{
		if (Pptr == NULL)
			return (NULL);
		else
			return (MONICPCR(Pptr));
	}
	else if (Pptr == NULL)
	{
		if (Qptr == NULL)
			return (NULL);
		else
			return (MONICPCR(Qptr));
	}
	else
	{
		Ptmp = COPYPCR(Pptr);
		Qtmp = COPYPCR(Qptr);
                j = DEGREEPCR(Qtmp);
		while ((Qtmp != NULL) && ((j = DEGREEPCR(Qtmp)) != 0))
		{
			Rtmp =	MODPCR(Ptmp, Qtmp);
			DELETE_PCR(Ptmp);
			Ptmp = Qtmp;
			Qtmp = Rtmp;
		}
		if (j == 0)
		{
			TempCR = Qtmp->COEF;
			Qtmp->COEF = ONECR();
			FREEMPCR(TempCR);
			DELETE_PCR(Ptmp);
			return (Qtmp);
		}
		else
		{
			Qtmp = MONICPCR(Ptmp);
			DELETE_PCR(Ptmp);
			return (Qtmp);
		}
	}
}

void SQUAREFREEPCR(POLYCR Pptr)
/*
 * returns the "squarefree decomposition" of Pptr:= g[1]g{2]^2...g[m]^m,
 * where g[i] is squarefree and is the product of the irreducible factors of
 * multiplicity i (if any), else equals 1. See L. Childs, Introduction to 
 * Higher Algebra, p.159.
 */
{
	POLYCR Dtmp, Ftmp, Gtmp, Htmp, Ptmp, Ttmp, Ttmp1;
	unsigned int i = 1;

	Ptmp = COPYPCR(Pptr);
	Ftmp = DERIVPCR(Ptmp);
	Dtmp = GCDPCR(Ptmp, Ftmp);
	if (DEGREEPCR(Dtmp) == 0)
	{
		if (Pptr->DEG)
		{
			printf("G[1] = ");
			PRINTPCR(Pptr);
			printf("\n");
		}
		DELETE_PCR(Ptmp);
		DELETE_PCR(Ftmp);
		DELETE_PCR(Dtmp);
		return;
	}
	else
	{
		Gtmp = DIVPCR(Ptmp, Dtmp);
		while (DEGREEPCR(Dtmp))
		{
			DELETE_PCR(Ptmp);
			DELETE_PCR(Ftmp);
			Htmp = Gtmp;
			Ptmp = Dtmp;
			Ftmp = DERIVPCR(Ptmp);
			Dtmp = GCDPCR(Ptmp, Ftmp);
			Gtmp = DIVPCR(Ptmp, Dtmp);
			Ttmp = DIVPCR(Htmp, Gtmp);
			DELETE_PCR(Htmp);
			Ttmp1 = MONICPCR(Ttmp);
			DELETE_PCR(Ttmp);
			if (Ttmp1->DEG)
			{
				printf("G[%u] = ", i);
				PRINTPCR(Ttmp1);
				printf("\n");
			}
			DELETE_PCR(Ttmp1);
			i++;
		}
		Ttmp = MONICPCR(Gtmp);
		if (Ttmp->DEG)
		{
			printf("G[%u] = ", i);
			PRINTPCR(Ttmp);
			printf("\n");
		}
		DELETE_PCR(Ptmp);
		DELETE_PCR(Ttmp);
		DELETE_PCR(Ftmp);
		DELETE_PCR(Gtmp);
		DELETE_PCR(Dtmp);
		return;
	}
}

POLYCR ONEPCR()
/*
 * returns the constant polynomial 1.
 */
{
	POLYCR Pptr;

	Pptr = CREATECR();
	Pptr->DEG = 0;
	Pptr->COEF = ONECR();
	Pptr->NEXT = NULL;
	return (Pptr);
}

POLYCR CONSTANTPCR(MPCR *Aptr)
/*
 * returns the constant polynomial whose constant term is *Aptr.
 */
{
	POLYCR Pptr;	

	if (EQZEROCR(Aptr))
		Pptr = NULL;
	else
	{
		Pptr = CREATECR();
		Pptr->DEG = 0;
		Pptr->COEF = COPYCR(Aptr);
		Pptr->NEXT = NULL;
	}
	return (Pptr);
}

POLYCR LINEARPCR(MPCR *Aptr)
/*
 * returns the linear polynomial x + *Aptr.
 */
{
	POLYCR Pptr;
	MPCR *TempCR;

	Pptr = CONSTANTPCR(Aptr);
	TempCR = ONECR();
	PINSERTPCR(1, TempCR, &Pptr, 0);
	FREEMPCR(TempCR);
	return (Pptr);
}

unsigned int CARDPCR(POLYCR Pptr)
/*
 * returns the number of terms in Pptr (0 if Pptr = NULL).
 */
{
	unsigned int i = 0;
	POLYCR CURSOR;

	if (Pptr == NULL)
		return (0);
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		i++;
		CURSOR = CURSOR->NEXT;
	}
	return (i);
}

POLYCR POWERPCR(POLYCR Pptr, USI n)
/*
 *  returns Pptr ^ n, where 0 <= n < R0 * R0.
 */
{
	POLYCR B, Eptr, Tmp;

	Eptr = ONEPCR();
	if (n == 0)
		return (Eptr);
	B = COPYPCR(Pptr);
	while (1)
        {
		if (n & 1) 
		{
			Tmp = Eptr;
			Eptr = MULTPCR(Tmp, B);
			DELETE_PCR(Tmp);
			if (n == 1)
			{
				DELETE_PCR(B);
				return (Eptr);
			}
		}
		Tmp = B;
		B = MULTPCR(Tmp, Tmp);
		DELETE_PCR(Tmp);
		n = n >> 1;
	}
}

POLYCR SHIFTPCR(POLYCR Pptr, MPCR *Aptr)
/*
 * returns Pptr(x + *Aptr).
 */
{
	POLYCR Sptr, Tptr1, Tptr2, Tptr3, CURSOR;

	if (DEGREEPCR(Pptr) <= 0)
		return (COPYPCR(Pptr));
	Sptr = NULL;
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		Tptr1 = LINEARPCR(Aptr);
		Tptr2 = POWERPCR(Tptr1, (unsigned int)(CURSOR->DEG));
		DELETE_PCR(Tptr1);
		Tptr1 = SCALARPCR(CURSOR->COEF, Tptr2);
		DELETE_PCR(Tptr2);
		Tptr3 = Sptr;
		Sptr = ADDPCR(Sptr, Tptr1);
		DELETE_PCR(Tptr1);
		DELETE_PCR(Tptr3);
		CURSOR = CURSOR->NEXT;
	}
	return (Sptr);
}

POLYCR CONJPCR(POLYCR Pptr)
/*
 * replacing the coefficients of Pptr by their conjugates.
 */
{
	POLYCR Sptr, Rtmp;
	MPCR *Temp2CR;

	if (Pptr == NULL)
		Sptr = NULL;
	else
	{
		Sptr = COPYPCR(Pptr);
		Rtmp = Sptr;
		while (Rtmp != NULL)
		{
			Temp2CR = Rtmp->COEF;
			Rtmp->COEF = CONJCR(Rtmp->COEF);
			FREEMPCR(Temp2CR);
			Rtmp = Rtmp->NEXT;
		}
	}
	return (Sptr);
}

POLYCR NORMPCR(POLYCR Pptr)
/*
 * returns the norm of Pptr.
 */
{
	POLYCR Tptr1, Tptr2;

	if (Pptr == NULL)
		return (NULL);
	Tptr1 = CONJPCR(Pptr);
	Tptr2 = MULTPCR(Pptr, Tptr1);
	DELETE_PCR(Tptr1);
	return (Tptr2);
	
}

POLYCR *ASQUAREFREEPCR(POLYCR Pptr, USI **D, USI *tptr)
/*
 * returns the "squarefree decomposition" of Pptr, a non-constant polynomial:
 * Pptr = G[0]^D[0]...G[*tpr - 1]^D[*tptr - 1],
 * where G[i] is squarefree and is the product of the irreducible factors of
 * multiplicity D[i]. See L. Childs, Introduction to Higher Algebra, p.159.
 */
{
	POLYCR Ttmp, Ttmp1, Dtmp, Ftmp, Gtmp, Htmp, Ptmp, *G;
	unsigned int i = 1, j = 0, n;

	n = (unsigned int)(Pptr->DEG);
	Ptmp = COPYPCR(Pptr);
	Ftmp = DERIVPCR(Ptmp);
	Dtmp = GCDPCR(Ptmp, Ftmp);
	G = (POLYCR *)mmalloc(n * sizeof(POLYCR));
	*D = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	if (DEGREEPCR(Dtmp) == 0)
	{
		G[j] = Ptmp;
		(*D)[j] = i;
		j++;
		DELETE_PCR(Ftmp);
		DELETE_PCR(Dtmp);
		*tptr = j;
		return (G);
	}
	else
	{
		Gtmp = DIVPCR(Ptmp, Dtmp);
		while (DEGREEPCR(Dtmp))
		{
			DELETE_PCR(Ptmp);
			DELETE_PCR(Ftmp);
			Htmp = Gtmp;
			Ptmp = Dtmp;
			Ftmp = DERIVPCR(Ptmp);
			Dtmp = GCDPCR(Ptmp, Ftmp);
			Gtmp = DIVPCR(Ptmp, Dtmp);
			Ttmp = DIVPCR(Htmp, Gtmp);
			DELETE_PCR(Htmp);
			Ttmp1 = MONICPCR(Ttmp);
			DELETE_PCR(Ttmp);
			if (Ttmp1->DEG > 0)
			{
				G[j] = Ttmp1;
				(*D)[j] = i;
				j++;
			}
			else
				DELETE_PCR(Ttmp1);
			i++;
		}
		DELETE_PCR(Ptmp);
		DELETE_PCR(Ftmp);
		G[j] = Gtmp;
		(*D)[j] = i;
		j++;
		DELETE_PCR(Dtmp);
		*tptr = j;
		return (G);
	}
}

POLYCR NORM_TESTPCR(POLYCR Pptr, POLYCR *Rptr, MPCR **Aptr)
/*
 * finds first non-negative integer *Aptr such that *Rptr = N(Pptr(x + *Aptr))
 * is squarefree. Returns Pptr(x + *Aptr).
 */
{
	MPCR *S, *T, *TmpCR;
	POLYCR Tmptr1, Tmptr2, Tmptr3, G, R;
	int t;
	
	S = BUILDMPCR();
	S->R = ZEROR();
	S->I = ONER();
	T = ZEROCR();
	G = COPYPCR(Pptr);
	while (1)
	{
		R = NORMPCR(G);
		Tmptr2 = DERIVPCR(R);
		Tmptr3 = GCDPCR(R, Tmptr2);
		DELETE_PCR(Tmptr2);
		t = Tmptr3->DEG;
		DELETE_PCR(Tmptr3);
		if (t == 0)
			break;
		else
		{
			TmpCR = T;
			T = ADDCR(S, T);
			FREEMPCR(TmpCR);
			Tmptr1 = G;
			G = SHIFTPCR(G, S);
			DELETE_PCR(Tmptr1);
			DELETE_PCR(R);
		}
	}
	*Aptr = T;
	FREEMPCR(S);
	*Rptr = R;
	return (G);
}

POLYCR PR_TO_PCR(POLYR Pptr)
/*
 * converts a polynomial with MPR coefficients into one with MPCR coefficients.
 */
{
	POLYR CURSOR;
	POLYCR P;
	MPCR *TempCR;

	P = NULL;
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		TempCR = BUILDMPCR();
		TempCR->R = COPYR(CURSOR->COEF);
		TempCR->I = ZEROR();
		PINSERTPCR(CURSOR->DEG, TempCR, &P, 0);
		FREEMPCR(TempCR);
		CURSOR = CURSOR->NEXT;
	}	
	return (P);
}

POLYR PCR_TO_PR(POLYCR Pptr)
/*
 * converts a polynomial with real MPCR coefficients into  a POLYR.
 */
{
	POLYR P;
	POLYCR CURSOR;
	MPR *TempR;

	P = NULL;
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		TempR = COPYR((CURSOR->COEF)->R);
		PINSERTPR(CURSOR->DEG, TempR, &P, 0);
		FREEMPR(TempR);
		CURSOR = CURSOR->NEXT;
	}	
	return (P);
}

POLYCR *TRAGERPCR(POLYCR Pptr, USI *nptr)
/*
 * B. Trager's way of factorizing Pptr using NORMPCR(Pptr). 
 * Returns the distinct irreducible factors of a squarefree POLYCR and their
 * number *nptr.  See B. Trager, "Algebraic Factoring and Rational Function 
 * Integration", Proc. of the 1976 ACM Symposium on Symbolic and Algebraic 
 * Computation.
 */
{
	MPCR *B, *TmpCR;
	POLYCR *FACTORSCR, Tmptr, Tmptr1, Tmptr2, Tmptr3;
	POLYR *FACTORSR, TmptrR;
	unsigned int *a, i, m, n, t;

	t = (unsigned int)DEGREEPCR(Pptr);
	m = 2 * t;
	Tmptr = NORM_TESTPCR(Pptr, &Tmptr1, &B);	
	TmptrR = PCR_TO_PR(Tmptr1);
	DELETE_PCR(Tmptr1);
	FACTORSR = KALTOFENPR(TmptrR, &a, &n);
	/* note: DEGREE(TmptrR) = m */
	DELETE_PR(TmptrR);
	FACTORSCR = (POLYCR *)mmalloc(t * sizeof(POLYCR));
	for (i = 0; i < n; i++)
	{
		FACTORSCR[i] = PR_TO_PCR(FACTORSR[i]);
		DELETE_PR(FACTORSR[i]);
	}
	ffree((char *)FACTORSR, m * sizeof(POLYR));
	ffree((char *)a, m * sizeof(unsigned int));
	*nptr = n;
	if (n == 1)
	{
		FREEMPCR(B);
		Tmptr1 = FACTORSCR[0];
		FACTORSCR[0] = MONICPCR(Pptr);
		DELETE_PCR(Tmptr1);
		DELETE_PCR(Tmptr);
		return(FACTORSCR);
	}
	TmpCR = B;
	B = MINUSCR(B);
	FREEMPCR(TmpCR);
	for (i = 0; i < n; i++)
	{
		Tmptr2 = GCDPCR(Tmptr, FACTORSCR[i]);
		Tmptr3 = Tmptr;
		Tmptr = DIVPCR(Tmptr, Tmptr2);
		DELETE_PCR(Tmptr3);
		Tmptr1 = FACTORSCR[i];
		FACTORSCR[i] = SHIFTPCR(Tmptr2, B);
		DELETE_PCR(Tmptr1);
		DELETE_PCR(Tmptr2);
	}
	FREEMPCR(B);
	DELETE_PCR(Tmptr);
	return (FACTORSCR);
}

POLYCR *TRAGER_PCR(POLYCR Pptr, USI **C, USI *nptr)
/*
 * Factors a polynomial over Q(i)[x] using B. Trager's algorithm.
 * Returns the content of Pptr and the factorization into irreducibles.
 * *Q[0],...,*Q[*nptr - 1] are the essentially distinct irreducibles,
 * C[0],...,C[*nptr - 1] are the corresponding multiplicities.
 */
{
	unsigned int k, m, n, i, j = 0, t, *D;
	POLYCR P, *SQ, *Q, *K;

	n = (unsigned int)DEGREEPCR(Pptr);
	P = COPYPCR(Pptr);
	Q = (POLYCR *)mmalloc(n * sizeof(POLYCR));	
	*C = (unsigned int *)mmalloc(n * sizeof(unsigned int));
	SQ = ASQUAREFREEPCR(P, &D, &t);
	DELETE_PCR(P);
	for (i = 0; i < t; i++)
	{
		K = TRAGERPCR(SQ[i], &m);
		for (k = 0; k <= m - 1; k++)
		{
			Q[j] = K[k];
			(*C)[j] = D[i];	
			j++;
		}
		ffree((char *)K, (unsigned int)(SQ[i]->DEG) * sizeof(POLYCR));
		DELETE_PCR(SQ[i]);
	}
	*nptr = j;
	ffree((char *)D, n * sizeof(unsigned int));
	ffree((char *)SQ, n * sizeof(POLYCR));
	return (Q);
}
