/* pR.c */

#include <stdio.h>
#include <stdlib.h>
#include "integer.h"
#include "fun.h"
extern long int nettbytes;

int DEGREEPR(POLYR Pptr)
/*
 * returns the degree of the polynomial Pptr.
 */
{
	if (Pptr == NULL)
		return (-1);
	else
		return (Pptr->DEG);
	
}

MPR *LEADPR(POLYR Pptr)
/*
 * *Aptr is the  leading coefficient of the polynomial Pptr.
 */
{
	if (Pptr == NULL)
		return ZEROR();
	else
		return COPYR(Pptr->COEF);
}

void DELETE_PR(POLYR Pptr)
/*
 * returns back to the system storage dynamically allocated to the 
 * linked list which Pptr points to.
 */
{
	POLYR Qtmp;
	POLYR orig;
	
	if(!Pptr) return;
	orig = Pptr;
	while (Pptr != NULL)
	{
 		Qtmp = Pptr->NEXT;
		FREEMPR(Pptr->COEF);
#ifdef DEBUG
		SAVEPR(Pptr);
#endif
		Pptr= Qtmp;
	}
#ifndef DEBUG
	SAVEPR(orig);
#endif
}

void PINSERTPR(int DEG, MPR *COEF, POLYR *Pptr, USI OPT)
/*
 * inserts a copy of a single term *Qptr into polynomial *Pptr.
 * It has two options that control the insertion in case a term of *Pptr already
 * has the degree of *Qptr, If OPT = 0, then the term is replaced; if OPT = 1,
 * then the coefficient of *Qptr is added to that of the corresponding term of
 *  *Pptr.
 */
{
	POLYR CURSOR, Ttmp;
	MPR *TempR;

	if (*Pptr == NULL)
	{
		*Pptr = CREATER();
		(*Pptr)->DEG = DEG;
		(*Pptr)->COEF = COPYR(COEF);
		(*Pptr)->NEXT = NULL;
		return;
	}
	else
	{
		CURSOR = *Pptr;
		while ((CURSOR->DEG > DEG) && (CURSOR->NEXT != NULL))
			CURSOR = CURSOR->NEXT;
		if (CURSOR->DEG < DEG)
		{
			/* Qptr inserted before CURSOR */
			Ttmp = CREATER();
			Ttmp->DEG = CURSOR->DEG;
			Ttmp->COEF = CURSOR->COEF;
			Ttmp->NEXT = CURSOR->NEXT;
			CURSOR->DEG = DEG;
			CURSOR->COEF = COPYR(COEF);
			CURSOR->NEXT = Ttmp;
			return;
		}
		else if (CURSOR->DEG == DEG)
		{
			if (OPT == 0)
				CURSOR->COEF = COPYR(COEF);
			else
			{
				TempR = CURSOR->COEF;
				CURSOR->COEF = ADDR(CURSOR->COEF, COEF);
				FREEMPR(TempR);
			}
			return;
		}
		else
		{
			/* CURSOR->DEG > Qptr->DEG and CURSOR->NEXT === NULL */
			/* Qptr inserted at end */
			Ttmp = CREATER();
			Ttmp->DEG = DEG;
			Ttmp->COEF = COPYR(COEF);
			Ttmp->NEXT = NULL;
			CURSOR->NEXT = Ttmp;
			return;
		}
	}
}

void PURGEPR(POLYR *Pptr)
/*
 * gets rid of any zero coefficients in the polynomial Pptr.
 */
{
	POLYR R1tmp, R2tmp;

	if (*Pptr != NULL)
	{
		R1tmp = *Pptr;
		R2tmp = (*Pptr)->NEXT;
		while (R2tmp != NULL)
		{
			/* purge non-leading terms */
			if (EQZEROR(R2tmp->COEF))
			{
				R1tmp->NEXT = R2tmp->NEXT;
				R2tmp->NEXT = NULL;
				DELETE_PR(R2tmp);
				R2tmp = R1tmp->NEXT;
			}
			else
			{
				R1tmp = R2tmp;
				R2tmp = R2tmp->NEXT;
			}
		}
		if (EQZEROR((*Pptr)->COEF))
		{
			/* purge leading term */
			R1tmp = *Pptr;
			*Pptr = (*Pptr)->NEXT;
			R1tmp->NEXT = NULL;
			DELETE_PR(R1tmp);
		}
	}
	return;
}

POLYR INPUTPR(USI *ptr)
/*
 * Inputting a polynomial Pptr from the terminal.
 * *ptr = 0 indicates some garbage input and a default POLYR as output. 
 */
{
	POLYR Pptr;
	int k;
	char *line, *p;
	MPR *TempR;
	
	Pptr = NULL;
	*ptr = 1;
	do
	{
		printf("Enter the degree of new term, (-1 to terminate): ");
		if (scanfd10(&k) == 0)
		{
			*ptr = 0;
			k = -1;
		}
		if (k >= 0)
		{
			printf("Enter the coefficient of x^%d: ", k);
			p = line = Gets();
		        TempR = INPUTSR(&p, ptr); 
			PINSERTPR(k, TempR, &Pptr, 0);
			FREEMPR(TempR);
			if (*ptr == 0)
				k = -1;
			free(line);
		}
	}
	while (k >= 0);
	PURGEPR(&Pptr);
	return Pptr;
}

POLYR COPYPR(POLYR Pptr)
/*
 * makes a copy of the polynomial Pptr.
 */
{
	POLYR Rtmp, Sptr;

	Sptr = NULL;
	Rtmp = Pptr;
	while (Rtmp != NULL)
	{
		PINSERTPR(Rtmp->DEG, Rtmp->COEF, &Sptr, 0);
		Rtmp = Rtmp->NEXT;
	}
	return (Sptr);
}
 
void PRINTPR(POLYR Pptr)
/*
 * outputs the terms of the polynomial Pptr.
 */
{
	FPRINTPR(stdout, Pptr);
	return;
}

POLYR SCALARPR(MPR *Cptr, POLYR Pptr)
/*
 * multiplying the polynomial Pptr by the MPR *Cptr.
 */
{
	POLYR Sptr, Rtmp;
	MPR *Temp2R;

	if (Pptr == NULL || EQZEROR(Cptr))
		Sptr = NULL;
	else
	{
		Sptr = COPYPR(Pptr);
		Rtmp = Sptr;
		while (Rtmp != NULL)
		{
			Temp2R = Rtmp->COEF;
			Rtmp->COEF = MULTR(Rtmp->COEF, Cptr);
			FREEMPR(Temp2R);
			Rtmp = Rtmp->NEXT;
		}
	}
	return (Sptr);
}

POLYR ADDPR(POLYR Pptr, POLYR Qptr)
/*
 * returns the sum of two polynomials.
 */
{
	POLYR CURSOR, Sptr;

		Sptr = COPYPR(Pptr);
		CURSOR = Qptr;
		while (CURSOR != NULL)
		{
			PINSERTPR(CURSOR->DEG, CURSOR->COEF, &Sptr, 1);
			CURSOR = CURSOR->NEXT;
		}
		PURGEPR(&Sptr);
		return (Sptr);
}

POLYR SUBPR(POLYR Pptr, POLYR Qptr)
/*
 * returns the difference Pptr - Qptr of two polynomials.
 */
{
	POLYR CURSOR, Sptr;
	MPR *TempR;

	Sptr = COPYPR(Pptr);
	if (Qptr == NULL)
		return (Sptr);
	CURSOR = Qptr;
	while (CURSOR != NULL)
	{
		TempR = MINUSR(CURSOR->COEF);
		PINSERTPR(CURSOR->DEG, TempR, &Sptr, 1);
		FREEMPR(TempR);
		CURSOR = CURSOR->NEXT;
	}
	PURGEPR(&Sptr);
	return (Sptr);
}

POLYR MULTPR(POLYR Pptr, POLYR Qptr)
/*
 * returns the product of two polynomials.
 */
{
	POLYR Sptr, CURSOR_P, CURSOR_Q;
	MPR *TempR;
	int k;

	if (Pptr == NULL || Qptr == NULL)
		return (NULL);
	Sptr = NULL;
	CURSOR_P = Pptr;
	while (CURSOR_P != NULL)
	{
		CURSOR_Q = Qptr;
		while (CURSOR_Q != NULL)
		{
			k = CURSOR_P->DEG + CURSOR_Q->DEG;
			TempR = MULTR(CURSOR_P->COEF, CURSOR_Q->COEF);
			PINSERTPR(k, TempR, &Sptr, 1);
			FREEMPR(TempR);
			CURSOR_Q = CURSOR_Q->NEXT;
		}
		CURSOR_P = CURSOR_P->NEXT;
	}
	PURGEPR(&Sptr);
	return (Sptr);
}

MPR *VALPR(POLYR Pptr, MPR *Cptr)
/*
 *  Evaluating the polynomial Pptr at the MPR *Cptr.
 */
{
	int k;
	MPR *B, *Aptr, *TempR;
	POLYR CURSOR;

	if (Pptr == NULL)
		return ZEROR();
	else
		Aptr = COPYR(Pptr->COEF);
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		if (CURSOR->NEXT == NULL)
			k = DEGREEPR(CURSOR);
		else
			k = DEGREEPR(CURSOR) - DEGREEPR(CURSOR->NEXT);
		B = POWERR(Cptr, (unsigned int)k);
		TempR = Aptr;
		Aptr = MULTR(Aptr, B);
		FREEMPR(B);
		FREEMPR(TempR);
		CURSOR = CURSOR->NEXT;
		if (CURSOR != NULL)
		{
			TempR = Aptr;
			Aptr = ADDR(Aptr, CURSOR->COEF);
			FREEMPR(TempR);
		}
	}
	return (Aptr);
}

POLYR DIVPR(POLYR Fptr, POLYR Gptr)
/*
 * division of polynomials: see Knuth, Vol 2, p.402 
 * and H. Flanders, Scientific Pascal, p.510.
 */
{
	POLYR Htmp, CURSOR, Qptr, Temp, F;
	int j, d;
	MPR *A, *G, *TempR;

	Qptr = NULL;
	d = DEGREEPR(Gptr);
	j = DEGREEPR(Fptr) - d;
	if (j < 0)
		return (Qptr);
	else
	{
		F = COPYPR(Fptr);
		G = LEADPR(Gptr);
		while ((j = DEGREEPR(F)) >= d)
		{
			j = j - d;
			A = LEADPR(F);
			TempR = A;
			A = RATIOR(A, G);
			FREEMPR(TempR);
			Htmp = COPYPR(Gptr);

			CURSOR = Htmp;
			do
			{
				CURSOR->DEG = j + CURSOR->DEG;
				TempR = CURSOR->COEF;
				CURSOR->COEF = MINUSR(CURSOR->COEF);
				FREEMPR(TempR);
				TempR = CURSOR->COEF;
				CURSOR->COEF = MULTR(CURSOR->COEF, A);
				FREEMPR(TempR);
				CURSOR = CURSOR->NEXT;
			}
			while (CURSOR != NULL);

			Temp = F;
			F = ADDPR(F, Htmp);
			DELETE_PR(Htmp);
			DELETE_PR(Temp);
			PINSERTPR(j, A, &Qptr, 0);
			FREEMPR(A);
		}
		FREEMPR(G);
		DELETE_PR(F);
		return (Qptr);
	}
}

POLYR MODPR(POLYR Fptr, POLYR Gptr)
/*
 * division of polynomials: see Knuth, Vol 2, p.402 
 * and H. Flanders, Scientific Pascal, p.510.
 */
{
	POLYR Htmp, CURSOR, Temp, F;
	int j, d;
	MPR *A, *G, *TempR;

	d = DEGREEPR(Gptr);
	j = DEGREEPR(Fptr) - d;
	F = COPYPR(Fptr);
	if (j < 0)
		return (F);
	else
	{
		G = LEADPR(Gptr);
		while ((j = DEGREEPR(F)) >= d)
		{
			j = j - d;
			A = LEADPR(F);
			TempR = A;
			A = RATIOR(A, G);
			FREEMPR(TempR);
			Htmp = COPYPR(Gptr);
			CURSOR = Htmp;
			do
			{
				CURSOR->DEG = j + CURSOR->DEG;
				TempR = CURSOR->COEF;
				CURSOR->COEF = MINUSR(CURSOR->COEF);
				FREEMPR(TempR);
				TempR = CURSOR->COEF;
				CURSOR->COEF = MULTR(CURSOR->COEF, A);
				FREEMPR(TempR);
				CURSOR = CURSOR->NEXT;
			}
			while (CURSOR != NULL);
			Temp = F;
			F = ADDPR(F, Htmp);
			DELETE_PR(Temp);
			DELETE_PR(Htmp);
			FREEMPR(A);
		}
		FREEMPR(G);
		return (F);
	}
}

POLYR MONICPR(POLYR Pptr)
/*
 * converts Pptr to a monic polynomial by multiplying it by 1/LEADPR(Pptr).
 */
{
	MPR *G, *TempR;
	POLYR P;

	G = LEADPR(Pptr);
	TempR = G;
	G = INVERSER(G);
	FREEMPR(TempR);
	P = SCALARPR(G, Pptr);
	FREEMPR(G);
	return (P);
}

MPI *LCM_PR(POLYR Pptr)
/*
 * returns the LCM of the denominators of the coefficients of Pptr.
 */
{
	MPI *TempI, *Aptr;
	POLYR CURSOR;

	CURSOR = Pptr;
	Aptr = COPYI((CURSOR->COEF)->D);
	CURSOR = CURSOR->NEXT;
	while (CURSOR != NULL)
	{
		TempI = Aptr;
		Aptr = LCM(Aptr, (CURSOR->COEF)->D);
		FREEMPI(TempI);
		CURSOR = CURSOR->NEXT;
	}
	return (Aptr);
}

POLYR COPYPI_TO_PR(POLYI Pptr)
/*
 * makes a POLYR copy of the POLYI Pptr.
 */
{
	POLYR Sptr;
	POLYI Rtmp;
	MPI *TempI;
	MPR *TempR;

	TempI = ONEI();
	Sptr = NULL;
	Rtmp = Pptr;
	while (Rtmp != NULL)
	{
		TempR = RATIOI(Rtmp->COEF, TempI);
		PINSERTPR(Rtmp->DEG, TempR, &Sptr, 0);
		FREEMPR(TempR);
		Rtmp = Rtmp->NEXT;
	}
	FREEMPI(TempI);
	return (Sptr);
}

POLYI COPYPR_TO_PI(POLYR Pptr)
/*
 * makes a POLYI copy of POLYR Pptr with rational integer coefficients.
 */
{
	POLYI Sptr;
	POLYR Rtmp; 

	Sptr = NULL;
	Rtmp = Pptr;
	while (Rtmp != NULL)
	{
		PINSERTPI(Rtmp->DEG, Rtmp->COEF->N, &Sptr, 0);
		Rtmp = Rtmp->NEXT;
	}
	return (Sptr);
}

POLYR GCDPR(POLYR Pptr, POLYR Qptr)
/*
 * returns the gcd of polynomials Pptr and Qptr.
 * see Knuth, Vol 2, p.403-408 and H. Flanders, Scientific Pascal, p.510.
 */
{
	POLYI P1tmp, Q1tmp, Stmp;
	POLYR Ptmp, Qtmp, S1tmp;
	MPI *F, *G, *TempI;
	MPR *J, *K;
	
	if (Qptr == NULL)
	{
		if (Pptr == NULL)
			return (NULL);
		else
			return (MONICPR(Pptr));
	}
	if (Pptr == NULL)
	{
		if (Qptr == NULL)
			return (NULL);
		else
			return (MONICPR(Qptr));
	}
	TempI = ONEI();
	F = LCM_PR(Pptr);
	G = LCM_PR(Qptr);
	J = RATIOI(F, TempI);
	K = RATIOI(G, TempI);
	FREEMPI(F);
	FREEMPI(G);
	FREEMPI(TempI);
	Ptmp = SCALARPR(J, Pptr);
	Qtmp = SCALARPR(K, Qptr);
	FREEMPR(J);
	FREEMPR(K);
	P1tmp = COPYPR_TO_PI(Ptmp);
	Q1tmp = COPYPR_TO_PI(Qtmp);
	DELETE_PR(Ptmp);
	DELETE_PR(Qtmp);
	Stmp = GCDPI(P1tmp, Q1tmp);
	DELETE_PI(P1tmp);
	DELETE_PI(Q1tmp);
	Ptmp = COPYPI_TO_PR(Stmp);
	DELETE_PI(Stmp);
	S1tmp = MONICPR(Ptmp);
	DELETE_PR(Ptmp);
	return (S1tmp);
}

POLYR DERIVPR(POLYR Pptr)
/*
 * returns the derivative of a polynomial.
 */
{
	POLYR CURSOR, Sptr;
	MPI *G, *TempI;
	MPR *F, *TempR;

	if (DEGREEPR(Pptr) == 0)
		return (NULL);
	else
	{
		TempI = ONEI();
		Sptr = COPYPR(Pptr);
  		CURSOR = Sptr;
		while (CURSOR != NULL)
		{
			G = CHANGE((unsigned int) CURSOR->DEG);
			F = RATIOI(G, TempI);
			FREEMPI(G);
			TempR = CURSOR->COEF;
			CURSOR->COEF = MULTR(CURSOR->COEF, F);
			FREEMPR(F);
			FREEMPR(TempR);
			CURSOR->DEG = CURSOR->DEG - 1;
			if (CURSOR->NEXT == NULL)
				break;
			else if ((CURSOR->NEXT)->DEG == 0)
			{
				DELETE_PR(CURSOR->NEXT);
				CURSOR->NEXT = NULL;
				break;
			}
			else
			{
				CURSOR = CURSOR->NEXT;
			}
		}
		FREEMPI(TempI);
		return (Sptr);
	}
}

void SQUAREFREEPR(POLYR Pptr)
/*
 * returns the "squarefree decomposition" of Pptr:= g[1]g{2]^2...g[m]^m,
 * where g[i] is squarefree and is the product of the irreducible factors of
 * multiplicity i (if any), else equals 1. See L. Childs, Introduction to Higher
 * Algebra, p.159.
 */
{
	POLYR Dtmp, Ftmp, Gtmp, Htmp, Ptmp, Ttmp, Ttmp1;
	unsigned int i = 1;

	Ptmp = COPYPR(Pptr);
	Ftmp = DERIVPR(Ptmp);
	Dtmp = GCDPR(Ptmp, Ftmp);
	if (DEGREEPR(Dtmp) == 0)
	{
		printf("G[1] = ");
		PRINTPR(Pptr);
		printf("\n");
		DELETE_PR(Ptmp);
		DELETE_PR(Ftmp);
		DELETE_PR(Dtmp);
		return;
	}
	else
	{
		Gtmp = DIVPR(Ptmp, Dtmp);
		while (DEGREEPR(Dtmp))
		{
			DELETE_PR(Ptmp);
			DELETE_PR(Ftmp);
			Htmp = Gtmp;
			Ptmp = Dtmp;
			Ftmp = DERIVPR(Ptmp);
			Dtmp = GCDPR(Ptmp, Ftmp);
			Gtmp = DIVPR(Ptmp, Dtmp);
			Ttmp = DIVPR(Htmp, Gtmp);
			DELETE_PR(Htmp);
			Ttmp1 = MONICPR(Ttmp);
			DELETE_PR(Ttmp);
			printf("G[%u] = ", i);
			PRINTPR(Ttmp1);
			printf("\n");
			DELETE_PR(Ttmp1);
			i++;
		}
		printf("G[%u] = ", i);
		Ttmp = MONICPR(Gtmp);
		PRINTPR(Ttmp);
		printf("\n");
		DELETE_PR(Ptmp);
		DELETE_PR(Ttmp);
		DELETE_PR(Ftmp);
		DELETE_PR(Gtmp);
		DELETE_PR(Dtmp);
		return;
	}
}

POLYR ONEPR()
/*
 * returns the constant polynomial 1.
 */
{
	POLYR Pptr;

	Pptr = CREATER();
	Pptr->DEG = 0;
	Pptr->COEF = ONER();
	Pptr->NEXT = NULL;
	return (Pptr);
}

POLYR CONSTANTPR(MPR *Aptr)
/*
 * returns the constant polynomial whose constant term is *Aptr.
 */
{
	POLYR Pptr;	

	if (EQZEROR(Aptr))
		Pptr = NULL;
	else
	{
		Pptr = CREATER();
		Pptr->DEG = 0;
		Pptr->COEF = COPYR(Aptr);
		Pptr->NEXT = NULL;
	}
	return (Pptr);
}

POLYR LINEARPR(MPR *Aptr)
/*
 * returns the linear polynomial x + *Aptr.
 */
{
	POLYR Pptr;
	MPR *TempR;

	Pptr = CONSTANTPR(Aptr);
	TempR = ONER();
	PINSERTPR(1, TempR, &Pptr, 0);
	FREEMPR(TempR);
	return (Pptr);
}

unsigned int CARDPR(POLYR Pptr)
/*
 * returns the number of terms in Pptr (0 if Pptr = NULL).
 */
{
	unsigned int i = 0;
	POLYR CURSOR;

	if (Pptr == NULL)
		return (0);
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		i++;
		CURSOR = CURSOR->NEXT;
	}
	return (i);
}

POLYR POWERPR(POLYR Pptr, USI n)
/*
 *  returns Pptr ^ n, where 0 <= n < R0 * R0.
 */
{
	POLYR B, Eptr, Tmp;

	Eptr = ONEPR();
	if (n == 0)
		return (Eptr);
	B = COPYPR(Pptr);
	while (1)
        {
		if (n & 1) 
		{
			Tmp = Eptr;
			Eptr = MULTPR(Tmp, B);
			DELETE_PR(Tmp);
			if (n == 1)
			{
				DELETE_PR(B);
				return (Eptr);
			}
		}
		Tmp = B;
		B = MULTPR(Tmp, Tmp);
		DELETE_PR(Tmp);
		n = n >> 1;
	}
}

POLYI CLEARDENOMPI(POLYR Pptr)
/*
 * multiplies a polynomial by the LCM of the denominators of its coefficients,
 * converting it to a POLYI.
 */
{
	MPI *F, *TempI;
	MPR *J;
	POLYR Ptmp;
	POLYI P1tmp;

	TempI = ONEI();
	F = LCM_PR(Pptr);
	J = RATIOI(F, TempI);
	FREEMPI(F);
	FREEMPI(TempI);
	Ptmp = SCALARPR(J, Pptr);
	FREEMPR(J);
	P1tmp = COPYPR_TO_PI(Ptmp);
	DELETE_PR(Ptmp);
	return (P1tmp);
}
