/* pm.c */

#include <stdio.h>
#include "integer.h"
#include "fun.h"

int SIGN(int m)
/*
 * returns the sign of m. 
 */
{
/*	return((m>0)?1:(m<0?-1:0)); */
	if (m > 0)
		return (1);
	else if (m < 0)
		return (-1);
	else
		return (0);
}

int DEGREEPm(POLYm Pptr)
/*
 * returns the degree of the polynomial Pptr, 
 * for use with polynomials mod m, 1 < m < R0.
 */
{
	if (Pptr == NULL)
		return (-1);
	else
		return (Pptr->DEG);
}

unsigned int LEADPm(POLYm Pptr)
/*
 * returns the  leading coefficient of the polynomial Pptr. 
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	if (Pptr == NULL)
		return (0);
	else
		return (Pptr->COEF);
}

void DELETE_Pm(POLYm Pptr)
/*
 * returns back to the system storage dynamically allocated to the linked list
 * which Pptr points to. For use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm Qtmp, orig;
	
	if(!Pptr) return;
	orig = Pptr;
	while (Pptr != NULL)
	{
 		Qtmp = Pptr->NEXT;
#ifdef DEBUG
		SAVEPm(Pptr);
#endif
		Pptr= Qtmp;
	}
#ifndef DEBUG
	SAVEPm(orig);
#endif
	return;
}


void PINSERTPm(int DEG, USI COEF, POLYm *Pptr, USI OPT, USI m)
/*
 * inserts a copy of a single term *Qptr into polynomial *Pptr.
 * It has two options that control the insertion in case a term of *Pptr
 * already has the degree of *Qptr, If OPT = 0, then the term is replaced;
 *  if OPT = 1, then the coefficient of *Qptr is added to that of the
 *  corresponding term of *Pptr mod m, 1 < m < R0.
 */
{
	POLYm CURSOR, Ttmp;

	if (*Pptr == NULL)
	{
		*Pptr = CREATEm();
		(*Pptr)->DEG = DEG;
		(*Pptr)->COEF = COEF;
		(*Pptr)->NEXT = NULL;
		return;
	}
	else
	{
		CURSOR = *Pptr;
		while ((CURSOR->DEG > DEG) && (CURSOR->NEXT != NULL))
			CURSOR = CURSOR->NEXT;
		if (CURSOR->DEG < DEG)
		{
			/* COEF inserted before CURSOR */
			Ttmp = CREATEm();
			Ttmp->DEG = CURSOR->DEG;
			Ttmp->COEF = CURSOR->COEF;
			Ttmp->NEXT = CURSOR->NEXT;
			CURSOR->DEG = DEG;
			CURSOR->COEF = COEF;
			CURSOR->NEXT = Ttmp;
			return;
		}
		else if (CURSOR->DEG == DEG)
		{
			if (OPT == 0)
				CURSOR->COEF = COEF;
			else
				CURSOR->COEF = ADDm(CURSOR->COEF, COEF, m);
			return;
		}
		else
		{
			/* CURSOR->DEG > DEG and CURSOR->NEXT === NULL */
			/* COEF inserted at end */
			Ttmp = CREATEm();
			Ttmp->DEG = DEG;
			Ttmp->COEF = COEF;
			Ttmp->NEXT = NULL;
			CURSOR->NEXT = Ttmp;
			return;
		}
	}
}

void PURGEPm(POLYm *Pptr)
/*
 * gets rid of any zero coefficients in the polynomial Pptr mod m.
 * for use with polynomials mod m, 1 < m < R0.
 */
{
	POLYm R1tmp, R2tmp;

	if (*Pptr != NULL)
	{
		R1tmp = *Pptr;
		R2tmp = (*Pptr)->NEXT;
		while (R2tmp != NULL)
		{
			/* purge non-leading terms */
			if (R2tmp->COEF == 0)
			{
				R1tmp->NEXT = R2tmp->NEXT;
				R2tmp->NEXT = NULL;
				DELETE_Pm(R2tmp);
				R2tmp = R1tmp->NEXT;
			}
			else
			{
				R1tmp = R2tmp;
				R2tmp = R2tmp->NEXT;
			}
		}
		if ((*Pptr)->COEF == 0)
		{
			/* purge leading term */
			R1tmp = *Pptr;
			*Pptr = (*Pptr)->NEXT;
			R1tmp->NEXT = NULL;
			DELETE_Pm(R1tmp);
		}
	}
	return;
}

POLYm INPUTPm(USI *ptr, USI m)
/*
 * inputting a polynomial Pptr.
 * for use with polynomials mod m, 1 < m < R0. 
 * *ptr = 0 indicates some garbage input and a default POLYm as output. 
 */
{
	POLYm Pptr;
	unsigned int u, COEF;
	int k;

	u = 1;
	Pptr = NULL;
	printf("Enter a (mod %u) polynomial: (degree -1 for the zero polynomial)\n", m);
	do
	{
		printf("Enter the degree of new term, (degree = -1 terminating input):\n");
		if (scanfd10(&k) == 0)
		{
			u = 0;
			k = -1;
		}
		if (k >= 0)
		{
			printf("Enter the coefficient c of x^%d (0 <= c < %u):\n", k, m);
			u = scanf("%u", &COEF);
			getchar();
			if (u != 1 || COEF >= m)
			{
				printf("inappropriate coefficient,\n");
				printf("polynomial input prematurely interrupted.\n");
				COEF = 0;
				GetReturn();
				k = -1;
				u = 0;
			}
			PINSERTPm(k, COEF, &Pptr, 0, 1);
		}
	}
	while (k >= 0);
	PURGEPm(&Pptr);
	*ptr = u;
	return (Pptr);
}

POLYm COPYPm(POLYm Pptr, USI m)
/*
 * makes a copy of the polynomial Pptr.
 * for use with polynomials mod m, 1 < m < R0.
 */
{
	POLYm Rtmp, Sptr;

	Sptr = NULL;
	Rtmp = Pptr;
	while (Rtmp != NULL)
	{
		PINSERTPm(Rtmp->DEG, Rtmp->COEF, &Sptr, 0, m);
		Rtmp = Rtmp->NEXT;
	}
	return (Sptr);
}
 
void PRINTPm(POLYm Pptr)
/*
 * outputs the terms of the polynomial Pptr.
 * for use with polynomials mod m, 1 < m < R0.
 */
{
	POLYm CURSOR;
	int k;

	if (Pptr == NULL)
	{
		printf("0");
		return;
	}
	else
	{
		CURSOR = Pptr;
		k = DEGREEPm(Pptr);
		while (CURSOR != NULL)
		{
			if (CURSOR->DEG != k)
				printf("+ ");
			if (CURSOR->DEG == 0 || CURSOR->COEF != 1)
				printf("%u", CURSOR->COEF);
			if (CURSOR->DEG == 1)
				printf("x ");
			if (CURSOR->DEG > 1)
				printf("x^%d ", CURSOR->DEG);
			CURSOR = CURSOR->NEXT;
		}
		return;
	}
}


POLYm SCALARPm(USI c, POLYm Pptr, USI m)
/*
 * multiplying the polynomial Pptr by the residue c, 0 <= c < m.
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm Sptr, Rtmp;

	if (Pptr == NULL || c == 0)
		Sptr = NULL;
	else
	{
		Sptr = COPYPm(Pptr, m);
		Rtmp = Sptr;
		while (Rtmp != NULL)
		{
			Rtmp->COEF = MULTm(Rtmp->COEF, c, m);
			Rtmp = Rtmp->NEXT;
		}
	}
	return (Sptr);
}

POLYm ADDPm(POLYm Pptr, POLYm Qptr, USI m)
/*
 * returns the sum of two polynomials.
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm CURSOR, Sptr;

		Sptr = COPYPm(Pptr, m);
		CURSOR = Qptr;
		while (CURSOR != NULL)
		{
			PINSERTPm(CURSOR->DEG, CURSOR->COEF, &Sptr, 1, m);
			CURSOR = CURSOR->NEXT;
		}
		PURGEPm(&Sptr);
		return (Sptr);
}

POLYm SUBPm(POLYm Pptr, POLYm Qptr, USI m)
/*
 * returns the difference Pptr - Qptr of two polynomials.
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm CURSOR, Sptr;
	unsigned int COEF;

		Sptr = COPYPm(Pptr, m);
		if (Qptr == NULL)
			return (Sptr);
		CURSOR = Qptr;
		while (CURSOR != NULL)
		{
			COEF = MINUSm(CURSOR->COEF, m);
			PINSERTPm(CURSOR->DEG, COEF, &Sptr, 1, m);
			CURSOR = CURSOR->NEXT;
		}
		PURGEPm(&Sptr);
		return (Sptr);
}

POLYm MULTPm(POLYm Pptr, POLYm Qptr, USI m)
/*
 * returns the product of two polynomials.
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm Sptr, CURSOR_P, CURSOR_Q;
	int DEG;
	unsigned int COEF;

	if (Pptr == NULL || Qptr == NULL)
		return (NULL);
	Sptr = NULL;
	CURSOR_P = Pptr;
	while (CURSOR_P != NULL)
	{
		CURSOR_Q = Qptr;
		while (CURSOR_Q != NULL)
		{
			DEG = CURSOR_P->DEG + CURSOR_Q->DEG;
			COEF = MULTm(CURSOR_P->COEF, CURSOR_Q->COEF, m);
			PINSERTPm(DEG, COEF, &Sptr, 1, m);
			CURSOR_Q = CURSOR_Q->NEXT;
		}
		CURSOR_P = CURSOR_P->NEXT;
	}
	PURGEPm(&Sptr);
	return (Sptr);
}

unsigned int VALPm(POLYm Pptr, USI c, USI m)
/*
 * Evaluating the polynomial Pptr at the residue c, 0 <= c < m.
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	int k;
	unsigned int a, b;
	POLYm CURSOR;

	if (Pptr == NULL)
		return (0);
	else
		a = Pptr->COEF;
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		if (CURSOR->NEXT == NULL)
			k = DEGREEPm(CURSOR);
		else
			k = DEGREEPm(CURSOR) - DEGREEPm(CURSOR->NEXT);
		b = POWER_m(c, (unsigned int)k, m);
		a = MULTm(a, b, m);
		CURSOR = CURSOR->NEXT;
		if (CURSOR != NULL)
			a = ADDm(a, CURSOR->COEF, m);
	}
	return (a);
}

POLYm DIVPm(POLYm Fptr, POLYm Gptr, USI m)
/*
 * division of polynomials: see Knuth, Vol 2, p.402
 * and H. Flanders, Scientific Pascal, p.510.
 * for use with polynomials mod m, 1 < m < R0.
 */
{
	POLYm Htmp, CURSOR, Qptr, Temp, F;
	int j, d;
	unsigned int a, g;

	Qptr = NULL;
	d = DEGREEPm(Gptr);
	j = DEGREEPm(Fptr) - d;
	if (j < 0)
		return (Qptr);
	else
	{
		F = COPYPm(Fptr, m);
		g = LEADPm(Gptr);
		while ((j = DEGREEPm(F)) >= d)
		{
			j = j - d;
			a = LEADPm(F);
			a = DIVm(a, g, m);	
			Htmp = COPYPm(Gptr, m);
			CURSOR = Htmp;
			do
			{
				CURSOR->DEG = j + CURSOR->DEG;
				CURSOR->COEF = MINUSm(CURSOR->COEF, m);
				CURSOR->COEF = MULTm(CURSOR->COEF, a, m);
				CURSOR = CURSOR->NEXT;
			}
			while (CURSOR != NULL);
			Temp = F;
			F = ADDPm(F, Htmp, m);
			DELETE_Pm(Htmp);
			DELETE_Pm(Temp);
			PINSERTPm(j, a, &Qptr, 0, m);
		}
		DELETE_Pm(F);
		return (Qptr);
	}
}

POLYm MODPm(POLYm Fptr, POLYm Gptr, USI m)
/*
 * division of polynomials: see Knuth, Vol 2, p.402
 * and H. Flanders, Scientific Pascal, p.510.
 * for use with polynomials mod m, 1 < m < R0.
 */
{
	POLYm Htmp, CURSOR, Temp, F;
	int j, d;
	unsigned int a, b, g;

	d = DEGREEPm(Gptr);
	j = DEGREEPm(Fptr) - d;
	if (j < 0)
		return COPYPm(Fptr, m);
	else if (d == 0)
		return NULL;
	else if (d == 1)
	{
		if (Gptr->NEXT == NULL)
		{
			b = VALPm(Fptr, 0, m);
			return (CONSTANTPm(b));
		}
		else
		{
			b = DIVm((Gptr->NEXT)->COEF, Gptr->COEF, m);
			b = MINUSm(b, m);
			b = VALPm(Fptr, b, m);
			return (CONSTANTPm(b));
		}
	}
	else
	{
		F = COPYPm(Fptr, m);
		g = LEADPm(Gptr);
		while ((j = DEGREEPm(F)) >= d)
		{
			j = j - d;
			a = LEADPm(F);
			a = DIVm(a, g, m);
			Htmp = COPYPm(Gptr, m);
			CURSOR = Htmp;
			do
			{
				CURSOR->DEG = j + CURSOR->DEG;
				CURSOR->COEF = MINUSm(CURSOR->COEF, m);
				CURSOR->COEF = MULTm(CURSOR->COEF, a, m);
				CURSOR = CURSOR->NEXT;
			}
			while (CURSOR != NULL);
			Temp = F;
			F = ADDPm(Temp, Htmp, m);
			DELETE_Pm(Temp);
			DELETE_Pm(Htmp);
		}
		return (F);
	}
}


POLYm MONICPm(POLYm Pptr, USI m)
/*
 * converts Pptr to a monic polynomial by multiplying it by 1/LEADPm(Pptr).
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	unsigned int g;

	g = LEADPm(Pptr);
	g = INVERSEm(g, m);
	return (SCALARPm(g, Pptr, m));
}

POLYm GCDPm(POLYm Pptr, POLYm Qptr, USI m)
/*
 * returns the gcd of polynomials Pptr and Qptr.
 * see Knuth, Vol 2, p.403-408 and H. Flanders, Scientific Pascal, p.510.
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm Ptmp, Qtmp, Rtmp;
	int j;
	
	if (Qptr == NULL)
	{
		if (Pptr == NULL)
			return (NULL);
		else
			return (MONICPm(Pptr, m));
	}
	else if (Pptr == NULL)
	{
		if (Qptr == NULL)
			return (NULL);
		else
			return (MONICPm(Qptr, m));
	}
	else
	{
		Ptmp = COPYPm(Pptr, m);
		Qtmp = COPYPm(Qptr, m);
                j = DEGREEPm(Qtmp); /* to keep the compiler quiet */
		while ((Qtmp != NULL) && ((j = DEGREEPm(Qtmp)) != 0))
		{
			Rtmp =	MODPm(Ptmp, Qtmp, m);
			DELETE_Pm(Ptmp);
			Ptmp = Qtmp;
			Qtmp = Rtmp;
		}
		if (j == 0)
		{
			Qtmp->COEF = 1;
			DELETE_Pm(Ptmp);
			return (Qtmp);
		}
		else
		{
			Qtmp = MONICPm(Ptmp, m);
			DELETE_Pm(Ptmp);
			return (Qtmp);
		}
	}
}

POLYm DERIVPm(POLYm Pptr, USI m)
/*
 * returns the derivative of a polynomial.
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm CURSOR, Sptr;

	if (DEGREEPm(Pptr) == 0)
		return (NULL);
	else
	{
		Sptr = COPYPm(Pptr, m);
  		CURSOR = Sptr;
		while (CURSOR != NULL)
		{
			CURSOR->COEF = MULTm(CURSOR->COEF, (unsigned int)CURSOR->DEG, m);
			CURSOR->DEG = CURSOR->DEG - 1;

			if (CURSOR->NEXT == NULL)
				break;
			else if ((CURSOR->NEXT)->DEG == 0)
			{
				DELETE_Pm(CURSOR->NEXT);
				CURSOR->NEXT = NULL;
				break;
			}
			else
				CURSOR = CURSOR->NEXT;
		}
		PURGEPm(&Sptr);
		return (Sptr);
	}
}

POLYm POWERPm(POLYm Pptr, USI n, USI m)
/*
 *  returns Pptr ^ n mod m, where 0 <= n < R0 * R0.
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm B, Eptr, Tmp;

	Eptr = CONSTANTPm(1);
	if (n == 0)
		return (Eptr);
	B = COPYPm(Pptr, m);
	while (1)
        {
		if (n & 1) 
		{
			Tmp = Eptr;
			Eptr = MULTPm(Tmp, B, m);
			DELETE_Pm(Tmp);
			if (n == 1)
			{
				DELETE_Pm(B);
				return (Eptr);
			}
		}
		Tmp = B;
		B = MULTPm(Tmp, Tmp, m);
		DELETE_Pm(Tmp);
		n = n >> 1;
	}
}

POLYm CONSTANTPm(USI a)
/*
 * returns the constant polynomial whose constant term is the residue a.
 * for use with polynomials mod m, 1 < m < R0, 0 <= a < m. 
 */
{
	POLYm Pptr;	

	if (a == 0)
		return (NULL);
	else
	{
		Pptr = CREATEm();
		Pptr->DEG = 0;
		Pptr->COEF = a;
		Pptr->NEXT = NULL;
		return (Pptr);
	}
}
POLYm LINEARPm(USI a, USI m)
/*
 * 0 <= a < m < R0, returns the polynomial x + a.
 */
{
	POLYm Pptr;

	Pptr = CONSTANTPm(a);
	PINSERTPm(1, 1, &Pptr, 0, m);
	return (Pptr);
}

POLYm POWERPPm(POLYm Pptr, MPI *Bptr, POLYm Qptr, USI m)
/*
 *  returns Pptr ^ *Bptr, mod Qptr, where 0 <= *Bptr; 
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm B, Eptr, Tmp;
	MPI *N, *Temp1I, *Temp2I;

	Eptr = CONSTANTPm(1);
	if (Bptr->S == 0)
		return (Eptr);
	N = COPYI(Bptr);
	B = COPYPm(Pptr, m);
	Temp1I = ONEI();
	while (1)
        {
		if (N->V[0] & 1) 
		{
			Tmp = Eptr;
			Eptr = MULTPm(Tmp, B, m);
			DELETE_Pm(Tmp);
			Tmp = Eptr;
			Eptr = MODPm(Tmp, Qptr, m);
			DELETE_Pm(Tmp);
			if (EQUALI(N, Temp1I))
			{
				DELETE_Pm(B);
				FREEMPI(N);
				FREEMPI(Temp1I);
				return (Eptr);
			}
		}
		Tmp = B;
		B = MULTPm(Tmp, Tmp, m);
		DELETE_Pm(Tmp);
		Tmp = B;
		B = MODPm(Tmp, Qptr, m);
		DELETE_Pm(Tmp);
		Temp2I = N;
		N = INT0_(N, 2);
		FREEMPI(Temp2I);
	}
}

POLYm POWER0Pm(POLYm Pptr, USI n, POLYm Qptr, USI m)
/*
 *  returns Pptr ^ n, mod Qptr, where 0 <= n, 
 * for use with polynomials mod m, 1 < m < R0. 
 */
{
	POLYm B, Eptr, Tmp;

	Eptr = CONSTANTPm(1);
	if (n == 0)
		return (Eptr);
	B = COPYPm(Pptr, m);
	while (1)
        {
		if (n & 1) 
		{
			Tmp = Eptr;
			Eptr = MULTPm(Tmp, B, m);
			DELETE_Pm(Tmp);
			Tmp = Eptr;
			Eptr = MODPm(Tmp, Qptr, m);
			DELETE_Pm(Tmp);
			if (n == 1)
			{
				DELETE_Pm(B);
				return (Eptr);
			}
		}
		Tmp = B;
		B = MULTPm(Tmp, Tmp, m);
		DELETE_Pm(Tmp);
		Tmp = B;
		B = MODPm(Tmp, Qptr, m);
		DELETE_Pm(Tmp);
		n = n >> 1;
	}
}

POLYm EUCLIDPm(POLYm Pptr, POLYm Qptr, POLYm *Hptr, POLYm *Kptr, USI m)
/*
 * gcd(Pptr, Qptr) = Hptr * Pptr + Kptr * Qptr.
 * deg(Hptr) < deg(Qptr), deg(Kptr) < deg(Pptr).
 */
{
	POLYm Q, A, B, C, H1, H2, K1, K2, L1, L2, Tmp, Tmp1, Tmp2;
	unsigned int g;

	if (Qptr == NULL)
	{
		if (Pptr != NULL)
		{
			g = LEADPm(Pptr);
			g = INVERSEm(g, m);
			*Hptr = CONSTANTPm(g);
			*Kptr = NULL;
			return (MONICPm(Pptr, m));
		}
		else
		{
			*Hptr = NULL;
			*Kptr = NULL;
			return (NULL);
		}
	}
	A = COPYPm(Pptr, m);
	B = COPYPm(Qptr, m);
	C = MODPm(A, B, m);
	if (C == NULL)
	{
		*Hptr = NULL;
		g = LEADPm(Qptr);
		g = INVERSEm(g, m);
		*Kptr = CONSTANTPm(g);
		DELETE_Pm(A);
		DELETE_Pm(B);
		return (MONICPm(Qptr, m));
	}
	L1 = CONSTANTPm(1);
	K1 = NULL;
	L2 = NULL;
	K2 = CONSTANTPm(1);
	while (C != NULL)
	{
		Q = DIVPm(A, B, m);
		DELETE_Pm(A);
		A = B;
		B = C;
		C = MODPm(A, B, m);
		Tmp1 = MULTPm(Q, K1, m);
		Tmp2 = MULTPm(Q, K2, m);
		DELETE_Pm(Q);
		H1 = SUBPm(L1, Tmp1, m);
		H2 = SUBPm(L2, Tmp2, m);
		DELETE_Pm(Tmp1);
		DELETE_Pm(Tmp2);
		DELETE_Pm(L1);
		L1 = K1;
		DELETE_Pm(L2);
		L2 = K2;
		K1 = H1;
		K2 = H2;
	}
	g = LEADPm(B);
	g = INVERSEm(g, m);
	Tmp = MODPm(K1, Qptr, m);
	*Hptr = SCALARPm(g, Tmp, m);
	DELETE_Pm(Tmp);
	Tmp = MODPm(K2, Pptr, m);
	*Kptr = SCALARPm(g, Tmp, m);
	DELETE_Pm(Tmp);
	DELETE_Pm(K1);
	DELETE_Pm(K2);
	DELETE_Pm(L1);
	DELETE_Pm(L2);
	DELETE_Pm(A);
	DELETE_Pm(C);
	Tmp = MONICPm(B, m);
	DELETE_Pm(B);
	return (Tmp);
}

POLYm CHANGETOPm(POLYI Pptr, USI m)
/*
 * converts a polynomial with MPI coefficients into one with coefficients mod m
 */
{
	POLYI CURSOR;
	POLYm P;
	int DEG;
	unsigned int COEF;

	P = NULL;
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		DEG = CURSOR->DEG;
		COEF = MOD_(CURSOR->COEF, m);
		PINSERTPm(DEG, COEF, &P, 0, m);
		CURSOR = CURSOR->NEXT;
	}	
	PURGEPm(&P);
	return (P);
}

POLYI CHANGETOPI(POLYm Pptr)
/*
 * Converts a POLYm into a POLYI. 
 */
{
	POLYm CURSOR;
	POLYI P;
	MPI *TempI;

	P = NULL;
	CURSOR = Pptr;
	while (CURSOR != NULL)
	{
		TempI = CHANGE(CURSOR->COEF);
		PINSERTPI(CURSOR->DEG, TempI, &P, 0);
		FREEMPI(TempI);
		CURSOR = CURSOR->NEXT;
	}	
	return (P);
}

void HENSELPm(POLYI Fptr, POLYm u1ptr, POLYm u2ptr, POLYm v1ptr, POLYm v2ptr, POLYm *U1ptr, POLYm *U2ptr, POLYm *V1ptr, POLYm *V2ptr, USI q)
/*
 * input: polynomials mod m : u1ptr, u2ptr, v1ptr, v2ptr, with Fptr a 
 * polynomial with MPI coefficients; gcd(m, LEADPm(Fptr)) = 1;
 * LEADPI(Fptr) = LEADPm(u1ptr) mod q; u2ptr monic;
 * DEGREEPm(viptr) < DEGREEPm(uiptr) for i = 1, 2;
 * u1ptr*v2ptr + u2ptr*v1ptr = 1 mod q;
 * LEADPI(Fptr) = LEADPm(U1ptr) mod q^2; U2ptr monic;
 * U1ptr*V2ptr + U2ptr*V1ptr = 1 mod q^2;
 * DEGREEPm(Viptr) < DEGREEPm(Viptr) for i = 1, 2;
 * Uiptr = uiptr mod q, Viptr = viptr mod q, i = 1, 2.
 *
 *The algorithm is called the quadratic lift version of Hensel lifting and is 
 * due to Zassenhaus. See E. Kaltofen, Factorization of polynomials, pp95-113,
 * Computing Supplement 4, Springer 1982.
 */
{
	POLYm w1, w2, f, t, CURSOR, one, b, Temp, Temp1, Temp2, Temp3;
	unsigned int m;

	m = q * q;
	/* Step H1 */
	Temp = COPYPm(u1ptr, m);
	Temp->COEF = MOD_(Fptr->COEF, m);
	Temp1 = MULTPm(Temp, u2ptr, m);
	f = CHANGETOPm(Fptr, m);
	t = SUBPm(f, Temp1, m);
	/* DEGREEPm(t) < DEGREEPI(Fptr) */
	DELETE_Pm(Temp1);
	DELETE_Pm(f);
	CURSOR = t;
	/* the coefficients of t are divisible by q */
	while (CURSOR != NULL)
	{
		CURSOR->COEF = CURSOR->COEF / q;
		CURSOR = CURSOR->NEXT;
	}
	Temp1 = MULTPm(t, v1ptr, q);
	Temp2 = MULTPm(t, v2ptr, q);
	w1 = MODPm(Temp1, u1ptr, q);
	w2 = MODPm(Temp2, u2ptr, q);
	DELETE_Pm(Temp1);
	DELETE_Pm(Temp2);
	Temp1 = SCALARPm(q, w1, m);
	Temp2 = SCALARPm(q, w2, m);
	DELETE_Pm(w1);
	DELETE_Pm(w2);
	*U1ptr = ADDPm(Temp, Temp1, m);
	*U2ptr = ADDPm(u2ptr, Temp2, m);
	DELETE_Pm(Temp1);
	DELETE_Pm(Temp2);
	/* at this stage t = w1*v2ptr + w2*v1ptr mod q */
	DELETE_Pm(t);

	/* Step H2 */
	Temp1 = MULTPm(u1ptr, v2ptr, m);
	Temp2 = MULTPm(u2ptr, v1ptr, m);
	Temp3 = ADDPm(Temp1, Temp2, m);
	DELETE_Pm(Temp1);
	DELETE_Pm(Temp2);
	one = CONSTANTPm(1);
	
	b = SUBPm(one, Temp3, m);
	/* DEGREEPm(b) < DEGREEPI(Fptr) */
	DELETE_Pm(one);
	DELETE_Pm(Temp3);
	CURSOR = b;
	/* the coefficients of b are divisible by q */
	while (CURSOR != NULL)
	{
		CURSOR->COEF = CURSOR->COEF / q;
		CURSOR = CURSOR->NEXT;
	}
	Temp1 = MULTPm(b, v1ptr, q);
	Temp2 = MULTPm(b, v2ptr, q);
	DELETE_Pm(b);
	w1 = MODPm(Temp1, u1ptr, q);
	w2 = MODPm(Temp2, u2ptr, q);
	DELETE_Pm(Temp);
	DELETE_Pm(Temp1);
	DELETE_Pm(Temp2);
	Temp1 = SCALARPm(q, w1, m);
	Temp2 = SCALARPm(q, w2, m);
	*V1ptr = ADDPm(v1ptr, Temp1, m);
	*V2ptr = ADDPm(v2ptr, Temp2, m);
	DELETE_Pm(Temp1);
	DELETE_Pm(Temp2);
	/* at this stage 1 = w1*v2ptr + w2*v1ptr mod m */

}
